@@ -22,7 +22,7 @@
 */
 
 #include "csoundCore.h"
-#include <sndfile.h>
+#include "soundio.h"
 #include "envvar.h"
 #include <ctype.h>
 #include <math.h>
@@ -31,13 +31,6 @@
 #include <fcntl.h>
 #endif
 
-#if defined(mac_classic) && defined(__MWERKS__)
-#include <unix.h>
-#endif
-#if defined(SYMANTEC)
-extern off_t lseek(int, off_t, int);
-#endif
-
 #include "namedins.h"
 
 /* list of environment variables used by Csound */
@@ -49,8 +42,8 @@ static const char *envVar_list[] = {
     "CS_LANG",
     "HOME",
     "INCDIR",
-    "OPCODEDIR",
-    "OPCODEDIR64",
+    "OPCODE6DIR",
+    "OPCODE6DIR64",
     "RAWWAVE_PATH",
     "SADIR",
     "SFDIR",
@@ -61,12 +54,26 @@ static const char *envVar_list[] = {
     NULL
 };
 
-#define ENV_DB          (((CSOUND*) csound)->envVarDB)
+typedef struct CSFILE_ {
+    struct CSFILE_  *nxt;
+    struct CSFILE_  *prv;
+    int             type;
+    int             fd;
+    FILE            *f;
+    SNDFILE         *sf;
+    void            *cb;
+    int    async_flag;
+    int    items;
+    int    pos;
+    MYFLT *buf;
+    int    bufsize;
+    char            fullName[1];
+} CSFILE;
 
 #if defined(MSVC)
 #define RD_OPTS  _O_RDONLY | _O_BINARY
 #define WR_OPTS  _O_TRUNC | _O_CREAT | _O_WRONLY | _O_BINARY,_S_IWRITE
-#elif defined(mac_classic) || defined(SYMANTEC) || defined(WIN32)
+#elif defined(WIN32)
 #define RD_OPTS  O_RDONLY | O_BINARY
 #define WR_OPTS  O_TRUNC | O_CREAT | O_WRONLY | O_BINARY, 0644
 #elif defined DOSGCC
@@ -80,12 +87,6 @@ static const char *envVar_list[] = {
 #define WR_OPTS  O_TRUNC | O_CREAT | O_WRONLY | O_BINARY, 0644
 #endif
 
-typedef struct envVarEntry_s {
-    struct envVarEntry_s *nxt;  /* pointer to next link in chain        */
-    char    *name;              /* name of environment variable         */
-    char    *value;             /* value of environment variable        */
-} envVarEntry_t;
-
 typedef struct searchPathCacheEntry_s {
     char    *name;
     struct searchPathCacheEntry_s   *nxt;
@@ -97,16 +98,6 @@ typedef struct nameChain_s {
     char    s[1];
 } nameChain_t;
 
-typedef struct CSFILE_ {
-    struct CSFILE_  *nxt;
-    struct CSFILE_  *prv;
-    int             type;
-    int             fd;
-    FILE            *f;
-    SNDFILE         *sf;
-    char            fullName[1];
-} CSFILE;
-
 /* Space for 16 global environment variables, */
 /* 32 bytes for name and 480 bytes for value. */
 /* Only written by csoundSetGlobalEnv().      */
@@ -116,18 +107,6 @@ static char globalEnvVars[8192] = { (char) 0 };
 #define globalEnvVarName(x)   ((char*) &(globalEnvVars[(int) (x) << 9]))
 #define globalEnvVarValue(x)  ((char*) &(globalEnvVars[((int) (x) << 9) + 32]))
 
-static inline envVarEntry_t **getEnvVarChain(CSOUND *csound, const char *name)
-{
-    unsigned char h;
-    /* check for trivial cases */
-    if (UNLIKELY(ENV_DB == NULL || name == NULL || name[0] == '\0'))
-      return NULL;
-    /* calculate hash value */
-    h = name_hash_2(csound, name);
-    /* return with pointer from table */
-    return &(((envVarEntry_t**) ENV_DB)[(int) h]);
-}
-
 static int is_valid_envvar_name(const char *name)
 {
     char *s;
@@ -151,8 +130,6 @@ static int is_valid_envvar_name(const char *name)
 
 PUBLIC const char *csoundGetEnv(CSOUND *csound, const char *name)
 {
-    envVarEntry_t **pp, *p;
-
     if (csound == NULL) {
       int i;
       if (name == NULL || name[0] == '\0')
@@ -163,15 +140,10 @@ PUBLIC const char *csoundGetEnv(CSOUND *csound, const char *name)
       }
       return (const char*) getenv(name);
     }
-    pp = getEnvVarChain(csound, name);
-    if (pp == NULL)
-      return (const char*) NULL;
-    p = *pp;
-    while (p != NULL && sCmp(p->name, name) != 0)
-      p = p->nxt;
-    if (p == NULL)
-      return (const char*) NULL;
-    return (const char*) p->value;
+
+    if (csound->envVarDB == NULL) return NULL;
+
+    return (const char*) cs_hash_table_get(csound, csound->envVarDB, (char*)name);
 }
 
 /**
@@ -215,15 +187,12 @@ PUBLIC int csoundSetGlobalEnv(const char *name, const char *value)
 int csoundSetEnv(CSOUND *csound, const char *name, const char *value)
 {
     searchPathCacheEntry_t  *ep, *nxt;
-    envVarEntry_t           **pp, *p;
-    char                    *s1, *s2;
+    char                    *oldValue;
 
     /* check for valid parameters */
     if (UNLIKELY(csound == NULL || !is_valid_envvar_name(name)))
       return CSOUND_ERROR;
-    pp = getEnvVarChain(csound, name);
-    if (UNLIKELY(pp == NULL))
-      return CSOUND_ERROR;
+
     /* invalidate search path cache */
     ep = (searchPathCacheEntry_t*) csound->searchPathCache;
     while (ep != NULL) {
@@ -232,49 +201,31 @@ int csoundSetEnv(CSOUND *csound, const char *name, const char *value)
       ep = nxt;
     }
     csound->searchPathCache = NULL;
-    p = *pp;
-    s1 = (char*) name;
-    s2 = NULL;
-    /* copy value */
-    if (value != NULL) {
-      s2 = (char*) mmalloc(csound, strlen(value) + 1);
-      strcpy(s2, value);
-    }
-    /* is this variable already defined ? */
-    while (p != NULL && sCmp(p->name, name) != 0)
-      p = p->nxt;
-    if (p != NULL) {
-      /* yes, only need to replace value */
-      if (p->value != NULL)
-        mfree(csound, p->value);
-      p->value = s2;
-    }
-    else {
-      /* no, need to allocate new entry, and copy name too */
-      p = (envVarEntry_t*) mmalloc(csound, sizeof(envVarEntry_t));
-      s1 = (char*) mmalloc(csound, (size_t) strlen(name) + (size_t) 1);
-      strcpy(s1, name);
-      /* store pointers to name and value, and link into chain */
-      p->nxt = *pp;
-      p->name = s1;
-      p->value = s2;
-      *pp = p;
+
+
+    oldValue = cs_hash_table_get(csound, csound->envVarDB, (char*)name);
+    if (oldValue != NULL) {
+        mfree(csound, oldValue);
     }
+
+    cs_hash_table_put(csound, csound->envVarDB,
+                      (char*)name, cs_strdup(csound, (char*)value));
+
     /* print debugging info if requested */
     if (csound->oparms->odebug) {
-      csound->Message(csound, Str("Environment variable '%s' has been set to "),
+      csoundMessage(csound, Str("Environment variable '%s' has been set to "),
                               name);
       if (value == NULL)
-        csound->Message(csound, "NULL\n");
+        csoundMessage(csound, "NULL\n");
       else
-        csound->Message(csound, "'%s'\n", s2);
+        csoundMessage(csound, "'%s'\n", value);
     }
     /* report success */
     return CSOUND_SUCCESS;
 }
 
 /**
- * Append 'value' to environment variable 'name', using ';' as
+ * Append 'value' to environment variable 'name', using ENVSEP as
  * separator character.
  * Returns CSOUND_SUCCESS on success, and CSOUND_ERROR or CSOUND_MEMORY
  * if the environment variable could not be set for some reason.
@@ -295,13 +246,18 @@ int csoundAppendEnv(CSOUND *csound, const char *name, const char *value)
       return csoundSetEnv(csound, name, value);
     if (value == NULL || value[0] == '\0')
       return CSOUND_SUCCESS;
-    /* allocate new value (+ 2 bytes for ';' and null character) */
+    /* allocate new value (+ 2 bytes for ENVSEP and null character) */
     newval = (char*) mmalloc(csound, (size_t) strlen(oldval)
                              + (size_t) strlen(value) + (size_t) 2);
     /* append to old value */
     strcpy(newval, oldval);     /* These are safe as space calculated above */
-    strcat(newval, ";");
+    //    printf("%d: newval = %s\n", __LINE__, newval);
+    // should be a better way
+    newval[strlen(oldval)]= ENVSEP;
+    newval[strlen(oldval)+1]= '\0';
+    //    printf("%d: newval = %s\n", __LINE__, newval);
     strcat(newval, value);
+    //    printf("%d: newval = %s\n", __LINE__, newval);
     /* set variable */
     retval = csoundSetEnv(csound, name, newval);
     mfree(csound, newval);
@@ -310,7 +266,7 @@ int csoundAppendEnv(CSOUND *csound, const char *name, const char *value)
 }
 
 /**
- * Prepend 'value' to environment variable 'name', using ';' as
+ * Prepend 'value' to environment variable 'name', using ENVSEP as
  * separator character.
  * Returns CSOUND_SUCCESS on success, and CSOUND_ERROR or CSOUND_MEMORY
  * if the environment variable could not be set for some reason.
@@ -336,8 +292,12 @@ int csoundPrependEnv(CSOUND *csound, const char *name, const char *value)
                                      + (size_t) strlen(value) + (size_t) 2);
     /* prepend to old value */
     strcpy(newval, value);
-    strcat(newval, ";");
+    //    printf("%d: newval = %s\n", __LINE__,  newval);
+    newval[strlen(value)]= ENVSEP;
+    newval[strlen(value)+1]= '\0';
+    //    printf("%d: newval = %s\n", __LINE__,  newval);
     strcat(newval, oldval);
+    //    printf("%d: newval = %s\n", __LINE__,  newval);
     /* set variable */
     retval = csoundSetEnv(csound, name, newval);
     mfree(csound, newval);
@@ -356,12 +316,10 @@ int csoundInitEnv(CSOUND *csound)
 {
     int i, retval;
     /* check if already initialised */
-    if (ENV_DB != NULL)
+    if (csound->envVarDB != NULL)
       return CSOUND_SUCCESS;
     /* allocate table */
-    ENV_DB = (void*) mmalloc(csound, sizeof(envVarEntry_t*) * (size_t) 256);
-    for (i = 0; i < 256; i++)
-      ((envVarEntry_t**) ENV_DB)[i] = (envVarEntry_t*) NULL;
+    csound->envVarDB = cs_hash_table_create(csound);
     /* copy standard Csound environment variables */
     for (i = 0; envVar_list[i] != NULL; i++) {
       const char  *name = envVar_list[i];
@@ -405,7 +363,7 @@ int csoundParseEnv(CSOUND *csound, const char *s)
     value = strchr(name, '=');
     append_mode = 0;
     if (UNLIKELY(value == NULL || value == name)) {
-      sprintf(msg, " *** invalid format for --env\n");
+      strcpy(msg, " *** invalid format for --env\n");
       retval = CSOUND_ERROR;
       goto err_return;
     }
@@ -415,7 +373,7 @@ int csoundParseEnv(CSOUND *csound, const char *s)
       *(value - 2) = '\0';
     }
     if (UNLIKELY(!is_valid_envvar_name(name))) {
-      sprintf(msg, " *** invalid environment variable name\n");
+      strcpy(msg, " *** invalid environment variable name\n");
       retval = CSOUND_ERROR;
       goto err_return;
     }
@@ -425,13 +383,13 @@ int csoundParseEnv(CSOUND *csound, const char *s)
     else
       retval = csoundAppendEnv(csound, name, value);
     if (UNLIKELY(retval == CSOUND_MEMORY))
-      sprintf(msg, " *** memory allocation failure\n");
+      strcpy(msg, " *** memory allocation failure\n");
     else
-      sprintf(msg, " *** error setting environment variable\n");
+      strcpy(msg, " *** error setting environment variable\n");
 
  err_return:
     if (UNLIKELY(retval != CSOUND_SUCCESS))
-      csound->Message(csound, Str(msg));
+      csoundMessage(csound, Str(msg));
     if (name != NULL)
       mfree(csound, name);
     return retval;
@@ -455,7 +413,7 @@ char **csoundGetSearchPathFromEnv(CSOUND *csound, const char *envList)
     len = (int) strlen(envList);
     /* split environment variable list to tokens */
     for (i = j = 0; i <= len; i++) {
-      if (envList[i] == ';' || envList[i] == '\0') {
+      if (envList[i] == ';' || envList[i] == ':' || envList[i] == '\0') {
         if (i > j) {
           tmp = (nameChain_t*) mmalloc(csound, sizeof(nameChain_t) + (i - j));
           for (k = 0; j < i; j++, k++)
@@ -492,7 +450,7 @@ char **csoundGetSearchPathFromEnv(CSOUND *csound, const char *envList)
       else
         len = -1;
       for (i = j = 0; i <= len; i++) {
-        if (s[i] == ';' || s[i] == '\0') {
+        if (s[i] == ';' || s[i] == ':' || s[i] == '\0') {
           if (i > j) {
             tmp = (nameChain_t*) mmalloc(csound, sizeof(nameChain_t)
                                                  + (i - j) + 1);
@@ -589,18 +547,11 @@ int csoundIsNameFullpath(const char *name)
 #ifdef WIN32
     if (isalpha(name[0]) && name[1] == ':') return 1;
 #endif
-#ifndef mac_classic
     if (name[0] == DIRSEP) /* ||
         (name[0] == '.' && (name[1] == DIRSEP ||
                             (name[1] == '.' && name[2] == DIRSEP)))) */
       return 1;
     return 0;
-#else
-    /* MacOS full paths contain DIRSEP but do not start with it */
-    if (name[0] == DIRSEP || strchr(name, DIRSEP) == NULL)
-      return 0;
-    return 1;
-#endif
 }
 
 /** Check if name is a relative pathname for this platform.  Bare
@@ -608,16 +559,9 @@ int csoundIsNameFullpath(const char *name)
  */
 int csoundIsNameRelativePath(const char *name)
 {
-#ifndef mac_classic
     if (name[0] != DIRSEP && strchr(name, DIRSEP) != NULL)
       return 1;
     return 0;
-#else
-    /* MacOS relative paths begin with DIRSEP */
-    if (name[0] == DIRSEP)
-      return 1;
-    return 0;
-#endif
 }
 
 /** Check if name is a "leaf" (bare) filename for this platform. */
@@ -654,13 +598,8 @@ char* csoundConcatenatePaths(CSOUND* csound, const char *path1,
     }
 
     start2 = path2;
-#ifndef mac_classic
     /* ignore "./" at the beginning */
     if (path2[0] == '.' && path2[1] == DIRSEP) start2 = path2 + 2;
-#else
-    /* ignore the first ':' of path2 if any are present */
-    if (path2[0] == ':') start2 = path2 + 1;
-#endif
 
     result = (char*) mmalloc(csound, (size_t)len1+(size_t)len2+2);
     strcpy(result, path1);
@@ -748,7 +687,6 @@ char *csoundGetDirectoryForPath(CSOUND* csound, const char * path) {
 
     if (csoundIsNameFullpath(tempPath))
     {
-#ifndef mac_classic
         /* check if root directory */
         if (lastIndex == tempPath) {
             partialPath = (char *)mmalloc(csound, 2);
@@ -774,8 +712,6 @@ char *csoundGetDirectoryForPath(CSOUND* csound, const char * path) {
             return partialPath;
         }
 #  endif
-#endif  /* no special case needed on OS 9 for root directory */
-        /* not the root directory or we are on OS 9 */
         len = (lastIndex - tempPath);
 
         partialPath = (char *)mcalloc(csound, len + 1);
@@ -790,7 +726,7 @@ char *csoundGetDirectoryForPath(CSOUND* csound, const char * path) {
     /* we have a relative path or just a filename */
     cwd = mmalloc(csound, 512);
     if (UNLIKELY(getcwd(cwd, 512)==NULL)) {
-      csoundDie(csound, "Current directory path name too long\n");
+      csoundDie(csound, Str("Current directory path name too long\n"));
     }
 
     if (lastIndex == NULL) {
@@ -948,9 +884,9 @@ static int csoundFindFile_Fd(CSOUND *csound, char **fullName,
  * and if it is still not found, a pathname list that is created the
  * following way is searched:
  *   1. if envList is NULL or empty, no directories are searched
- *   2. envList is parsed as a ';' separated list of environment variable
- *      names, and all environment variables are expanded and expected to
- *      contain a ';' separated list of directory names
+ *   2. envList is parsed as a ';' or ':' separated list of environment
+ *      variable names, and all environment variables are expanded and
+ *      expected to contain a ';' or ':' separated list of directory names
  *   2. all directories in the resulting pathname list are searched, starting
  *      from the last and towards the first one, and the directory where the
  *      file is found first will be used
@@ -1013,9 +949,6 @@ char *csoundFindOutputFile(CSOUND *csound,
 /**
  * Open a file and return handle.
  *
- * This function has been replaced by csoundFileOpenWithType().
- * Please do not use it in new code.
- *
  * CSOUND *csound:
  *   Csound instance pointer
  * void *fd:
@@ -1043,33 +976,17 @@ char *csoundFindOutputFile(CSOUND *csound,
  *   list of environment variables for search path (see csoundFindInputFile()
  *   for details); if NULL, the specified name is used as it is, without any
  *   conversion or search.
+ * int csFileType:
+ *   A value from the enumeration CSOUND_FILETYPES (see soundCore.h)
+ * int isTemporary:
+ *   1 if this file will be deleted when Csound is finished.
+ *   Otherwise, 0.
  * return value:
  *   opaque handle to the opened file, for use with csoundGetFileName() or
  *   csoundFileClose(), or storing in FDCH.fd.
  *   On failure, NULL is returned.
  */
 
-void *csoundFileOpen(CSOUND *csound, void *fd, int type,
-                     const char *name, void *param, const char *env)
-{
-    return csoundFileOpenWithType(csound, fd, type, name, param, env,
-                                  CSFTYPE_UNKNOWN, 0);
-}
-
-/**
- * Open a file and return handle.
- *
- * Same as csoundFileOpen() with two additional parameters describing the
- * type of file being opened and whether it is a temporary file.  This
- * function replaces csoundFileOpen().  This additional information is
- * used as metadata to be passed to the host application's FileOpenCallback.
- *
- * int csFileType:
- *   A value from the enumeration CSOUND_FILETYPES (see soundCore.h)
- * int isTemporary:
- *   1 if this file will be deleted when Csound is finished.
- *   Otherwise, 0.
- */
 void *csoundFileOpenWithType(CSOUND *csound, void *fd, int type,
                      const char *name, void *param, const char *env,
                      int csFileType, int isTemporary)
@@ -1080,6 +997,7 @@ void *csoundFileOpenWithType(CSOUND *csound, void *fd, int type,
     SF_INFO sfinfo;
     int     tmp_fd = -1, nbytes = (int) sizeof(CSFILE);
 
+
     /* check file type */
     if (UNLIKELY((unsigned int) (type - 1) >= (unsigned int) CSFILE_SND_W)) {
       csoundErrorMsg(csound, Str("internal error: csoundFileOpen(): "
@@ -1134,6 +1052,7 @@ void *csoundFileOpenWithType(CSOUND *csound, void *fd, int type,
       mfree(csound, fullName);
       env = NULL;
     }
+
     /* if sound file, re-open file descriptor with libsndfile */
     switch (type) {
       case CSFILE_STD:                          /* stdio */
@@ -1198,6 +1117,10 @@ void *csoundFileOpenWithType(CSOUND *csound, void *fd, int type,
                                 writing, isTemporary);
     }
     /* return with opaque file handle */
+    p->cb = NULL;
+    p->async_flag = 0;
+    p->buf = NULL;
+    p->bufsize = 0;
     return (void*) p;
 
  err_return:
@@ -1219,6 +1142,7 @@ void *csoundFileOpenWithType(CSOUND *csound, void *fd, int type,
     return NULL;
 }
 
+
 /**
  * Allocate a file handle for an existing file already opened with open(),
  * fopen(), or sf_open(), for later use with csoundFileClose() or
@@ -1250,6 +1174,7 @@ void *csoundCreateFileHandle(CSOUND *csound,
     p->fd = -1;
     p->f = (FILE*) NULL;
     p->sf = (SNDFILE*) NULL;
+    p->cb = NULL;
     strcpy(&(p->fullName[0]), fullName);
     /* open file */
     switch (type) {
@@ -1275,6 +1200,7 @@ void *csoundCreateFileHandle(CSOUND *csound,
       ((CSFILE*) csound->open_files)->prv = p;
     csound->open_files = (void*) p;
     /* return with opaque file handle */
+    p->cb = NULL;
     return (void*) p;
 }
 
@@ -1295,8 +1221,39 @@ int csoundFileClose(CSOUND *csound, void *fd)
 {
     CSFILE  *p = (CSFILE*) fd;
     int     retval = -1;
-
-    /* close file */
+   if(p->async_flag == ASYNC_GLOBAL) {
+     csound->WaitThreadLockNoTimeout(csound->file_io_threadlock);
+     /* close file */
+    switch (p->type) {
+      case CSFILE_FD_R:
+      case CSFILE_FD_W:
+        retval = close(p->fd);
+        break;
+      case CSFILE_STD:
+        retval = fclose(p->f);
+        break;
+      case CSFILE_SND_R:
+      case CSFILE_SND_W:
+        if(p->sf)
+        retval = sf_close(p->sf);
+        p->sf = NULL;
+        if (p->fd >= 0)
+          retval |= close(p->fd);
+        break;
+    }
+    /* unlink from chain of open files */
+    if (p->prv == NULL)
+      csound->open_files = (void*) p->nxt;
+    else
+      p->prv->nxt = p->nxt;
+    if (p->nxt != NULL)
+      p->nxt->prv = p->prv;
+    if(p->buf != NULL) mfree(csound, p->buf);
+    p->bufsize = 0;
+    csound->DestroyCircularBuffer(csound, p->cb);
+    csound->NotifyThreadLock(csound->file_io_threadlock);
+   } else {
+   /* close file */
     switch (p->type) {
       case CSFILE_FD_R:
       case CSFILE_FD_W:
@@ -1312,15 +1269,17 @@ int csoundFileClose(CSOUND *csound, void *fd)
           retval |= close(p->fd);
         break;
     }
-    /* unlink from chain of open files */
+   /* unlink from chain of open files */
     if (p->prv == NULL)
       csound->open_files = (void*) p->nxt;
     else
       p->prv->nxt = p->nxt;
     if (p->nxt != NULL)
       p->nxt->prv = p->prv;
+   }
     /* free allocated memory */
     mfree(csound, fd);
+
     /* return with error value */
     return retval;
 }
@@ -1331,5 +1290,174 @@ void close_all_files(CSOUND *csound)
 {
     while (csound->open_files != NULL)
       csoundFileClose(csound, csound->open_files);
+    if (csound->file_io_start) {
+        pthread_join(csound->file_io_thread, NULL);
+        if (csound->file_io_threadlock != NULL)
+         csound->DestroyThreadLock(csound->file_io_threadlock);
+    }
 }
 
+/* The fromScore parameter should be 1 if opening a score include file,
+   0 if opening an orchestra include file */
+void *fopen_path(CSOUND *csound, FILE **fp, char *name, char *basename,
+                 char *env, int fromScore)
+{
+  void *fd;
+  int  csftype = (fromScore ? CSFTYPE_SCO_INCLUDE : CSFTYPE_ORC_INCLUDE);
+
+  /* First try to open name given */
+  fd = csound->FileOpen2(csound, fp, CSFILE_STD, name, "rb", NULL,
+                         csftype, 0);
+  if (fd != NULL)
+    return fd;
+  /* if that fails try in base directory */
+  if (basename != NULL) {
+    char *dir, *name_full;
+    if ((dir = csoundSplitDirectoryFromPath(csound, basename)) != NULL) {
+      name_full = csoundConcatenatePaths(csound, dir, name);
+      fd = csound->FileOpen2(csound, fp, CSFILE_STD, name_full, "rb", NULL,
+                             csftype, 0);
+      mfree(csound, dir);
+      mfree(csound, name_full);
+      if (fd != NULL)
+        return fd;
+    }
+  }
+  /* or use env argument */
+  fd = csound->FileOpen2(csound, fp, CSFILE_STD, name, "rb", env,
+                         csftype, 0);
+  return fd;
+}
+
+void *file_iothread(void *p);
+
+void *csoundFileOpenWithType_Async(CSOUND *csound, void *fd, int type,
+                     const char *name, void *param, const char *env,
+                                   int csFileType, int buffsize, int isTemporary)
+{
+    CSFILE *p;
+    if ((p = (CSFILE *) csoundFileOpenWithType(csound,fd,type,name,param,env,
+                                               csFileType,isTemporary)) == NULL)
+      return NULL;
+
+    if (csound->file_io_start == 0) {
+      csound->file_io_start = 1;
+      csound->file_io_threadlock = csound->CreateThreadLock();
+      csound->NotifyThreadLock(csound->file_io_threadlock);
+      pthread_create(&csound->file_io_thread,NULL, file_iothread, (void *) csound);
+    }
+    csound->WaitThreadLockNoTimeout(csound->file_io_threadlock);
+    p->async_flag = ASYNC_GLOBAL;
+
+    p->cb = csound->CreateCircularBuffer(csound, buffsize*4, sizeof(MYFLT));
+    p->items = 0;
+    p->pos = 0;
+    p->bufsize = buffsize;
+    p->buf = (MYFLT *) mcalloc(csound, sizeof(MYFLT)*buffsize);
+    csound->NotifyThreadLock(csound->file_io_threadlock);
+
+    if (p->cb == NULL || p->buf == NULL) {
+      /* close file immediately */
+      csoundFileClose(csound, (void *) p);
+      return NULL;
+    }
+    return (void *) p;
+}
+
+unsigned int csoundReadAsync(CSOUND *csound, void *handle,
+                             MYFLT *buf, int items)
+{
+    CSFILE *p = handle;
+    if(p != NULL &&  p->cb != NULL)
+    return csound->ReadCircularBuffer(csound, p->cb, buf, items);
+    else return 0;
+}
+
+unsigned int csoundWriteAsync(CSOUND *csound, void *handle,
+                              MYFLT *buf, int items)
+{
+    CSFILE *p = handle;
+    if(p != NULL &&  p->cb != NULL)
+    return csound->WriteCircularBuffer(csound, p->cb, buf, items);
+    else return 0;
+}
+
+int csoundFSeekAsync(CSOUND *csound, void *handle, int pos, int whence){
+     CSFILE *p = handle;
+     int ret = 0;
+     csound->WaitThreadLockNoTimeout(csound->file_io_threadlock);
+     switch (p->type) {
+      case CSFILE_FD_R:
+        break;
+      case CSFILE_FD_W:
+        break;
+      case CSFILE_STD:
+        break;
+     case CSFILE_SND_R:
+     case CSFILE_SND_W:
+       ret = sf_seek(p->sf,pos,whence);
+       //csoundMessage(csound, "seek set %d \n", pos);
+       csound->FlushCircularBuffer(csound, p->cb);
+       p->items = 0;
+      break;
+    }
+    csound->NotifyThreadLock(csound->file_io_threadlock);
+    return ret;
+}
+
+
+static int read_files(CSOUND *csound){
+  CSFILE *current = (CSFILE *) csound->open_files;
+  if (current == NULL) return 0;
+  while (current) {
+    if(current->async_flag == ASYNC_GLOBAL) {
+    int m = current->pos, l, n = current->items;
+    int items = current->bufsize;
+    MYFLT *buf = current->buf;
+    switch (current->type) {
+      case CSFILE_FD_R:
+        break;
+      case CSFILE_FD_W:
+        break;
+      case CSFILE_STD:
+        break;
+      case CSFILE_SND_R:
+       if(n == 0) {
+           n = sf_read_MYFLT(current->sf, buf, items);
+           m = 0;
+        }
+        l = csound->WriteCircularBuffer(csound,current->cb,&buf[m],n);
+        m += l;
+        n -= l;
+        current->items = n;
+        current->pos = m;
+        break;
+      case CSFILE_SND_W:
+        items = csound->ReadCircularBuffer(csound, current->cb, buf, items);
+        if(items == 0) { csoundSleep(10); break;}
+        sf_write_MYFLT(current->sf, buf, items);
+        break;
+    }
+    }
+    current = current->nxt;
+  }
+  return 1;
+}
+
+
+
+
+void *file_iothread(void *p){
+  int res = 1;
+  CSOUND *csound = p;
+  int wakeup = (int) (1000*csound->ksmps/csound->esr);
+  if(wakeup == 0) wakeup = 1;
+  while(res){
+    csoundSleep(wakeup);
+    csound->WaitThreadLockNoTimeout(csound->file_io_threadlock);
+    res = read_files(csound);
+    csound->NotifyThreadLock(csound->file_io_threadlock);
+  }
+  csound->file_io_start = 0;
+  return NULL;
+}
