@@ -22,11 +22,7 @@
 */
 
 #include "csoundCore.h"     /*                              LINEVENT.C      */
-#include "text.h"
 #include <ctype.h>
-#if (defined(mac_classic) && defined(__MWERKS__)) || defined(SYMANTEC)
-#include <console.h>
-#endif
 
 #include "linevent.h"
 
@@ -43,51 +39,40 @@
 # endif
 #endif
 
-#define LBUFSIZ   32768
+//#define LBUFSIZ   32768
 #define LF        '\n'
 
-typedef struct {
-    char    *Linep, *Linebufend;
-    FILE    *Linecons;
-    int     stdmode;
-    EVTBLK  prve;
-    char    Linebuf[LBUFSIZ];
-} LINEVENT_GLOBALS;
+/* typedef struct { */
+/*     char    *Linep, *Linebufend; */
+/*     FILE    *Linecons; */
+/*     int     stdmode; */
+/*     EVTBLK  prve; */
+/*     char    Linebuf[LBUFSIZ]; */
+/* } LINEVENT_GLOBALS; */
 
 static void sensLine(CSOUND *csound, void *userData);
 
-#define ST(x)   (((LINEVENT_GLOBALS*) ((CSOUND*) csound)->lineventGlobals)->x)
+#define STA(x)   (csound->lineventStatics.x)
 
 void RTLineset(CSOUND *csound)      /* set up Linebuf & ready the input files */
 {                                   /*     callable once from musmon.c        */
     OPARMS  *O = csound->oparms;
-    csound->lineventGlobals = (LINEVENT_GLOBALS*)
-                               csound->Calloc(csound, sizeof(LINEVENT_GLOBALS));
-    ST(prve).opcod = ' ';
-    ST(Linebufend) = ST(Linebuf) + LBUFSIZ;
-    ST(Linep) = ST(Linebuf);
+    /* csound->lineventGlobals = (LINEVENT_GLOBALS*) */
+    /*                            csound->Calloc(csound, */
+    /*                            sizeof(LINEVENT_GLOBALS)); */
+    STA(linebufsiz) = LBUFSIZ;
+    STA(Linebuf) = (char *) csound->Calloc(csound, STA(linebufsiz));
+    STA(prve).opcod = ' ';
+    STA(Linebufend) = STA(Linebuf) + STA(linebufsiz);
+    STA(Linep) = STA(Linebuf);
     if (strcmp(O->Linename, "stdin") == 0) {
-#ifdef SYMANTEC
-      console_options.top += 10;
-      console_options.left += 10;
-      console_options.title = "\pRT Line_events";
-      console_options.nrows = 10;
-      console_options.ncols = 50;
-      ST(Linecons) = fopenc();
-      cshow(ST(Linecons));
-#elif defined(mills_macintosh)
-      ST(Linecons) = stdin;
+#if defined(DOSGCC) || defined(WIN32)
       setvbuf(stdin, NULL, _IONBF, 0);
+      /* WARNING("-L stdin:  system has no fcntl function to get stdin"); */
 #else
-  #if defined(DOSGCC) || defined(__WATCOMC__) || defined(WIN32) || \
-      defined(mills_macintosh)
-      setvbuf(stdin, NULL, _IONBF, 0);
-   /* WARNING("-L stdin:  system has no fcntl function to get stdin"); */
-  #else
-      ST(stdmode) = fcntl(csound->Linefd, F_GETFL, 0);
-      if (UNLIKELY(fcntl(csound->Linefd, F_SETFL, ST(stdmode) | O_NDELAY) < 0))
+      STA(stdmode) = fcntl(csound->Linefd, F_GETFL, 0);
+      if (UNLIKELY(fcntl(csound->Linefd, F_SETFL, STA(stdmode) | O_NDELAY) < 0))
         csoundDie(csound, Str("-L stdin fcntl failed"));
-  #endif
 #endif
     }
 #ifdef PIPES
@@ -99,11 +84,7 @@ void RTLineset(CSOUND *csound)      /* set up Linebuf & ready the input files */
       else csoundDie(csound, Str("Cannot open %s"), O->Linename);
     }
 #endif
-#if defined(mills_macintosh) || defined(SYMANTEC)
-#define MODE
-#else
 #define MODE ,0
-#endif
 #if defined(MSVC)
 #define O_RDONLY _O_RDONLY
 #endif
@@ -111,7 +92,7 @@ void RTLineset(CSOUND *csound)      /* set up Linebuf & ready the input files */
       if (UNLIKELY((csound->Linefd=open(O->Linename, O_RDONLY|O_NDELAY MODE)) < 0))
         csoundDie(csound, Str("Cannot open %s"), O->Linename);
     csound->Message(csound, Str("stdmode = %.8x Linefd = %d\n"),
-                    ST(stdmode), csound->Linefd);
+                    STA(stdmode), csound->Linefd);
     csound->RegisterSenseEventCallback(csound, sensLine, NULL);
 }
 
@@ -121,32 +102,26 @@ int _pclose(FILE*);
 
 void RTclose(CSOUND *csound)
 {
-    if (csound->oparms->Linein == 0 || csound->lineventGlobals == NULL)
+    if (csound->oparms->Linein == 0)
       return;
     csound->oparms->Linein = 0;
     csound->Message(csound, Str("stdmode = %.8x Linefd = %d\n"),
-                            ST(stdmode), csound->Linefd);
-#if defined(mills_macintosh) || defined(SYMANTEC)
-    if (ST(Linecons) != NULL)
-      fclose(ST(Linecons));
-#else
-  #ifdef PIPES
+                    STA(stdmode), csound->Linefd);
+#ifdef PIPES
     if (csound->oparms->Linename[0] == '|')
       _pclose(csound->Linepipe);
     else
+#endif
+      {
+        if (strcmp(csound->oparms->Linename, "stdin") != 0)
+          close(csound->Linefd);
+  #if !defined(DOSGCC) && !defined(WIN32)
+        else
+          fcntl(csound->Linefd, F_SETFL, STA(stdmode));
   #endif
-    {
-      if (strcmp(csound->oparms->Linename, "stdin") != 0)
-        close(csound->Linefd);
-  #if !defined(DOSGCC) && !defined(__WATCOMC__) && !defined(WIN32) && \
-      !defined(mills_macintosh)
-      else
-        fcntl(csound->Linefd, F_SETFL, ST(stdmode));
-  #endif
-    }
-#endif      /* !(mills_macintosh || SYMANTEC) */
-    csound->Free(csound, csound->lineventGlobals);
-    csound->lineventGlobals = NULL;
+}
+//csound->Free(csound, csound->lineventGlobals);
+//csound->lineventGlobals = NULL;
 }
 
 /* does string segment contain LF? */
@@ -160,24 +135,36 @@ static inline int containsLF(char *cp, char *endp)
     return 0;
 }
 
-static CS_NOINLINE int linevent_alloc(CSOUND *csound)
+static CS_NOINLINE int linevent_alloc(CSOUND *csound, int reallocsize)
 {
     volatile jmp_buf tmpExitJmp;
     int         err;
 
+    if(reallocsize > 0) {
+      STA(Linebuf) = (char *) csound->ReAlloc(csound,
+                                              (void *) STA(Linebuf), reallocsize);
+      STA(linebufsiz) = reallocsize;
+      STA(Linebufend) = STA(Linebuf) + STA(linebufsiz);
+    } else if(STA(Linebuf)==NULL) {
+       STA(linebufsiz) = LBUFSIZ;
+       STA(Linebuf) = (char *) csound->Calloc(csound, STA(linebufsiz));
+    }
+    if(STA(Linebuf) == NULL) return 1;
+
+    if (STA(Linep)) return 0;
     csound->Linefd = -1;
     memcpy((void*) &tmpExitJmp, (void*) &csound->exitjmp, sizeof(jmp_buf));
     if ((err = setjmp(csound->exitjmp)) != 0) {
       memcpy((void*) &csound->exitjmp, (void*) &tmpExitJmp, sizeof(jmp_buf));
-      csound->lineventGlobals = NULL;
+      //csound->lineventGlobals = NULL;
       return -1;
     }
-    csound->lineventGlobals =
-        (LINEVENT_GLOBALS*) mcalloc(csound, sizeof(LINEVENT_GLOBALS));
+    /* csound->lineventGlobals = */
+    /*     (LINEVENT_GLOBALS*) mcalloc(csound, sizeof(LINEVENT_GLOBALS)); */
     memcpy((void*) &csound->exitjmp, (void*) &tmpExitJmp, sizeof(jmp_buf));
-    ST(prve).opcod = ' ';
-    ST(Linebufend) = ST(Linebuf) + LBUFSIZ;
-    ST(Linep) = ST(Linebuf);
+    STA(prve).opcod = ' ';
+    STA(Linebufend) = STA(Linebuf) + STA(linebufsiz);
+    STA(Linep) = STA(Linebuf);
     csound->RegisterSenseEventCallback(csound, sensLine, NULL);
 
     return 0;
@@ -186,25 +173,25 @@ static CS_NOINLINE int linevent_alloc(CSOUND *csound)
 /* insert text from an external source,
    to be interpreted as if coming in from stdin/Linefd for -L */
 
-PUBLIC void csoundInputMessage(CSOUND *csound, const char *message)
+void csoundInputMessageInternal(CSOUND *csound, const char *message)
 {
     int32  size = (int32) strlen(message);
+    int n;
 
-    if (!csound->lineventGlobals) {
-      if (linevent_alloc(csound) != 0)
-        return;
-    }
-    if (!size)
-      return;
-    if (UNLIKELY((ST(Linep) + size) >= ST(Linebufend))) {
+    if ((n=linevent_alloc(csound, 0)) != 0) return;
+    if (!size) return;
+    if (UNLIKELY((STA(Linep) + size) >= STA(Linebufend))) {
+      int extralloc = STA(Linep) + size - STA(Linebufend);
+      if ((n=linevent_alloc(csound, STA(linebufsiz) + extralloc ), 0) != 0){
       csoundErrorMsg(csound, Str("LineBuffer Overflow - "
                                  "Input Data has been Lost"));
       return;
+      }
     }
-    memcpy(ST(Linep), message, size);
-    if (ST(Linep)[size - 1] != (char) '\n')
-      ST(Linep)[size++] = (char) '\n';
-    ST(Linep) += size;
+    memcpy(STA(Linep), message, size);
+    if (STA(Linep)[size - 1] != (char) '\n')
+      STA(Linep)[size++] = (char) '\n';
+    STA(Linep) += size;
 }
 
 /* accumlate RT Linein buffer, & place completed events in EVTBLK */
@@ -214,26 +201,25 @@ static void sensLine(CSOUND *csound, void *userData)
 {
     char    *cp, *Linestart, *Linend;
     int     c, n, pcnt;
+    IGN(userData);
 
     while (1) {
-      Linend = ST(Linep);
+      Linend = STA(Linep);
       if (csound->Linefd >= 0) {
-#if defined(mills_macintosh) || defined(SYMANTEC)
-        n = fread((void *) Linend, (size_t) 1,
-                  (size_t) (ST(Linebufend) - Linend), ST(Linecons));
-#else
-        n = read(csound->Linefd, Linend, ST(Linebufend) - Linend);
-#endif
+        n = read(csound->Linefd, Linend, STA(Linebufend) - Linend);
         Linend += (n > 0 ? n : 0);
       }
-      if (Linend <= ST(Linebuf))
+      if (Linend <= STA(Linebuf))
         break;
-      Linestart = ST(Linebuf);
+      Linestart = STA(Linebuf);
       cp = Linestart;
+
       while (containsLF(Linestart, Linend)) {
         EVTBLK  e;
-        char    sstrp[SSTRSIZ];
-        e.strarg = NULL;
+        char    *sstrp = NULL;
+        int     scnt = 0;
+        int     strsiz = 0;
+        e.strarg = NULL; e.scnt = 0;
         c = *cp;
         while (c == ' ' || c == '\t')   /* skip initial white space */
           c = *(++cp);
@@ -263,10 +249,10 @@ static void sensLine(CSOUND *csound, void *userData)
             break;
           pcnt++;
           if (c == '"') {                       /* if find character string */
-            if (UNLIKELY(e.strarg != NULL)) {
-              csound->ErrorMsg(csound, Str("multiple string p-fields"));
-              goto Lerr;
-            }
+            if (e.strarg == NULL)
+              e.strarg = sstrp = mmalloc(csound, strsiz=SSTRSIZ);
+            n = scnt;
+            while (n-->0) sstrp += strlen(sstrp)+1;
             n = 0;
             while ((c = *(++cp)) != '"') {
               if (UNLIKELY(c == LF)) {
@@ -274,14 +260,21 @@ static void sensLine(CSOUND *csound, void *userData)
                 goto Lerr;
               }
               sstrp[n++] = c;                   /*   save in private strbuf */
-              if (UNLIKELY(n >= SSTRSIZ)) {
-                csound->ErrorMsg(csound, Str("string p-field is too long"));
-                goto Lerr;
+              if (UNLIKELY((sstrp-e.strarg)+n >= strsiz-10)) {
+                e.strarg = mrealloc(csound, e.strarg, strsiz+=SSTRSIZ);
+                sstrp = e.strarg+n;
               }
             }
             sstrp[n] = '\0';
-            e.strarg = &(sstrp[0]);
-            e.p[pcnt] = SSTRCOD;                /*   & store coded float   */
+            {
+              union {
+                MYFLT d;
+                int32 i;
+              } ch;
+              ch.d = SSTRCOD; ch.i += scnt++;
+              e.p[pcnt] = ch.d;           /* set as string with count */
+            }
+            e.scnt = scnt;
             continue;
           }
           if (UNLIKELY(!(isdigit(c) || c == '+' || c == '-' || c == '.')))
@@ -289,18 +282,18 @@ static void sensLine(CSOUND *csound, void *userData)
           if (c == '.' &&                       /*  if lone dot,       */
               ((n = cp[1]) == ' ' || n == '\t' || n == LF)) {
             if (UNLIKELY(e.opcod != 'i' ||
-                         ST(prve).opcod != 'i' || pcnt > ST(prve).pcnt)) {
+                         STA(prve).opcod != 'i' || pcnt > STA(prve).pcnt)) {
               csound->ErrorMsg(csound, Str("dot carry has no reference"));
               goto Lerr;
             }                                   /*        pfld carry   */
-            e.p[pcnt] = ST(prve).p[pcnt];
-            if (UNLIKELY(e.p[pcnt] == SSTRCOD)) {
+            e.p[pcnt] = STA(prve).p[pcnt];
+            if (UNLIKELY(ISSTRCOD(e.p[pcnt]))) {
               csound->ErrorMsg(csound, Str("cannot carry string p-field"));
               goto Lerr;
             }
             continue;
           }
-          e.p[pcnt] = (MYFLT) strtod(cp, &newcp);
+          e.p[pcnt] = (MYFLT) cs_strtod(cp, &newcp);
           cp = newcp - 1;
         } while (pcnt < PMAX);
         if (e.opcod =='f' && e.p[1]<FL(0.0)); /* an OK case */
@@ -315,10 +308,10 @@ static void sensLine(CSOUND *csound, void *userData)
         }
         e.pcnt = pcnt;                          /*   &  record pfld count    */
         if (e.opcod == 'i') {                   /* do carries for instr data */
-          memcpy((void*) &ST(prve), (void*) &e,
+          memcpy((void*) &STA(prve), (void*) &e,
                  (size_t) ((char*) &(e.p[pcnt + 1]) - (char*) &e));
           /* FIXME: how to carry string args ? */
-          ST(prve).strarg = NULL;
+          STA(prve).strarg = NULL;
         }
         if (UNLIKELY(pcnt >= PMAX && c != LF)) {
           csound->ErrorMsg(csound, Str("too many pfields"));
@@ -337,18 +330,18 @@ static void sensLine(CSOUND *csound, void *userData)
                                  Linestart, n + 1, "^");  /* mark the error */
         Linestart = (++cp);
       }
-      if (Linestart != &(ST(Linebuf)[0])) {
+      if (Linestart != &(STA(Linebuf)[0])) {
         int len = (int) (Linend - Linestart);
         /* move any remaining characters to the beginning of the buffer */
         for (n = 0; n < len; n++)
-          ST(Linebuf)[n] = Linestart[n];
-        n = (int) (Linestart - &(ST(Linebuf)[0]));
-        ST(Linep) -= n;
+          STA(Linebuf)[n] = Linestart[n];
+        n = (int) (Linestart - &(STA(Linebuf)[0]));
+        STA(Linep) -= n;
         Linend -= n;
       }
-      if (Linend == ST(Linep))      /* return if no more data is available  */
+      if (Linend == STA(Linep))      /* return if no more data is available  */
         break;
-      ST(Linep) = Linend;                       /* accum the chars          */
+      STA(Linep) = Linend;                       /* accum the chars          */
     }
 }
 
@@ -359,69 +352,100 @@ static const char *errmsg_1 =
 static const char *errmsg_2 =
   Str_noop("event: string name is allowed only for \"i\" and \"q\" events");
 
-int eventOpcode(CSOUND *csound, LINEVENT *p)
+int eventOpcode_(CSOUND *csound, LINEVENT *p, int insname, char p1)
 {
     EVTBLK  evt;
     int     i;
     char    opcod;
 
-    opcod = ((char*) p->args[0])[0];
-    if ((opcod != 'a' && opcod != 'i' && opcod != 'q' && opcod != 'f' &&
-         opcod != 'e') || ((char*) p->args[0])[1] != '\0')
-      return csound->PerfError(csound, Str(errmsg_1));
-    evt.strarg = NULL;
+    if(p1==0)
+         opcod = *((STRINGDAT*) p->args[0])->data;
+    else  opcod = p1;
+
+    if (UNLIKELY((opcod != 'a' && opcod != 'i' && opcod != 'q' && opcod != 'f' &&
+                  opcod != 'e') /*|| ((STRINGDAT*) p->args[0])->data[1] != '\0'*/))
+      return csound->PerfError(csound, p->h.insdshead,Str(errmsg_1));
+    evt.strarg = NULL; evt.scnt = 0;
     evt.opcod = opcod;
-    evt.pcnt = p->INOCOUNT - 1;
+    if(p->flag==1) evt.pcnt = p->argno-2;
+    else
+      evt.pcnt = p->INOCOUNT - 1;
     /* IV - Oct 31 2002: allow string argument */
     if (evt.pcnt > 0) {
-      if (p->XSTRCODE & 2) {
+      if (insname) {
         if (UNLIKELY(evt.opcod != 'i' && evt.opcod != 'q'))
-          return csound->PerfError(csound, Str(errmsg_2));
-        evt.p[1] = SSTRCOD;
-        evt.strarg = (char*) p->args[1];
+          return csound->PerfError(csound, p->h.insdshead,Str(errmsg_2));
+        evt.p[1] =  csound->strarg2insno(csound,
+                                           ((STRINGDAT*) p->args[1])->data, 1);
+        evt.strarg = NULL; evt.scnt = 0;
       }
       else {
-        evt.p[1] = *p->args[1];
-        evt.strarg = NULL;
+        if (ISSTRCOD(*p->args[1])) {
+          evt.p[1]  = csound->strarg2insno(csound,
+                                           get_arg_string(csound, *p->args[1]), 1);
+        } else evt.p[1] = *p->args[1];
+        evt.strarg = NULL; evt.scnt = 0;
       }
       for (i = 2; i <= evt.pcnt; i++)
         evt.p[i] = *p->args[i];
     }
     if (insert_score_event_at_sample(csound, &evt, csound->icurTime) != 0)
-      return csound->PerfError(csound, Str("event: error creating '%c' event"),
-                                       opcod);
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("event: error creating '%c' event"),
+                               opcod);
     return OK;
 }
 
+int eventOpcode(CSOUND *csound, LINEVENT *p)
+{
+  return eventOpcode_(csound, p, 0, 0);
+}
+
+int eventOpcode_S(CSOUND *csound, LINEVENT *p)
+{
+  return eventOpcode_(csound, p, 1, 0);
+}
+
+
+
 /* i-time version of event opcode */
 
-int eventOpcodeI(CSOUND *csound, LINEVENT *p)
+int eventOpcodeI_(CSOUND *csound, LINEVENT *p, int insname, char p1)
 {
     EVTBLK  evt;
     int     i, err = 0;
     char    opcod;
 
-    opcod = ((char*) p->args[0])[0];
+    if(p1==0)
+         opcod = *((STRINGDAT*) p->args[0])->data;
+    else  opcod = p1;
     if (UNLIKELY((opcod != 'a' && opcod != 'i' && opcod != 'q' && opcod != 'f' &&
-                  opcod != 'e') || ((char*) p->args[0])[1] != '\0'))
+                  opcod != 'e') /*|| ((STRINGDAT*) p->args[0])->data[1] != '\0'*/))
       return csound->InitError(csound, Str(errmsg_1));
-    evt.strarg = NULL;
+    evt.strarg = NULL; evt.scnt = 0;
     evt.opcod = opcod;
-    evt.pcnt = p->INOCOUNT - 1;
+    if(p->flag==1) evt.pcnt = p->argno-1;
+    else
+      evt.pcnt = p->INOCOUNT - 1;
     /* IV - Oct 31 2002: allow string argument */
     if (evt.pcnt > 0) {
-      if (p->XSTRCODE & 2) {
+      if (insname) {
         if (UNLIKELY(evt.opcod != 'i' && evt.opcod != 'q'))
           return csound->InitError(csound, Str(errmsg_2));
-        evt.p[1] = SSTRCOD;
-        evt.strarg = (char*) p->args[1];
+        evt.p[1] = csound->strarg2insno(csound,((STRINGDAT *)p->args[1])->data, 1);
+        evt.strarg = NULL; evt.scnt = 0;
+        for (i = 2; i <= evt.pcnt; i++)
+           evt.p[i] = *p->args[i];
       }
       else {
-        evt.p[1] = *p->args[1];
-        evt.strarg = NULL;
+        evt.strarg = NULL; evt.scnt = 0;
+        if (ISSTRCOD(*p->args[1])) {
+          evt.p[1]  = csound->strarg2insno(csound,
+                                           get_arg_string(csound, *p->args[1]), 1);
+        } else evt.p[1] = *p->args[1];
+        for (i = 2; i <= evt.pcnt; i++)
+          evt.p[i] = *p->args[i];
       }
-      for (i = 2; i <= evt.pcnt; i++)
-        evt.p[i] = *p->args[i];
     }
     if (opcod == 'f' && (int) evt.pcnt >= 2 && evt.p[2] <= FL(0.0)) {
       FUNC  *dummyftp;
@@ -435,3 +459,12 @@ int eventOpcodeI(CSOUND *csound, LINEVENT *p)
     return (err == 0 ? OK : NOTOK);
 }
 
+int eventOpcodeI(CSOUND *csound, LINEVENT *p)
+{
+  return eventOpcodeI_(csound, p, 0, 0);
+}
+
+int eventOpcodeI_S(CSOUND *csound, LINEVENT *p)
+{
+  return eventOpcodeI_(csound, p, 1, 0);
+}
