@@ -29,7 +29,6 @@
 #include "oload.h"
 #include "remote.h"
 #include <math.h>
-# include "cscore.h"
 #include "corfile.h"
 
 #define SEGAMPS AMPLMSG
@@ -39,7 +38,7 @@ extern  int     MIDIinsert(CSOUND *, int, MCHNBLK*, MEVENT*);
 extern  int     insert(CSOUND *, int, EVTBLK*);
 extern  void    MidiOpen(CSOUND *);
 extern  void    m_chn_init_all(CSOUND *);
-extern  void    scsortstr(CSOUND *, CORFIL *);
+//extern  char *  scsortstr(CSOUND *, CORFIL *);
 extern  void    infoff(CSOUND*, MYFLT), orcompact(CSOUND*);
 extern  void    beatexpire(CSOUND *, double), timexpire(CSOUND *, double);
 extern  void    sfopenin(CSOUND *), sfopenout(CSOUND*), sfnopenout(CSOUND*);
@@ -48,6 +47,7 @@ extern  void    MidiClose(CSOUND *);
 extern  void    RTclose(CSOUND *);
 extern  void    remote_Cleanup(CSOUND *);
 extern  char    **csoundGetSearchPathFromEnv(CSOUND *, const char *);
+/* extern  void    initialize_instrument0(CSOUND *); */
 
 typedef struct evt_cb_func {
     void    (*func)(CSOUND *, void *);
@@ -55,17 +55,7 @@ typedef struct evt_cb_func {
     struct evt_cb_func  *nxt;
 } EVT_CB_FUNC;
 
-typedef struct {
-    int32   srngcnt[MAXCHNLS], orngcnt[MAXCHNLS];
-    int16   srngflg;
-    int16   sectno;
-    int     lplayed;
-    int     segamps, sormsg;
-    EVENT   **ep, **epend;      /* pointers for stepping through lplay list */
-    EVENT   *lsect;
-} MUSMON_GLOBALS;
-
-#define ST(x)   (((MUSMON_GLOBALS*) csound->musmonGlobals)->x)
+#define STA(x)   (csound->musmonStatics.x)
 
 /* IV - Jan 28 2005 */
 void print_benchmark_info(CSOUND *csound, const char *s)
@@ -86,7 +76,7 @@ static void settempo(CSOUND *csound, MYFLT tempo)
     if (tempo <= FL(0.0)) return;
     if (csound->oparms->Beatmode==1)
       csound->ibeatTime = (int64_t)(csound->esr*60.0 / (double) tempo);
-    csound->curBeat_inc = (double) tempo / (60.0 * (double) csound->global_ekr);
+    csound->curBeat_inc = (double) tempo / (60.0 * (double) csound->ekr);
 }
 
 int gettempo(CSOUND *csound, GTEMPO *p)
@@ -195,8 +185,6 @@ int musmon(CSOUND *csound)
                             CS_PACKAGE_VERSION, __DATE__);
 #endif
 #endif
-    if (LIKELY(csound->musmonGlobals == NULL))
-      csound->musmonGlobals = csound->Calloc(csound, sizeof(MUSMON_GLOBALS));
     /* initialise search path cache */
     csoundGetSearchPathFromEnv(csound, "SNAPDIR");
     csoundGetSearchPathFromEnv(csound, "SFDIR;SSDIR;INCDIR");
@@ -206,11 +194,42 @@ int musmon(CSOUND *csound)
 
     m_chn_init_all(csound);     /* allocate MIDI channels */
     dispinit(csound);           /* initialise graphics or character display */
-    oload(csound);              /* set globals and run inits */
+
+    reverbinit(csound);
+    dbfs_init(csound, csound->e0dbfs);
+    csound->nspout = csound->ksmps * csound->nchnls;  /* alloc spin & spout */
+    csound->nspin = csound->ksmps * csound->inchnls; /* JPff: in preparation */
+    csound->spin  = (MYFLT *) mcalloc(csound, csound->nspin * sizeof(MYFLT));
+    csound->spout = (MYFLT *) mcalloc(csound, csound->nspout * sizeof(MYFLT));
+    csound->auxspin  = (MYFLT *) mcalloc(csound, csound->nspin * sizeof(MYFLT));
+    /* memset(csound->maxamp, '\0', sizeof(MYFLT)*MAXCHNLS); */
+    /* memset(csound->smaxamp, '\0', sizeof(MYFLT)*MAXCHNLS); */
+    /* memset(csound->omaxamp, '\0', sizeof(MYFLT)*MAXCHNLS); */
+
+    /* initialise sensevents state */
+    csound->prvbt = csound->curbt = csound->nxtbt = 0.0;
+    csound->curp2 = csound->nxtim = csound->timeOffs = csound->beatOffs = 0.0;
+    csound->icurTime = 0L;
+    if (O->Beatmode && O->cmdTempo > 0) {
+      /* if performing from beats, set the initial tempo */
+     csound->curBeat_inc = (double) O->cmdTempo / (60.0 * (double) csound->ekr);
+     csound->ibeatTime = (int64_t)(csound->esr*60.0 / (double) O->cmdTempo);
+    }
+    else {
+      csound->curBeat_inc = 1.0 / (double) csound->ekr;
+      csound->ibeatTime = 1;
+    }
+    csound->cyclesRemaining = 0;
+    memset(&(csound->evt), 0, sizeof(EVTBLK));
+
+  /* run instr 0 inits */
+    if (UNLIKELY(init0(csound) != 0))
+    csoundDie(csound, Str("header init errors"));
 
     /* kperf() will not call csoundYield() more than 250 times per second */
-    csound->evt_poll_cnt = 0;
-    csound->evt_poll_maxcnt = (int) ((double) csound->ekr / 250.0);
+    csound->evt_poll_cnt    = 0;
+    csound->evt_poll_maxcnt =
+      (int)(250.0 /(double) csound->ekr); /* VL this was wrong: kr/250 originally */
     /* Enable musmon to handle external MIDI input, if it has been enabled. */
     if (O->Midiin || O->FMidiin || O->RMidiin) {
       O->RTevents = 1;
@@ -219,18 +238,18 @@ int musmon(CSOUND *csound)
     csound->Message(csound, Str("orch now loaded\n"));
 
     csound->multichan = (csound->nchnls > 1 ? 1 : 0);
-    ST(segamps) = O->msglevel & SEGAMPS;
-    ST(sormsg) = O->msglevel & SORMSG;
+    STA(segamps) = O->msglevel & SEGAMPS;
+    STA(sormsg)  = O->msglevel & SORMSG;
 
     if (O->Linein)
       RTLineset(csound);                /* if realtime input expected   */
 
     if (csound->enableHostImplementedAudioIO &&
         csound->hostRequestedBufferSize) {
-      int bufsize = (int) csound->hostRequestedBufferSize;
-      int ksmps = (int) csound->ksmps;
-      bufsize = (bufsize + (ksmps >> 1)) / ksmps;
-      bufsize = (bufsize ? bufsize * ksmps : ksmps);
+      int bufsize    = (int) csound->hostRequestedBufferSize;
+      int ksmps      = (int) csound->ksmps;
+      bufsize        = (bufsize + (ksmps >> 1)) / ksmps;
+      bufsize        = (bufsize ? bufsize * ksmps : ksmps);
       O->outbufsamps = O->inbufsamps = bufsize;
     }
     else {
@@ -259,7 +278,7 @@ int musmon(CSOUND *csound)
     }
     csound->Message(csound, Str("audio buffered in %d sample-frame blocks\n"),
                             (int) O->outbufsamps);
-    O->inbufsamps *= csound->inchnls;    /* now adjusted for n channels  */
+    O->inbufsamps  *= csound->inchnls;    /* now adjusted for n channels  */
     O->outbufsamps *= csound->nchnls;
     iotranset(csound);          /* point recv & tran to audio formatter */
       /* open audio file or device for input first, and then for output */
@@ -271,13 +290,12 @@ int musmon(CSOUND *csound)
       else
         sfnopenout(csound);
     }
-
-    corfile_flush(O->playscore);
+    if(O->playscore!=NULL) corfile_flush(O->playscore);
     //csound->scfp
     if (UNLIKELY(O->usingcscore)) {
-      if (ST(lsect) == NULL) {
-        ST(lsect) = (EVENT*) mmalloc(csound, sizeof(EVENT));
-        ST(lsect)->op = 'l';
+      if (STA(lsect) == NULL) {
+        STA(lsect) = (EVENT*) mmalloc(csound, sizeof(EVENT));
+        STA(lsect)->op = 'l';
       }
       csound->Message(csound, Str("using Cscore processing\n"));
       /* override stdout in */
@@ -293,7 +311,7 @@ int musmon(CSOUND *csound)
         fclose(csound->scfp);
         csound->scfp = NULL;
       }
-      if (ST(lplayed))
+      if (STA(lplayed))
         return 0;
 
       /*  read from cscore.out */
@@ -324,10 +342,22 @@ int musmon(CSOUND *csound)
       O->usingcscore = 0;
     }
 
-    csound->Message(csound, Str("SECTION %d:\n"), ++ST(sectno));
+    csound->Message(csound, Str("SECTION %d:\n"), ++STA(sectno));
     /* apply score offset if non-zero */
     if (csound->csoundScoreOffsetSeconds_ > FL(0.0))
-      csound->SetScoreOffsetSeconds(csound, csound->csoundScoreOffsetSeconds_);
+      csoundSetScoreOffsetSeconds(csound, csound->csoundScoreOffsetSeconds_);
+
+
+    if(csound->realtime_audio_flag && csound->init_pass_loop == 0){
+      extern void *init_pass_thread(void *);
+      pthread_attr_t attr;
+      csound->init_pass_loop = 1;
+      csound->init_pass_threadlock = csoundCreateMutex(0);
+      pthread_attr_init(&attr);
+      //pthread_attr_setstacksize(&attr, 1048576);
+      pthread_create(&csound->init_pass_thread,&attr,init_pass_thread, csound);
+    }
+
 
     /* since we are running in components, we exit here to playevents later */
     return 0;
@@ -362,12 +392,17 @@ static void delete_pending_rt_events(CSOUND *csound)
     csound->OrcTrigEvts = NULL;
 }
 
+static void cs_beep(CSOUND *csound)
+{
+    csound->Message(csound, Str("%c\tbeep!\n"), '\a');
+}
+
 PUBLIC int csoundCleanup(CSOUND *csound)
 {
     void    *p;
     MYFLT   *maxp;
     int32    *rngp;
-    int     n;
+    uint32_t n;
 
     while (csound->evtFuncChain != NULL) {
       p = (void*) csound->evtFuncChain;
@@ -376,49 +411,62 @@ PUBLIC int csoundCleanup(CSOUND *csound)
     }
 
     /* check if we have already cleaned up */
-    if (!(csound->engineState & CS_STATE_CLN))
+    if (!(csound->engineStatus & CS_STATE_CLN))
       return 0;
     /* will not clean up more than once */
-    csound->engineState &= ~(CS_STATE_CLN);
+    csound->engineStatus &= ~(CS_STATE_CLN);
 
     deactivate_all_notes(csound);
-    if (csound->instrtxtp &&
-        csound->instrtxtp[0] &&
-        csound->instrtxtp[0]->instance &&
-        csound->instrtxtp[0]->instance->actflg)
-      xturnoff_now(csound, csound->instrtxtp[0]->instance);
-    delete_pending_rt_events(csound);
+
+    if (csound->engineState.instrtxtp &&
+        csound->engineState.instrtxtp[0] &&
+        csound->engineState.instrtxtp[0]->instance &&
+        csound->engineState.instrtxtp[0]->instance->actflg)
+      xturnoff_now(csound, csound->engineState.instrtxtp[0]->instance);
+      delete_pending_rt_events(csound);
+
+    if(csound->init_pass_loop == 1) {
+      csoundLockMutex(csound->init_pass_threadlock);
+      csound->init_pass_loop = 0;
+      csoundUnlockMutex(csound->init_pass_threadlock);
+      pthread_join(csound->init_pass_thread, NULL);
+      csoundDestroyMutex(csound->init_pass_threadlock);
+      csound->init_pass_threadlock = 0;
+    }
+
     while (csound->freeEvtNodes != NULL) {
       p = (void*) csound->freeEvtNodes;
       csound->freeEvtNodes = ((EVTNODE*) p)->nxt;
       free(p);
     }
+
     orcompact(csound);
+
     corfile_rm(&csound->scstr);
 
     /* print stats only if musmon was actually run */
-    if (UNLIKELY(csound->musmonGlobals != NULL)) {
+    /* NOT SURE HOW   ************************** */
+    {
       csound->Message(csound, Str("end of score.\t\t   overall amps:"));
       for (n = 0; n < csound->nchnls; n++) {
         if (csound->smaxamp[n] > csound->omaxamp[n])
           csound->omaxamp[n] = csound->smaxamp[n];
         if (csound->maxamp[n] > csound->omaxamp[n])
           csound->omaxamp[n] = csound->maxamp[n];
-        ST(orngcnt)[n] += (ST(srngcnt)[n] + csound->rngcnt[n]);
+        STA(orngcnt)[n] += (STA(srngcnt)[n] + csound->rngcnt[n]);
       }
       for (maxp = csound->omaxamp, n = csound->nchnls; n--; )
         print_maxamp(csound, *maxp++);
       if (csound->oparms->outformat != AE_FLOAT) {
         csound->Message(csound, Str("\n\t   overall samples out of range:"));
-        for (rngp = ST(orngcnt), n = csound->nchnls; n--; )
+        for (rngp = STA(orngcnt), n = csound->nchnls; n--; )
           csound->Message(csound, "%9d", *rngp++);
       }
       csound->Message(csound, Str("\n%d errors in performance\n"),
-                              csound->perferrcnt);
+                      csound->perferrcnt);
       print_benchmark_info(csound, Str("end of performance"));
     }
-
-    /* close line input (-L) */
+/* close line input (-L) */
     RTclose(csound);
     /* close MIDI input */
     MidiClose(csound);
@@ -436,29 +484,21 @@ PUBLIC int csoundCleanup(CSOUND *csound)
     if (csound->oparms->ringbell)
       cs_beep(csound);
 
-    return dispexit(csound);    /* hold or terminate the display output     */
-}
 
-void cs_beep(CSOUND *csound)
-{
-#ifdef mac_classic
-    SysBeep(30L);
-#else
-    csound->Message(csound, Str("%c\tbeep!\n"), '\007');
-#endif
+    return dispexit(csound);    /* hold or terminate the display output     */
 }
 
 int lplay(CSOUND *csound, EVLIST *a)    /* cscore re-entry into musmon */
 {
-    if (csound->musmonGlobals == NULL)
-      csound->musmonGlobals = csound->Calloc(csound, sizeof(MUSMON_GLOBALS));
-    ST(lplayed) = 1;
-    if (!ST(sectno))
-      csound->Message(csound, Str("SECTION %d:\n"), ++ST(sectno));
-    ST(ep) = &a->e[1];                  /* from 1st evlist member */
-    ST(epend) = ST(ep) + a->nevents;    /*   to last              */
+    /* if (csound->musmonGlobals == NULL) */
+    /*  csound->musmonGlobals = csound->Calloc(csound, sizeof(MUSMON_GLOBALS)); */
+    STA(lplayed) = 1;
+    if (!STA(sectno))
+      csound->Message(csound, Str("SECTION %d:\n"), ++STA(sectno));
+    STA(ep) = &a->e[1];                  /* from 1st evlist member */
+    STA(epend) = STA(ep) + a->nevents;    /*   to last              */
     while (csoundPerform(csound) == 0)  /* play list members      */
-      ;
+      ;                                 /* NB: empoty loop */
     return OK;
 }
 
@@ -468,20 +508,39 @@ int lplay(CSOUND *csound, EVLIST *a)    /* cscore re-entry into musmon */
 int turnon(CSOUND *csound, TURNON *p)
 {
     EVTBLK  evt;
-    int     isNamedInstr;
-
-    evt.strarg = NULL;
+    int insno;
+    evt.strarg = NULL; evt.scnt = 0;
     evt.opcod = 'i';
     evt.pcnt = 3;
-    isNamedInstr = (int) csound->GetInputArgSMask(p);
-    if (isNamedInstr) {
-      int32  insno = csound->strarg2insno(csound, p->insno, isNamedInstr);
-      if (insno <= 0L)
+
+    if(ISSTRCOD(*p->insno)) {
+    char *ss = get_arg_string(csound,*p->insno);
+    insno = csound->strarg2insno(csound,ss,1);
+    if (insno <= 0L)
         return NOTOK;
-      evt.p[1] = (MYFLT) insno;
-    }
-    else
-      evt.p[1] = *p->insno;
+    } else insno = *p->insno;
+    evt.p[1] = (MYFLT) insno;
+    evt.p[2] = *p->itime;
+    evt.p[3] = FL(-1.0);
+    evt.c.extra = NULL;
+    return insert_score_event_at_sample(csound, &evt, csound->icurTime);
+}
+
+/* make list to turn on instrs for indef */
+/* perf called from i0 for execution in playevents */
+
+int turnon_S(CSOUND *csound, TURNON *p)
+{
+    EVTBLK  evt;
+    int     insno;
+
+    evt.strarg = NULL; evt.scnt = 0;
+    evt.opcod = 'i';
+    evt.pcnt = 3;
+    insno = csound->strarg2insno(csound, ((STRINGDAT *)p->insno)->data, 1);
+    if (insno <= 0L)
+        return NOTOK;
+    evt.p[1] = (MYFLT) insno;
     evt.p[2] = *p->itime;
     evt.p[3] = FL(-1.0);
     evt.c.extra = NULL;
@@ -498,7 +557,7 @@ static void print_amp_values(CSOUND *csound, int score_evt)
     int32          *rngp, *srngp;
     int           n;
 
-    if (ST(segamps) || (p->rngflg && ST(sormsg))) {
+    if (STA(segamps) || (p->rngflg && STA(sormsg))) {
       if (score_evt)
         p->Message(p, "B%7.3f ..%7.3f T%7.3f TT%7.3f M:",
                       p->prvbt - p->beatOffs,  p->curbt - p->beatOffs,
@@ -518,12 +577,12 @@ static void print_amp_values(CSOUND *csound, int score_evt)
     }
     if (p->rngflg) {
       p->rngflg = 0;
-      ST(srngflg)++;
+      STA(srngflg)++;
     }
     for (n = p->nchnls,
          maxp = p->maxamp - 1, smaxp = p->smaxamp - 1,
          maxps = p->maxpos - 1, smaxps = p->smaxpos - 1,
-         rngp = p->rngcnt, srngp = ST(srngcnt); n--; ) {
+         rngp = p->rngcnt, srngp = STA(srngcnt); n--; ) {
       ++maxps; ++smaxps;
       if (*++maxp > *++smaxp) {
         *smaxp = *maxp;
@@ -549,24 +608,24 @@ static void section_amps(CSOUND *csound, int enable_msgs)
 
     if (enable_msgs) {
       if (enable_msgs == 1)
-        p->Message(p, Str("end of section %d\t sect peak amps:"), ST(sectno));
+        p->Message(p, Str("end of section %d\t sect peak amps:"), STA(sectno));
       else if (enable_msgs == 2)
         p->Message(p, Str("end of lplay event list\t      peak amps:"));
       for (n = p->nchnls, maxp = p->smaxamp; n--; )
         print_maxamp(p, *maxp++);               /* IV - Jul 9 2002 */
       p->Message(p, "\n");
-      if (UNLIKELY(ST(srngflg))) {
+      if (UNLIKELY(STA(srngflg))) {
         p->Message(p, Str("\t number of samples out of range:"));
-        for (n = p->nchnls, srngp = ST(srngcnt); n--; )
+        for (n = p->nchnls, srngp = STA(srngcnt); n--; )
           p->Message(p, "%9d", *srngp++);
         p->Message(p, "\n");
       }
     }
-    ST(srngflg) = 0;
+    STA(srngflg) = 0;
     for (n = p->nchnls,
          smaxp = p->smaxamp - 1, maxp = p->omaxamp - 1,
          smaxps = p->smaxpos - 1, maxps = p->omaxpos - 1,
-         srngp = ST(srngcnt), rngp = ST(orngcnt); n--; ) {
+         srngp = STA(srngcnt), rngp = STA(orngcnt); n--; ) {
       ++maxps; ++smaxps;
       if (UNLIKELY(*++smaxp > *++maxp)) {
         *maxp = *smaxp;                 /* keep ovrl maxamps */
@@ -613,50 +672,54 @@ static int process_score_event(CSOUND *csound, EVTBLK *evt, int rtEvt)
       csound->currevent = saved_currevent;
       return (evt->opcod == 'l' ? 3 : (evt->opcod == 's' ? 1 : 2));
     case 'q':
-      if (evt->p[1] == SSTRCOD && evt->strarg) {    /* IV - Oct 31 2002 */
+      if (ISSTRCOD(evt->p[1]) && evt->strarg) {    /* IV - Oct 31 2002 */
         if (UNLIKELY((insno = (int) named_instr_find(csound, evt->strarg)) < 1)) {
-          printScoreError(csound, rtEvt, Str(" - note deleted. instr %s undefined"),
-                                    evt->strarg);
+          printScoreError(csound, rtEvt,
+                          Str(" - note deleted. instr %s undefined"),
+                          evt->strarg);
           break;
         }
         csound->Message(csound, Str("Setting instrument %s %s\n"),
                       evt->strarg, (evt->p[3] == 0 ? Str("off") : Str("on")));
-        csound->instrtxtp[insno]->muted = (int16) evt->p[3];
+        csound->engineState.instrtxtp[insno]->muted = (int16) evt->p[3];
       }
       else {                                        /* IV - Oct 31 2002 */
         insno = abs((int) evt->p[1]);
-        if (UNLIKELY((unsigned int)(insno-1) >= (unsigned int) csound->maxinsno ||
-                     csound->instrtxtp[insno] == NULL)) {
+        if (UNLIKELY((unsigned int)(insno-1) >=
+                     (unsigned int) csound->engineState.maxinsno ||
+                     csound->engineState.instrtxtp[insno] == NULL)) {
           printScoreError(csound, rtEvt,
                           Str(" - note deleted. instr %d(%d) undefined"),
-                          insno, csound->maxinsno);
+                          insno, csound->engineState.maxinsno);
           break;
         }
         csound->Message(csound, Str("Setting instrument %d %s\n"),
                       insno, (evt->p[3] == 0 ? Str("off") : (Str("on"))));
-        csound->instrtxtp[insno]->muted = (int16) evt->p[3];
+        csound->engineState.instrtxtp[insno]->muted = (int16) evt->p[3];
       }
       break;
     case 'i':
-      if (evt->p[1] == SSTRCOD && evt->strarg) {    /* IV - Oct 31 2002 */
+      if (ISSTRCOD(evt->p[1]) && evt->strarg) {    /* IV - Oct 31 2002 */
         if (UNLIKELY((insno = (int) named_instr_find(csound, evt->strarg)) < 1)) {
-          printScoreError(csound, rtEvt, Str(" - note deleted. instr %s undefined"),
-                                    evt->strarg);
+          printScoreError(csound, rtEvt,
+                          Str(" - note deleted. instr %s undefined"),
+                          evt->strarg);
           break;
         }
         if ((rfd = getRemoteInsRfd(csound, insno))) {
           /* RM: if this note labeled as remote */
           if (rfd == GLOBAL_REMOT)
-            insGlobevt(csound, evt);     /* RM: do a global send and allow local */
+            insGlobevt(csound, evt);  /* RM: do a global send and allow local */
           else {
-            insSendevt(csound, evt, rfd);/* RM: or send to single remote Csound  */
+            insSendevt(csound, evt, rfd);/* RM: or send to single remote Csound */
             break;                       /* RM: and quit */
           }
         }
         evt->p[1] = (MYFLT) insno;
         if (csound->oparms->Beatmode && !rtEvt && evt->p3orig > FL(0.0))
           evt->p[3] = evt->p3orig * (MYFLT) csound->ibeatTime/csound->esr;
-        if (UNLIKELY((n = insert(csound, insno, evt)))) {  /* else alloc, init, activate */
+        /* else alloc, init, activate */
+        if (UNLIKELY((n = insert(csound, insno, evt)))) {
           printScoreError(csound, rtEvt,
                           Str(" - note deleted.  i%d (%s) had %d init errors"),
                           insno, evt->strarg, n);
@@ -664,11 +727,12 @@ static int process_score_event(CSOUND *csound, EVTBLK *evt, int rtEvt)
       }
       else {                                        /* IV - Oct 31 2002 */
         insno = abs((int) evt->p[1]);
-        if (UNLIKELY((unsigned int)(insno-1) >= (unsigned int)csound->maxinsno ||
-                     csound->instrtxtp[insno] == NULL)) {
+        if (UNLIKELY((unsigned int)(insno-1) >=
+                     (unsigned int)csound->engineState.maxinsno ||
+                     csound->engineState.instrtxtp[insno] == NULL)) {
           printScoreError(csound, rtEvt,
                           Str(" - note deleted. instr %d(%d) undefined"),
-                          insno, csound->maxinsno);
+                          insno, csound->engineState.maxinsno);
           break;
         }
         if ((rfd = getRemoteInsRfd(csound, insno))) {
@@ -705,7 +769,7 @@ static int process_score_event(CSOUND *csound, EVTBLK *evt, int rtEvt)
     case 'a':
       {
         int64_t kCnt;
-        kCnt = (int64_t) ((double) csound->global_ekr * (double) evt->p[3] + 0.5);
+        kCnt = (int64_t) ((double) csound->ekr * (double) evt->p[3] + 0.5);
         if (kCnt > csound->advanceCnt) {
           csound->advanceCnt = kCnt;
           csound->Message(csound,
@@ -729,11 +793,13 @@ static void process_midi_event(CSOUND *csound, MEVENT *mep, MCHNBLK *chn)
         csound->Message(csound,
                         Str("\t\t   T%7.3f - note deleted. "), csound->curp2);
         {
-          char *name = csound->instrtxtp[insno]->insname;
+          char *name = csound->engineState.instrtxtp[insno]->insname;
           if (name)
-            csound->Message(csound, Str("instr %s had %d init errors\n"), name, n);
+            csound->Message(csound, Str("instr %s had %d init errors\n"),
+                            name, n);
           else
-            csound->Message(csound, Str("instr %d had %d init errors\n"), insno, n);
+            csound->Message(csound, Str("instr %d had %d init errors\n"),
+                            insno, n);
         }
         csound->perferrcnt++;
       }
@@ -869,6 +935,7 @@ int sensevents(CSOUND *csound)
             }
             /* end of: 1: section, 2: score, 3: lplay list */
             retval = (e->opcod == 'l' ? 3 : (e->opcod == 's' ? 1 : 2));
+
             goto scode;
           default:                            /* q, i, f, a:              */
             process_score_event(csound, e, 0);/*   handle event now       */
@@ -878,12 +945,12 @@ int sensevents(CSOUND *csound)
       }
       else {
         /* else read next score event */
-        if (UNLIKELY(O->usingcscore)) {           /*    get next lplay event      */
+        if (UNLIKELY(O->usingcscore)) {       /*    get next lplay event  */
           /* FIXME: this may be non-portable */
-          if (ST(ep) < ST(epend))                       /* nxt event    */
-            memcpy((void*) e, (void*) &((*ST(ep)++)->strarg), sizeof(EVTBLK));
-          else                                          /* else lcode   */
-            memcpy((void*) e, (void*) &(ST(lsect)->strarg), sizeof(EVTBLK));
+          if (STA(ep) < STA(epend))           /* nxt event    */
+            memcpy((void*) e, (void*) &((*STA(ep)++)->strarg), sizeof(EVTBLK));
+          else                                /* else lcode   */
+            memcpy((void*) e, (void*) &(STA(lsect)->strarg), sizeof(EVTBLK));
         } else
           if (!(rdscor(csound, e)))           /* or rd nxt evt from scstr */
             e->opcod = 'e';
@@ -916,17 +983,40 @@ int sensevents(CSOUND *csound)
           continue;
         }
       }
+
+
       /* calculate the number of k-periods remaining until next event */
-      if (O->Beatmode)
-        csound->cyclesRemaining =
-          RNDINT64((csound->nxtbt - csound->curBeat) / csound->curBeat_inc);
+      if (!O->sampleAccurate) {
+        if (O->Beatmode)
+          csound->cyclesRemaining =
+            RNDINT64((csound->nxtbt - csound->curBeat) / csound->curBeat_inc);
+        else {
+          csound->cyclesRemaining =
+            RNDINT64((csound->nxtim*csound->esr - csound->icurTime)/csound->ksmps);
+          csound->nxtim =
+            (csound->cyclesRemaining*csound->ksmps+csound->icurTime)/csound->esr;
+        }
+      }
       else {
-        csound->cyclesRemaining =
-          RNDINT64((csound->nxtim*csound->esr - csound->icurTime) / csound->ksmps);
-        csound->nxtim = (csound->cyclesRemaining*csound->ksmps+csound->icurTime)/csound->esr;
+      /* VL 30-11-2012
+        new code for sample-accurate timing needs to truncate cyclesRemaining
+      */
+        if (O->Beatmode)
+          csound->cyclesRemaining = (int64_t)
+            ((csound->nxtbt - csound->curBeat) / csound->curBeat_inc);
+        else {
+          csound->cyclesRemaining = (int64_t)
+            FLOOR((csound->nxtim*csound->esr -
+                   csound->icurTime+csound->onedsr*0.5) / csound->ksmps);
+          csound->nxtim =
+            (csound->cyclesRemaining*csound->ksmps+csound->icurTime)/csound->esr;
+        }
       }
     }
 
+
+
+
     /* handle any real time events now: */
     /* FIXME: the initialisation pass of real time */
     /*   events is not sorted by instrument number */
@@ -1009,30 +1099,34 @@ int sensevents(CSOUND *csound)
       return 1;
     }
     /* for s, or e after s */
-    if (retval == 1 || (retval == 2 && ST(sectno) > 1)) {
+    if (retval == 1 || (retval == 2 && STA(sectno) > 1)) {
       delete_pending_rt_events(csound);
       if (O->Beatmode)
         csound->curbt = csound->curBeat;
-      csound->curp2 = csound->nxtim = csound->timeOffs = csound->icurTime/csound->esr;
+      csound->curp2 = csound->nxtim =
+        csound->timeOffs = csound->icurTime/csound->esr;
       csound->prvbt = csound->nxtbt = csound->beatOffs = csound->curbt;
       section_amps(csound, 1);
     }
-    else
+    else{
       section_amps(csound, 0);
+
+    }
     if (retval == 1) {                        /* if s code,        */
       orcompact(csound);                      /*   rtn inactiv spc */
       if (csound->actanchor.nxtact == NULL)   /*   if no indef ins */
         rlsmemfiles(csound);                  /*    purge memfiles */
-      csound->Message(csound, Str("SECTION %d:\n"), ++ST(sectno));
+      csound->Message(csound, Str("SECTION %d:\n"), ++STA(sectno));
       goto retest;                            /*   & back for more */
     }
+
     return 2;                   /* done with entire score */
 }
 
 static inline uint64_t time2kcnt(CSOUND *csound, double tval)
 {
     if (tval > 0.0) {
-      tval *= (double) csound->global_ekr;
+      tval *= (double) csound->ekr;
 #ifdef HAVE_C99
       return (uint64_t) llrint(tval);
 #else
@@ -1078,12 +1172,17 @@ int insert_score_event_at_sample(CSOUND *csound, EVTBLK *evt, int64_t time_ofs)
         return CSOUND_MEMORY;
     }
     if (evt->strarg != NULL) {  /* copy string argument if present */
-      e->evt.strarg = (char*) malloc((size_t) strlen(evt->strarg) + (size_t) 1);
+      /* NEED TO COPY WHOLE STRING STRUCTURE */
+      int n = evt->scnt;
+      char *p = evt->strarg;
+      while (n--) { p += strlen(p)+1; };
+      e->evt.strarg = (char*) malloc((size_t) (p-evt->strarg)+1);
       if (UNLIKELY(e->evt.strarg == NULL)) {
         free(e);
         return CSOUND_MEMORY;
       }
-      strcpy(e->evt.strarg, evt->strarg);
+      memcpy(e->evt.strarg, evt->strarg, p-evt->strarg+1 );
+      e->evt.scnt = evt->scnt;
     }
     e->evt.opcod = evt->opcod;
     e->evt.pcnt = evt->pcnt;
@@ -1114,7 +1213,8 @@ int insert_score_event_at_sample(CSOUND *csound, EVTBLK *evt, int64_t time_ofs)
         if (p[2] < FL(0.0))
           p[2] = FL(0.0);
         /* start beat: this is possibly wrong */
-        evt->p2orig = (MYFLT) (((start_time - st->icurTime/st->esr) / st->ibeatTime)
+        evt->p2orig = (MYFLT) (((start_time - st->icurTime/st->esr) /
+                                st->ibeatTime)
                                + (st->curBeat - st->beatOffs));
         if (evt->p2orig < FL(0.0))
           evt->p2orig = FL(0.0);
@@ -1131,14 +1231,15 @@ int insert_score_event_at_sample(CSOUND *csound, EVTBLK *evt, int64_t time_ofs)
           evt->p3orig = (MYFLT) ((double) evt->p3orig / st->ibeatTime);
       case 'q':                         /* mute instrument */
         /* check for a valid instrument number or name */
-        if (evt->strarg != NULL && p[1] == SSTRCOD)
+        if (evt->strarg != NULL && ISSTRCOD(p[1]))
           i = (int) named_instr_find(csound, evt->strarg);
         else
           i = (int) fabs((double) p[1]);
-        if (UNLIKELY((unsigned int) (i - 1) >= (unsigned int) csound->maxinsno ||
-                     csound->instrtxtp[i] == NULL)) {
+        if (UNLIKELY((unsigned int) (i - 1) >=
+                     (unsigned int) csound->engineState.maxinsno ||
+                     csound->engineState.instrtxtp[i] == NULL)) {
           csoundMessage(csound, Str("insert_score_event(): invalid instrument "
-                                    "number or name\n"));
+                                    "number or name\n" ));
           goto err_return;
         }
         break;
@@ -1211,11 +1312,12 @@ void musmon_rewind_score(CSOUND *csound)
       csound->nxtbt = csound->curbt = csound->prvbt = 0.0;
       csound->nxtim = csound->curp2 = 0.0;
       csound->beatOffs = csound->timeOffs = 0.0;
-      csound->curBeat = 0.0;
+      csound->curBeat  = 0.0;
       csound->icurTime = 0L;
       csound->cyclesRemaining = 0;
       csound->evt.strarg = NULL;
-      csound->evt.opcod = '\0';
+      csound->evt.scnt = 0;
+      csound->evt.opcod  = '\0';
       /* reset tempo */
       if (csound->oparms->Beatmode)
         settempo(csound, (MYFLT) csound->oparms->cmdTempo);
@@ -1223,16 +1325,17 @@ void musmon_rewind_score(CSOUND *csound)
         settempo(csound, FL(60.0));
       /* update section/overall amplitudes, reset to section 1 */
       section_amps(csound, 1);
-      ST(sectno) = 1;
-      csound->Message(csound, Str("SECTION %d:\n"), ST(sectno));
+      STA(sectno) = 1;
+      csound->Message(csound, Str("SECTION %d:\n"), STA(sectno));
     }
 
     /* apply score offset if non-zero */
     csound->advanceCnt = 0;
     if (csound->csoundScoreOffsetSeconds_ > FL(0.0))
-      csound->SetScoreOffsetSeconds(csound, csound->csoundScoreOffsetSeconds_);
-
-    corfile_rewind(csound->scstr);
+      csoundSetScoreOffsetSeconds(csound, csound->csoundScoreOffsetSeconds_);
+    if(csound->scstr)
+      corfile_rewind(csound->scstr);
+    else csound->Warning(csound, Str("cannot rewind score: no score in memory \n"));
 }
 
 /**
