@@ -24,6 +24,19 @@
 #include "csoundCore.h"         /*                  RDSCORSTR.C */
 #include "corfile.h"
 
+char* get_arg_string(CSOUND *csound, MYFLT p)
+{
+    int32 n;
+    char *ss = csound->ids->insdshead->strarg;  /* look at this instr's strarg */
+    union {
+      MYFLT d;
+      int32 i;
+    } ch;
+    ch.d = p; n = ch.i&0xffff;
+    while (n-- > 0) ss += strlen(ss)+1;
+    return ss;
+}
+
 static void dumpline(CSOUND *);
 
 static void flushline(CSOUND *csound)   /* flush scorefile to next newline */
@@ -46,32 +59,32 @@ static int scanflt(CSOUND *csound, MYFLT *pfld)
     }
     if (c == '"') {                             /* if find a quoted string  */
       char *sstrp;
-      if (csound->scnt0==0) {
-        if ((sstrp = csound->sstrbuf) == NULL)
-          sstrp = csound->sstrbuf = mmalloc(csound, SSTRSIZ);
-        while ((c = corfile_getc(csound->scstr)) != '"') {
-          if (c=='\\') c = corfile_getc(csound->scstr);
-          *sstrp++ = c;
+      int n = csound->scnt;
+      if ((sstrp = csound->sstrbuf) == NULL)
+        sstrp = csound->sstrbuf = mmalloc(csound, csound->strsiz=SSTRSIZ);
+      while (n--!=0) sstrp += strlen(sstrp)+1;
+      n = sstrp-csound->sstrbuf;
+      while ((c = corfile_getc(csound->scstr)) != '"') {
+        //if (c=='\\') c = corfile_getc(csound->scstr);
+        *sstrp++ = c;
+        n++;
+        if (n > csound->strsiz-10) {
+          csound->sstrbuf = mrealloc(csound, csound->sstrbuf,
+                                     csound->strsiz+=SSTRSIZ);
+          sstrp = csound->sstrbuf+n;
         }
-        *sstrp++ = '\0';
-        *pfld = SSTRCOD;                        /*   flag with hifloat      */
-        csound->sstrlen = sstrp - csound->sstrbuf;  /*    & overall length  */
       }
-      else {
-        int n = csound->scnt0;
-        printf("***Entering dubious code; n=%d\n", n);
-        if ((sstrp = csound->sstrbuf0[n]) == NULL)
-          sstrp = csound->sstrbuf0[n] = mmalloc(csound, SSTRSIZ);
-        while ((c = corfile_getc(csound->scstr)) != '"') {
-          if (c=='\\') c = corfile_getc(csound->scstr);
-          *sstrp++ = c;
-        }
-        *sstrp++ = '\0';
-        *pfld = ((int[4]){SSTRCOD,SSTRCOD1,SSTRCOD2,SSTRCOD3})[n]; /* flag with hifloat */
-                                                          /* Net  line is wrong*/
-        csound->sstrlen0[n] = sstrp - csound->sstrbuf0[n];  /* & overall length */
+      *sstrp++ = '\0';
+      {
+        union {
+          MYFLT d;
+          int32 i;
+        } ch;
+        ch.d = SSTRCOD; ch.i += csound->scnt++;
+        *pfld = ch.d;           /* set as string with count */
       }
-      csound->scnt0++;
+      csound->sstrlen = sstrp - csound->sstrbuf;  /*    & overall length  */
+      //printf("csound->sstrlen = %d\n", csound->sstrlen);
       return(1);
     }
     if (UNLIKELY(!((c>='0' && c<='9') || c=='+' || c=='-' || c=='.'))) {
@@ -108,9 +121,6 @@ int rdscor(CSOUND *csound, EVTBLK *e) /* read next score-line from scorefile */
 
     if (csound->scstr == NULL ||
         csound->scstr->body[0] == '\0') {   /* if no concurrent scorefile  */
-#ifdef BETA
-      csound->Message(csound, "THIS SHOULD NOT HAPPEN -- CONTACT jpff");
-#endif
       e->opcod = 'f';             /*     return an 'f 0 3600'    */
       e->p[1] = FL(0.0);
       e->p[2] = FL(INF);
@@ -118,8 +128,9 @@ int rdscor(CSOUND *csound, EVTBLK *e) /* read next score-line from scorefile */
       e->pcnt = 2;
       return(1);
     }
-    while ((c = corfile_getc(csound->scstr)) != '\0') {  /* else read the real score */
-      csound->scnt0 = 0;
+  /* else read the real score */
+    while ((c = corfile_getc(csound->scstr)) != '\0') {
+      csound->scnt = 0;
       switch (c) {
       case ' ':
       case '\t':
@@ -134,7 +145,7 @@ int rdscor(CSOUND *csound, EVTBLK *e) /* read next score-line from scorefile */
         goto unwarped;
       case 'w':
         csound->warped = 1;     /* w statement is itself unwarped */
-      unwarped:   e->opcod = c;                   /*  UNWARPED scorefile:         */
+      unwarped:   e->opcod = c;         /*  UNWARPED scorefile:         */
         pp = &e->p[0];
         plim = &e->p[PMAX];             /*    caution, irregular format */
         while (1) {
@@ -142,7 +153,7 @@ int rdscor(CSOUND *csound, EVTBLK *e) /* read next score-line from scorefile */
                  c=='\t'); /* eat whitespace */
           if (c == ';') { flushline(csound); break; } /* comments? skip */
           if (c == '\n' || c == '\0')   break;    /* newline? done  */
-          corfile_ungetc(csound->scstr);       /* pfld:  back up */
+          corfile_ungetc(csound->scstr);          /* pfld:  back up */
           if (!scanflt(csound, ++pp))  break;     /*   & read value */
             if (UNLIKELY(pp >= plim)) {
             csound->Message(csound, Str("ERROR: too many pfields: "));
@@ -157,6 +168,7 @@ int rdscor(CSOUND *csound, EVTBLK *e) /* read next score-line from scorefile */
       case 'e':
         e->opcod = c;
         e->pcnt = 0;
+
         return(1);
       case EOF:                          /* necessary for cscoreGetEvent */
         return(0);
@@ -181,49 +193,62 @@ int rdscor(CSOUND *csound, EVTBLK *e) /* read next score-line from scorefile */
                          scanflt(csound, ++pp))
                     /* p4....  */
                     if (pp >= plim) {
+                      MYFLT *new;
                       MYFLT *q;
                       int c=1;
                       csound->DebugMsg(csound, "Extra p-fields (%d %d %d %d)\n",
-                              (int)e->p[1],(int)e->p[2],
-                              (int)e->p[3],(int)e->p[4]);
-                      e->c.extra = (MYFLT*)realloc(e->c.extra,sizeof(MYFLT)*PMAX);
+                                       (int)e->p[1],(int)e->p[2],
+                                       (int)e->p[3],(int)e->p[4]);
+                      new = (MYFLT*)realloc(e->c.extra,sizeof(MYFLT)*PMAX);
+                      if (new==NULL) {
+                        fprintf(stderr, "Out of Mdemory\n");
+                        exit(7);
+                      }
+                      e->c.extra = new;
                       e->c.extra[0] = PMAX-2;
                       q = e->c.extra;
                       while ((corfile_getc(csound->scstr) != '\n') &&
                              (scanflt(csound, &q[c++]))) {
                         if (c > (int) e->c.extra[0]) {
-                          csound->DebugMsg(csound, "and more extra p-fields [%d](%d)%d\n",
-                                  c, (int) e->c.extra[0],
-                                  sizeof(MYFLT)*((int)e->c.extra[0]+PMAX) );
-                          q = e->c.extra =
+                          csound->DebugMsg(csound,
+                                           "and more extra p-fields [%d](%d)%d\n",
+                                           c, (int) e->c.extra[0],
+                                           sizeof(MYFLT)*((int)e->c.extra[0]+PMAX));
+                          new =
                             (MYFLT *)realloc(e->c.extra,
-                                 sizeof(MYFLT)*((int) e->c.extra[0]+PMAX));
+                                             sizeof(MYFLT)*((int) e->c.extra[0]+
+                                                            PMAX));
+                          if (new==NULL) {
+                            fprintf(stderr, "Out of Mdemory\n");
+                            exit(7);
+                          }
+                          q = e->c.extra = new;
                           e->c.extra[0] = e->c.extra[0]+PMAX-1;
                         }
                       }
                       e->c.extra[0] = c;
                       /* flushline(csound); */
-                       goto setp;
+                      goto setp;
                     }
       setp:
         if (!csound->csoundIsScorePending_ && e->opcod == 'i') {
           /* FIXME: should pause and not mute */
           csound->sstrlen = 0;
-          e->opcod = 'f'; e->p[1] = FL(0.0); e->pcnt = 2;
+          e->opcod = 'f'; e->p[1] = FL(0.0); e->pcnt = 2; e->scnt = 0;
           return 1;
         }
         e->pcnt = pp - &e->p[0];                   /* count the pfields */
         if (e->pcnt>=PMAX) e->pcnt += e->c.extra[0]; /* and overflow fields */
         if (csound->sstrlen) {        /* if string arg present, save it */
           e->strarg = mmalloc(csound, csound->sstrlen); /* FIXME:       */
-          strcpy(e->strarg, csound->sstrbuf);           /* leaks memory */
+          memcpy(e->strarg, csound->sstrbuf, csound->sstrlen); /* leaks memory */
+          e->scnt = csound->scnt;
           csound->sstrlen = 0;
         }
+        else { e->strarg = NULL; e->scnt = 0; } /* is this necessary?? */
         return 1;
       }
     }
     corfile_rm(&(csound->scstr));
     return 0;
 }
-
-
