@@ -31,7 +31,8 @@
 typedef struct {
     OPDS   h;
     MYFLT  *args[VARGMAX];
+    int argno;
+    int flag;
 } LINEVENT;
 
 #endif      /* CSOUND_LINEVENT_H */
-
