@@ -34,30 +34,26 @@ void    *mmalloc(CSOUND *, size_t);
 void    *mcalloc(CSOUND *, size_t);
 void    *mrealloc(CSOUND *, void *, size_t);
 void    mfree(CSOUND *, void *);
+char    *cs_strdup(CSOUND*, char*);
+char    *cs_strndup(CSOUND*, char*, size_t);
 void    csoundAuxAlloc(CSOUND *, size_t, AUXCH *), auxchfree(CSOUND *, INSDS *);
 void    fdrecord(CSOUND *, FDCH *), fdclose(CSOUND *, FDCH *);
 void    fdchclose(CSOUND *, INSDS *);
 CS_PRINTF2  void    synterr(CSOUND *, const char *, ...);
 CS_NORETURN CS_PRINTF2  void    csoundDie(CSOUND *, const char *, ...);
 CS_PRINTF2  int     csoundInitError(CSOUND *, const char *, ...);
-CS_PRINTF2  int     csoundPerfError(CSOUND *, const char *, ...);
+CS_PRINTF3  int     csoundPerfError(CSOUND *, INSDS *ip, const char *, ...);
 CS_PRINTF2  void    csoundWarning(CSOUND *, const char *, ...);
 CS_PRINTF2  void    csoundDebugMsg(CSOUND *, const char *, ...);
 CS_PRINTF2  void    csoundErrorMsg(CSOUND *, const char *, ...);
 void    csoundErrMsgV(CSOUND *, const char *, const char *, va_list);
 CS_NORETURN void    csoundLongJmp(CSOUND *, int retval);
-void    putop(CSOUND *, TEXT *);
-void    rdorchfile(CSOUND *), otran(CSOUND *);
-char    argtyp(CSOUND *, char *);
 TEXT    *getoptxt(CSOUND *, int *);
-int     express(CSOUND *, char *);
-int     lgexist(CSOUND *, const char *);
-void    oload(CSOUND *);
 void    reverbinit(CSOUND *);
 void    dispinit(CSOUND *);
 int     init0(CSOUND *);
 void    scsort(CSOUND *, FILE *, FILE *);
-void    scsortstr(CSOUND *, CORFIL *);
+char    *scsortstr(CSOUND *, CORFIL *);
 int     scxtract(CSOUND *, CORFIL *, FILE *);
 int     rdscor(CSOUND *, EVTBLK *);
 int     musmon(CSOUND *);
@@ -65,7 +61,6 @@ void    RTLineset(CSOUND *);
 FUNC    *csoundFTFind(CSOUND *, MYFLT *);
 FUNC    *csoundFTFindP(CSOUND *, MYFLT *);
 FUNC    *csoundFTnp2Find(CSOUND *, MYFLT *);
-void    cs_beep(CSOUND *);
 MYFLT   intpow(MYFLT, int32);
 void    list_opcodes(CSOUND *, int);
 char    *getstrformat(int format);
@@ -81,8 +76,8 @@ void    add_tmpfile(CSOUND *, char *);
 void    xturnoff(CSOUND *, INSDS *);
 void    xturnoff_now(CSOUND *, INSDS *);
 int     insert_score_event(CSOUND *, EVTBLK *, double);
-MEMFIL  *ldmemfile(CSOUND *, const char *);
-MEMFIL  *ldmemfile2(CSOUND *, const char *, int);
+  //MEMFIL  *ldmemfile(CSOUND *, const char *);
+  //MEMFIL  *ldmemfile2(CSOUND *, const char *, int);
 MEMFIL  *ldmemfile2withCB(CSOUND *csound, const char *filnam, int csFileType,
                          int (*callback)(CSOUND*, MEMFIL*));
 void    rlsmemfiles(CSOUND *);
@@ -100,11 +95,13 @@ int     check_rtaudio_name(char *fName, char **devName, int isOutput);
 int     csoundLoadOpcodeDB(CSOUND *, const char *);
 void    csoundDestroyOpcodeDB(CSOUND *);
 int     csoundCheckOpcodePluginFile(CSOUND *, const char *);
-int     csoundLoadAllPluginOpcodes(CSOUND *);
+  //int     csoundLoadAllPluginOpcodes(CSOUND *);
 int     csoundLoadAndInitModule(CSOUND *, const char *);
 void    csoundNotifyFileOpened(CSOUND *, const char *, int, int, int);
 int     insert_score_event_at_sample(CSOUND *, EVTBLK *, int64_t);
 
+char *get_arg_string(CSOUND *, MYFLT);
+
 /**
  * Register a function to be called at note deactivation.
  * Should be called from the initialisation routine of an opcode.
