@@ -41,6 +41,15 @@ typedef float AudioUnitSampleType;
 #endif
 
 
+typedef struct {
+  char name[128];
+  int outchannels;
+  int inchannels;
+  int indevnum;
+  int outdevnum;
+} Device_Info;
+
+
 typedef struct csdata_ {
   AudioDeviceID dev;
   AudioStreamBasicDescription format;
@@ -50,10 +59,6 @@ typedef struct csdata_ {
   int         currentOutputIndex;
   MYFLT       *inputBuffer;
   MYFLT       *outputBuffer;
-  void        *auLock_in;                /* thread lock for au callback  */
-  void        *clientLock_in;            /* thread lock for rtrecord     */
-  void        *auLock_out;               /* thread lock for au callback  */
-  void        *clientLock_out;           /* thread lock for rtplay       */
   csRtAudioParams *inParm;
   csRtAudioParams *outParm;
   int onchnls, inchnls;
@@ -62,12 +67,16 @@ typedef struct csdata_ {
   CSOUND *csound;
   AudioBufferList *inputdata;
   int disp;
-  int isInputRunning;
-  int isOutputRunning;
   AudioDeviceID defdevin;
   AudioDeviceID defdevout;
+  int devnos;
+  int devin;
+  int devout;
+  void *incb;
+  void *outcb;
 } csdata;
 
+
 OSStatus  Csound_Input(void *inRefCon,
                        AudioUnitRenderActionFlags *ioActionFlags,
                        const AudioTimeStamp *inTimeStamp,
@@ -86,11 +95,13 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
                csdata *cdata, int isInput)
 {
     UInt32  psize, devnum, devnos;
-    AudioDeviceID *sysdevs;
     AudioDeviceID dev;
+    AudioDeviceID *sysdevs;
     AudioStreamBasicDescription format;
     int     i;
+    Device_Info *devinfo;
     UInt32  bufframes, nchnls;
+    int devouts = 0, devins = 0;
     double srate;
     UInt32 enableIO, maxFPS;
     AudioComponent HALOutput;
@@ -106,9 +117,11 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
     CFStringRef devName;
     CFStringEncoding defaultEncoding = CFStringGetSystemEncoding();
 
+
+
     prop.mSelector = (isInput ?
                       kAudioHardwarePropertyDefaultInputDevice :
-                      kAudioHardwarePropertyDefaultOutputDevice );
+                      kAudioHardwarePropertyDefaultOutputDevice);
 
     psize = sizeof(AudioDeviceID);
     AudioObjectGetPropertyData(kAudioObjectSystemObject,
@@ -118,52 +131,143 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
     else cdata->defdevout = dev;
 
     prop.mSelector = kAudioHardwarePropertyDevices;
-
     AudioObjectGetPropertyDataSize(kAudioObjectSystemObject,
-                                 &prop, 0, NULL, &psize);
+                                   &prop, 0, NULL, &psize);
     devnos = psize / sizeof(AudioDeviceID);
     sysdevs = (AudioDeviceID *) malloc(psize);
-
+    devinfo = (Device_Info *) malloc(devnos*sizeof(Device_Info));
     AudioObjectGetPropertyData(kAudioObjectSystemObject,
                                &prop, 0, NULL, &psize, sysdevs);
-    if(cdata->disp){
-      csound->Message(csound,
-                 "==========================================================\n"
-                 "AuHAL Module: found %d device(s):\n", (int) devnos);
-      for (i = 0; i < devnos; i++) {
-        psize = sizeof(CFStringRef);
-        prop.mSelector = kAudioObjectPropertyName;
-        AudioObjectGetPropertyData(sysdevs[i],
-                                   &prop, 0, NULL, &psize, &devName);
-        csound->Message(csound, "=> AuHAL device %d: %s \n", i,
-                   CFStringGetCStringPtr(devName, defaultEncoding));
-        CFRelease(devName);
 
+    cdata->devnos = devnos;
+    for (i = 0; (unsigned int) i < devnos; i++) {
+      AudioBufferList *b;
+      int devchannels, k, n;
+      int numlists;
+      psize = sizeof(CFStringRef);
+      prop.mScope = kAudioObjectPropertyScopeGlobal;
+      prop.mSelector = kAudioObjectPropertyName;
+      AudioObjectGetPropertyData(sysdevs[i],
+                                 &prop, 0, NULL, &psize, &devName);
+      strcpy(devinfo[i].name, CFStringGetCStringPtr(devName, defaultEncoding));
+      CFRelease(devName);
+
+      devchannels = 0;
+      prop.mScope = kAudioDevicePropertyScopeInput;
+      prop.mSelector =  kAudioDevicePropertyStreamConfiguration;
+      AudioObjectGetPropertyDataSize(sysdevs[i],
+                                     &prop, 0, NULL, &psize);
+      b = (AudioBufferList *) malloc(psize);
+      numlists = psize / sizeof(AudioBufferList);
+      AudioObjectGetPropertyData(sysdevs[i],
+                                 &prop, 0, NULL, &psize, b);
+      for(n=0; n < numlists; n++){
+        for(k=0; (unsigned int) k < b[n].mNumberBuffers; k++)
+          devchannels += b[n].mBuffers[k].mNumberChannels;
       }
-      cdata->disp = 0;
-    }
-    if (parm->devName != NULL) {
-      devnum = atoi(parm->devName);
-      if (devnum >= 0 && devnum < devnos)
-        dev  = sysdevs[devnum];
-      prop.mSelector = (isInput ?
-                        kAudioHardwarePropertyDefaultInputDevice :
-                        kAudioHardwarePropertyDefaultOutputDevice );
-      AudioObjectSetPropertyData(kAudioObjectSystemObject, &prop,
-                                 0, NULL, sizeof(AudioDeviceID), &dev);
+      devinfo[i].inchannels = devchannels;
+      if(devchannels) {
+        devins++;
+        devinfo[i].indevnum = devins;
+      } else devinfo[i].indevnum = -1;
+      free(b);
 
-      free(sysdevs);
+      devchannels = 0;
+      prop.mScope = kAudioDevicePropertyScopeOutput;
+      AudioObjectGetPropertyDataSize(sysdevs[i],
+                                     &prop, 0, NULL, &psize);
+      b = (AudioBufferList *) malloc(psize);
+      numlists = psize /sizeof(AudioBufferList);
+      AudioObjectGetPropertyData(sysdevs[i],
+                                 &prop, 0, NULL, &psize, b);
+      for(n=0; n < numlists; n++){
+        for(k=0; (unsigned int) k < b[n].mNumberBuffers; k++)
+          devchannels += b[n].mBuffers[k].mNumberChannels;
+      }
+      devinfo[i].outchannels = devchannels;
+      if(devchannels) {
+        devouts++;
+        devinfo[i].outdevnum = devouts;
+      } else devinfo[i].outdevnum = -1;
+      free(b);
     }
+
+
+    if(cdata->disp)
+      csound->Message(csound,
+         "==========================================================\n");
+
+    if (isInput)
+      csound->Message(csound,
+                      Str("AuHAL Module: found %d input device(s):\n"), devins);
+    else csound->Message(csound,
+                         Str("AuHAL Module: found %d output device(s):\n"),
+                         devouts);
+
+    for (i = 0; (unsigned int)  i < devnos; i++) {
+      if (isInput) {
+        if(devinfo[i].inchannels) {
+          csound->Message(csound, Str("%d: %s (%d channels) \n"),
+                          devinfo[i].indevnum, devinfo[i].name,
+                          devinfo[i].inchannels);
+        }
+      }
+      else {
+        if(devinfo[i].outchannels)
+          csound->Message(csound, Str("%d: %s (%d channels) \n"),
+                          devinfo[i].outdevnum, devinfo[i].name,
+                          devinfo[i].outchannels);
+      }
+    }
+
+    if (parm->devName != NULL) devnum = atoi(parm->devName);
+    else devnum = parm->devNum;
+
+    if (devnum > 0 && devnum < 1024) {
+      int CoreAudioDev = -1;
+      prop.mSelector = kAudioHardwarePropertyDevices;
+      if (isInput) {
+        for(i=0; (unsigned int)  i < devnos; i++) {
+          if((unsigned int) devinfo[i].indevnum == devnum) CoreAudioDev = i;
+        }
+        if(CoreAudioDev >= 0) {
+          prop.mSelector = kAudioHardwarePropertyDefaultInputDevice;
+          dev  = sysdevs[CoreAudioDev];
+          AudioObjectSetPropertyData(kAudioObjectSystemObject, &prop,
+                                     0, NULL, sizeof(AudioDeviceID), &dev);
+        }
+        else csound->Warning(csound, Str("requested device %d out of range"),
+                             devnum);
+      }
+      else {
+        prop.mSelector = kAudioHardwarePropertyDefaultOutputDevice;
+        for(i=0;(unsigned int)  i < devnos; i++) {
+          if((unsigned int) devinfo[i].outdevnum == devnum)  CoreAudioDev = i;
+        }
+        if(CoreAudioDev >= 0) {
+          dev  = sysdevs[CoreAudioDev];
+          AudioObjectSetPropertyData(kAudioObjectSystemObject, &prop,
+                                     0, NULL, sizeof(AudioDeviceID), &dev);
+        }
+        else csound->Warning(csound, Str("requested device %d out of range"),
+                             devnum, devinfo[CoreAudioDev].name);
+      }
+    }
+
+
+    free(sysdevs);
+    free(devinfo);
+
     psize = sizeof(CFStringRef);
     prop.mSelector = kAudioObjectPropertyName;
     AudioObjectGetPropertyData(dev,
                                &prop, 0, NULL, &psize, &devName);
     if(isInput)
       csound->Message(csound, Str("selected input device: %s \n"),
-                 CFStringGetCStringPtr(devName, defaultEncoding));
+                      CFStringGetCStringPtr(devName, defaultEncoding));
     else
       csound->Message(csound, Str("selected output device: %s \n"),
-                 CFStringGetCStringPtr(devName, defaultEncoding));
+                      CFStringGetCStringPtr(devName, defaultEncoding));
 
     CFRelease(devName);
 
@@ -186,12 +290,13 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
     double sr;
     AudioObjectGetPropertyData(dev, &prop, 0, NULL, &psize, &sr);
     if(sr != srate) {
-       csound->Die(csound,
-            "could not set SR, tried %.1f, got %.1f \n", srate, sr);
+      csound->Warning(csound,
+                      Str("Attempted to set device SR, tried %.1f, got %.1f \n"),
+                      srate, sr);
     }
 
     HALOutput = AudioComponentFindNext(NULL, &cd);
-    if(isInput){
+    if (isInput) {
       AudioComponentInstanceNew(HALOutput, &(cdata->inunit));
       enableIO = 1;
       AudioUnitSetProperty(cdata->inunit, kAudioOutputUnitProperty_EnableIO,
@@ -204,12 +309,13 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
       AudioUnitSetProperty(cdata->inunit,kAudioOutputUnitProperty_CurrentDevice,
                            kAudioUnitScope_Global, isInput, &dev, psize);
       aunit = &(cdata->inunit);
-    } else {
+    }
+    else {
       AudioComponentInstanceNew(HALOutput, &(cdata->outunit));
       psize = sizeof(AudioDeviceID);
       /* for output, select device BEFORE enabling IO */
       AudioUnitSetProperty(cdata->outunit, kAudioOutputUnitProperty_CurrentDevice,
-      kAudioUnitScope_Global, isInput, &dev, psize);
+                           kAudioUnitScope_Global, isInput, &dev, psize);
       enableIO = 1;
       AudioUnitSetProperty(cdata->outunit, kAudioOutputUnitProperty_EnableIO,
                            kAudioUnitScope_Output, 0, &enableIO, sizeof(enableIO));
@@ -260,9 +366,10 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
       AudioOutputUnitStart(*aunit);
 
       csound->Message(csound,
-                 "AuHAL module: output device open with %d buffer frames\n"
-                 "======================================================\n",
-                 bufframes);
+                      Str("***** AuHAL module: output device open with %d "
+                          "buffer frames\n"),
+                          bufframes);
+      cdata->disp = 0;
     }
     else {
       AURenderCallbackStruct input;
@@ -286,70 +393,186 @@ int AuHAL_open(CSOUND *csound, const csRtAudioParams * parm,
       AudioUnitInitialize(*aunit);
       AudioOutputUnitStart(*aunit);
       csound->Message(csound,
-                 "AuHAL module: input device open with %d buffer frames\n"
-                 "==============================================\n",
-                 bufframes);
-
+              "***** AuHAL module: input device open with %d buffer frames\n",
+                      bufframes);
     }
+    if(!cdata->disp)
+      csound->Message(csound,
+              "==========================================================\n");
 
+    cdata->disp = 0;
     return 0;
 
 }
 
+int listDevices(CSOUND *csound, CS_AUDIODEVICE *list, int isOutput){
+    UInt32  psize, devnos;
+    AudioDeviceID *sysdevs;
+    Device_Info *devinfo;
+    int     i;
+    int devouts = 0, devins = 0;
+
+    AudioObjectPropertyAddress prop = {
+      kAudioObjectPropertyName,
+      kAudioObjectPropertyScopeGlobal,
+      kAudioObjectPropertyElementMaster
+    };
+    CFStringRef devName;
+    CFStringEncoding defaultEncoding = CFStringGetSystemEncoding();
+    prop.mSelector = kAudioHardwarePropertyDevices;
+    AudioObjectGetPropertyDataSize(kAudioObjectSystemObject,
+                                   &prop, 0, NULL, &psize);
+    devnos = psize / sizeof(AudioDeviceID);
+    sysdevs = (AudioDeviceID *) malloc(psize);
+    devinfo = (Device_Info *) malloc(devnos*sizeof*devinfo);
+    AudioObjectGetPropertyData(kAudioObjectSystemObject,
+                               &prop, 0, NULL, &psize, sysdevs);
+
+    for (i = 0; (unsigned int) i < devnos; i++) {
+      AudioBufferList *b;
+      int devchannels, k, n;
+      int numlists;
+      psize = sizeof(CFStringRef);
+      prop.mScope = kAudioObjectPropertyScopeGlobal;
+      prop.mSelector = kAudioObjectPropertyName;
+      AudioObjectGetPropertyData(sysdevs[i],
+                                 &prop, 0, NULL, &psize, &devName);
+      strcpy(devinfo[i].name, CFStringGetCStringPtr(devName, defaultEncoding));
+      CFRelease(devName);
+
+      devchannels = 0;
+      prop.mScope = kAudioDevicePropertyScopeInput;
+      prop.mSelector =  kAudioDevicePropertyStreamConfiguration;
+      AudioObjectGetPropertyDataSize(sysdevs[i],
+                                     &prop, 0, NULL, &psize);
+      b = (AudioBufferList *) malloc(psize);
+      numlists = psize / sizeof(AudioBufferList);
+      AudioObjectGetPropertyData(sysdevs[i],
+                                 &prop, 0, NULL, &psize, b);
+      for(n=0; n < numlists; n++){
+        for(k=0; (unsigned int)  k < b[n].mNumberBuffers; k++)
+          devchannels += b[n].mBuffers[k].mNumberChannels;
+      }
+      devinfo[i].inchannels = devchannels;
+      if(devchannels) {
+        devins++;
+        devinfo[i].indevnum = devins;
+      } else devinfo[i].indevnum = -1;
+      free(b);
+
+      devchannels = 0;
+      prop.mScope = kAudioDevicePropertyScopeOutput;
+      AudioObjectGetPropertyDataSize(sysdevs[i],
+                                     &prop, 0, NULL, &psize);
+      b = (AudioBufferList *) malloc(psize);
+      numlists = psize /sizeof(AudioBufferList);
+      AudioObjectGetPropertyData(sysdevs[i],
+                                 &prop, 0, NULL, &psize, b);
+      for(n=0; n < numlists; n++){
+        for(k=0; (unsigned int) k < b[n].mNumberBuffers; k++)
+          devchannels += b[n].mBuffers[k].mNumberChannels;
+      }
+      devinfo[i].outchannels = devchannels;
+      if(devchannels) {
+        devouts++;
+        devinfo[i].outdevnum = devouts;
+      } else devinfo[i].outdevnum = -1;
+      free(b);
+    }
+    if(list==NULL){
+      return (isOutput ? devouts : devins);
+    } else {
+
+      char tmp[64], *s;
+      int n=0, i;
+
+      if ((s = (char*) csound->QueryGlobalVariable(csound, "_RTAUDIO")) == NULL)
+        return 0;
+
+      if(!isOutput){
+        for(i=0; (unsigned int)  i < devnos; i++) {
+          if(devinfo[i].inchannels) {
+            strncpy(list[n].device_name,  devinfo[i].name, 63);
+            sprintf(tmp, "dac%d", devinfo[i].indevnum);
+            strncpy(list[n].device_id, tmp, 63);
+            strncpy(list[n].rt_module, s, 63);
+            list[n].max_nchnls = devinfo[i].inchannels;
+            list[n].isOutput = 0;
+            n++;
+          }
+        }
+        return n;
+      } else {
+        for(i=0;(unsigned int) i < devnos; i++){
+          if(devinfo[i].outchannels) {
+            strncpy(list[n].device_name,  devinfo[i].name, 63);
+            sprintf(tmp, "dac%d", devinfo[i].outdevnum);
+            strncpy(list[n].device_id, tmp, 63);
+            strncpy(list[n].rt_module, s, 63);
+            list[n].max_nchnls = devinfo[i].outchannels;
+            list[n].isOutput = 1;
+            n++;
+          }
+        }
+        return n;
+      }
+    }
+}
+
 /* open for audio input */
 static int recopen_(CSOUND *csound, const csRtAudioParams * parm)
 {
     csdata  *cdata;
-
-    if (csound->rtRecord_userdata != NULL)
+    void **recordata = csound->GetRtRecordUserData(csound);
+    if (*(csound->GetRtRecordUserData(csound)) != NULL)
       return 0;
 
     /* allocate structure */
 
-    if(csound->rtPlay_userdata != NULL)
-      cdata = (csdata *) csound->rtPlay_userdata;
+    if(*(csound->GetRtPlayUserData(csound) )!= NULL)
+       cdata = (csdata *) *(csound->GetRtPlayUserData(csound));
     else {
-      cdata = (csdata *) calloc(1, sizeof(csdata));
+       cdata = (csdata *) calloc(1, sizeof(csdata));
       cdata->disp = 1;
     }
 
     cdata->inunit = NULL;
-    cdata->auLock_in = csound->CreateThreadLock();
-    cdata->clientLock_in = csound->CreateThreadLock();
-    csound->WaitThreadLock(cdata->auLock_in, (size_t) 500);
-    csound->WaitThreadLock(cdata->clientLock_in, (size_t) 500);
-    csound->rtRecord_userdata = (void *) cdata;
+    *recordata = (void *) cdata;
     cdata->inParm =  (csRtAudioParams *) parm;
     cdata->csound = cdata->csound;
     cdata->inputBuffer =
       (MYFLT *) calloc (csound->GetInputBufferSize(csound), sizeof(MYFLT));
-    cdata->isInputRunning = 1;
-    return AuHAL_open(csound, parm, cdata, 1);
+    cdata->incb =
+      csound->CreateCircularBuffer(csound,
+                                   parm->bufSamp_HW*parm->nChannels, sizeof(MYFLT));
+    int ret = AuHAL_open(csound, parm, cdata, 1);
+    return ret;
 }
 
 /* open for audio output */
 static int playopen_(CSOUND *csound, const csRtAudioParams * parm)
 {
     csdata  *cdata;
-    if(csound->rtRecord_userdata != NULL)
-      cdata = (csdata *) csound->rtRecord_userdata;
+    void    **playdata = csound->GetRtPlayUserData(csound);
+
+    if(*(csound->GetRtRecordUserData(csound)) != NULL)
+      cdata = (csdata *) *(csound->GetRtRecordUserData(csound));
     else {
       cdata = (csdata *) calloc(1, sizeof(csdata));
       cdata->disp = 1;
     }
     cdata->outunit = NULL;
-    cdata->auLock_out = csound->CreateThreadLock();
-    cdata->clientLock_out = csound->CreateThreadLock();
-    csound->WaitThreadLock(cdata->auLock_out, (size_t) 500);
-    csound->WaitThreadLock(cdata->clientLock_out, (size_t) 500);
-
-    csound->rtPlay_userdata = (void *) cdata;
+    *playdata = (void *) cdata;
     cdata->outParm =  (csRtAudioParams *) parm;
     cdata->csound = csound;
     cdata->outputBuffer =
       (MYFLT *) calloc (csound->GetOutputBufferSize(csound), sizeof(MYFLT));
-    cdata->isOutputRunning = 1;
-    return AuHAL_open(csound, parm, cdata, 0);
+    memset(cdata->outputBuffer, 0,
+           csound->GetOutputBufferSize(csound)*sizeof(MYFLT));
+    cdata->outcb =
+      csound->CreateCircularBuffer(csound,
+                                   parm->bufSamp_HW*parm->nChannels, sizeof(MYFLT));
+    return AuHAL_open(csound, parm,cdata,0);
 }
 
 OSStatus  Csound_Input(void *inRefCon,
@@ -365,28 +588,36 @@ OSStatus  Csound_Input(void *inRefCon,
     MYFLT *inputBuffer = cdata->inputBuffer;
     int j,k;
     AudioUnitSampleType *buffer;
-    if(!cdata->isInputRunning) return 0;
-    csound->WaitThreadLock(cdata->auLock_in,10);
+    int n = inNumberFrames*inchnls;
+    int l;
+    IGN(ioData);
+
     AudioUnitRender(cdata->inunit, ioActionFlags, inTimeStamp, inBusNumber,
                     inNumberFrames, cdata->inputdata);
     for (k = 0; k < inchnls; k++){
       buffer = (AudioUnitSampleType *) cdata->inputdata->mBuffers[k].mData;
-      for(j=0; j < inNumberFrames; j++){
+      for(j=0; (unsigned int) j < inNumberFrames; j++){
         inputBuffer[j*inchnls+k] = buffer[j];
       }
     }
-    csound->NotifyThreadLock(cdata->clientLock_in);
-
+    l = csound->WriteCircularBuffer(csound, cdata->incb,inputBuffer,n);
     return 0;
 }
 
+#define MICROS 1000000
 static int rtrecord_(CSOUND *csound, MYFLT *inbuff_, int nbytes)
 {
     csdata  *cdata;
+    int n = nbytes/sizeof(MYFLT);
+    int m = 0, l, w = n;
+    MYFLT sr = csound->GetSr(csound);
     cdata = (csdata *) *(csound->GetRtRecordUserData(csound));
-    csound->WaitThreadLock(cdata->clientLock_in, (size_t) 500);
-    memcpy(inbuff_,cdata->inputBuffer,nbytes);
-    csound->NotifyThreadLock(cdata->auLock_in);
+    do{
+      l = csound->ReadCircularBuffer(csound,cdata->incb,&inbuff_[m],n);
+      m += l;
+      n -= l;
+      if(n) usleep(MICROS*w/sr);
+    } while(n);
     return nbytes;
 }
 
@@ -397,36 +628,40 @@ OSStatus  Csound_Render(void *inRefCon,
                         UInt32 inNumberFrames,
                         AudioBufferList *ioData)
 {
-
     csdata *cdata = (csdata *) inRefCon;
     CSOUND *csound = cdata->csound;
-    int onchnls = cdata->onchnls;;
+    int onchnls = cdata->onchnls;
     MYFLT *outputBuffer = cdata->outputBuffer;
     int j,k;
     AudioUnitSampleType *buffer;
+    int n = inNumberFrames*onchnls;
+    IGN(ioActionFlags);
+    IGN(inTimeStamp);
+    IGN(inBusNumber);
 
-    if(!cdata->isOutputRunning) return 0;
-    csound->WaitThreadLock(cdata->auLock_out, 10);
+    n = csound->ReadCircularBuffer(csound,cdata->outcb,outputBuffer,n);
     for (k = 0; k < onchnls; k++) {
       buffer = (AudioUnitSampleType *) ioData->mBuffers[k].mData;
-      for(j=0; j < inNumberFrames; j++){
+      for(j=0; (unsigned int) j < inNumberFrames; j++){
         buffer[j] = (AudioUnitSampleType) outputBuffer[j*onchnls+k] ;
       }
     }
-    csound->NotifyThreadLock(cdata->clientLock_out);
-
     return 0;
 }
 
 static void rtplay_(CSOUND *csound, const MYFLT *outbuff_, int nbytes)
 {
-
     csdata  *cdata;
+    int n = nbytes/sizeof(MYFLT);
+    int m = 0, l, w = n;
+    MYFLT sr = csound->GetSr(csound);
     cdata = (csdata *) *(csound->GetRtPlayUserData(csound));
-    csound->WaitThreadLock(cdata->clientLock_out, (size_t) 500);
-    memcpy(cdata->outputBuffer,outbuff_,nbytes);
-    csound->NotifyThreadLock(cdata->auLock_out);
-
+    do {
+      l = csound->WriteCircularBuffer(csound, cdata->outcb,&outbuff_[m],n);
+      m += l;
+      n -= l;
+      if(n) usleep(MICROS*w/sr);
+    } while(n);
 }
 
 /* close the I/O device entirely  */
@@ -448,38 +683,12 @@ static void rtclose_(CSOUND *csound)
         AudioUnitUninitialize(cdata->inunit);
         AudioComponentInstanceDispose(cdata->inunit);
       }
-      cdata->isInputRunning = 0;
 
       if(cdata->outunit != NULL){
         AudioOutputUnitStop(cdata->outunit);
         AudioUnitUninitialize(cdata->outunit);
         AudioComponentInstanceDispose(cdata->outunit);
       }
-      cdata->isOutputRunning = 0;
-
-      if (cdata->clientLock_in != NULL) {
-        csound->NotifyThreadLock(cdata->clientLock_in);
-        csound->DestroyThreadLock(cdata->clientLock_in);
-        cdata->clientLock_in = NULL;
-      }
-      if (cdata->auLock_in != NULL) {
-        csound->NotifyThreadLock(cdata->auLock_in);
-        csound->DestroyThreadLock(cdata->auLock_in);
-        cdata->auLock_in = NULL;
-      }
-
-      if (cdata->clientLock_out != NULL) {
-        csound->NotifyThreadLock(cdata->clientLock_out);
-        csound->DestroyThreadLock(cdata->clientLock_out);
-        cdata->clientLock_in = NULL;
-      }
-
-      if (cdata->auLock_out != NULL) {
-        csound->NotifyThreadLock(cdata->auLock_out);
-        csound->DestroyThreadLock(cdata->auLock_out);
-        cdata->auLock_out = NULL;
-      }
-
 
       if (cdata->outputBuffer != NULL) {
         free(cdata->outputBuffer);
@@ -489,8 +698,9 @@ static void rtclose_(CSOUND *csound)
         free(cdata->inputBuffer);
         cdata->inputBuffer = NULL;
       }
-      csound->rtRecord_userdata = NULL;
-      csound->rtPlay_userdata = NULL;
+
+      *(csound->GetRtRecordUserData(csound)) = NULL;
+      *(csound->GetRtPlayUserData(csound)) = NULL;
 
       if(cdata->inputdata) {
         int i;
@@ -500,34 +710,36 @@ static void rtclose_(CSOUND *csound)
       }
 
       if(cdata->defdevin) {
-      AudioObjectPropertyAddress prop = {
-      kAudioHardwarePropertyDefaultInputDevice,
-      kAudioObjectPropertyScopeGlobal,
-      kAudioObjectPropertyElementMaster
-      };
-      UInt32 psize = sizeof(AudioDeviceID);
-      AudioObjectSetPropertyData(kAudioObjectSystemObject,
-                               &prop, 0, NULL, psize, &cdata->defdevin);
+        AudioObjectPropertyAddress prop = {
+          kAudioHardwarePropertyDefaultInputDevice,
+          kAudioObjectPropertyScopeGlobal,
+          kAudioObjectPropertyElementMaster
+        };
+        UInt32 psize = sizeof(AudioDeviceID);
+        AudioObjectSetPropertyData(kAudioObjectSystemObject,
+                                   &prop, 0, NULL, psize, &cdata->defdevin);
       }
       if(cdata->defdevout) {
-      AudioObjectPropertyAddress prop = {
-     kAudioHardwarePropertyDefaultOutputDevice,
-      kAudioObjectPropertyScopeGlobal,
-      kAudioObjectPropertyElementMaster
-      };
-      UInt32 psize = sizeof(AudioDeviceID);
-      AudioObjectSetPropertyData(kAudioObjectSystemObject,
-                               &prop, 0, NULL, psize, &cdata->defdevout);
-    }
+        AudioObjectPropertyAddress prop = {
+          kAudioHardwarePropertyDefaultOutputDevice,
+          kAudioObjectPropertyScopeGlobal,
+          kAudioObjectPropertyElementMaster
+        };
+        UInt32 psize = sizeof(AudioDeviceID);
+        AudioObjectSetPropertyData(kAudioObjectSystemObject,
+                                   &prop, 0, NULL, psize, &cdata->defdevout);
+      }
+      csound->DestroyCircularBuffer(csound, cdata->incb);
+      csound->DestroyCircularBuffer(csound, cdata->outcb);
       free(cdata);
       csound->Message(csound, Str("AuHAL module: device closed\n"));
-
     }
 }
 
 int csoundModuleInit(CSOUND *csound)
 {
     char   *drv;
+    csound->module_list_add(csound, "auhal", "audio");
     drv = (char *) csound->QueryGlobalVariable(csound, "_RTAUDIO");
     if (drv == NULL)
       return 0;
@@ -536,18 +748,19 @@ int csoundModuleInit(CSOUND *csound)
           strcmp(drv, "coreaudio") == 0 || strcmp(drv, "CoreAudio") == 0 ||
           strcmp(drv, "COREAUDIO") == 0))
       return 0;
-    if (csound->oparms->msglevel & 0x400)
-      csound->Message(csound, Str("rtaudio: coreaaudio-AuHAL module enabled\n"));
+    //if (csound->oparms->msglevel & 0x400)
+    csound->Message(csound, Str("rtaudio: coreaaudio-AuHAL module enabled\n"));
     csound->SetPlayopenCallback(csound, playopen_);
     csound->SetRecopenCallback(csound, recopen_);
     csound->SetRtplayCallback(csound, rtplay_);
     csound->SetRtrecordCallback(csound, rtrecord_);
     csound->SetRtcloseCallback(csound, rtclose_);
+    csound->SetAudioDeviceListCallback(csound, listDevices);
     return 0;
 }
 
 int csoundModuleCreate(CSOUND *csound)
 {
-    CSOUND *p = csound;
+    IGN(csound);
     return 0;
 }
