@@ -250,4 +250,4 @@ typedef struct {
 } WIDGET_GLOBALS;
 }
 
-#define ST(x)   (((WIDGET_GLOBALS*) csound->widgetGlobals)->x)
+#define ST(x)   ((widgetGlobals)->x)
