@@ -30,9 +30,6 @@
 #ifdef MSVC                   /* Thanks to Richard Dobson */
 # define hypot _hypot
 #endif
-#if defined(__WATCOMC__) || defined(__ncc)
-extern double hypot(double, double);
-#endif
 
 int printv(CSOUND *csound, PRINTV *p)
 {
@@ -57,7 +54,7 @@ int fdspset(CSOUND *csound, FSIGDISP *p){
       p->size = *p->points;
     }
     if ((p->fdata.auxp == NULL) ||
-        (p->fdata.size < (int) (p->size*sizeof(MYFLT)))) {
+        (p->fdata.size < (unsigned int) (p->size*sizeof(MYFLT)))) {
       csound->AuxAlloc(csound, p->size*sizeof(MYFLT), &p->fdata);
     }
     sprintf(strmsg, Str("instr %d, pvs-signal %s:"),
@@ -153,16 +150,20 @@ int kdsplay(CSOUND *csound, DSPLAY *p)
     p->nxtp = fp;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("display: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("display: not initialised"));
 }
 
 int dsplay(CSOUND *csound, DSPLAY *p)
 {
     MYFLT  *fp = p->nxtp, *sp = p->signal, *endp = p->endp;
-    int    n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
+    nsmps -= early;
     if (!p->nprds) {
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         fp[n] = sp[n];
         if (fp >= endp) {
           fp = p->begp;
@@ -172,7 +173,7 @@ int dsplay(CSOUND *csound, DSPLAY *p)
     }
     else {
       MYFLT *fp2 = fp + p->bufpts;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         *fp++ = sp[n];
         *fp2++ = sp[n];
         if (!(--p->pntcnt)) {
@@ -377,16 +378,20 @@ int kdspfft(CSOUND *csound, DSPFFT *p)
     p->bufp = bufp;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("dispfft: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("dispfft: not initialised"));
 }
 
 int dspfft(CSOUND *csound, DSPFFT *p)
 {
     MYFLT *sigp = p->signal, *bufp = p->bufp, *endp = p->endp;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (UNLIKELY(p->auxch.auxp==NULL)) goto err1;
-    for (n=0; n<nsmps; n++) {
+    nsmps -= early;
+    for (n=offset; n<nsmps; n++) {
       if (bufp < p->sampbuf) {            /* skip any spare samples */
         bufp++; sigp++;
       }
@@ -417,7 +422,8 @@ int dspfft(CSOUND *csound, DSPFFT *p)
     p->bufp = bufp;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("dispfft: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("dispfft: not initialised"));
 }
 
 #define NTERMS  4
@@ -518,11 +524,11 @@ int tempeset(CSOUND *csound, TEMPEST *p)
       }
     }
     /* calc input lo-pass filter coefs */
-    b = FL(2.0) - COS((*p->ihp * 6.28318 * csound->onedkr));
+    b = FL(2.0) - COS((*p->ihp * 6.28318 * CS_ONEDKR));
     p->coef1 = b - SQRT(b * b - FL(1.0));
     p->coef0 = FL(1.0) - p->coef1;
     p->yt1 = FL(0.0);
-    p->fwdcoef = POWER(FL(0.5), p->timcount*csound->onedkr/(*p->ihtim));
+    p->fwdcoef = POWER(FL(0.5), p->timcount*CS_ONEDKR/(*p->ihtim));
     p->fwdmask = FL(0.0);
 #ifdef DEBUG
     csound->Message(csound,
@@ -557,9 +563,9 @@ int tempest(CSOUND *csound, TEMPEST *p)
     p->yt1 = p->coef0 * *p->kin + p->coef1 * p->yt1; /* get lo-pass of kinput */
 
     if (UNLIKELY(p->auxch.auxp==NULL)) goto err1; /* RWD fix */
-    if (!(--p->countdown)) {                        /* then on countdown:    */
+    if (!(--p->countdown)) {                         /* then on countdown:    */
       MYFLT *memp;
-      MYFLT kin, expect, *xcur = p->xcur;           /* xcur from prv pass    */
+      MYFLT kin, expect, *xcur = p->xcur;            /* xcur from prv pass    */
       MYFLT lamtot = FL(0.0), weightot = FL(0.0);
 
       p->countdown = p->timcount;           /* reset the countdown            */
@@ -697,6 +703,7 @@ int tempest(CSOUND *csound, TEMPEST *p)
     *p->kout = p->tempo;                    /* put current tempo */
     return OK;
  err1:
-      return csound->PerfError(csound, Str("tempest: not initialised"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("tempest: not initialised"));
 }
 
