@@ -37,7 +37,33 @@ static const int dumpf_format_table[9] = {
   CSFTYPE_FLOATS_TEXT,
 };
 
-int kdmpset(CSOUND *csound, KDUMP *p)
+int kdmpset_S(CSOUND *csound, KDUMP *p) {
+    /* open in curdir or pathname */
+    char soundoname[1024];
+    if (UNLIKELY((p->format = (int)*p->iformat) < 1 || p->format > 8)) {
+      return csound->InitError(csound, Str("unknown format request"));
+    }
+    if (UNLIKELY(p->format == 2 || p->format == 3)) {
+      return csound->InitError(csound,
+                               Str("alaw and ulaw not implemented here"));
+    }
+    strncpy(soundoname,  ((STRINGDAT *)p->ifilcod)->data, 1023);
+    if (p->fdch.fd != NULL)
+      fdclose(csound, &(p->fdch));
+    p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundoname,
+                                   "wb", "", dumpf_format_table[p->format], 0);
+    if (UNLIKELY(p->fdch.fd == NULL))
+      return csound->InitError(csound, Str("Cannot open %s"), soundoname);
+    fdrecord(csound, &p->fdch);
+    if ((p->timcount = (int32)(*p->iprd * csound->ekr)) <= 0)
+      p->timcount = 1;
+    p->countdown = p->timcount;
+    return OK;
+
+}
+
+
+int kdmpset_p(CSOUND *csound, KDUMP *p)
 {
     /* open in curdir or pathname */
     char soundoname[1024];
@@ -48,7 +74,9 @@ int kdmpset(CSOUND *csound, KDUMP *p)
       return csound->InitError(csound,
                                Str("alaw and ulaw not implemented here"));
     }
-    csound->strarg2name(csound, soundoname, p->ifilcod, "dumpk.", p->XSTRCODE);
+    if(ISSTRCOD(*p->ifilcod))
+      strncpy(soundoname, get_arg_string(csound, *p->ifilcod), 1023);
+    else csound->strarg2name(csound, soundoname, p->ifilcod, "dumpk.", 0);
     if (p->fdch.fd != NULL)
       fdclose(csound, &(p->fdch));
     p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundoname,
@@ -62,7 +90,7 @@ int kdmpset(CSOUND *csound, KDUMP *p)
     return OK;
 }
 
-int kdmp2set(CSOUND *csound, KDUMP2 *p)
+int kdmp2set_S(CSOUND *csound, KDUMP2 *p)
 {
     /* open in curdir or pathname */
     char soundoname[1024];
@@ -73,7 +101,7 @@ int kdmp2set(CSOUND *csound, KDUMP2 *p)
       return csound->InitError(csound,
                                Str("alaw and ulaw not implemented here"));
     }
-    csound->strarg2name(csound, soundoname, p->ifilcod, "dumpk.", p->XSTRCODE);
+    strncpy(soundoname,  ((STRINGDAT *)p->ifilcod)->data, 1023);
     if (p->fdch.fd != NULL)
       fdclose(csound, &(p->fdch));
     p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundoname,
@@ -87,7 +115,8 @@ int kdmp2set(CSOUND *csound, KDUMP2 *p)
     return OK;
 }
 
-int kdmp3set(CSOUND *csound, KDUMP3 *p)
+
+int kdmp2set_p(CSOUND *csound, KDUMP2 *p)
 {
     /* open in curdir or pathname */
     char soundoname[1024];
@@ -98,7 +127,9 @@ int kdmp3set(CSOUND *csound, KDUMP3 *p)
       return csound->InitError(csound,
                                Str("alaw and ulaw not implemented here"));
     }
-    csound->strarg2name(csound, soundoname, p->ifilcod, "dumpk.", p->XSTRCODE);
+    if(ISSTRCOD(*p->ifilcod))
+      strncpy(soundoname, get_arg_string(csound, *p->ifilcod), 1023);
+    else csound->strarg2name(csound, soundoname, p->ifilcod, "dumpk.", 0);
     if (p->fdch.fd != NULL)
       fdclose(csound, &(p->fdch));
     p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundoname,
@@ -112,7 +143,8 @@ int kdmp3set(CSOUND *csound, KDUMP3 *p)
     return OK;
 }
 
-int kdmp4set(CSOUND *csound, KDUMP4 *p)
+
+int kdmp3set_S(CSOUND *csound, KDUMP3 *p)
 {
     /* open in curdir or pathname */
     char soundoname[1024];
@@ -123,7 +155,61 @@ int kdmp4set(CSOUND *csound, KDUMP4 *p)
       return csound->InitError(csound,
                                Str("alaw and ulaw not implemented here"));
     }
-    csound->strarg2name(csound, soundoname, p->ifilcod, "dumpk.", p->XSTRCODE);
+    strncpy(soundoname,  ((STRINGDAT *)p->ifilcod)->data, 1023);
+    if (p->fdch.fd != NULL)
+      fdclose(csound, &(p->fdch));
+    p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundoname,
+                                   "wb", "", dumpf_format_table[p->format], 0);
+    if (UNLIKELY(p->fdch.fd == NULL))
+      return csound->InitError(csound, Str("Cannot open %s"), soundoname);
+    fdrecord(csound, &p->fdch);
+    if ((p->timcount = (int32)(*p->iprd * csound->ekr)) <= 0)
+      p->timcount = 1;
+    p->countdown = p->timcount;
+    return OK;
+}
+
+
+
+int kdmp3set_p(CSOUND *csound, KDUMP3 *p)
+{
+    /* open in curdir or pathname */
+    char soundoname[1024];
+    if (UNLIKELY((p->format = (int)*p->iformat) < 1 || p->format > 8)) {
+      return csound->InitError(csound, Str("unknown format request"));
+    }
+    if (UNLIKELY(p->format == 2 || p->format == 3)) {
+      return csound->InitError(csound,
+                               Str("alaw and ulaw not implemented here"));
+    }
+    if(ISSTRCOD(*p->ifilcod))
+      strncpy(soundoname, get_arg_string(csound, *p->ifilcod), 1023);
+    else csound->strarg2name(csound, soundoname, p->ifilcod, "dumpk.", 0);
+    if (p->fdch.fd != NULL)
+      fdclose(csound, &(p->fdch));
+    p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundoname,
+                                   "wb", "", dumpf_format_table[p->format], 0);
+    if (UNLIKELY(p->fdch.fd == NULL))
+      return csound->InitError(csound, Str("Cannot open %s"), soundoname);
+    fdrecord(csound, &p->fdch);
+    if ((p->timcount = (int32)(*p->iprd * csound->ekr)) <= 0)
+      p->timcount = 1;
+    p->countdown = p->timcount;
+    return OK;
+}
+
+int kdmp4set_S(CSOUND *csound, KDUMP4 *p)
+{
+    /* open in curdir or pathname */
+    char soundoname[1024];
+    if (UNLIKELY((p->format = (int)*p->iformat) < 1 || p->format > 8)) {
+      return csound->InitError(csound, Str("unknown format request"));
+    }
+    if (UNLIKELY(p->format == 2 || p->format == 3)) {
+      return csound->InitError(csound,
+                               Str("alaw and ulaw not implemented here"));
+    }
+   strncpy(soundoname,  ((STRINGDAT *)p->ifilcod)->data, 1023);
     if (p->fdch.fd != NULL)
       fdclose(csound, &(p->fdch));
     p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundoname,
@@ -137,7 +223,35 @@ int kdmp4set(CSOUND *csound, KDUMP4 *p)
     return OK;
 }
 
-static void nkdump(CSOUND *csound, MYFLT *kp, FILE *ofd, int format, int nk)
+int kdmp4set_p(CSOUND *csound, KDUMP4 *p)
+{
+    /* open in curdir or pathname */
+    char soundoname[1024];
+    if (UNLIKELY((p->format = (int)*p->iformat) < 1 || p->format > 8)) {
+      return csound->InitError(csound, Str("unknown format request"));
+    }
+    if (UNLIKELY(p->format == 2 || p->format == 3)) {
+      return csound->InitError(csound,
+                               Str("alaw and ulaw not implemented here"));
+    }
+    if(ISSTRCOD(*p->ifilcod))
+      strncpy(soundoname, get_arg_string(csound, *p->ifilcod), 1023);
+    else csound->strarg2name(csound, soundoname, p->ifilcod, "dumpk.", 0);
+    if (p->fdch.fd != NULL)
+      fdclose(csound, &(p->fdch));
+    p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundoname,
+                                   "wb", "", dumpf_format_table[p->format], 0);
+    if (p->fdch.fd == NULL)
+      return csound->InitError(csound, Str("Cannot open %s"), soundoname);
+    fdrecord(csound, &p->fdch);
+    if ((p->timcount = (int32)(*p->iprd * csound->ekr)) <= 0)
+      p->timcount = 1;
+    p->countdown = p->timcount;
+    return OK;
+}
+
+static void nkdump(CSOUND *csound, MYFLT *kp, FILE *ofd, int format,
+                   int nk, void *p)
 {
     char  buf1[256], outbuf[256];
     int   len = 0;
@@ -183,17 +297,20 @@ static void nkdump(CSOUND *csound, MYFLT *kp, FILE *ofd, int format, int nk)
       break;
     case 8: *outbuf = '\0';
       while (--nk) {
-        sprintf(buf1, "%6.4f\t", *kp++);
+        CS_SPRINTF(buf1, "%6.4f\t", *kp++);
         strncat(outbuf, buf1, 256);
       }
-      sprintf(buf1, "%6.4f\n", *kp);
+      CS_SPRINTF(buf1, "%6.4f\n", *kp);
       strncat(outbuf, buf1, 256);
       len = strlen(outbuf);
       break;
-    default: csound->Die(csound, Str("unknown kdump format"));
+    default:
+      csound->PerfError(csound,((KDUMP *)p)->h.insdshead,
+                        Str("unknown kdump format"));
     }
     if (UNLIKELY(fwrite(outbuf, len, 1, ofd)!=1)) { /* now write the buffer */
-      csound->PerfError(csound, Str("write failure in dumpk"));
+      csound->PerfError(csound, ((KDUMP *)p)->h.insdshead,
+                        Str("write failure in dumpk"));
     }
 }
 
@@ -204,7 +321,7 @@ int kdump(CSOUND *csound, KDUMP *p)
     if (--p->countdown <= 0) {
       p->countdown = p->timcount;
       kval[0] = *p->ksig;
-      nkdump(csound, kval, p->f, p->format, 1);
+      nkdump(csound, kval, p->f, p->format, 1, p);
     }
     return OK;
 }
@@ -217,7 +334,7 @@ int kdump2(CSOUND *csound, KDUMP2 *p)
       p->countdown = p->timcount;
       kval[0] = *p->ksig1;
       kval[1] = *p->ksig2;
-      nkdump(csound, kval, p->f, p->format, 2);
+      nkdump(csound, kval, p->f, p->format, 2, p);
     }
     return OK;
 }
@@ -231,7 +348,7 @@ int kdump3(CSOUND *csound, KDUMP3 *p)
       kval[0] = *p->ksig1;
       kval[1] = *p->ksig2;
       kval[2] = *p->ksig3;
-      nkdump(csound, kval, p->f, p->format, 3);
+      nkdump(csound, kval, p->f, p->format, 3, p);
     }
     return OK;
 }
@@ -246,7 +363,7 @@ int kdump4(CSOUND *csound, KDUMP4 *p)
       kval[1] = *p->ksig2;
       kval[2] = *p->ksig3;
       kval[3] = *p->ksig4;
-      nkdump(csound, kval, p->f, p->format, 4);
+      nkdump(csound, kval, p->f, p->format, 4, p);
     }
     return OK;
 }
@@ -255,7 +372,7 @@ int kdump4(CSOUND *csound, KDUMP4 *p)
 /* ******** READK and friends; new code 1999 Feb 14 by JPff    ******** */
 /* ******************************************************************** */
 
-int krdset(CSOUND *csound, KREAD *p)
+int krdset_p(CSOUND *csound, KREAD *p)
 {
     /* open in curdir or pathname */
     char soundiname[1024];
@@ -266,8 +383,9 @@ int krdset(CSOUND *csound, KREAD *p)
       return csound->InitError(csound,
                                Str("alaw and ulaw not implemented here"));
     }
-    csound->strarg2name(csound, soundiname,
-                        p->ifilcod, "readk.", p->XSTRCODE);
+     if(ISSTRCOD(*p->ifilcod))
+       strncpy(soundiname, get_arg_string(csound, *p->ifilcod), 1023);
+    else csound->strarg2name(csound, soundiname, p->ifilcod, "readk.", 0);
     if (p->fdch.fd != NULL)
       fdclose(csound, &(p->fdch));
     p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundiname, "rb",
@@ -282,7 +400,7 @@ int krdset(CSOUND *csound, KREAD *p)
     return OK;
 }
 
-int krd2set(CSOUND *csound, KREAD2 *p)
+int krdset_S(CSOUND *csound, KREAD *p)
 {
     /* open in curdir or pathname */
     char soundiname[1024];
@@ -293,8 +411,7 @@ int krd2set(CSOUND *csound, KREAD2 *p)
       return csound->InitError(csound,
                                Str("alaw and ulaw not implemented here"));
     }
-    csound->strarg2name(csound, soundiname, p->ifilcod,
-                        "readk.", p->XSTRCODE);
+    strncpy(soundiname,  ((STRINGDAT *)p->ifilcod)->data, 1023);
     if (p->fdch.fd != NULL)
       fdclose(csound, &(p->fdch));
     p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundiname, "rb",
@@ -309,7 +426,7 @@ int krd2set(CSOUND *csound, KREAD2 *p)
     return OK;
 }
 
-int krd3set(CSOUND *csound, KREAD3 *p)
+int krd2set_S(CSOUND *csound, KREAD2 *p)
 {
     /* open in curdir or pathname */
     char soundiname[1024];
@@ -320,8 +437,7 @@ int krd3set(CSOUND *csound, KREAD3 *p)
       return csound->InitError(csound,
                                Str("alaw and ulaw not implemented here"));
     }
-    csound->strarg2name(csound, soundiname, p->ifilcod,
-                        "readk.", p->XSTRCODE);
+   strncpy(soundiname,  ((STRINGDAT *)p->ifilcod)->data, 1023);
     if (p->fdch.fd != NULL)
       fdclose(csound, &(p->fdch));
     p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundiname, "rb",
@@ -336,7 +452,7 @@ int krd3set(CSOUND *csound, KREAD3 *p)
     return OK;
 }
 
-int krd4set(CSOUND *csound, KREAD4 *p)
+int krd2set_p(CSOUND *csound, KREAD2 *p)
 {
     /* open in curdir or pathname */
     char soundiname[1024];
@@ -347,8 +463,9 @@ int krd4set(CSOUND *csound, KREAD4 *p)
       return csound->InitError(csound,
                                Str("alaw and ulaw not implemented here"));
     }
-    csound->strarg2name(csound, soundiname, p->ifilcod,
-                        "readk.", p->XSTRCODE);
+    if(ISSTRCOD(*p->ifilcod))
+      strncpy(soundiname, get_arg_string(csound, *p->ifilcod), 1023);
+    else csound->strarg2name(csound, soundiname, p->ifilcod, "readk.", 0);
     if (p->fdch.fd != NULL)
       fdclose(csound, &(p->fdch));
     p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundiname, "rb",
@@ -363,6 +480,115 @@ int krd4set(CSOUND *csound, KREAD4 *p)
     return OK;
 }
 
+int krd3set_S(CSOUND *csound, KREAD3 *p)
+{
+    /* open in curdir or pathname */
+    char soundiname[1024];
+    if (UNLIKELY((p->format = (int)*p->iformat) < 1 || p->format > 8)) {
+      return csound->InitError(csound, Str("unknown format request"));
+    }
+    if (UNLIKELY(p->format == 2 || p->format == 3)) {
+      return csound->InitError(csound,
+                               Str("alaw and ulaw not implemented here"));
+    }
+    strncpy(soundiname,  ((STRINGDAT *)p->ifilcod)->data, 1023);
+    if (p->fdch.fd != NULL)
+      fdclose(csound, &(p->fdch));
+    p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundiname, "rb",
+                                   "SFDIR;SSDIR", dumpf_format_table[p->format], 0);
+    if (UNLIKELY(p->fdch.fd == NULL))
+      return csound->InitError(csound, Str("Cannot open %s"), soundiname);
+    fdrecord(csound, &p->fdch);
+    if ((p->timcount = (int32)(*p->iprd * csound->ekr)) <= 0)
+      p->timcount = 1;
+    p->countdown = 0;
+    p->k[0] = p->k[1] = p->k[2] = p->k[3] = FL(0.0);
+    return OK;
+}
+
+int krd3set_p(CSOUND *csound, KREAD3 *p)
+{
+    /* open in curdir or pathname */
+    char soundiname[1024];
+    if (UNLIKELY((p->format = (int)*p->iformat) < 1 || p->format > 8)) {
+      return csound->InitError(csound, Str("unknown format request"));
+    }
+    if (UNLIKELY(p->format == 2 || p->format == 3)) {
+      return csound->InitError(csound,
+                               Str("alaw and ulaw not implemented here"));
+    }
+    if(ISSTRCOD(*p->ifilcod))
+      strncpy(soundiname, get_arg_string(csound, *p->ifilcod), 1023);
+    else csound->strarg2name(csound, soundiname, p->ifilcod, "readk.", 0);
+    if (p->fdch.fd != NULL)
+      fdclose(csound, &(p->fdch));
+    p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundiname, "rb",
+                                   "SFDIR;SSDIR", dumpf_format_table[p->format], 0);
+    if (UNLIKELY(p->fdch.fd == NULL))
+      return csound->InitError(csound, Str("Cannot open %s"), soundiname);
+    fdrecord(csound, &p->fdch);
+    if ((p->timcount = (int32)(*p->iprd * csound->ekr)) <= 0)
+      p->timcount = 1;
+    p->countdown = 0;
+    p->k[0] = p->k[1] = p->k[2] = p->k[3] = FL(0.0);
+    return OK;
+}
+
+int krd4set_S(CSOUND *csound, KREAD4 *p)
+{
+    /* open in curdir or pathname */
+    char soundiname[1024];
+    if (UNLIKELY((p->format = (int)*p->iformat) < 1 || p->format > 8)) {
+      return csound->InitError(csound, Str("unknown format request"));
+    }
+    if (UNLIKELY(p->format == 2 || p->format == 3)) {
+      return csound->InitError(csound,
+                               Str("alaw and ulaw not implemented here"));
+    }
+    strncpy(soundiname,  ((STRINGDAT *)p->ifilcod)->data, 1023);
+    if (p->fdch.fd != NULL)
+      fdclose(csound, &(p->fdch));
+    p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundiname, "rb",
+                                   "SFDIR;SSDIR", dumpf_format_table[p->format], 0);
+    if (UNLIKELY(p->fdch.fd == NULL))
+      return csound->InitError(csound, Str("Cannot open %s"), soundiname);
+    fdrecord(csound, &p->fdch);
+    if ((p->timcount = (int32)(*p->iprd * csound->ekr)) <= 0)
+      p->timcount = 1;
+    p->countdown = 0;
+    p->k[0] = p->k[1] = p->k[2] = p->k[3] = FL(0.0);
+    return OK;
+}
+
+int krd4set_p(CSOUND *csound, KREAD4 *p)
+{
+    /* open in curdir or pathname */
+    char soundiname[1024];
+    if (UNLIKELY((p->format = (int)*p->iformat) < 1 || p->format > 8)) {
+      return csound->InitError(csound, Str("unknown format request"));
+    }
+    if (UNLIKELY(p->format == 2 || p->format == 3)) {
+      return csound->InitError(csound,
+                               Str("alaw and ulaw not implemented here"));
+    }
+    if(ISSTRCOD(*p->ifilcod))
+      strncpy(soundiname, get_arg_string(csound, *p->ifilcod), 1023);
+    else csound->strarg2name(csound, soundiname, p->ifilcod, "readk.", 0);
+    if (p->fdch.fd != NULL)
+      fdclose(csound, &(p->fdch));
+    p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundiname, "rb",
+                                   "SFDIR;SSDIR", dumpf_format_table[p->format], 0);
+    if (UNLIKELY(p->fdch.fd == NULL))
+      return csound->InitError(csound, Str("Cannot open %s"), soundiname);
+    fdrecord(csound, &p->fdch);
+    if ((p->timcount = (int32)(*p->iprd * csound->ekr)) <= 0)
+      p->timcount = 1;
+    p->countdown = 0;
+    p->k[0] = p->k[1] = p->k[2] = p->k[3] = FL(0.0);
+    return OK;
+}
+
+
 static void nkread(CSOUND *csound, MYFLT *kp, FILE *ifd, int format, int nk)
 {
     int   len;
@@ -414,9 +640,9 @@ static void nkread(CSOUND *csound, MYFLT *kp, FILE *ifd, int format, int nk)
         fseek(ifd, -1L, SEEK_CUR);
         *bp = '\0';
 #ifndef USE_DOUBLE
-        sscanf(inbuf,"%f", kp);
+        CS_SSCANF(inbuf,"%f", kp);
 #else
-        sscanf(inbuf,"%lf", kp);
+        CS_SSCANF(inbuf,"%lf", kp);
 #endif
         kp++;
       }
@@ -433,14 +659,14 @@ static void nkread(CSOUND *csound, MYFLT *kp, FILE *ifd, int format, int nk)
         fseek(ifd, -1L, SEEK_CUR);
         *bp = '\0';
 #ifndef USE_DOUBLE
-        sscanf(inbuf,"%f", kp);
+        CS_SSCANF(inbuf,"%f", kp);
 #else
-        sscanf(inbuf,"%lf", kp);
+        CS_SSCANF(inbuf,"%lf", kp);
 #endif
         kp++;
       }
       break;
-    default: csound->Die(csound, Str("unknown kdump format"));
+    default: csound->Warning(csound,Str("unknown kdump format"));
     }
 }
 
@@ -514,12 +740,13 @@ int kread4(CSOUND *csound, KREAD4 *p)
     return OK;
 }
 
-int krdsset(CSOUND *csound, KREADS *p)
+#define INITSIZE 1024
+
+int krdsset_S(CSOUND *csound, KREADS *p)
 {
     /* open in curdir or pathname */
     char soundiname[1024];
-    csound->strarg2name(csound, soundiname,
-                        p->ifilcod, "readk.", p->XSTRCODE);
+    strncpy(soundiname, ((STRINGDAT *)p->ifilcod)->data, 1023);
     if (p->fdch.fd != NULL)
       fdclose(csound, &(p->fdch));
     p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundiname, "rb",
@@ -530,19 +757,51 @@ int krdsset(CSOUND *csound, KREADS *p)
     if ((p->timcount = (int32)(*p->iprd * csound->ekr)) <= 0)
       p->timcount = 1;
     p->countdown = 0;
-    p->lasts = (char*)csound->Malloc(csound, 1+csound->strVarMaxLen);
+    p->lasts = (char*)csound->Calloc(csound, INITSIZE);
     p->lasts[0] = '\0';
+     if(p->str->data == NULL) {
+       p->str->data = mcalloc(csound, INITSIZE);
+        p->str->size = INITSIZE;
+    }
     return OK;
 }
 
+
+int krdsset_p(CSOUND *csound, KREADS *p)
+{
+    /* open in curdir or pathname */
+    char soundiname[1024];
+    if(ISSTRCOD(*p->ifilcod))
+      strncpy(soundiname, get_arg_string(csound, *p->ifilcod), 1023);
+    else csound->strarg2name(csound, soundiname, p->ifilcod, "readk.", 0);
+    if (p->fdch.fd != NULL)
+      fdclose(csound, &(p->fdch));
+    p->fdch.fd = csound->FileOpen2(csound, &(p->f), CSFILE_STD, soundiname, "rb",
+                                   "SFDIR;SSDIR", 0, 0);
+    if (UNLIKELY(p->fdch.fd == NULL))
+      return csound->InitError(csound, Str("Cannot open %s"), soundiname);
+    fdrecord(csound, &p->fdch);
+    if ((p->timcount = (int32)(*p->iprd * csound->ekr)) <= 0)
+      p->timcount = 1;
+    p->countdown = 0;
+    p->lasts = (char*)csound->Malloc(csound, INITSIZE);
+    p->lasts[0] = '\0';
+     if(p->str->data == NULL) {
+       p->str->data = mcalloc(csound, INITSIZE);
+        p->str->size = INITSIZE;
+    }
+    return OK;
+}
+
+
 int kreads(CSOUND *csound, KREADS *p)
 {
     if (--p->countdown <= 0) {
       p->countdown = p->timcount;
-      if (UNLIKELY(fgets(p->lasts, csound->strVarMaxLen,  p->f)==NULL)) {
-        csound->PerfError(csound, "Read failure in readks");
+      if (UNLIKELY(fgets(p->lasts, INITSIZE-1,  p->f)==NULL)) {
+        csound->PerfError(csound, p->h.insdshead, Str("Read failure in readks"));
       }
     }
-    strncpy((char*) p->str, p->lasts, csound->strVarMaxLen);
+    strncpy((char*) p->str->data, p->lasts, INITSIZE);
     return OK;
 }
