@@ -24,7 +24,6 @@
 */
 
 #include "csoundCore.h" /*                            GOTO_OPS.C        */
-#include "oload.h"
 #include "insert.h"     /* for goto's */
 #include "aops.h"       /* for cond's */
 extern int32 strarg2insno(CSOUND *, void *p, int is_string);
@@ -37,7 +36,8 @@ int igoto(CSOUND *csound, GOTO *p)
 
 int kgoto(CSOUND *csound, GOTO *p)
 {
-    csound->pds = p->lblblk->prvp;
+    IGN(csound);
+    CS_PDS = p->lblblk->prvp;
     return OK;
 }
 
@@ -50,8 +50,10 @@ int icgoto(CSOUND *csound, CGOTO *p)
 
 int kcgoto(CSOUND *csound, CGOTO *p)
 {
+   IGN(csound);
     if (*p->cond)
-      csound->pds = p->lblblk->prvp;
+      CS_PDS = p->lblblk->prvp;
+
     return OK;
 }
 
@@ -66,8 +68,9 @@ int ingoto(CSOUND *csound, CGOTO *p)
 
 int kngoto(CSOUND *csound, CGOTO *p)
 {
+    IGN(csound);
     if (!*p->cond)
-      csound->pds = p->lblblk->prvp;
+      CS_PDS = p->lblblk->prvp;
     return OK;
 }
 
@@ -83,7 +86,8 @@ int iingoto(CSOUND *csound, CGOTO *p)
 
 int kingoto(CSOUND *csound, CGOTO *p)
 {
-    csound->pds = p->lblblk->prvp;
+    IGN(csound);
+    CS_PDS = p->lblblk->prvp;
     return OK;
 }
 #endif
@@ -98,10 +102,11 @@ int timset(CSOUND *csound, TIMOUT *p)
 
 int timout(CSOUND *csound, TIMOUT *p)
 {
+   IGN(csound);
     if (p->cnt1)                            /* once delay has expired, */
       p->cnt1--;
     else if (--p->cnt2 >= 0L)               /*  br during idur countdown */
-      csound->pds = p->lblblk->prvp;
+      CS_PDS = p->lblblk->prvp;
     return OK;
 }
 
@@ -114,40 +119,46 @@ int rireturn(CSOUND *csound, LINK *p)
 
 int reinit(CSOUND *csound, GOTO *p)
 {
-    csound->reinitflag = 1;
+    csound->reinitflag = p->h.insdshead->reinitflag = 1;
     csound->curip = p->h.insdshead;
     csound->ids = p->lblblk->prvi;        /* now, despite ANSI C warning:  */
+    if(csound->realtime_audio_flag == 0) {
     while ((csound->ids = csound->ids->nxti) != NULL &&
            csound->ids->iopadr != (SUBR) rireturn)
       (*csound->ids->iopadr)(csound, csound->ids);
-    csound->reinitflag = 0;
+     csound->reinitflag = p->h.insdshead->reinitflag = 0;
+     } else {
+    csound->curip->init_done = 0;
+    }
     return OK;
 }
 
 int rigoto(CSOUND *csound, GOTO *p)
 {
-    if (csound->reinitflag)
+    if (p->h.insdshead->reinitflag)
       csound->ids = p->lblblk->prvi;
     return OK;
 }
 
 int tigoto(CSOUND *csound, GOTO *p)     /* I-time only, NOP at reinit */
 {
-    if (csound->tieflag && !csound->reinitflag)
+    if (p->h.insdshead->tieflag && !p->h.insdshead->reinitflag)
       csound->ids = p->lblblk->prvi;
     return OK;
 }
 
 int tival(CSOUND *csound, EVAL *p)      /* I-time only, NOP at reinit */
 {
-    if (!csound->reinitflag)
-      *p->r = (csound->tieflag ? FL(1.0) : FL(0.0));
+  if (!p->h.insdshead->reinitflag)
+    *p->r = p->h.insdshead->tieflag;
+    /* *p->r = (csound->tieflag ? FL(1.0) : FL(0.0)); */
     return OK;
 }
 
 int ihold(CSOUND *csound, LINK *p)      /* make this note indefinit duration */
 {                                       /* called by ihold statmnt at Itime  */
-    if (!csound->reinitflag) {          /* no-op at reinit                   */
+    IGN(csound);
+    if (!p->h.insdshead->reinitflag) {  /* no-op at reinit                   */
       csound->curip->offbet = -1.0;
       csound->curip->offtim = -1.0;
     }
@@ -156,15 +167,16 @@ int ihold(CSOUND *csound, LINK *p)      /* make this note indefinit duration */
 
 int turnoff(CSOUND *csound, LINK *p)    /* terminate the current instrument  */
 {                                       /* called by turnoff statmt at Ptime */
-    INSDS  *lcurip = csound->pds->insdshead;
+    IGN(csound);
+    INSDS  *lcurip = CS_PDS->insdshead;
     /* IV - Oct 16 2002: check for subinstr and user opcode */
     /* find top level instrument instance */
     while (lcurip->opcod_iobufs)
       lcurip = ((OPCOD_IOBUFS*) lcurip->opcod_iobufs)->parent_ip;
     xturnoff(csound, lcurip);
     if (lcurip->xtratim <= 0)
-      while (csound->pds->nxtp != NULL)
-        csound->pds = csound->pds->nxtp;                /* loop to last opds */
+      while (CS_PDS->nxtp != NULL)
+        CS_PDS = CS_PDS->nxtp;                /* loop to last opds */
     return OK;
 }
 
@@ -176,26 +188,33 @@ int turnoff2(CSOUND *csound, TURNOFF2 *p, int isStringArg)
     int   mode, insno, allow_release;
 
     if(isStringArg){
-     p1 = (MYFLT) strarg2insno(csound, p->kInsNo, (p->XSTRCODE & 1));
-    } else p1 = *(p->kInsNo);
+      p1 = (MYFLT) strarg2insno(csound, ((STRINGDAT *)p->kInsNo)->data, 1);
+    }
+    else  if(ISSTRCOD(*p->kInsNo)) {
+      p1 = (MYFLT) strarg2insno(csound, get_arg_string(csound, *p->kInsNo), 1);
+    }
+    else p1 = *(p->kInsNo);
 
     if (p1 <= FL(0.0))
       return OK;    /* not triggered */
 
     insno = (int) p1;
-    if (UNLIKELY(insno < 1 || insno > (int) csound->maxinsno ||
-                 csound->instrtxtp[insno] == NULL)) {
-      return csoundPerfError(csound, Str("turnoff2: invalid instrument number"));
+    if (UNLIKELY(insno < 1 || insno > (int) csound->engineState.maxinsno ||
+                 csound->engineState.instrtxtp[insno] == NULL)) {
+      return csoundPerfError(csound, p->h.insdshead,
+                             Str("turnoff2: invalid instrument number"));
     }
     mode = (int) (*(p->kFlags) + FL(0.5));
     allow_release = (*(p->kRelease) == FL(0.0) ? 0 : 1);
     if (UNLIKELY(mode < 0 || mode > 15 || (mode & 3) == 3)) {
-      return csoundPerfError(csound, Str("turnoff2: invalid mode parameter"));
+      return csoundPerfError(csound, p->h.insdshead,
+                             Str("turnoff2: invalid mode parameter"));
     }
     ip = &(csound->actanchor);
     ip2 = NULL;
 /*     if ((mode & 4) && !ip->p1){ */
-/*       return csoundPerfError(csound, Str("turnoff2: invalid instrument number")); */
+/*       return csoundPerfError(csound, p->h.insdshead, */
+/*                              Str("turnoff2: invalid instrument number")); */
 /*     }   */
     while ((ip = ip->nxtact) != NULL && (int) ip->insno != insno);
     if (ip == NULL)
@@ -233,8 +252,8 @@ int turnoff2(CSOUND *csound, TURNOFF2 *p, int isStringArg)
       }
     }
     if (!p->h.insdshead->actflg) {  /* if current note was deactivated: */
-      while (csound->pds->nxtp != NULL)
-        csound->pds = csound->pds->nxtp;            /* loop to last opds */
+      while (CS_PDS->nxtp != NULL)
+        CS_PDS = CS_PDS->nxtp;            /* loop to last opds */
     }
     return OK;
 }
@@ -286,35 +305,39 @@ int loop_ge_i(CSOUND *csound, LOOP_OPS *p)
 int loop_l_p(CSOUND *csound, LOOP_OPS *p)
 {
     /* if ((kndxvar += kincr) < klimit) kgoto l */
+    IGN(csound);
     *(p->ndxvar) += *(p->incr);
     if (*(p->ndxvar) < *(p->limit))
-      csound->pds = p->l->prvp;
+      CS_PDS = p->l->prvp;
     return OK;
 }
 
 int loop_le_p(CSOUND *csound, LOOP_OPS *p)
 {
     /* if ((kndxvar += kincr) <= klimit) kgoto l */
+    IGN(csound);
     *(p->ndxvar) += *(p->incr);
     if (*(p->ndxvar) <= *(p->limit))
-      csound->pds = p->l->prvp;
+      CS_PDS = p->l->prvp;
     return OK;
 }
 
 int loop_g_p(CSOUND *csound, LOOP_OPS *p)
 {
     /* if ((kndxvar -= kdecr) > klimit) kgoto l */
+    IGN(csound);
     *(p->ndxvar) -= *(p->incr);
     if (*(p->ndxvar) > *(p->limit))
-      csound->pds = p->l->prvp;
+      CS_PDS = p->l->prvp;
     return OK;
 }
 
 int loop_ge_p(CSOUND *csound, LOOP_OPS *p)
 {
     /* if ((kndxvar -= kdecr) >= klimit) kgoto l */
+    IGN(csound);
     *(p->ndxvar) -= *(p->incr);
     if (*(p->ndxvar) >= *(p->limit))
-      csound->pds = p->l->prvp;
+      CS_PDS = p->l->prvp;
     return OK;
 }
