@@ -49,6 +49,7 @@ void send_midi_message(CSOUND *, int status, int data1, int data2);
 
 int release_set(CSOUND *csound, REL *p)
 {
+    IGN(csound);
     if (p->h.insdshead->xtratim < EXTRA_TIME)
       /* if not initialised by another opcode */
       p->h.insdshead->xtratim = EXTRA_TIME;
@@ -57,6 +58,7 @@ int release_set(CSOUND *csound, REL *p)
 
 int release(CSOUND *csound, REL *p)
 {
+    IGN(csound);
     if (p->h.insdshead->relesing)
       *p->r = FL(1.0); /* TRUE */
     else
@@ -88,7 +90,7 @@ int mclock(CSOUND *csound, MCLOCK *p)
       p->beginning_flag=FALSE;
       return OK;
     }
-    else if ((MYFLT) csound->kcounter > p->clock_tics) {
+    else if ((MYFLT) CS_KCNT > p->clock_tics) {
       send_midi_message(csound, 0xF8, 0, 0);    /* clock message */
       p->clock_tics += p->period;
     }
@@ -143,7 +145,7 @@ int iout_on_dur_set(CSOUND *csound, OUT_ON_DUR *p)
     p->vel = (temp = abs((int)*p->ivel)) < 128 ? temp : 127;
 
     note_on(csound, p->chn,p->num, p->vel);
-    p->istart_time = (MYFLT)csound->kcounter * csound->onedkr;
+    p->istart_time = (MYFLT)CS_KCNT * CS_ONEDKR;
     p->fl_expired = FALSE;
     p->fl_extra_dur = FALSE;
     return OK;
@@ -152,7 +154,7 @@ int iout_on_dur_set(CSOUND *csound, OUT_ON_DUR *p)
 int iout_on_dur(CSOUND *csound, OUT_ON_DUR *p)
 {
     if (!(p->fl_expired)) {
-      MYFLT actual_dur = (MYFLT) csound->kcounter * csound->onedkr
+      MYFLT actual_dur = (MYFLT) CS_KCNT * CS_ONEDKR
                          - p->istart_time;
       MYFLT dur = *p->idur;
       if (dur < actual_dur) {
@@ -170,7 +172,7 @@ int iout_on_dur(CSOUND *csound, OUT_ON_DUR *p)
 int iout_on_dur2(CSOUND *csound, OUT_ON_DUR *p)
 {
     if (!(p->fl_expired)) {
-      MYFLT actual_dur = (MYFLT)csound->kcounter * csound->onedkr
+      MYFLT actual_dur = (MYFLT)CS_KCNT * CS_ONEDKR
                          - p->istart_time;
       MYFLT dur = *p->idur;
       if (dur < actual_dur) {
@@ -198,7 +200,7 @@ int moscil_set(CSOUND *csound, MOSCIL *p)
     if (p->h.insdshead->xtratim < EXTRA_TIME)
       /* if not initialised by another opcode */
       p->h.insdshead->xtratim = EXTRA_TIME;
-    p->istart_time = (MYFLT)csound->kcounter * csound->onedkr;
+    p->istart_time = (MYFLT)CS_KCNT * CS_ONEDKR;
     p->fl_first_note   = TRUE;
     p->fl_note_expired = TRUE;
     p->fl_end_note     = FALSE;
@@ -217,7 +219,7 @@ int moscil(CSOUND *csound, MOSCIL *p)
         p->fl_end_note     = TRUE;
         note_off(csound, p->last_chn, p->last_num, p->last_vel);
       }
-      else if (p->last_dur < (MYFLT)csound->kcounter * csound->onedkr
+      else if (p->last_dur < (MYFLT)CS_KCNT * CS_ONEDKR
                              - p->istart_time) {
         p->fl_note_expired = TRUE;
         note_off(csound, p->last_chn, p->last_num, p->last_vel);
@@ -226,13 +228,13 @@ int moscil(CSOUND *csound, MOSCIL *p)
     else {
       if (!p->fl_end_note
           && p->last_pause + p->last_dur <
-             (MYFLT)csound->kcounter * csound->onedkr - p->istart_time
+             (MYFLT)CS_KCNT * CS_ONEDKR - p->istart_time
           && !(p->h.insdshead->relesing)) {
         MYFLT ftemp;
         p->istart_time = p->istart_time + p->last_pause + p->last_dur;
         p->last_dur   =         /* dur must be at least 1/kr */
-          (ftemp = *p->kdur) > 0 ? ftemp : csound->onedkr;
-        p->last_pause = (ftemp = *p->kpause) > 0 ? ftemp : csound->onedkr;
+          (ftemp = *p->kdur) > 0 ? ftemp : CS_ONEDKR;
+        p->last_pause = (ftemp = *p->kpause) > 0 ? ftemp : CS_ONEDKR;
  first_note:
         {
           int temp;
@@ -250,6 +252,7 @@ int moscil(CSOUND *csound, MOSCIL *p)
 
 int kvar_out_on_set(CSOUND *csound, KOUT_ON *p)
 {
+    IGN(csound);
     if (p->h.insdshead->xtratim < EXTRA_TIME)
       /* if not initialised by another opcode */
       p->h.insdshead->xtratim = EXTRA_TIME;
@@ -308,7 +311,9 @@ int out_controller (CSOUND *csound, OUT_CONTR *p)
       value =  (int)((*p->value - min) * FL(127.0) / (*p->max - min));
       value = (value < 128) ?  value : 127;
       value = (value > -1) ?  value : 0;
-      if (value != p->last_value || *p->chn != p->lastchn || *p->num != p->lastctrl) {
+      if (value != p->last_value ||
+          *p->chn != p->lastchn ||
+          *p->num != p->lastctrl) {
         /* csound->Message(csound, "out contr value: %d\n", value); */
         control_change(csound, (int)*p->chn-1,(int)*p->num ,value);
         p->last_value = value;
@@ -344,7 +349,9 @@ int out_poly_aftertouch (CSOUND *csound, OUT_POLYATOUCH *p)
     value =  (int)((*p->value - min) * FL(127.0) / (*p->max - min));
     value = value < 128 ?  value : 127;
     value = value > -1  ?  value : 0;
-    if (value != p->last_value  || *p->chn != p->lastchn || *p->num != p->lastctrl) {
+    if (value != p->last_value ||
+        *p->chn != p->lastchn  ||
+        *p->num != p->lastctrl) {
       poly_after_touch(csound, (int)*p->chn-1, (int)*p->num, value);
       p->last_value = value;
       p->lastchn = *p->chn;
@@ -383,7 +390,9 @@ int out_controller14 (CSOUND *csound, OUT_CONTR14 *p)
       value = (value < 16384) ?  value : 16383;
       value = (value > -1) ?  value : 0;
 
-      if (value != p->last_value  || *p->chn != p->lastchn || *p->msb_num != p->lastctrl) {
+      if (value != p->last_value  ||
+          *p->chn != p->lastchn   ||
+          *p->msb_num != p->lastctrl) {
         unsigned int msb = value >> 7;
         unsigned int lsb = value & 0x7F;
         csound->Warning(csound, Str("out contr14 msb:%x lsb:%x\n"), msb, lsb);
@@ -423,6 +432,7 @@ int out_pitch_bend(CSOUND *csound, OUT_PB *p)
 
 int kon2_set(CSOUND *csound, KON2 *p)
 {
+   IGN(csound);
     /* if not initialised by another opcode */
     if (p->h.insdshead->xtratim < EXTRA_TIME)
       p->h.insdshead->xtratim = EXTRA_TIME;
@@ -519,6 +529,7 @@ int nrpn(CSOUND *csound, NRPN *p)
 
 int mdelay_set(CSOUND *csound, MDELAY *p)
 {
+   IGN(csound);
     p->read_index = 0;
     p->write_index = 0;
     memset(p->status, 0, DELTAB_LENGTH);
@@ -529,7 +540,7 @@ int mdelay(CSOUND *csound, MDELAY *p)                   /*gab-A6 fixed*/
 {
     int read_index = p->read_index % DELTAB_LENGTH;
     int write_index = p->write_index % DELTAB_LENGTH;
-    MYFLT present_time =  csound->kcounter * csound->onedkr;
+    MYFLT present_time =  CS_KCNT * CS_ONEDKR;
 
     if (((int)*p->in_status == 0x90 || (int)*p->in_status == 0x80)) {
       p->status[write_index] = (int)*p->in_status;
