@@ -24,18 +24,24 @@
 #include "csoundCore.h"
 #include "remote.h"
 
+/* Somewhat revised from the original.  Pete G. Nov 2012
+   More correct, I think, but I could be wrong... (:-/)
+*/
 #ifdef HAVE_SOCKETS
-#ifndef WIN32
-#include <sys/ioctl.h>
-#ifdef LINUX
-#include <linux/if.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-extern int inet_aton (const char *, struct in_addr *);
-#endif
-#include <net/if.h>
-#endif
+  #ifndef WIN32
+    #include <sys/ioctl.h>
+    #ifdef LINUX
+      #include <linux/if.h>
+    #endif
+    #ifdef __HAIKU__
+      #include <sys/sockio.h>
+    #endif
+    #include <sys/socket.h>
+    #include <netinet/in.h>
+    #include <arpa/inet.h>
+    extern int inet_aton (const char *, struct in_addr *);
+    #include <net/if.h>
+  #endif /* not WIN32 */
 #endif /* HAVE_SOCKETS */
 
 #define MAXREMOTES 10
@@ -49,44 +55,79 @@ void remoteRESET(CSOUND *csound)
     csound->remoteGlobals = NULL;
 }
 
-#ifdef HAVE_SOCKETS
-
- /* get the IPaddress of this machine */
-static int getIpAddress(char *ipaddr, char *ifname)
+#if defined(HAVE_SOCKETS)
+#ifndef WIN32
+#include <netdb.h>
+#endif
+#if 0
+static int foo(char *ipaddr)
 {
-    int ret = 1;
-#ifdef WIN32
     /* VL 12/10/06: something needs to go here */
     /* gethostbyname is the real answer; code below is unsafe */
     char hostname[1024];
     struct hostent *he;
     struct sockaddr_in sin;
     gethostname(hostname, sizeof(hostname));
+    printf("hostname=%s\n", hostname);
     he = gethostbyname(hostname);
 
     memset(&sin, 0, sizeof (struct sockaddr_in));
     memmove(&sin.sin_addr, he->h_addr_list[0], he->h_length);
     strcpy(ipaddr, inet_ntoa (sin.sin_addr));
-    ret = 0;
+    printf("IP: %s\n", ipaddr);
+    return 0;
+}
+#endif
 
+ /* get the IPaddress of this machine */
+static int getIpAddress(char *ipaddr)
+{
+#ifdef WIN32
+    /* VL 12/10/06: something needs to go here */
+    /* gethostbyname is the real answer; code below is unsafe */
+    char hostname[1024];
+    struct hostent *he;
+    struct sockaddr_in sin;
+    if (gethostname(hostname, sizeof(hostname))<0) return 1;
+    if ((he = gethostbyname(hostname))==NULL) return 1;
+
+    memset(&sin, 0, sizeof (struct sockaddr_in));
+    memmove(&sin.sin_addr, he->h_addr_list[0], he->h_length);
+    strcpy(ipaddr, inet_ntoa (sin.sin_addr));
+    return 0;
 #else
+    int ret = 1;
     struct ifreq ifr;
     int fd;
 
     fd = socket(AF_INET,SOCK_DGRAM, 0);
     if (fd >= 0) {
-      strcpy(ifr.ifr_name, ifname);
-
+#ifdef MACOSX
+      strcpy(ifr.ifr_name, "en0");
+#else
+      strcpy(ifr.ifr_name, "eth0");
+#endif
       if (ioctl(fd, SIOCGIFADDR, &ifr) == 0) {
         char *local;
         local = inet_ntoa(((struct sockaddr_in *)(&ifr.ifr_addr))->sin_addr);
         strcpy(ipaddr, local);
+        printf("IP for remote: %s\n", ipaddr);
+        ret = 0;
+      }
+      else {
+        strcpy(ifr.ifr_name, "wlan0");
+      if (ioctl(fd, SIOCGIFADDR, &ifr) == 0) {
+        char *local;
+        local = inet_ntoa(((struct sockaddr_in *)(&ifr.ifr_addr))->sin_addr);
+        strcpy(ipaddr, local);
+          printf("IP for remote: %s\n", ipaddr);
         ret = 0;
       }
     }
+    }
     close(fd);
-#endif
     return ret;
+#endif
 }
 
 char remoteID(CSOUND *csound)
@@ -160,7 +201,7 @@ static int callox(CSOUND *csound)
 
     /* get IP adrs of this machine */
     /* FIXME: don't hardcode eth0 */
-    if (UNLIKELY(getIpAddress(ST(ipadrs), "eth0") < 0)) {
+    if (UNLIKELY(getIpAddress(ST(ipadrs)) != 0)) {
       csound->Message(csound, Str("unable to get local ip address."));
       goto error;
     }
@@ -236,7 +277,8 @@ static int CLopen(CSOUND *csound, char *ipadrs)     /* Client -- open to send */
     ST(to_addr).sin_port = htons((int) ST(remote_port)); /* port we will listen on,
                                                             network byte order */
     for (i=0; i<10; i++){
-      if (UNLIKELY(connect(rfd, (struct sockaddr *) &ST(to_addr), sizeof(ST(to_addr))) < 0))
+      if (UNLIKELY(connect(rfd, (struct sockaddr *) &ST(to_addr),
+                           sizeof(ST(to_addr))) < 0))
         csound->Message(csound, Str("---> Could not connect \n"));
       else goto conok;
     }
@@ -258,13 +300,14 @@ int CLsend(CSOUND *csound, int conn, void *data, int length)
 {
     int nbytes;
     if (UNLIKELY((nbytes = write(conn, data, length)) <= 0)) {
-      return csound->PerfError(csound, Str("write to socket failed"));
+      csound->ErrorMsg(csound, Str("write to socket failed"));
+      return NOTOK;
     }
     /*    csound->Message(csound, "nbytes sent: %d \n", nbytes); */
     return OK;
 }
 
-static int SVopen(CSOUND *csound, char *ipadrs_local)
+static int SVopen(CSOUND *csound)
            /* Server -- open to receive */
 {
     int conn, socklisten,opt;
@@ -303,8 +346,8 @@ static int SVopen(CSOUND *csound, char *ipadrs_local)
     inet_aton((const char *)ipadrs, &(ST(local_addr).sin_addr));
 #endif
 /*     ST(local_addr).sin_port = htons((int)REMOT_PORT); */
-    ST(local_addr).sin_port = htons((int) ST(remote_port)); /* port we will listen
-                                                             on, netwrk byt order */
+    ST(local_addr).sin_port =
+      htons((int) ST(remote_port)); /* port we will listen on, netwrk byt order */
     /* associate the socket with the address and port */
     if (UNLIKELY(bind (socklisten,
               (struct sockaddr *) &ST(local_addr),
@@ -334,6 +377,7 @@ static int SVopen(CSOUND *csound, char *ipadrs_local)
 int SVrecv(CSOUND *csound, int conn, void *data, int length)
 {
     struct sockaddr from;
+    IGN(csound);
 #ifdef WIN32 /* VL, 12/10/06: I'm guessing here. If someone knows better, fix it */
 #define MSG_DONTWAIT  0
    int clilen = sizeof(from);
@@ -382,10 +426,12 @@ int insremot(CSOUND *csound, INSREMOT *p)
     }
 /*     csound->Message(csound, Str("*** str1: %s own:%s\n"), */
 /*                     (char *)p->str1 , ST(ipadrs)); */
-    if (strcmp(ST(ipadrs), (char *)p->str1) == 0) {  /* if client is this adrs */
+    if (strcmp(ST(ipadrs), (char *)p->str1->data) == 0) {
+      /* if client is this adrs */
       MYFLT   **argp = p->insno;
       int rfd = 0;
-      if ((rfd = CLopen(csound, (char *)p->str2)) <= 0) /* open port to remote */
+      if ((rfd = CLopen(csound, (char *)p->str2->data)) <= 0)
+        /* open port to remote */
         return NOTOK;
       for (nargs -= 2; nargs--; ) {
         int16 insno = (int16)**argp++;     /* & for each insno */
@@ -399,10 +445,12 @@ int insremot(CSOUND *csound, INSREMOT *p)
       }
       ST(insrfd_list)[ST(insrfd_count)++] = rfd;   /*  and make a list    */
     }
-    else if (!strcmp(ST(ipadrs),(char *)p->str2)) { /* if server is this adrs*/
+    else if (!strcmp(ST(ipadrs),(char *)p->str2->data)) {
+      /* if server is this adrs*/
 /*       csound->Message(csound, Str("*** str2: %s own:%s\n"), */
 /*                       (char *)p->str2 , ST(ipadrs)); */
-      if (UNLIKELY(SVopen(csound, (char *)p->str2) == NOTOK)){ /* open port to listen */
+      /* open port to listen */
+      if (UNLIKELY(SVopen(csound) == NOTOK)){
         return csound->InitError(csound, Str("Failed to open port to listen"));
       }
     }
@@ -424,8 +472,9 @@ int insglobal(CSOUND *csound, INSGLOBAL *p)
       return csound->InitError(csound, Str("missing instr nos"));
     }
     csound->Message(csound, Str("*** str1: %s own:%s\n"),
-                    (char *)p->str1 , ST(ipadrs));
-    if (strcmp(ST(ipadrs), (char *)p->str1) == 0) { /* if client is this adrs */
+                    (char *)p->str1->data , ST(ipadrs));
+    if (strcmp(ST(ipadrs), (char *)p->str1->data) == 0) {
+      /* if client is this adrs */
       MYFLT   **argp = p->insno;
       for (nargs -= 1; nargs--; ) {
         int16 insno = (int16)**argp++;             /* for each insno */
@@ -455,14 +504,16 @@ int midremot(CSOUND *csound, MIDREMOT *p)    /* declare certain channels for
     if (UNLIKELY(nargs < 3)) {
       return csound->InitError(csound, Str("missing channel nos"));
     }
-    if (strcmp(ST(ipadrs), (char *)p->str1) == 0) {  /* if client is this adrs */
+    if (strcmp(ST(ipadrs), (char *)p->str1->data) == 0) {
+      /* if client is this adrs */
       MYFLT   **argp = p->chnum;
       int  rfd;
-      if (UNLIKELY((rfd = CLopen(csound, (char *)p->str2)) <= 0)) /* open port to remot */
+        /* open port to remote */
+      if (UNLIKELY((rfd = CLopen(csound, (char *)p->str2->data)) <= 0))
         return NOTOK;
       for (nargs -= 2; nargs--; ) {
         int16 chnum = (int16)**argp++;               /* & for each channel   */
-        if (UNLIKELY(chnum <= 0 || chnum > 16)) {              /* THESE ARE MIDCHANS+1 */
+        if (UNLIKELY(chnum <= 0 || chnum > 16)) {    /* THESE ARE MIDCHANS+1 */
           return csound->InitError(csound, Str("illegal channel no"));
         }
         if (UNLIKELY(ST(chnrfd)[chnum])) {
@@ -472,8 +523,10 @@ int midremot(CSOUND *csound, MIDREMOT *p)    /* declare certain channels for
                 }
                 ST(chnrfd_list)[ST(chnrfd_count)++] = rfd;   /* and make a list */
     }
-    else if (!strcmp(ST(ipadrs), (char *)p->str2)) { /* if server is this adrs */
-      if (UNLIKELY(SVopen(csound, (char *)p->str2) == NOTOK)){  /* open port to listen */
+    else if (!strcmp(ST(ipadrs), (char *)p->str2->data)) {
+      /* if server is this adrs */
+      /* open port to listen */
+      if (UNLIKELY(SVopen(csound) == NOTOK)){
         return csound->InitError(csound, Str("Failed to open port to listen"));
       }
       csound->oparms->RMidiin = 1;            /* & enable rtevents in */
@@ -497,7 +550,8 @@ int midglobal(CSOUND *csound, MIDGLOBAL *p)
     }
 /*     csound->Message(csound, Str("*** str1: %s own:%s\n"), */
 /*                     (char *)p->str1 , ST(ipadrs)); */
-    if (strcmp(ST(ipadrs), (char *)p->str1) == 0) { /* if client is this adrs */
+    if (strcmp(ST(ipadrs), (char *)p->str1->data) == 0) {
+      /* if client is this adrs */
       MYFLT   **argp = p->chnum;
       for (nargs -= 1; nargs--; ) {
         int16 chnum = (int16)**argp++;             /* for each channel */
@@ -522,6 +576,7 @@ int insSendevt(CSOUND *csound, EVTBLK *evt, int rfd)
     int nn;
     MYFLT *f, *g;
     cpp->strarg = NULL;                     /* copy the initial header */
+    cpp->scnt = 0;
     cpp->opcod = evt->opcod;
     cpp->pcnt = evt->pcnt;
     f = &evt->p2orig;
@@ -531,7 +586,8 @@ int insSendevt(CSOUND *csound, EVTBLK *evt, int rfd)
     bp->type = SCOR_EVT;                    /* insert type and len */
     bp->len = (char *)g - (char *)bp;
     if (UNLIKELY(CLsend(csound, rfd, (void *)bp, (int)bp->len) < 0)) {
-      return csound->PerfError(csound, Str("CLsend failed"));
+      csound->ErrorMsg(csound, Str("CLsend failed"));
+      return NOTOK;
     }
     else return OK;
 }
@@ -555,7 +611,8 @@ int MIDIsendevt(CSOUND *csound, MEVENT *evt, int rfd)
     bp->len = sizeof(int) * 2 + sizeof(MEVENT);
 
     if (UNLIKELY(CLsend(csound, rfd, (void *)bp, (size_t)bp->len) < 0)) {
-      return csound->PerfError(csound, Str("CLsend failed"));
+      csound->ErrorMsg(csound, Str("CLsend failed"));
+      return NOTOK;
     }
     else return OK;
 }
@@ -579,7 +636,8 @@ int MIDIsend_msg(CSOUND *csound, MEVENT *evt, int rfd)
     bp->len = sizeof(int) * 2 + sizeof(MEVENT);
 
     if (UNLIKELY(CLsend(csound, rfd, (void *)bp, (size_t)bp->len) < 0)) {
-      return csound->PerfError(csound, Str("CLsend failed"));
+      csound->ErrorMsg(csound, Str("CLsend failed"));
+      return NOTOK;
     }
     else return OK;
 }
