@@ -3,6 +3,7 @@
 
     Copyright (C) 2005, 2006 Istvan Varga
               (C) 2005       Matt J. Ingalls, John ffitch
+              (C) 2013   V Lazzarini (new string code)
 
     This file is part of Csound.
 
@@ -55,8 +56,10 @@ static void str_set(CSOUND *csound, int ndx, const char *s)
         csound->strsets[i] = NULL;
       csound->strsmax = newmax;
     }
-    if (UNLIKELY(ndx < 0))    /* -ve index */
-      csound->Die(csound, Str("illegal strset index"));
+    if (UNLIKELY(ndx < 0))  {  /* -ve index */
+     csound->InitError(csound, Str("illegal strset index"));
+     return;
+    }
 
     if (csound->strsets[ndx] != NULL) {
       if (strcmp(s, csound->strsets[ndx]) == 0)
@@ -76,7 +79,7 @@ static void str_set(CSOUND *csound, int ndx, const char *s)
 
 int strset_init(CSOUND *csound, STRSET_OP *p)
 {
-    str_set(csound, (int) MYFLT2LRND(*p->indx), (char*) p->str);
+    str_set(csound, (int) MYFLT2LRND(*p->indx), p->str->data);
     return OK;
 }
 
@@ -86,36 +89,50 @@ void strset_option(CSOUND *csound, char *s)
 {
     int indx = 0;
 
-    if (UNLIKELY(!isdigit(*s)))
-      csound->Die(csound, Str("--strset: invalid format"));
+    if (UNLIKELY(!isdigit(*s))) {
+       csound->Warning(csound, Str("--strset: invalid format"));
+       return;
+    }
     do {
       indx = (indx * 10) + (int) (*s++ - '0');
     } while (isdigit(*s));
-    if (UNLIKELY(*s++ != '='))
-      csound->Die(csound, Str("--strset: invalid format"));
+    if (UNLIKELY(*s++ != '=')){
+      csound->Warning(csound, Str("--strset: invalid format"));
+      return;
+    }
     str_set(csound, indx, s);
 }
 
 int strget_init(CSOUND *csound, STRGET_OP *p)
 {
     int   indx;
-
-    ((char*) p->r)[0] = '\0';
-    if (*(p->indx) == SSTRCOD) {
-      if (csound->currevent->strarg == NULL)
+    if (ISSTRCOD(*(p->indx))) {
+      char *ss = csound->currevent->strarg;
+      if (ss == NULL)
         return OK;
-      if ((int) strlen(csound->currevent->strarg) >= csound->strVarMaxLen)
-        return csound->InitError(csound, Str("strget: buffer overflow"));
-      strcpy((char*) p->r, csound->currevent->strarg);
+      ss = get_arg_string(csound, *p->indx);
+      if(p->r->data == NULL) {
+        p->r->data = cs_strdup(csound, ss);
+        p->r->size = strlen(ss)+1;
+       }
+      else if ((int) strlen(ss) >= p->r->size) {
+        mfree(csound, p->r->data);
+        p->r->data = cs_strdup(csound, ss);
+        p->r->size = strlen(ss) + 1;
+      }
+      else strcpy(p->r->data, ss);
       return OK;
     }
     indx = (int)((double)*(p->indx) + (*(p->indx) >= FL(0.0) ? 0.5 : -0.5));
     if (indx < 0 || indx > (int) csound->strsmax ||
         csound->strsets == NULL || csound->strsets[indx] == NULL)
       return OK;
-    if (UNLIKELY((int) strlen(csound->strsets[indx]) >= csound->strVarMaxLen))
-      return csound->InitError(csound, Str("strget: buffer overflow"));
-    strcpy((char*) p->r, csound->strsets[indx]);
+    if (UNLIKELY((int) strlen(csound->strsets[indx]) >= p->r->size)){
+      int size = strlen(csound->strsets[indx]);
+      p->r->data = mrealloc(csound, p->r->data, size + 1);
+      p->r->size = size + 1;
+    }
+    strcpy((char*) p->r->data, csound->strsets[indx]);
     return OK;
 }
 
@@ -126,68 +143,128 @@ static CS_NOINLINE int StrOp_ErrMsg(void *p, const char *msg)
 
     if (UNLIKELY(csound->ids != NULL && csound->ids->insdshead == csound->curip))
       return csound->InitError(csound, "%s: %s", opname, Str(msg));
-    else if (UNLIKELY(csound->pds != NULL))
-      return csound->PerfError(csound, "%s: %s", opname, Str(msg));
+    else if (UNLIKELY(((OPDS*) p)->insdshead->pds != NULL))
+      return csound->PerfError(csound, ((OPDS*)p)->insdshead,
+                               "%s: %s", opname, Str(msg));
     else
-      csound->Die(csound, "%s: %s", opname, Str(msg));
+      csound->Warning(csound, "%s: %s", opname, Str(msg));
 
     return NOTOK;
 }
-
-static CS_NOINLINE CS_NORETURN void StrOp_FatalError(void *p, const char *msg)
-{
-    CSOUND      *csound = ((OPDS*) p)->insdshead->csound;
-    const char  *opname = csound->GetOpcodeName(p);
-
-    csound->Die(csound, "%s: %s", opname, Str(msg));
-}
-
 /* strcpy */
-
-int strcpy_opcode(CSOUND *csound, STRCPY_OP *p)
+int strcpy_opcode_S(CSOUND *csound, STRCPY_OP *p)
 {
-    char  *newVal = (char*) p->str;
-
-    if (p->r == p->str)
-      return OK;
-    if(*p->str == SSTRCOD){
-       csound->strarg2name(csound, (char *)p->r, p->str, "soundin.", p->XSTRCODE);
-       return OK;
+    char  *newVal = p->str->data;
+    if(p->r->data == NULL) {
+      p->r->data =  cs_strdup(csound, newVal);
+      p->r->size =  strlen(p->str->data) + 1;
+        return OK;
     }
-    if (UNLIKELY((int) strlen(newVal) >= csound->strVarMaxLen))
-      return StrOp_ErrMsg(p, "buffer overflow");
-    strcpy((char*) p->r, newVal);
+    if (p->r->data == p->str->data)
+      return OK;
+    if (UNLIKELY((int) strlen(newVal) >= p->r->size)){
+        mfree(csound, p->r->data);
+        p->r->data = cs_strdup(csound, newVal);
+        p->r->size = strlen(newVal) + 1;
+
+    }
+    else strcpy((char*) p->r->data, newVal);
 
     return OK;
 }
 
-/* strcat */
+int strassign_opcode_S(CSOUND *csound, STRCPY_OP *p)
+{
+  p->r->data = p->str->data;
+  p->r->size = p->str->size;
+  return OK;
+}
+int strassign_opcode_Sk(CSOUND *csound, STRCPY_OP *p)
+{
+  if(strcmp(p->r->data, p->str->data)!=0){
+       p->r->data = p->str->data;
+       p->r->size = p->str->size;
+  }
+  //csound->Message(csound, p->r->data);
+  return OK;
+}
 
+int str_changed(CSOUND *csound, STRCHGD *p){
+  if(p->mem == NULL) {
+    mfree(csound, p->mem);
+    p->mem = cs_strdup(csound, p->str->data);
+  }
+  *p->r = 0;
+  return OK;
+}
+
+int str_changed_k(CSOUND *csound, STRCHGD *p){
+
+  if(strcmp(p->str->data, p->mem)!=0 || p->mem == NULL) {
+    mfree(csound, p->mem);
+    p->mem = cs_strdup(csound, p->str->data);
+    *p->r = 1;
+  }
+  else *p->r = 0;
+  return OK;
+}
+extern char* get_strarg(CSOUND *csound, MYFLT p, char *strarg);
+int strcpy_opcode_p(CSOUND *csound, STRGET_OP *p)
+{
+
+    if (ISSTRCOD(*p->indx)) {
+      char *ss;
+      //printf("here\n ");
+      ss = get_arg_string(csound, *p->indx);
+      //csound->Message(csound, "%s \n", ss);
+      if(p->r->data == NULL) {
+        p->r->data = cs_strdup(csound, ss);
+        p->r->size = strlen(ss)+1;
+      }
+      else if ((int) strlen(ss) >= p->r->size) {
+        mfree(csound, p->r->data);
+        p->r->data = cs_strdup(csound, ss);
+        p->r->size = strlen(ss) + 1;
+      }
+      else strcpy(p->r->data,ss);
+
+    }
+    else{
+      p->r->data = csound->strarg2name(csound, NULL, p->indx, "soundin.", 0);
+    }
+
+    return OK;
+}
+
+
+/* strcat */
 int strcat_opcode(CSOUND *csound, STRCAT_OP *p)
 {
-    char  *newVal1 = (char*) p->str1;
-    char  *newVal2 = (char*) p->str2;
+;
+  int size;
 
-    if (UNLIKELY((int) (strlen(newVal1) + strlen(newVal2)) >= csound->strVarMaxLen))
-      return StrOp_ErrMsg(p, "buffer overflow");
-    if (p->r != p->str2) {
-      if (p->r != p->str1)
-        strcpy((char*) p->r, newVal1);
-      strcat((char*) p->r, newVal2);
-      return OK;
+  size = strlen(p->str1->data) +
+         strlen(p->str2->data);
+
+    if(p->r->data == NULL) {
+        p->r->data = mcalloc(csound, size+1);
+        p->r->size = size+1;
     }
-    if (newVal1[0] == '\0')
-      return OK;
-    memmove(newVal2 + strlen(newVal1), newVal2, strlen(newVal2) + 1);
-    if (p->r != p->str1)
-      memcpy(newVal2, newVal1, strlen(newVal1));
+    else if(UNLIKELY((int) size >= p->r->size)) {
+      p->r->data = mrealloc(csound, p->r->data, size + 1);
+      p->r->size = size + 1;
+    }
+
+    if(p->r->data != p->str1->data)
+     strncpy((char*) p->r->data,  p->str1->data, p->r->size);
+    strcat((char*) p->r->data, p->str2->data);
 
     return OK;
 }
 
 /* strcmp */
 
-int strcmp_opcode(CSOUND *csound, STRCAT_OP *p)
+int strcmp_opcode(CSOUND *csound, STRCMP_OP *p)
 {
     int     i;
 
@@ -195,7 +272,7 @@ int strcmp_opcode(CSOUND *csound, STRCAT_OP *p)
     *(p->r) = FL(0.0);
     if (p->str1 == p->str2)
       return OK;
-    i = strcmp((char*) p->str1, (char*) p->str2);
+    i = strcmp((char*) p->str1->data, (char*) p->str2->data);
     if (i < 0)
       *(p->r) = FL(-1.0);
     else if (i > 0)
@@ -207,29 +284,36 @@ int strcmp_opcode(CSOUND *csound, STRCAT_OP *p)
 /* perform a sprintf-style format -- based on code by Matt J. Ingalls */
 
 static CS_NOINLINE int
-    sprintf_opcode_(void *p,          /* opcode data structure pointer       */
-                    char *dst,        /* pointer to space for output string  */
+sprintf_opcode_(CSOUND *csound,
+                    void *p,          /* opcode data structure pointer       */
+                    STRINGDAT *str,   /* pointer to space for output string  */
                     const char *fmt,  /* format string                       */
                     MYFLT **kvals,    /* array of argument pointers          */
                     int numVals,      /* number of arguments                 */
-                    int strCode,      /* bit mask for string arguments       */
-                    int maxLen)       /* available space in output buffer    */
+                    int strCode)      /* bit mask for string arguments       */
 {
     int     len = 0;
-    char    strseg[2048], *outstring = dst;
+    char    *strseg, *outstring = str->data;
     MYFLT   *parm = NULL;
     int     i = 0, j = 0, n;
     const char  *segwaiting = NULL;
-    int     maxChars;
+    int     maxChars, siz = strlen(fmt) + numVals*7 + 1;
 
     if (UNLIKELY((int) ((OPDS*) p)->optext->t.xincod != 0))
       return StrOp_ErrMsg(p, "a-rate argument not allowed");
-    if (UNLIKELY((int) ((OPDS*) p)->optext->t.inoffs->count > 31))
-      StrOp_FatalError(p, "too many arguments");
+    if (UNLIKELY((int) ((OPDS*) p)->optext->t.inArgCount > 31)){
+      StrOp_ErrMsg(p, "too many arguments");
+      return NOTOK;
+    }
+
+
+    strseg = malloc(siz);
 
     while (1) {
-      if (UNLIKELY(i >= 2047)) {
-        return StrOp_ErrMsg(p, "format string too long");
+      if (UNLIKELY(i >= siz)) {
+        // return StrOp_ErrMsg(p, "format string too long");
+        siz *= 2;
+        strseg = realloc(strseg, siz);
       }
       if (*fmt != '%' && *fmt != '\0') {
         strseg[i++] = *fmt++;
@@ -244,16 +328,16 @@ static CS_NOINLINE int
       }
       /* if already a segment waiting, then lets print it */
       if (segwaiting != NULL) {
-        maxChars = maxLen - len;
+        maxChars = str->size - len;
         strseg[i] = '\0';
         if (UNLIKELY(numVals <= 0)) {
           return StrOp_ErrMsg(p, "insufficient arguments for format");
         }
         numVals--;
-        if (UNLIKELY((*segwaiting == 's' && !(strCode & 1)) ||
-                     (*segwaiting != 's' && (strCode & 1)))) {
-          return StrOp_ErrMsg(p, "argument type inconsistent with format");
-        }
+        /* if (UNLIKELY((*segwaiting == 's' && !(strCode & 1)) || */
+        /*              (*segwaiting != 's' && (strCode & 1)))) { */
+        /*   return StrOp_ErrMsg(p, "argument type inconsistent with format"); */
+        /* } */
         strCode >>= 1;
         parm = kvals[j++];
         switch (*segwaiting) {
@@ -265,6 +349,15 @@ static CS_NOINLINE int
         case 'u':
         case 'c':
 #ifdef HAVE_SNPRINTF
+          if ((int)strlen(strseg) + 24 > (int)maxChars) {
+            int offs = outstring - str->data;
+            str->data = mrealloc(csound, str->data,
+                                 str->size  + 13);
+            str->size += 24;
+            maxChars += 24;
+            outstring = str->data + offs;
+            //printf("maxchars = %d  %s\n", maxChars, strseg);
+          }
           n = snprintf(outstring, maxChars, strseg, (int) MYFLT2LRND(*parm));
 #else
           n = sprintf(outstring, strseg, (int) MYFLT2LRND(*parm));
@@ -277,33 +370,46 @@ static CS_NOINLINE int
         case 'g':
         case 'G':
 #ifdef HAVE_SNPRINTF
+          if(strlen(strseg) + 24 > (unsigned)maxChars) {
+           int offs = outstring - str->data;
+            str->data = mrealloc(csound, str->data,
+                                 str->size  + 13);
+            str->size += 24;
+            maxChars += 24;
+            outstring = str->data + offs;
+            //printf("maxchars = %d  %s\n", maxChars, strseg);
+          }
           n = snprintf(outstring, maxChars, strseg, (double)*parm);
 #else
           n = sprintf(outstring, strseg, (double)*parm);
 #endif
           break;
         case 's':
-          if ((char*)parm == dst) {
+          if (((STRINGDAT*)parm)->data == str->data) {
             return StrOp_ErrMsg(p, "output argument may not be "
                                    "the same as any of the input args");
           }
-#ifdef HAVE_SNPRINTF
-          n = snprintf(outstring, maxChars, strseg, (char*) parm);
-#else
-          n = sprintf(outstring, strseg, (char*)parm);
-#endif
+          if(((STRINGDAT*)parm)->size >= maxChars) {
+            int offs = outstring - str->data;
+            str->data = mrealloc(csound, str->data,
+                                 str->size  + ((STRINGDAT*)parm)->size);
+            str->size += ((STRINGDAT*)parm)->size;
+            maxChars += ((STRINGDAT*)parm)->size;
+            outstring = str->data + offs;
+          }
+          n = sprintf(outstring, strseg, ((STRINGDAT*)parm)->data);
           break;
         default:
           return StrOp_ErrMsg(p, "invalid format string");
         }
-        if (UNLIKELY(n < 0 || n >= maxChars)) {
-#ifdef HAVE_SNPRINTF
+        if (n < 0 || n >= maxChars) {
           /* safely detected excess string length */
-          return StrOp_ErrMsg(p, "buffer overflow");
-#else
-          /* wrote past end of buffer - hope that did not already crash ! */
-          StrOp_FatalError(p, "buffer overflow");
-#endif
+            int offs = outstring - str->data;
+            str->data = mrealloc(csound, str->data, maxChars*2);
+            outstring = str->data + offs;
+            str->size = maxChars*2;
+            maxChars += str->size;
+
         }
         outstring += n;
         len += n;
@@ -321,15 +427,21 @@ static CS_NOINLINE int
     if (UNLIKELY(numVals > 0)) {
       return StrOp_ErrMsg(p, "too many arguments for format");
     }
-    return 0;
+    free(strseg);
+    return OK;
 }
 
 int sprintf_opcode(CSOUND *csound, SPRINTF_OP *p)
 {
-    if (UNLIKELY(sprintf_opcode_(p, (char*) p->r, (char*) p->sfmt, &(p->args[0]),
-                        (int) p->INOCOUNT - 1, ((int) p->XSTRCODE >> 1),
-                                 csound->strVarMaxLen) != 0)) {
-      ((char*) p->r)[0] = '\0';
+  if(p->r->data == NULL) {
+        int size = p->sfmt->size+ 10*((int) p->INOCOUNT);
+        p->r->data = mcalloc(csound, size);
+        p->r->size = size;
+    }
+  if (UNLIKELY(sprintf_opcode_(csound, p, p->r,
+                               (char*) p->sfmt->data, &(p->args[0]),
+                        (int) p->INOCOUNT - 1,0) == NOTOK)) {
+      ((char*) p->r->data)[0] = '\0';
       return NOTOK;
     }
     return OK;
@@ -337,13 +449,16 @@ int sprintf_opcode(CSOUND *csound, SPRINTF_OP *p)
 
 static CS_NOINLINE int printf_opcode_(CSOUND *csound, PRINTF_OP *p)
 {
-    char  buf[3072];
+    STRINGDAT buf;
     int   err;
-    err = sprintf_opcode_(p, buf, (char*) p->sfmt, &(p->args[0]),
-                          (int) p->INOCOUNT - 2, ((int) p->XSTRCODE >> 2),
-                          3072);
+    buf.size = 3072;
+    buf.data = mcalloc(csound, buf.size);
+
+    err = sprintf_opcode_(csound, p, &buf, (char*) p->sfmt->data, &(p->args[0]),
+                          (int) p->INOCOUNT - 2,0);
     if (LIKELY(err == OK))
-      csound->MessageS(csound, CSOUNDMSG_ORCH, "%s", buf);
+      csound->MessageS(csound, CSOUNDMSG_ORCH, buf.data);
+    mfree(csound, buf.data);
 
     return err;
 }
@@ -377,9 +492,9 @@ int puts_opcode_init(CSOUND *csound, PUTS_OP *p)
     p->noNewLine = (*p->no_newline == FL(0.0) ? 0 : 1);
     if (*p->ktrig > FL(0.0)) {
       if (!p->noNewLine)
-        csound->MessageS(csound, CSOUNDMSG_ORCH, "%s\n", (char*) p->str);
+        csound->MessageS(csound, CSOUNDMSG_ORCH, "%s\n", (char*) p->str->data);
       else
-        csound->MessageS(csound, CSOUNDMSG_ORCH, "%s", (char*) p->str);
+        csound->MessageS(csound, CSOUNDMSG_ORCH, (char*) p->str->data);
     }
     p->prv_ktrig = *p->ktrig;
 
@@ -391,36 +506,32 @@ int puts_opcode_perf(CSOUND *csound, PUTS_OP *p)
     if (*p->ktrig != p->prv_ktrig && *p->ktrig > FL(0.0)) {
       p->prv_ktrig = *p->ktrig;
       if (!p->noNewLine)
-        csound->MessageS(csound, CSOUNDMSG_ORCH, "%s\n", (char*) p->str);
+        csound->MessageS(csound, CSOUNDMSG_ORCH, "%s\n", (char*) p->str->data);
       else
-        csound->MessageS(csound, CSOUNDMSG_ORCH, "%s", (char*) p->str);
+        csound->MessageS(csound, CSOUNDMSG_ORCH, (char*) p->str->data);
     }
 
     return OK;
 }
 
-int strtod_opcode(CSOUND *csound, STRSET_OP *p)
+int strtod_opcode_p(CSOUND *csound, STRTOD_OP *p)
 {
     char    *s = NULL, *tmp;
     double  x;
 
-    if (p->XSTRCODE)
-      s = (char*) p->str;
-    else {
-      if (*p->str == SSTRCOD)
-        s = csound->currevent->strarg;
+    if (ISSTRCOD(*p->str))
+        s = get_arg_string(csound, *p->str);
       else {
         int ndx = (int) MYFLT2LRND(*p->str);
         if (ndx >= 0 && ndx <= (int) csound->strsmax && csound->strsets != NULL)
           s = csound->strsets[ndx];
       }
-      if (UNLIKELY(s == NULL))
+   if (UNLIKELY(s == NULL))
         return StrOp_ErrMsg(p, "empty string");
-    }
     while (*s == ' ' || *s == '\t') s++;
     if (UNLIKELY(*s == '\0'))
       return StrOp_ErrMsg(p, "empty string");
-    x = strtod(s, &tmp);
+    x = cs_strtod(s, &tmp);
     if (UNLIKELY(*tmp != '\0'))
       return StrOp_ErrMsg(p, "invalid format");
     *p->indx = (MYFLT) x;
@@ -428,17 +539,82 @@ int strtod_opcode(CSOUND *csound, STRSET_OP *p)
     return OK;
 }
 
-int strtol_opcode(CSOUND *csound, STRSET_OP *p)
-{ 
+int strtod_opcode_S(CSOUND *csound, STRSET_OP *p)
+{
+    char    *s = NULL, *tmp;
+    double  x;
+    s = (char*) p->str->data;
+    while (*s == ' ' || *s == '\t') s++;
+    if (UNLIKELY(*s == '\0'))
+      return StrOp_ErrMsg(p, "empty string");
+    x = cs_strtod(s, &tmp);
+    if (UNLIKELY(*tmp != '\0'))
+      return StrOp_ErrMsg(p, "invalid format");
+    *p->indx = (MYFLT) x;
+
+    return OK;
+}
+
+int strtol_opcode_S(CSOUND *csound, STRSET_OP *p)
+{
     char  *s = NULL;
     int   sgn = 0, radix = 10;
     int32  x = 0L;
 
-    if (p->XSTRCODE)
-      s = (char*) p->str;
-    else {
-      if (*p->str == SSTRCOD)
-        s = csound->currevent->strarg;
+    s = (char*) p->str->data;
+    while (*s == ' ' || *s == '\t') s++;
+    if (UNLIKELY(*s == '\0'))
+      return StrOp_ErrMsg(p, "empty string");
+    if (*s == '+') s++;
+    else if (*s == '-') sgn++, s++;
+    if (*s == '0') {
+      if (s[1] == 'x' || s[1] == 'X')
+        radix = 16, s += 2;
+      else if (s[1] != '\0')
+        radix = 8, s++;
+      else {
+        *p->indx = FL(0.0);
+        return OK;
+      }
+    }
+    if (UNLIKELY(*s == '\0'))
+      return StrOp_ErrMsg(p, "invalid format");
+    switch (radix) {
+      case 8:
+        while (*s >= '0' && *s <= '7') x = (x * 8L) + (int32) (*s++ - '0');
+        break;
+      case 10:
+        while (*s >= '0' && *s <= '9') x = (x * 10L) + (int32) (*s++ - '0');
+        break;
+      default:
+        while (1) {
+          if (*s >= '0' && *s <= '9')
+            x = (x * 16L) + (int32) (*s++ - '0');
+          else if (*s >= 'A' && *s <= 'F')
+            x = (x * 16L) + (int32) (*s++ - 'A') + 10L;
+          else if (*s >= 'a' && *s <= 'f')
+            x = (x * 16L) + (int32) (*s++ - 'a') + 10L;
+          else
+            break;
+        }
+    }
+    if (UNLIKELY(*s != '\0'))
+      return StrOp_ErrMsg(p, "invalid format");
+    if (sgn) x = -x;
+    *p->indx = (MYFLT) x;
+
+    return OK;
+}
+
+
+int strtol_opcode_p(CSOUND *csound, STRTOD_OP *p)
+{
+    char  *s = NULL;
+    int   sgn = 0, radix = 10;
+    int32  x = 0L;
+
+    if (ISSTRCOD(*p->str))
+        s = get_arg_string(csound, *p->str);
       else {
         int ndx = (int) MYFLT2LRND(*p->str);
         if (ndx >= 0 && ndx <= (int) csound->strsmax && csound->strsets != NULL)
@@ -446,7 +622,7 @@ int strtol_opcode(CSOUND *csound, STRSET_OP *p)
       }
       if (UNLIKELY(s == NULL))
         return StrOp_ErrMsg(p, "empty string");
-    }
+
     while (*s == ' ' || *s == '\t') s++;
     if (UNLIKELY(*s == '\0'))
       return StrOp_ErrMsg(p, "empty string");
@@ -507,8 +683,15 @@ int strsub_opcode(CSOUND *csound, STRSUB_OP *p)
     char        *dst;
     int         i, len, strt, end, rev = 0;
 
-    src = (char*) p->Ssrc;
-    dst = (char*) p->Sdst;
+    if(p->Ssrc->data == NULL) return NOTOK;
+    if(p->Sdst->data == NULL) {
+        int size = p->Ssrc->size;
+        p->Sdst->data = mcalloc(csound, size);
+        p->Sdst->size = size;
+    }
+
+    src = (char*) p->Ssrc->data;
+    dst = (char*) p->Sdst->data;
     len = (int) strlen(src);
 #if defined(MSVC) || (defined(__GNUC__) && defined(__i386__))
     strt = (int) MYFLT2LRND(*(p->istart));
@@ -533,14 +716,16 @@ int strsub_opcode(CSOUND *csound, STRSUB_OP *p)
       end = tmp;
       rev = 1;
     }
+
     src += strt;
     len = end - strt;
-    if (UNLIKELY(len >= csound->strVarMaxLen)) {
-      ((char*) p->Sdst)[0] = '\0';
-      return StrOp_ErrMsg(p, "buffer overflow");
+    if (UNLIKELY(len >=  p->Sdst->size)) {
+      p->Sdst->data = mrealloc(csound, p->Sdst->data, len+1);
+      p->Sdst->size = len+1;
+      dst = (char*) p->Sdst->data;
     }
     i = 0;
-    if (!rev || p->Sdst == p->Ssrc) {
+    if (!rev || p->Sdst->data == p->Ssrc->data) {
       /* copying in forward direction is safe */
       /* even if Ssrc and Sdst are the same */
       do {
@@ -582,7 +767,7 @@ int strsub_opcode(CSOUND *csound, STRSUB_OP *p)
 
 int strchar_opcode(CSOUND *csound, STRCHAR_OP *p)
 {
-    int     len = (int) strlen((char*) p->Ssrc);
+    int     len = (int) strlen((char*) p->Ssrc->data);
 #if defined(MSVC) || (defined(__GNUC__) && defined(__i386__))
     int     pos = (int) MYFLT2LRND(*(p->ipos));
 #else
@@ -593,7 +778,7 @@ int strchar_opcode(CSOUND *csound, STRCHAR_OP *p)
     if (pos < 0 || pos >= len)
       *(p->ichr) = FL(0.0);
     else
-      *(p->ichr) = (MYFLT) ((int) ((unsigned char) ((char*) p->Ssrc)[pos]));
+      *(p->ichr) = (MYFLT) ((int) ((unsigned char) ((char*) p->Ssrc->data)[pos]));
 
     return OK;
 }
@@ -608,7 +793,9 @@ int strchar_opcode(CSOUND *csound, STRCHAR_OP *p)
 int strlen_opcode(CSOUND *csound, STRLEN_OP *p)
 {
     (void) csound;
-    *(p->ilen) = (MYFLT) ((int) strlen((char*) p->Ssrc));
+    if(p->Ssrc->size)
+      *(p->ilen) = (MYFLT) strlen(p->Ssrc->data);
+    else *(p->ilen) = FL(0.0);
     return OK;
 }
 
@@ -626,10 +813,16 @@ int strupper_opcode(CSOUND *csound, STRUPPER_OP *p)
     const char  *src;
     char        *dst;
     int         i;
+    if(p->Ssrc->data == NULL) return NOTOK;
+    if(p->Sdst->data == NULL) {
+        int size = p->Ssrc->size;
+        p->Sdst->data = mcalloc(csound, size);
+        p->Sdst->size = size;
+    }
 
     (void) csound;
-    src = (char*) p->Ssrc;
-    dst = (char*) p->Sdst;
+    src = (char*) p->Ssrc->data;
+    dst = (char*) p->Sdst->data;
     for (i = 0; src[i] != '\0'; i++) {
       unsigned char   tmp;
       tmp = (unsigned char) src[i];
@@ -644,10 +837,16 @@ int strlower_opcode(CSOUND *csound, STRUPPER_OP *p)
     const char  *src;
     char        *dst;
     int         i;
+    if(p->Ssrc->data == NULL) return NOTOK;
+    if(p->Sdst->data == NULL) {
+        int size = p->Ssrc->size;
+        p->Sdst->data = mcalloc(csound, size);
+        p->Sdst->size = size;
+    }
 
     (void) csound;
-    src = (char*) p->Ssrc;
-    dst = (char*) p->Sdst;
+    src = (char*) p->Ssrc->data;
+    dst = (char*) p->Sdst->data;
     for (i = 0; src[i] != '\0'; i++) {
       unsigned char   tmp;
       tmp = (unsigned char) src[i];
@@ -672,14 +871,15 @@ int getcfg_opcode(CSOUND *csound, GETCFG_OP *p)
 #else
     int         opt = (int) (*(p->iopt) + FL(0.5));
 #endif
-    char        buf[64];
+    char        buf[32];
 
-    ((char*) p->Sdst)[0] = (char) 0;
-    buf[0] = (char) 0;
+
+    ((char*) p->Sdst->data)[0] = '\0';
+    buf[0] = '\0';
     s = &(buf[0]);
     switch (opt) {
-    case 1:             /* maximum length of string variables */
-      sprintf(&(buf[0]), "%d", (int) csound->strVarMaxLen - 1);
+    case 1:             /* maximum length of variable */
+      sprintf(&(buf[0]), "%d", (int) p->Sdst->size - 1);
       break;
     case 2:             /* input sound file name */
       s = (csound->oparms->sfread && !csound->initonly ?
@@ -709,15 +909,12 @@ int getcfg_opcode(CSOUND *csound, GETCFG_OP *p)
       s = "Win32";
 #elif defined(MACOSX)
       s = "MacOSX";
-#elif defined(mac_classic)
-      s = "MacOS";
 #else
       s = "unknown";
 #endif
       break;
     case 7:             /* is the channel I/O callback set ? (0: no, 1: yes) */
-      buf[0] = (csound->channelIOCallback_
-                == (CsoundChannelIOCallback_t) NULL ? '0' : '1');
+      buf[0] = 0;
       buf[1] = '\0';
       break;
     default:
@@ -725,9 +922,17 @@ int getcfg_opcode(CSOUND *csound, GETCFG_OP *p)
                                        *(p->iopt));
     }
     if (s != NULL) {
-      if (UNLIKELY((int) strlen(s) >= csound->strVarMaxLen))
-        return csound->InitError(csound, Str("getcfg: buffer overflow"));
-      strcpy((char*) p->Sdst, s);
+
+    if(p->Sdst->data == NULL) {
+        int size = strlen(s) + 1;
+        p->Sdst->data = mcalloc(csound, size);
+        p->Sdst->size = size;
+    }
+    else if (UNLIKELY((int) strlen(s) >=  p->Sdst->size)) {
+        p->Sdst->data = mrealloc(csound, p->Sdst->data, strlen(s) + 1);
+        p->Sdst->size = strlen(s) + 1;
+      }
+      strcpy((char*) p->Sdst->data, s);
     }
 
     return OK;
@@ -743,8 +948,8 @@ int getcfg_opcode(CSOUND *csound, GETCFG_OP *p)
 
 int strindex_opcode(CSOUND *csound, STRINDEX_OP *p)
 {
-    const char  *s1 = (char*) p->Ssrc1;
-    const char  *s2 = (char*) p->Ssrc2;
+    const char  *s1 = (char*) p->Ssrc1->data;
+    const char  *s2 = (char*) p->Ssrc2->data;
     int         i, j;
 
     (void) csound;
@@ -775,8 +980,8 @@ int strindex_opcode(CSOUND *csound, STRINDEX_OP *p)
 
 int strrindex_opcode(CSOUND *csound, STRINDEX_OP *p)
 {
-    const char  *s1 = (char*) p->Ssrc1;
-    const char  *s2 = (char*) p->Ssrc2;
+    const char  *s1 = (char*) p->Ssrc1->data;
+    const char  *s2 = (char*) p->Ssrc2->data;
     int         i, j, k;
 
     (void) csound;
@@ -798,4 +1003,3 @@ int strrindex_opcode(CSOUND *csound, STRINDEX_OP *p)
 
     return OK;
 }
-
