@@ -69,17 +69,24 @@ int ephsset(CSOUND *csound, EPHSOR *p)
 int ephsor(CSOUND *csound, EPHSOR *p)
 {
     double      phase;
-    int         n, nsmps=csound->ksmps;
+    uint32_t    offset = p->h.insdshead->ksmps_offset;
+    uint32_t    early  = p->h.insdshead->ksmps_no_end;
+    uint32_t    n, nsmps = CS_KSMPS;
     MYFLT       *rs, *aphs, onedsr = csound->onedsr;
-    double b = p->b;
+    double      b = p->b;
     double      incr, R = *p->kR;
 
     rs = p->sr;
+    if (UNLIKELY(offset)) memset(rs, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rs[nsmps], '\0', early*sizeof(MYFLT));
+    }
     aphs = p->aphs;
     phase = p->curphs;
     if (p->XINCODE) {
       MYFLT *cps = p->xcps;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         incr = (double)(cps[n] * onedsr);
         rs[n] = (MYFLT) b;
         aphs[n] = (MYFLT) phase;
@@ -97,7 +104,7 @@ int ephsor(CSOUND *csound, EPHSOR *p)
     }
     else {
       incr = (double)(*p->xcps * onedsr);
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         rs[n] = (MYFLT) b;
         aphs[n] = (MYFLT) phase;
         phase += incr;
@@ -121,7 +128,7 @@ int kphsor(CSOUND *csound, PHSOR *p)
 {
     double      phs;
     *p->sr = (MYFLT)(phs = p->curphs);
-    if (UNLIKELY((phs += (double)*p->xcps * csound->onedkr) >= 1.0))
+    if (UNLIKELY((phs += (double)*p->xcps * CS_ONEDKR) >= 1.0))
       phs -= 1.0;
     else if (UNLIKELY(phs < 0.0))
       phs += 1.0;
@@ -132,15 +139,22 @@ int kphsor(CSOUND *csound, PHSOR *p)
 int phsor(CSOUND *csound, PHSOR *p)
 {
     double      phase;
-    int         n, nsmps=csound->ksmps;
+    uint32_t    offset = p->h.insdshead->ksmps_offset;
+    uint32_t    early  = p->h.insdshead->ksmps_no_end;
+    uint32_t    n, nsmps = CS_KSMPS;
     MYFLT       *rs, onedsr = csound->onedsr;
     double      incr;
 
     rs = p->sr;
+    if (UNLIKELY(offset)) memset(rs, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rs[nsmps], '\0', early*sizeof(MYFLT));
+    }
     phase = p->curphs;
     if (p->XINCODE) {
       MYFLT *cps = p->xcps;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         incr = (double)(cps[n] * onedsr);
         rs[n] = (MYFLT)phase;
         phase += incr;
@@ -154,7 +168,7 @@ int phsor(CSOUND *csound, PHSOR *p)
     }
     else {
       incr = (double)(*p->xcps * onedsr);
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         rs[n] = (MYFLT)phase;
         phase += incr;
         if (UNLIKELY((MYFLT)phase >= 1.0)){
@@ -168,6 +182,7 @@ int phsor(CSOUND *csound, PHSOR *p)
     return OK;
 }
 
+#ifdef SOME_FINE_DAY
 /*****************************************************************************/
 /*****************************************************************************/
 
@@ -194,7 +209,7 @@ int phsor(CSOUND *csound, PHSOR *p)
 
 static int itblchk(CSOUND *csound, TABLE *p)
 {
-    if (UNLIKELY((p->ftp = csound->FTFind(csound, p->xfn)) == NULL))
+    if (UNLIKELY((p->ftp = csound->FTnp2Find(csound, p->xfn)) == NULL))
       return NOTOK;
 
     /* Although TABLE has an integer variable for the table number
@@ -271,7 +286,7 @@ int tblset(CSOUND *csound, TABLE *p)
     if (UNLIKELY(p->XINCODE != p->XOUTCODE)) {
       const char  *opname = csound->GetOpcodeName(p);
       const char  *msg = Str("%s: table index type inconsistent with output");
-      if (UNLIKELY(csound->ksmps == 1))
+      if (UNLIKELY(CS_KSMPS == 1))
         csound->Warning(csound, msg, opname);
       else {
         return csound->InitError(csound, msg, opname);
@@ -288,7 +303,7 @@ int tblsetkt(CSOUND *csound, TABLE *p)
     if (UNLIKELY(p->XINCODE != p->XOUTCODE)) {
       const char  *opname = csound->GetOpcodeName(p);
       const char  *msg = Str("%s: table index type inconsistent with output");
-      if (UNLIKELY(csound->ksmps == 1))
+      if (UNLIKELY(CS_KSMPS == 1))
         csound->Warning(csound, msg, opname);
       else {
         return csound->InitError(csound, msg, opname);
@@ -365,7 +380,7 @@ int itabl3(CSOUND *csound, TABLE *p)
  * with interpolation.
  *
  * This is typically used for k rate reading - where they are called
- * as a result of being listed in a line in opcodlst.  They are also
+ * as a result of being listed in a line in engineState.opcodlst.  They are also
  * called by two functions which after they have coped with any change
  * in the k rate function table number.
  *
@@ -380,7 +395,7 @@ int itabl3(CSOUND *csound, TABLE *p)
  *
  * These do the reading at a rate with an a rate index.
  *
- * They are called directly via their entries in opcodlst, and also by
+ * They are called directly via their entries in engineState.opcodlst, and also by
  * two functions which call them after they have coped with any change
  * in the k rate function table number.
  *
@@ -459,7 +474,8 @@ int ktable(CSOUND *csound, TABLE   *p)
     *p->rslt = *(ftp->ftable + indx);
     return OK;
  err1:
-    return csound->PerfError(csound, Str("table(krate): not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("table(krate): not initialised"));
 }
 
 /* tablefn()  */
@@ -473,20 +489,27 @@ int tablefn(CSOUND *csound, TABLE *p)
     FUNC        *ftp;
     MYFLT       *rslt, *pxndx, *tab;
     int32       indx, mask, length;
-    int         n, nsmps=csound->ksmps;
+    uint32_t    koffset = p->h.insdshead->ksmps_offset;
+    uint32_t    early  = p->h.insdshead->ksmps_no_end;
+    uint32_t    n, nsmps = CS_KSMPS;
     MYFLT       ndx, xbmul, offset;
     int         wrap = p->wrap;
 
     ftp = p->ftp;
     if (UNLIKELY(ftp==NULL)) goto err1;            /* RWD fix */
     rslt = p->rslt;
+    if (UNLIKELY(koffset)) memset(rslt, '\0', koffset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rslt[nsmps], '\0', early*sizeof(MYFLT));
+    }
     length = ftp->flen;
     pxndx = p->xndx;
     xbmul = (MYFLT)p->xbmul;
     offset = p->offset;
     mask = ftp->lenmask;
     tab = ftp->ftable;
-    for (n=0; n<nsmps; n++) {
+    for (n=koffset; n<nsmps; n++) {
       /* Read in the next raw index and increment the pointer ready
        * for the next cycle.
        *
@@ -511,7 +534,8 @@ int tablefn(CSOUND *csound, TABLE *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("table: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("table: not initialised"));
 }
 
 /* ktabli() */
@@ -603,7 +627,8 @@ int ktabli(CSOUND *csound, TABLE   *p)
     *p->rslt = v1 + (v2 - v1) * fract;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("tablei(krate): not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("tablei(krate): not initialised"));
 }
 
 
@@ -694,7 +719,8 @@ int ktabl3(CSOUND *csound, TABLE   *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("table3(krate): not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("table3(krate): not initialised"));
 }
 
 /* tabli() */
@@ -706,13 +732,20 @@ int tabli(CSOUND *csound, TABLE   *p)
 {
     FUNC        *ftp;
     int32        indx, mask, length;
-    int         n, nsmps=csound->ksmps;
+    uint32_t     koffset = p->h.insdshead->ksmps_offset;
+    uint32_t     early  = p->h.insdshead->ksmps_no_end;
+    uint32_t     n, nsmps = CS_KSMPS;
     MYFLT       *rslt, *pxndx, *tab;
-    MYFLT       fract, v1, v2, ndx, xbmul, offset;
+    MYFLT        fract, v1, v2, ndx, xbmul, offset;
 
     ftp = p->ftp;
     if (UNLIKELY(ftp==NULL)) goto err1;
     rslt   = p->rslt;
+    if (UNLIKELY(koffset)) memset(rslt, '\0', koffset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rslt[nsmps], '\0', early*sizeof(MYFLT));
+    }
     length = ftp->flen;
     pxndx  = p->xndx;
     xbmul  = (MYFLT)p->xbmul;
@@ -721,7 +754,7 @@ int tabli(CSOUND *csound, TABLE   *p)
     tab    = ftp->ftable;
     /* As for ktabli() code to handle non wrap mode, and wrap mode.  */
     if (!p->wrap) {
-      for (n=0; n<nsmps; n++) {
+      for (n=koffset; n<nsmps; n++) {
         /* Read in the next raw index and increment the pointer ready
          * for the next cycle.
          * Then multiply the ndx by the denormalising factor and add in
@@ -747,7 +780,7 @@ int tabli(CSOUND *csound, TABLE   *p)
       }
     }
     else {                      /* wrap mode */
-      for (n=0; n<nsmps; n++) {
+      for (n=koffset; n<nsmps; n++) {
         /* Read in the next raw index and increment the pointer ready
          * for the next cycle.
          * Then multiply the ndx by the denormalising factor and add in
@@ -767,28 +800,36 @@ int tabli(CSOUND *csound, TABLE   *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("tablei: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("tablei: not initialised"));
 }
 
 int tabl3(CSOUND *csound, TABLE *p)     /* Like tabli but cubic interpolation */
 {
     FUNC        *ftp;
     int32        indx, mask, length;
-    int         n, nsmps=csound->ksmps;
+    uint32_t     koffset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t     n, nsmps = CS_KSMPS;
     MYFLT       *rslt, *pxndx, *tab;
-    MYFLT       fract, v1, v2, ndx, xbmul, offset;
-    int         wrap = p->wrap;
+    MYFLT        fract, v1, v2, ndx, xbmul, offset;
+    int          wrap = p->wrap;
 
     ftp = p->ftp;
     if (UNLIKELY(ftp==NULL)) goto err1;
     rslt = p->rslt;
+    if (UNLIKELY(koffset)) memset(rslt, '\0', koffset*sizeof(MYFLT));
+        if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rslt[nsmps], '\0', early*sizeof(MYFLT));
+    }
     length = ftp->flen;
     pxndx = p->xndx;
     xbmul = (MYFLT)p->xbmul;
     offset = p->offset;
     mask = ftp->lenmask;
     tab = ftp->ftable;
-    for (n=0; n<nsmps; n++) {
+    for (n=koffset; n<nsmps; n++) {
       /* Read in the next raw index and increment the pointer ready
        * for the next cycle.
        * Then multiply the ndx by the denormalising factor and add in
@@ -833,7 +874,8 @@ int tabl3(CSOUND *csound, TABLE *p)     /* Like tabli but cubic interpolation */
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("table3: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("table3: not initialised"));
 }
 
 /*************************************/
@@ -909,15 +951,17 @@ static int ftkrchk(CSOUND *csound, TABLE *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("k rate function table no. %f < 1"),
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("k rate function table no. %f < 1"),
                              *p->xfn);
  err2:
-    return csound->PerfError(csound, Str("Offset %f < 0 or > tablelength"),
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("Offset %f < 0 or > tablelength"),
                              p->offset);
 }
 
 /* Now for the four functions, which are called as a result of being
- * listed in opcodlst in entry.c */
+ * listed in engineState.opcodlst in entry.c */
 
 int    ktablekt(CSOUND *csound, TABLE *p)
 {
@@ -954,6 +998,7 @@ int    tabl3kt(CSOUND *csound, TABLE *p)
     if (LIKELY(ftkrchk(csound,p)==OK)) return tabl3(csound,p);
     return NOTOK;
 }
+#endif /* SOME_FINE_DAY */
 
 int ko1set(CSOUND *csound, OSCIL1 *p)
 {
@@ -967,7 +1012,7 @@ int ko1set(CSOUND *csound, OSCIL1 *p)
     p->ftp = ftp;
     p->phs = 0;
     p->dcnt = (int32)(*p->idel * csound->ekr);
-    p->kinc = (int32) (csound->kicvt / *p->idur);
+    p->kinc = (int32) (CS_KICVT / *p->idur);
     if (p->kinc==0) p->kinc = 1;
     return OK;
 }
@@ -994,7 +1039,8 @@ int kosc1(CSOUND *csound, OSCIL1 *p)
     p->dcnt = dcnt;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscil1(krate): not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscil1(krate): not initialised"));
 }
 
 int kosc1i(CSOUND *csound, OSCIL1   *p)
@@ -1025,14 +1071,15 @@ int kosc1i(CSOUND *csound, OSCIL1   *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscil1i(krate): not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscil1i(krate): not initialised"));
 }
 
 int oscnset(CSOUND *csound, OSCILN *p)
 {
     FUNC        *ftp;
 
-    if (LIKELY((ftp = csound->FTFind(csound, p->ifn)) != NULL)) {
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL)) {
       p->ftp = ftp;
       p->inc = ftp->flen * *p->ifrq * csound->onedsr;
       p->index = FL(0.0);
@@ -1046,16 +1093,23 @@ int oscnset(CSOUND *csound, OSCILN *p)
 int osciln(CSOUND *csound, OSCILN *p)
 {
     MYFLT *rs = p->rslt;
-    int32  n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (UNLIKELY(p->ftp==NULL)) goto err1;
+    if (UNLIKELY(offset)) memset(rs, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rs[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->ntimes) {
       MYFLT *ftbl = p->ftp->ftable;
       MYFLT amp = *p->kamp;
       MYFLT ndx = p->index;
       MYFLT inc = p->inc;
       MYFLT maxndx = p->maxndx;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         rs[n] = ftbl[(int32)ndx] * amp;
         if (UNLIKELY((ndx += inc) > maxndx)) {
           if (--p->ntimes)
@@ -1078,7 +1132,8 @@ int osciln(CSOUND *csound, OSCILN *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("osciln: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("osciln: not initialised"));
 }
 
 int oscset(CSOUND *csound, OSC *p)
@@ -1102,14 +1157,15 @@ int koscil(CSOUND *csound, OSC *p)
     ftp = p->ftp;
     if (UNLIKELY(ftp==NULL)) goto err1;
     phs = p->lphs;
-    inc = (int32) (*p->xcps * csound->kicvt);
+    inc = (int32) (*p->xcps * CS_KICVT);
     *p->sr = ftp->ftable[phs >> ftp->lobits] * *p->xamp;
     phs += inc;
     phs &= PHMASK;
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscil(krate): not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscil(krate): not initialised"));
 }
 
 int osckk(CSOUND *csound, OSC *p)
@@ -1117,7 +1173,9 @@ int osckk(CSOUND *csound, OSC *p)
     FUNC    *ftp;
     MYFLT   amp, *ar, *ftbl;
     int32   phs, inc, lobits;
-    int     n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     ftp = p->ftp;
     if (UNLIKELY(ftp==NULL)) goto err1;
@@ -1127,7 +1185,12 @@ int osckk(CSOUND *csound, OSC *p)
     lobits = ftp->lobits;
     amp = *p->xamp;
     ar = p->sr;
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       ar[n] = ftbl[phs >> lobits] * amp;
       /* phs += inc; */
       /* phs &= PHMASK; */
@@ -1136,7 +1199,8 @@ int osckk(CSOUND *csound, OSC *p)
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscil: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscil: not initialised"));
 }
 
 int oscka(CSOUND *csound, OSC *p)
@@ -1144,7 +1208,9 @@ int oscka(CSOUND *csound, OSC *p)
     FUNC    *ftp;
     MYFLT   *ar, amp, *cpsp, *ftbl;
     int32    phs, lobits;
-    int     n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   sicvt = csound->sicvt;
 
     ftp = p->ftp;
@@ -1155,7 +1221,12 @@ int oscka(CSOUND *csound, OSC *p)
     cpsp = p->xcps;
     phs = p->lphs;
     ar = p->sr;
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       int32 inc = MYFLT2LONG(cpsp[n] * sicvt);
       ar[n] = ftbl[phs >> lobits] * amp;
       phs += inc;
@@ -1164,7 +1235,8 @@ int oscka(CSOUND *csound, OSC *p)
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscil: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscil: not initialised"));
 }
 
 int oscak(CSOUND *csound, OSC *p)
@@ -1172,7 +1244,9 @@ int oscak(CSOUND *csound, OSC *p)
     FUNC    *ftp;
     MYFLT   *ar, *ampp, *ftbl;
     int32    phs, inc, lobits;
-    int     n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     ftp = p->ftp;
     if (UNLIKELY(ftp==NULL)) goto err1;
@@ -1182,14 +1256,20 @@ int oscak(CSOUND *csound, OSC *p)
     inc = MYFLT2LONG(*p->xcps * csound->sicvt);
     ampp = p->xamp;
     ar = p->sr;
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       ar[n] = ftbl[phs >> lobits] * ampp[n];
       phs = (phs+inc) & PHMASK;
     }
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscil: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscil: not initialised"));
 }
 
 int oscaa(CSOUND *csound, OSC *p)
@@ -1197,7 +1277,9 @@ int oscaa(CSOUND *csound, OSC *p)
     FUNC    *ftp;
     MYFLT   *ar, *ampp, *cpsp, *ftbl;
     int32    phs, lobits;
-    int     n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   sicvt = csound->sicvt;
 
     ftp = p->ftp;
@@ -1208,7 +1290,12 @@ int oscaa(CSOUND *csound, OSC *p)
     ampp = p->xamp;
     cpsp = p->xcps;
     ar = p->sr;
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       int32 inc = MYFLT2LONG(cpsp[n] * sicvt);
       ar[n] = ftbl[phs >> lobits] * ampp[n];
       phs = (phs+inc) & PHMASK;
@@ -1216,7 +1303,8 @@ int oscaa(CSOUND *csound, OSC *p)
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscil: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscil: not initialised"));
 }
 
 int koscli(CSOUND *csound, OSC   *p)
@@ -1232,13 +1320,14 @@ int koscli(CSOUND *csound, OSC   *p)
     ftab = ftp->ftable + (phs >> ftp->lobits);
     v1 = ftab[0];
     *p->sr = (v1 + (ftab[1] - v1) * fract) * *p->xamp;
-    inc = (int32)(*p->xcps * csound->kicvt);
+    inc = (int32)(*p->xcps * CS_KICVT);
     phs += inc;
     phs &= PHMASK;
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscili(krate): not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscili(krate): not initialised"));
 }
 
 int osckki(CSOUND *csound, OSC   *p)
@@ -1246,7 +1335,9 @@ int osckki(CSOUND *csound, OSC   *p)
     FUNC    *ftp;
     MYFLT   fract, v1, amp, *ar, *ft, *ftab;
     int32   phs, inc, lobits;
-    int     n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (UNLIKELY((ftp = p->ftp)==NULL)) goto err1;
     lobits = ftp->lobits;
@@ -1254,8 +1345,13 @@ int osckki(CSOUND *csound, OSC   *p)
     inc = MYFLT2LONG(*p->xcps * csound->sicvt);
     amp = *p->xamp;
     ar = p->sr;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     ft = ftp->ftable;
-    for (n=0; n<nsmps; n++) {
+    for (n=offset; n<nsmps; n++) {
       fract = PFRAC(phs);
       ftab = ft + (phs >> lobits);
       v1 = ftab[0];
@@ -1265,7 +1361,8 @@ int osckki(CSOUND *csound, OSC   *p)
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscili: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscili: not initialised"));
 }
 
 int osckai(CSOUND *csound, OSC   *p)
@@ -1273,7 +1370,9 @@ int osckai(CSOUND *csound, OSC   *p)
     FUNC    *ftp;
     MYFLT   *ar, amp, *cpsp, fract, v1, *ftab, *ft;
     int32    phs, lobits;
-    int     n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   sicvt = csound->sicvt;
 
     ftp = p->ftp;
@@ -1283,8 +1382,13 @@ int osckai(CSOUND *csound, OSC   *p)
     cpsp = p->xcps;
     phs = p->lphs;
     ar = p->sr;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     ft = ftp->ftable;
-    for (n=0;n<nsmps;n++) {
+    for (n=offset;n<nsmps;n++) {
       int32 inc;
       inc = MYFLT2LONG(cpsp[n] * sicvt);
       fract = PFRAC(phs);
@@ -1297,15 +1401,18 @@ int osckai(CSOUND *csound, OSC   *p)
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscili: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscili: not initialised"));
 }
 
 int oscaki(CSOUND *csound, OSC   *p)
 {
     FUNC    *ftp;
-    MYFLT   v1, fract, *ar, *ampp, *ftab, *ft;
+    MYFLT    v1, fract, *ar, *ampp, *ftab, *ft;
     int32    phs, inc, lobits;
-    int     n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     ftp = p->ftp;
     if (UNLIKELY(ftp==NULL)) goto err1;
@@ -1314,8 +1421,13 @@ int oscaki(CSOUND *csound, OSC   *p)
     inc = MYFLT2LONG(*p->xcps * csound->sicvt);
     ampp = p->xamp;
     ar = p->sr;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     ft = ftp->ftable;
-    for (n=0;n<nsmps;n++) {
+    for (n=offset;n<nsmps;n++) {
       fract = (MYFLT) PFRAC(phs);
       ftab = ft + (phs >> lobits);
       v1 = ftab[0];
@@ -1325,7 +1437,8 @@ int oscaki(CSOUND *csound, OSC   *p)
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscili: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscili: not initialised"));
 }
 
 int oscaai(CSOUND *csound, OSC   *p)
@@ -1333,7 +1446,9 @@ int oscaai(CSOUND *csound, OSC   *p)
     FUNC    *ftp;
     MYFLT   v1, fract, *ar, *ampp, *cpsp, *ftab, *ft;
     int32   phs, lobits;
-    int     n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   sicvt = csound->sicvt;
 
     ftp = p->ftp;
@@ -1344,7 +1459,12 @@ int oscaai(CSOUND *csound, OSC   *p)
     ampp = p->xamp;
     cpsp = p->xcps;
     ar = p->sr;
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       int32 inc;
       inc = MYFLT2LONG(cpsp[n] * sicvt);
       fract = (MYFLT) PFRAC(phs);
@@ -1356,7 +1476,8 @@ int oscaai(CSOUND *csound, OSC   *p)
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscili: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscili: not initialised"));
 }
 
 int koscl3(CSOUND *csound, OSC   *p)
@@ -1364,7 +1485,7 @@ int koscl3(CSOUND *csound, OSC   *p)
     FUNC    *ftp;
     int32    phs, inc;
     MYFLT  *ftab, fract;
-    int     x0;
+    int32   x0;
     MYFLT   y0, y1, ym1, y2, amp = *p->xamp;
 
     phs = p->lphs;
@@ -1380,7 +1501,7 @@ int koscl3(CSOUND *csound, OSC   *p)
     else ym1 = ftab[x0++];
     y0 = ftab[x0++];
     y1 = ftab[x0++];
-    if (UNLIKELY(x0>ftp->flen)) y2 = ftab[1]; else y2 = ftab[x0];
+    if (UNLIKELY(x0>(int32)ftp->flen)) y2 = ftab[1]; else y2 = ftab[x0];
     {
       MYFLT frsq = fract*fract;
       MYFLT frcu = frsq*ym1;
@@ -1390,13 +1511,14 @@ int koscl3(CSOUND *csound, OSC   *p)
                       frsq*fract*(t1/FL(6.0) - FL(0.5)*y1) +
                       frsq*(FL(0.5)* y1 - y0));
     }
-    inc = (int32)(*p->xcps * csound->kicvt);
+    inc = (int32)(*p->xcps * CS_KICVT);
     phs += inc;
     phs &= PHMASK;
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscil3(krate): not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscil3(krate): not initialised"));
 }
 
 int osckk3(CSOUND *csound, OSC   *p)
@@ -1404,8 +1526,10 @@ int osckk3(CSOUND *csound, OSC   *p)
     FUNC    *ftp;
     MYFLT   fract, amp, *ar, *ftab;
     int32    phs, inc, lobits;
-    int     n, nsmps=csound->ksmps;
-    int     x0;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int32   x0;
     MYFLT   y0, y1, ym1, y2;
 
     ftp = p->ftp;
@@ -1416,7 +1540,12 @@ int osckk3(CSOUND *csound, OSC   *p)
     inc = MYFLT2LONG(*p->xcps * csound->sicvt);
     amp = *p->xamp;
     ar = p->sr;
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       fract = PFRAC(phs);
       x0 = (phs >> lobits);
       x0--;
@@ -1426,7 +1555,7 @@ int osckk3(CSOUND *csound, OSC   *p)
       else ym1 = ftab[x0++];
       y0 = ftab[x0++];
       y1 = ftab[x0++];
-      if (UNLIKELY(x0>ftp->flen)) y2 = ftab[1]; else y2 = ftab[x0];
+      if (UNLIKELY(x0>(int32)ftp->flen)) y2 = ftab[1]; else y2 = ftab[x0];
 /*    printf("fract = %f; y = %f, %f, %f, %f\n", fract,ym1,y0,y1,y2); */
       {
         MYFLT frsq = fract*fract;
@@ -1447,7 +1576,8 @@ int osckk3(CSOUND *csound, OSC   *p)
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscil3: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscil3: not initialised"));
 }
 
 int oscka3(CSOUND *csound, OSC   *p)
@@ -1455,8 +1585,10 @@ int oscka3(CSOUND *csound, OSC   *p)
     FUNC    *ftp;
     MYFLT   *ar, amp, *cpsp, fract, *ftab;
     int32    phs, lobits;
-    int     n, nsmps=csound->ksmps;
-    int     x0;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int32   x0;
     MYFLT   y0, y1, ym1, y2;
     MYFLT   sicvt = csound->sicvt;
 
@@ -1468,7 +1600,12 @@ int oscka3(CSOUND *csound, OSC   *p)
     cpsp = p->xcps;
     phs = p->lphs;
     ar = p->sr;
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       int32 inc;
       inc = MYFLT2LONG(cpsp[n] * sicvt);
       fract = PFRAC(phs);
@@ -1480,7 +1617,7 @@ int oscka3(CSOUND *csound, OSC   *p)
       else ym1 = ftab[x0++];
       y0 = ftab[x0++];
       y1 = ftab[x0++];
-      if (UNLIKELY(x0>ftp->flen)) y2 = ftab[1]; else y2 = ftab[x0];
+      if (UNLIKELY(x0>(int32)ftp->flen)) y2 = ftab[1]; else y2 = ftab[x0];
       {
         MYFLT frsq = fract*fract;
         MYFLT frcu = frsq*ym1;
@@ -1495,7 +1632,8 @@ int oscka3(CSOUND *csound, OSC   *p)
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscil3: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscil3: not initialised"));
 }
 
 int oscak3(CSOUND *csound, OSC   *p)
@@ -1503,8 +1641,10 @@ int oscak3(CSOUND *csound, OSC   *p)
     FUNC    *ftp;
     MYFLT   fract, *ar, *ampp, *ftab;
     int32    phs, inc, lobits;
-    int     n, nsmps=csound->ksmps;
-    int     x0;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int32   x0;
     MYFLT   y0, y1, ym1, y2;
 
     ftp = p->ftp;
@@ -1515,7 +1655,12 @@ int oscak3(CSOUND *csound, OSC   *p)
     inc = MYFLT2LONG(*p->xcps * csound->sicvt);
     ampp = p->xamp;
     ar = p->sr;
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       fract = (MYFLT) PFRAC(phs);
       x0 = (phs >> lobits);
       x0--;
@@ -1525,7 +1670,7 @@ int oscak3(CSOUND *csound, OSC   *p)
       else ym1 = ftab[x0++];
       y0 = ftab[x0++];
       y1 = ftab[x0++];
-      if (UNLIKELY(x0>ftp->flen)) y2 = ftab[1]; else y2 = ftab[x0];
+      if (UNLIKELY(x0>(int32)ftp->flen)) y2 = ftab[1]; else y2 = ftab[x0];
       {
         MYFLT frsq = fract*fract;
         MYFLT frcu = frsq*ym1;
@@ -1540,18 +1685,21 @@ int oscak3(CSOUND *csound, OSC   *p)
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscil3: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscil3: not initialised"));
 }
 
 int oscaa3(CSOUND *csound, OSC   *p)
 {
     FUNC    *ftp;
-    MYFLT   fract, *ar, *ampp, *cpsp, *ftab;
-    int32   phs, lobits;
-    int     n, nsmps=csound->ksmps;
-    int     x0;
-    MYFLT   y0, y1, ym1, y2;
-    MYFLT   sicvt = csound->sicvt;
+    MYFLT    fract, *ar, *ampp, *cpsp, *ftab;
+    int32    phs, lobits;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int32    x0;
+    MYFLT    y0, y1, ym1, y2;
+    MYFLT    sicvt = csound->sicvt;
 
     ftp = p->ftp;
     if (UNLIKELY(ftp==NULL)) goto err1;
@@ -1561,7 +1709,12 @@ int oscaa3(CSOUND *csound, OSC   *p)
     ampp = p->xamp;
     cpsp = p->xcps;
     ar = p->sr;
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       int32 inc = MYFLT2LONG(cpsp[n] * sicvt);
       fract = (MYFLT) PFRAC(phs);
       x0 = (phs >> lobits);
@@ -1572,7 +1725,7 @@ int oscaa3(CSOUND *csound, OSC   *p)
       else ym1 = ftab[x0++];
       y0 = ftab[x0++];
       y1 = ftab[x0++];
-      if (UNLIKELY(x0>ftp->flen)) y2 = ftab[1]; else y2 = ftab[x0];
+      if (UNLIKELY(x0>(int32)ftp->flen)) y2 = ftab[1]; else y2 = ftab[x0];
       {
         MYFLT frsq = fract*fract;
         MYFLT frcu = frsq*ym1;
@@ -1587,5 +1740,6 @@ int oscaa3(CSOUND *csound, OSC   *p)
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("oscil3: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("oscil3: not initialised"));
 }
