@@ -24,7 +24,6 @@
 #include "csoundCore.h"         /*                      UGENS5.C        */
 #include "ugens5.h"
 #include <math.h>
-#include "oload.h"
 
 /*
  * LPC storage slots
@@ -32,14 +31,17 @@
 
 #define MAX_LPC_SLOT 20
 
+
 int porset(CSOUND *csound, PORT *p)
 {
-    p->c2 = pow(0.5, (double)csound->onedkr / *p->ihtim);
+    p->c2 = pow(0.5, (double)CS_ONEDKR / *p->ihtim);
     p->c1 = 1.0 - p->c2;
     if (LIKELY(*p->isig >= FL(0.0)))
       p->yt1 = (double)(*p->isig);
+    p->ihtim_old = *p->ihtim;
     return OK;
 }
+int kporset(CSOUND *csound, PORT *p) { return porset(csound, p); }
 
 int port(CSOUND *csound, PORT *p)
 {
@@ -48,6 +50,18 @@ int port(CSOUND *csound, PORT *p)
     return OK;
 }
 
+int kport(CSOUND *csound, KPORT *p)
+{
+  if(p->ihtim_old != *p->ihtim) {
+     p->c2 = pow(0.5, (double)CS_ONEDKR / *p->ihtim);
+     p->c1 = 1.0 - p->c2;
+     p->ihtim_old = *p->ihtim;
+  }
+    p->yt1 = p->c1 * (double)*p->ksig + p->c2 * p->yt1;
+    *p->kr = (MYFLT)p->yt1;
+    return OK;
+}
+
 int tonset(CSOUND *csound, TONE *p)
 {
     double b;
@@ -61,10 +75,43 @@ int tonset(CSOUND *csound, TONE *p)
     return OK;
 }
 
+int ktonset(CSOUND *csound, TONE *p) {
+   double b;
+    p->prvhp = (double)*p->khp;
+    b = 2.0 - cos((double)(p->prvhp * CS_ONEDKR *TWOPI));
+    p->c2 = b - sqrt(b * b - 1.0);
+    p->c1 = 1.0 - p->c2;
+
+    if (LIKELY(!(*p->istor)))
+      p->yt1 = 0.0;
+    return OK;
+
+ }
+
+int ktone(CSOUND *csound, TONE *p)
+{
+    double      c1 = p->c1, c2 = p->c2;
+    double      yt1 = p->yt1;
+
+    if (*p->khp != (MYFLT)p->prvhp) {
+      double b;
+      p->prvhp = (double)*p->khp;
+      b = 2.0 - cos((double)(p->prvhp * CS_ONEDKR *TWOPI));
+      p->c2 = c2 = b - sqrt(b * b - 1.0);
+      p->c1 = c1 = 1.0 - c2;
+    }
+    yt1 = c1 * (double)(*p->asig) + c2 * yt1;
+    *p->ar = (MYFLT)yt1;
+    p->yt1 = yt1;
+    return OK;
+}
+
 int tone(CSOUND *csound, TONE *p)
 {
     MYFLT       *ar, *asig;
-    int         n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     double      c1 = p->c1, c2 = p->c2;
     double      yt1 = p->yt1;
 
@@ -77,7 +124,12 @@ int tone(CSOUND *csound, TONE *p)
     }
     ar = p->ar;
     asig = p->asig;
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       yt1 = c1 * (double)(asig[n]) + c2 * yt1;
       ar[n] = (MYFLT)yt1;
     }
@@ -96,21 +148,24 @@ int tonsetx(CSOUND *csound, TONEX *p)
     }
     if (UNLIKELY((p->loop = (int) (*p->ord + FL(0.5))) < 1)) p->loop = 4;
     if (!*p->istor && (p->aux.auxp == NULL ||
-                       (int)(p->loop*sizeof(double)) > p->aux.size))
-      csound->AuxAlloc(csound, (int32)(p->loop*sizeof(double)), &p->aux);
+                    (unsigned int)(p->loop*sizeof(double)) > p->aux.size))
+        csound->AuxAlloc(csound, (int32)(p->loop*sizeof(double)), &p->aux);
     p->yt1 = (double*)p->aux.auxp;
     if (LIKELY(!(*p->istor))) {
-      memset(p->yt1, 0, p->loop*sizeof(double)); /* Punning zero and 0.0 */
+    memset(p->yt1, 0, p->loop*sizeof(double)); /* Punning zero and 0.0 */
     }
     return OK;
 }
 
 int tonex(CSOUND *csound, TONEX *p)      /* From Gabriel Maldonado, modified */
 {
-    int j;
-    int nsmps;
-    MYFLT *asig, *ar;
-    double c1, c2, *yt1;
+    MYFLT       *ar = p->ar;
+    double      c2 = p->c2, *yt1 = p->yt1,c1 = p->c1;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int      j, lp = p->loop;
+
     if (*p->khp != p->prvhp) {
       double b;
       p->prvhp = (double)*p->khp;
@@ -118,30 +173,52 @@ int tonex(CSOUND *csound, TONEX *p)      /* From Gabriel Maldonado, modified */
       p->c2 = b - sqrt(b * b - 1.0);
       p->c1 = 1.0 - p->c2;
     }
-    c1 = p->c1;
-    c2 = p->c2;
-    yt1= p->yt1;
-    nsmps = csound->ksmps;
-    ar = p->ar;
-    memmove(ar,p->asig,sizeof(MYFLT)*nsmps);
 
-    for (j=0; j< p->loop; j++) {
-      int n;
+    memmove(ar,p->asig,sizeof(MYFLT)*nsmps);
+    if (UNLIKELY(offset))  memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (j=0; j< lp; j++) {
+      /* Should *yt1 be reset to something?? */
       for (n=0; n<nsmps; n++) {
-        double x = c1 * ar[n] + c2 * *yt1;
-        *yt1 = x;
+        double x = c1 * ar[n] + c2 * yt1[j];
+        yt1[j] = x;
         ar[n] = (MYFLT)x;
       }
-      asig = p->ar;
-      yt1++;
     }
     return OK;
 }
 
+int katone(CSOUND *csound, TONE *p)
+{
+    double     sig, x;
+    double      c2 = p->c2, yt1 = p->yt1;
+
+    if (*p->khp != p->prvhp) {
+      double b;
+      p->prvhp = *p->khp;
+      b = 2.0 - cos((double)(*p->khp * CS_ONEDKR *TWOPI));
+      p->c2 = c2 = b - sqrt(b * b - 1.0);
+/*      p->c1 = c1 = 1.0 - c2; */
+    }
+      sig = *p->asig;
+      x = yt1 = c2 * (yt1 + sig);
+      *p->ar = (MYFLT)x;
+      yt1 -= sig;               /* yt1 contains yt1-xt1 */
+
+    p->yt1 = yt1;
+    return OK;
+}
+
+
 int atone(CSOUND *csound, TONE *p)
 {
     MYFLT       *ar, *asig;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     double      c2 = p->c2, yt1 = p->yt1;
 
     if (*p->khp != p->prvhp) {
@@ -152,8 +229,13 @@ int atone(CSOUND *csound, TONE *p)
 /*      p->c1 = c1 = 1.0 - c2; */
     }
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     asig = p->asig;
-    for (n=0; n<nsmps; n++) {
+    for (n=offset; n<nsmps; n++) {
       double sig = (double)asig[n];
       double x = yt1 = c2 * (yt1 + sig);
       ar[n] = (MYFLT)x;
@@ -167,8 +249,10 @@ int atonex(CSOUND *csound, TONEX *p)      /* Gabriel Maldonado, modified */
 {
     MYFLT       *ar = p->ar;
     double      c2 = p->c2, *yt1 = p->yt1;
-    int         n, nsmps=csound->ksmps, j;
-    int lp = p->loop;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int      j, lp = p->loop;
 
     if (*p->khp != p->prvhp) {
       double b;
@@ -179,8 +263,13 @@ int atonex(CSOUND *csound, TONEX *p)      /* Gabriel Maldonado, modified */
     }
 
     memmove(ar,p->asig,sizeof(MYFLT)*nsmps);
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     for (j=1; j<lp; j++) {
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         double sig = (double)ar[n];
         double x = c2 * (yt1[j] + sig);
         yt1[j] = x - sig;            /* yt1 contains yt1-xt1 */
@@ -204,9 +293,51 @@ int rsnset(CSOUND *csound, RESON *p)
     return OK;
 }
 
+int krsnset(CSOUND *csound, RESON *p){ return rsnset(csound,p); }
+
+int kreson(CSOUND *csound, RESON *p)
+{
+    uint32_t flag = 0;
+    double      c3p1, c3t4, omc3, c2sqr;
+    double      yt0, yt1, yt2, c1 = p->c1, c2 = p->c2, c3 = p->c3;
+    IGN(csound);
+
+    if (*p->kcf != (MYFLT)p->prvcf) {
+      p->prvcf = (double)*p->kcf;
+      p->cosf = cos(p->prvcf * (double)(CS_ONEDKR *TWOPI));
+      flag = 1;                 /* Mark as changed */
+    }
+    if (*p->kbw != (MYFLT)p->prvbw) {
+      p->prvbw = (double)*p->kbw;
+      c3 = p->c3 = exp(p->prvbw * (double)(-CS_ONEDKR *TWOPI));
+      flag = 1;                /* Mark as changed */
+    }
+    if (flag) {
+      c3p1 = c3 + 1.0;
+      c3t4 = c3 * 4.0;
+      omc3 = 1.0 - c3;
+      c2 = p->c2 = c3t4 * p->cosf / c3p1;               /* -B, so + below */
+      c2sqr = c2 * c2;
+      if (p->scale == 1)
+        c1 = p->c1 = omc3 * sqrt(1.0 - c2sqr / c3t4);
+      else if (p->scale == 2)
+        c1 = p->c1 = sqrt((c3p1*c3p1-c2sqr) * omc3/c3p1);
+      else c1 = p->c1 = 1.0;
+    }
+
+    yt1 = p->yt1; yt2 = p->yt2;
+    yt0 = c1 * ((double)*p->asig) + c2 * yt1 - c3 * yt2;
+    *p->ar = (MYFLT)yt0;
+    p->yt1 = yt0; p->yt2 = yt1; /* Write back for next cycle */
+    return OK;
+}
+
+
 int reson(CSOUND *csound, RESON *p)
 {
-    int flag = 0, n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t flag = 0, n, nsmps = CS_KSMPS;
     MYFLT       *ar, *asig;
     double      c3p1, c3t4, omc3, c2sqr;
     double      yt1, yt2, c1 = p->c1, c2 = p->c2, c3 = p->c3;
@@ -235,8 +366,13 @@ int reson(CSOUND *csound, RESON *p)
     }
     asig = p->asig;
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     yt1 = p->yt1; yt2 = p->yt2;
-    for (n=0; n<nsmps; n++) {
+    for (n=offset; n<nsmps; n++) {
       double yt0 = c1 * ((double)asig[n]) + c2 * yt1 - c3 * yt2;
       ar[n] = (MYFLT)yt0;
       yt2 = yt1;
@@ -253,7 +389,7 @@ int rsnsetx(CSOUND *csound, RESONX *p)
     if ((p->loop = (int) (*p->ord + FL(0.5))) < 1)
       p->loop = 4; /* default value */
     if (!*p->istor && (p->aux.auxp == NULL ||
-                       (int)(p->loop*2*sizeof(double)) > p->aux.size))
+                       (unsigned int)(p->loop*2*sizeof(double)) > p->aux.size))
       csound->AuxAlloc(csound, (int32)(p->loop*2*sizeof(double)), &p->aux);
     p->yt1 = (double*)p->aux.auxp; p->yt2 = (double*)p->aux.auxp + p->loop;
     if (UNLIKELY(scale && scale != 1 && scale != 2)) {
@@ -271,7 +407,10 @@ int rsnsetx(CSOUND *csound, RESONX *p)
 
 int resonx(CSOUND *csound, RESONX *p)   /* Gabriel Maldonado, modified  */
 {
-    int flag = 0, nsmps = csound->ksmps, j;
+    uint32_t    offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t    flag = 0, n, nsmps = CS_KSMPS;
+    int         j;
     MYFLT       *ar;
     double      c3p1, c3t4, omc3, c2sqr;
     double      *yt1, *yt2, c1,c2,c3;
@@ -307,9 +446,13 @@ int resonx(CSOUND *csound, RESONX *p)   /* Gabriel Maldonado, modified  */
     yt2  = p->yt2;
     ar = p->ar;
     memmove(ar,p->asig,sizeof(MYFLT)*nsmps);
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     for (j=0; j< p->loop; j++) {
-      int n;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         double x =
           c1 * ((double)ar[n]) + c2 * yt1[j] - c3 * yt2[j];
         yt2[j] = yt1[j];
@@ -320,9 +463,61 @@ int resonx(CSOUND *csound, RESONX *p)   /* Gabriel Maldonado, modified  */
     return OK;
 }
 
+int kareson(CSOUND *csound, RESON *p)
+{
+    uint32_t flag = 0;
+    double      c3p1, c3t4, omc3, c2sqr, D = 2.0; /* 1/RMS = root2 (rand) */
+                                                   /*      or 1/.5  (sine) */
+    double      yt1, yt2, c1, c2, c3;
+    IGN(csound);
+
+    if (*p->kcf != (MYFLT)p->prvcf) {
+      p->prvcf = (double)*p->kcf;
+      p->cosf = cos(p->prvcf * (double)(CS_ONEDKR *TWOPI));
+      flag = 1;
+    }
+    if (*p->kbw != (MYFLT)p->prvbw) {
+      p->prvbw = (double)*p->kbw;
+      p->c3 = exp(p->prvbw * (double)(-CS_ONEDKR *TWOPI));
+      flag = 1;
+    }
+    if (flag) {
+      c3p1 = p->c3 + 1.0;
+      c3t4 = p->c3 * 4.0;
+      omc3 = 1.0 - p->c3;
+      p->c2 = c3t4 * p->cosf / c3p1;
+      c2sqr = p->c2 * p->c2;
+      if (p->scale == 1)                        /* i.e. 1 - A(reson) */
+        p->c1 = 1.0 - omc3 * sqrt(1.0 - c2sqr / c3t4);
+      else if (p->scale == 2)                 /* i.e. D - A(reson) */
+        p->c1 = D - sqrt((c3p1*c3p1-c2sqr)*omc3/c3p1);
+      else p->c1 = 0.0;                        /* cannot tell        */
+    }
+
+    c1 = p->c1; c2 = p->c2; c3 = p->c3; yt1 = p->yt1; yt2 = p->yt2;
+    if (p->scale == 1 || p->scale == 0) {
+        double sig = (double) *p->asig;
+        double ans = c1 * sig + c2 * yt1 - c3 * yt2;
+        yt2 = yt1;
+        yt1 = ans - sig;  /* yt1 contains yt1-xt1 */
+        *p->ar = (MYFLT)ans;
+    }
+    else if (p->scale == 2) {
+        double sig = (double) *p->asig;
+        double ans = c1 * sig + c2 * yt1 - c3 * yt2;
+        yt2 = yt1;
+        yt1 = ans - D * sig;      /* yt1 contains yt1-D*xt1 */
+        *p->ar = (MYFLT)ans;
+    }
+    p->yt1 = yt1; p->yt2 = yt2;
+    return OK;
+}
+
 int areson(CSOUND *csound, RESON *p)
 {
-    int flag = 0, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t flag = 0, n, nsmps = CS_KSMPS;
     MYFLT       *ar, *asig;
     double      c3p1, c3t4, omc3, c2sqr, D = 2.0; /* 1/RMS = root2 (rand) */
                                                    /*      or 1/.5  (sine) */
@@ -352,10 +547,14 @@ int areson(CSOUND *csound, RESON *p)
     }
     asig = p->asig;
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     c1 = p->c1; c2 = p->c2; c3 = p->c3; yt1 = p->yt1; yt2 = p->yt2;
     if (p->scale == 1 || p->scale == 0) {
-      int n;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         double sig = (double)asig[n];
         double ans = c1 * sig + c2 * yt1 - c3 * yt2;
         yt2 = yt1;
@@ -364,8 +563,7 @@ int areson(CSOUND *csound, RESON *p)
       }
     }
     else if (p->scale == 2) {
-      int n;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         double sig = (double)asig[n];
         double ans = c1 * sig + c2 * yt1 - c3 * yt2;
         yt2 = yt1;
@@ -384,7 +582,7 @@ int areson(CSOUND *csound, RESON *p)
  *
  */
 
-int lprdset(CSOUND *csound, LPREAD *p)
+int lprdset_(CSOUND *csound, LPREAD *p, int stringname)
 {
     LPHEADER *lph;
     MEMFIL   *mfp;
@@ -403,14 +601,18 @@ int lprdset(CSOUND *csound, LPREAD *p)
     ((LPREAD**) csound->lprdaddr)[csound->currentLPCSlot] = p;
 
     /* Build file name */
-    csound->strarg2name(csound, lpfilname, p->ifilno, "lp.", p->XSTRCODE);
+    if(stringname) strncpy(lpfilname, ((STRINGDAT*)p->ifilcod)->data, MAXNAME-1);
+    else if (ISSTRCOD(*p->ifilcod))
+      strncpy(lpfilname, get_arg_string(csound, *p->ifilcod), MAXNAME-1);
+    else csound->strarg2name(csound, lpfilname, p->ifilcod, "lp.", 0);
 
     /* Do not reload existing file ? */
     if (UNLIKELY((mfp = p->mfp) != NULL && strcmp(mfp->filename, lpfilname) == 0))
       goto lpend;                             /* rtn if file prv known */
     /* Load analysis in memory file */
     /* else read file  */
-    if (UNLIKELY((mfp = ldmemfile2(csound, lpfilname, CSFTYPE_LPC)) == NULL)) {
+    if (UNLIKELY((mfp = ldmemfile2withCB(csound, lpfilname, CSFTYPE_LPC, NULL))
+                 == NULL)) {
       return csound->InitError(csound, Str("LPREAD cannot load %s"), lpfilname);
     }
     /* Store memory file location in opcode */
@@ -470,6 +672,14 @@ int lprdset(CSOUND *csound, LPREAD *p)
     return OK;
 }
 
+int lprdset(CSOUND *csound, LPREAD *p){
+  return lprdset_(csound, p, 0);
+  }
+int lprdset_S(CSOUND *csound, LPREAD *p){
+  return lprdset_(csound, p, 1);
+}
+
+
 #ifdef TRACE_POLES
 static void
  DumpPolesF(int poleCount, MYFLT *part1, MYFLT *part2, int isMagn, char *where)
@@ -649,12 +859,14 @@ int lpread(CSOUND *csound, LPREAD *p)
     MYFLT   interMagn[MAXPOLES], interPhas[MAXPOLES];
 
     if (UNLIKELY(p->mfp==NULL)) {
-      return csound->PerfError(csound, Str("lpread: not initialised"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("lpread: not initialised"));
     }
     /* Locate frame position range */
     if (UNLIKELY((framphase = (int32)(*p->ktimpt*p->framrat16)) < 0)) {
       /* for kfram reqd*/
-      return csound->PerfError(csound, Str("lpread timpnt < 0"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("lpread timpnt < 0"));
     }
     if (framphase > p->lastfram16) {                /* not past last one */
       framphase = p->lastfram16;
@@ -690,7 +902,8 @@ int lpread(CSOUND *csound, LPREAD *p)
         DoPoleInterpolation(p->npoles,poleMagn1,polePhas1,poleMagn2,
                             polePhas2,fract,interMagn,interPhas);
       if (UNLIKELY(!status)) {
-        return csound->PerfError(csound, Str("Interpolation failed"));
+        return csound->PerfError(csound, p->h.insdshead,
+                                 Str("Interpolation failed"));
       }
       for (i=0; i<p->npoles; i++) {
         *cp++ =  interMagn[i];
@@ -747,8 +960,9 @@ int lpformant(CSOUND *csound, LPFORM *p)
       }
     }
     else {
-      csound->PerfError(csound, Str("this opcode only works with LPC "
-                                    "pole analysis type (-a)\n"));
+      csound->PerfError(csound, p->h.insdshead,
+                        Str("this opcode only works with LPC "
+                            "pole analysis type (-a)\n"));
       return NOTOK;
     }
 
@@ -756,7 +970,7 @@ int lpformant(CSOUND *csound, LPFORM *p)
     if (bws[j] > sr/2 || isnan(bws[j])) bws[j] = sr/2;
     if (bws[j] < 1.0) bws[j] = 1.0;
     if (cfs[j] > sr/2 || isnan(cfs[j])) cfs[j] = sr/2;
-    if (cfs[j] < 0) cfs[j] = -cfs[j];
+    cfs[j] = FABS(cfs[j]);
     *p->kcf = cfs[j];
     *p->kbw = bws[j];
 
@@ -793,12 +1007,14 @@ int lprsnset(CSOUND *csound, LPRESON *p)
 int lpreson(CSOUND *csound, LPRESON *p)
 {
     LPREAD *q = p->lpread;
-    int     nn, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   *coefp, *pastp, *jp, *jp2, *rslt = p->ar, *asig = p->asig;
     MYFLT   x;
     double  poleReal[MAXPOLES], poleImag[MAXPOLES];
     double  polyReal[MAXPOLES+1], polyImag[MAXPOLES+1];
-    int     i;
+    int     i, nn;
     double  pm,pp;
 
     jp = p->circjp;
@@ -828,14 +1044,19 @@ int lpreson(CSOUND *csound, LPRESON *p)
       }
     }
 
+    if (UNLIKELY(offset)) memset(rslt, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rslt[nsmps], '\0', early*sizeof(MYFLT));
+    }
     /* For each sample */
-    do {
+    for (n=offset; n<nsmps; n++) {
       /* Compute Xn = Yn + CkXn-k */
 
 #ifdef TRACE_FILTER
       csound->Message(csound, "Asig=%f\n", *asig);
 #endif
-      x = *asig++;
+      x = asig[n];
       coefp = q->kcoefs;              /* using lpread interp coefs */
       pastp = jp;
       nn = q->npoles;
@@ -851,14 +1072,14 @@ int lpreson(CSOUND *csound, LPRESON *p)
       /* Store result signal in circular and output buffers */
 
       *jp++ = *jp2++ = x;
-      *rslt++ = x;
+      rslt[n] = x;
 
       /* Check if end of buffer reached */
       if (jp2 >= p->jp2lim) {
         jp2 = jp;
         jp = p->circbuf;
       }
-    } while (--nsmps);
+    }
     p->circjp = jp;
     return OK;
 }
@@ -892,13 +1113,16 @@ int lpfrsnset(CSOUND *csound, LPFRESON *p)
 int lpfreson(CSOUND *csound, LPFRESON *p)
 {
     LPREAD  *q = p->lpread;
-    int     nn, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t nn, n, nsmps = CS_KSMPS;
     MYFLT   *coefp, *pastp, *pastp1, *rslt = p->ar, *asig = p->asig;
     MYFLT   x, temp1, temp2, ampscale, cq;
 
     if (*p->kfrqratio != p->prvratio) {             /* for new freqratio */
       if (*p->kfrqratio <= FL(0.0)) {
-        return csound->PerfError(csound, Str("illegal frqratio, %5.2f"),
+        return csound->PerfError(csound, p->h.insdshead,
+                                 Str("illegal frqratio, %5.2f"),
                                          *p->kfrqratio);
       }                                             /*      calculate d  */
       p->d = (*p->kfrqratio - FL(1.0)) / (*p->kfrqratio + FL(1.0));
@@ -920,7 +1144,12 @@ int lpfreson(CSOUND *csound, LPFRESON *p)
       ampscale = FL(1.0);
     }
     x = p->prvout;
-    do {
+    if (UNLIKELY(offset)) memset(rslt, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rslt[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       nn = q->npoles - 1;
       pastp  = pastp1 = p->past + nn;
       temp1 = *pastp;
@@ -932,14 +1161,15 @@ int lpfreson(CSOUND *csound, LPFRESON *p)
         pastp--;   pastp1--;
         temp1 = temp2;
       } while (--nn);
-      x = *asig++;
+      x = asig[n];
       pastp = p->past;
       coefp = q->kcoefs;
       nn = q->npoles;
-      do  x += *coefp++ * *pastp++;
-      while (--nn);
-      *rslt++ = x * ampscale;
-    } while (--nsmps);
+      do  {
+        x += *coefp++ * *pastp++;
+      } while (--nn);
+      rslt[n] = x * ampscale;
+    }
     p->prvout = x;
     return OK;
 }
@@ -982,14 +1212,17 @@ int balnset(CSOUND *csound, BALANCE *p)
 
 int rms(CSOUND *csound, RMS *p)
 {
-    int     n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   *asig;
     double  q;
     double  c1 = p->c1, c2 = p->c2;
 
     q = p->prvq;
     asig = p->asig;
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(early)) nsmps -= early;
+    for (n=offset; n<nsmps; n++) {
       double as = (double)asig[n];
       q = c1 * as * as + c2 * q;
     }
@@ -1000,15 +1233,17 @@ int rms(CSOUND *csound, RMS *p)
 
 int gain(CSOUND *csound, GAIN *p)
 {
-    int     nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   *ar, *asig;
     double  q, a, m, diff, inc;
     double  c1 = p->c1, c2 = p->c2;
-    int     n;
 
     q = p->prvq;
     asig = p->asig;
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(early)) nsmps -= early;
+    for (n = offset; n < nsmps-early; n++) {
       double as = (double)asig[n];
       q = c1 * as * as + c2 * q;
     }
@@ -1018,17 +1253,22 @@ int gain(CSOUND *csound, GAIN *p)
     else
       a = *p->krms;
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if ((diff = a - p->prva) != 0.0) {
       m = p->prva;
-      inc = diff * (double)csound->onedksmps;
-      for (n = 0; n < nsmps; n++) {
+      inc = diff / (double)(nsmps-offset);
+      for (n = offset; n < nsmps; n++) {
         ar[n] = asig[n] * m;
         m += inc;
       }
       p->prva = a;
     }
     else {
-      for (n = 0; n < nsmps; n++) {
+      for (n = offset; n < nsmps; n++) {
         ar[n] = asig[n] * a;
       }
     }
@@ -1037,7 +1277,9 @@ int gain(CSOUND *csound, GAIN *p)
 
 int balance(CSOUND *csound, BALANCE *p)
 {
-    int     n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   *ar, *asig, *csig;
     double  q, r, a, m, diff, inc;
     double  c1 = p->c1, c2 = p->c2;
@@ -1046,7 +1288,13 @@ int balance(CSOUND *csound, BALANCE *p)
     r = p->prvr;
     asig = p->asig;
     csig = p->csig;
-    for (n = 0; n < nsmps; n++) {
+    ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       double as = (double)asig[n];
       double cs = (double)csig[n];
       q = c1 * as * as + c2 * q;
@@ -1058,18 +1306,17 @@ int balance(CSOUND *csound, BALANCE *p)
       a = sqrt(r/q);
     else
       a = sqrt(r);
-    ar = p->ar;
     if ((diff = a - p->prva) != 0.0) {
       m = p->prva;
-      inc = diff * (double)csound->onedksmps;
-      for (n = 0; n < nsmps; n++) {
+      inc = diff / (double)(nsmps-offset);
+      for (n = offset; n < nsmps; n++) {
         ar[n] = asig[n] * m;
         m += inc;
       }
       p->prva = a;
     }
     else {
-      for (n = 0; n < nsmps; n++) {
+      for (n = offset; n < nsmps; n++) {
         ar[n] = asig[n] * a;
       }
     }
@@ -1142,7 +1389,8 @@ int lpinterpol(CSOUND *csound, LPINTERPOL *p)
 
     /* RWD: guessing this... */
     if (UNLIKELY(p->lp1==NULL || p->lp2==NULL)) {
-      return csound->PerfError(csound, Str("lpinterpol: not initialised"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("lpinterpol: not initialised"));
     }
     cp1 =  p->lp1->kcoefs;
     cp2 =  p->lp2->kcoefs;
@@ -1157,7 +1405,8 @@ int lpinterpol(CSOUND *csound, LPINTERPOL *p)
     status = DoPoleInterpolation(p->npoles,poleMagn1,polePhas1,poleMagn2,
                                      polePhas2,*p->kmix,interMagn,interPhas);
     if (UNLIKELY(!status)) {
-      return csound->PerfError(csound, Str("Interpolation failed"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("Interpolation failed"));
     }
 
     cp = p->kcoefs;      /* This is where the coefs get stored */
@@ -1168,3 +1417,59 @@ int lpinterpol(CSOUND *csound, LPINTERPOL *p)
     return OK;
 }
 
+
+int klimit(CSOUND *csound, LIMIT *p)
+{
+    MYFLT       sig=*p->sig, min=*p->min, max=*p->max;
+    if (LIKELY((sig <= max) && (sig >= min))) {
+      *p->ans = sig;
+    }
+    else {
+     if ( min >= max) {
+        *p->ans = FL(0.5) * (min + max);
+      }
+      else {
+        if (sig > max)
+          *p->ans = max;
+        else
+          *p->ans = min;
+      }
+    }
+    return OK;
+}
+
+int limit(CSOUND *csound, LIMIT *p)
+{
+    MYFLT       *ans, *asig;
+    MYFLT       min=*p->min, max=*p->max, aver;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    ans = p->ans;
+    asig  = p->sig;
+
+    if (UNLIKELY(offset)) memset(ans, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ans[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    if (min >= max) {
+      aver = (min + max) * FL(0.5);
+      for (n=offset; n<nsmps; n++) {
+        ans[n] = aver;
+      }
+    }
+    else
+      for (n=offset; n<nsmps; n++) {
+        if ((asig[n] <= max) && (asig[n] >= min)) {
+          ans[n] = asig[n];
+        }
+        else {
+          if (asig[n] > max)
+            ans[n] = max;
+          else
+            ans[n] = min;
+        }
+      }
+    return OK;
+}
