@@ -30,7 +30,7 @@
 
 int downset(CSOUND *csound, DOWNSAMP *p)
 {
-    if (UNLIKELY((p->len = (int)*p->ilen) > csound->ksmps))
+    if (UNLIKELY((p->len = (unsigned int)*p->ilen) > CS_KSMPS))
       return csound->InitError(csound, "ilen > ksmps");
     return OK;
 }
@@ -38,15 +38,18 @@ int downset(CSOUND *csound, DOWNSAMP *p)
 int downsamp(CSOUND *csound, DOWNSAMP *p)
 {
     MYFLT       *asig, sum;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
     int len, n;
 
     if (p->len <= 1)
-      *p->kr = *p->asig;
+      *p->kr = p->asig[offset];
     else {
       asig = p->asig;
       sum = FL(0.0);
       len = p->len;
-      for (n=0; n<len; n++) {
+      if (len>(int)(CS_KSMPS-early)) len = early;
+      for (n=offset; n<len; n++) {
         sum += asig[n];
       }
       *p->kr = sum / p->len;
@@ -58,9 +61,16 @@ int upsamp(CSOUND *csound, UPSAMP *p)
 {
     MYFLT kval = *p->ksig;
     MYFLT *ar = p->ar;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
-    for (n = 0; n < nsmps; n++)
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++)
       ar[n] = kval;
     return OK;
 }
@@ -84,16 +94,23 @@ int interpset(CSOUND *csound, INTERP *p)
 int interp(CSOUND *csound, INTERP *p)
 {
     MYFLT *ar, val, incr;
-    int   n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     ar = p->rslt;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->init_k) {
       p->init_k = 0;
       p->prev = *p->xsig;
     }
     val = p->prev;
-    incr = (*p->xsig - val) * csound->onedksmps;
-    for (n=0; n<nsmps; n++) {
+    incr = (*p->xsig - val) / (nsmps-offset);
+    for (n=offset; n<nsmps; n++) {
       ar[n] = val += incr;
     }
     p->prev = val;
@@ -116,12 +133,19 @@ int kntegrate(CSOUND *csound, INDIFF *p)
 int integrate(CSOUND *csound, INDIFF *p)
 {
     MYFLT       *rslt, *asig, sum;
-    int n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     rslt = p->rslt;
+    if (UNLIKELY(offset)) memset(rslt, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rslt[nsmps], '\0', early*sizeof(MYFLT));
+    }
     asig = p->xsig;
     sum = p->prev;
-    for (n=0; n<nsmps; n++) {
+    for (n=offset; n<nsmps; n++) {
       rslt[n] = sum += asig[n];
     }
     p->prev = sum;
@@ -140,12 +164,19 @@ int kdiff(CSOUND *csound, INDIFF *p)
 int diff(CSOUND *csound, INDIFF *p)
 {
     MYFLT       *ar, *asig, prev, tmp;
-    int n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     ar = p->rslt;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     asig = p->xsig;
     prev = p->prev;
-    for (n=0; n<nsmps; n++) {
+    for (n=offset; n<nsmps; n++) {
       tmp = asig[n];            /* IV - Sep 5 2002: fix to make */
       ar[n] = tmp - prev;       /* diff work when the input and */
       prev = tmp;               /* output argument is the same  */
@@ -173,14 +204,21 @@ int ksmphold(CSOUND *csound, SAMPHOLD *p)
 int samphold(CSOUND *csound, SAMPHOLD *p)
 {
     MYFLT       *ar, *asig, *agate, state;
-    int n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     ar = p->xr;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     asig = p->xsig;
     state = p->state;
     if (p->audiogate) {
       agate = p->xgate;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         if (agate[n] > FL(0.0))
           state = asig[n];
         ar[n] = state;
@@ -188,12 +226,12 @@ int samphold(CSOUND *csound, SAMPHOLD *p)
     }
     else {
       if (*p->xgate > FL(0.0)) {
-        for (n=0; n<nsmps; n++) {
+        for (n=offset; n<nsmps; n++) {
           ar[n] = state = asig[n];
         }
       }
       else {
-        for (n=0; n<nsmps; n++) {
+        for (n=offset; n<nsmps; n++) {
           ar[n] = state;
         }
       }
@@ -230,7 +268,7 @@ int delset(CSOUND *csound, DELAY *p)
 
 int delrset(CSOUND *csound, DELAYR *p)
 {
-    int32       npts;
+    uint32_t    npts;
     MYFLT       *auxp;
 
     if (UNLIKELY(p->XOUTCODE != 1))
@@ -252,7 +290,7 @@ int delrset(CSOUND *csound, DELAYR *p)
     if (UNLIKELY(*p->istor != FL(0.0) && p->auxch.auxp != NULL))
       return OK;
     /* ksmps is min dely */
-    if (UNLIKELY((npts=(int32)(FL(0.5) + *p->idlt*csound->esr)) < csound->ksmps)) {
+    if (UNLIKELY((npts=(uint32_t)(FL(0.5) + *p->idlt*csound->esr)) < CS_KSMPS)) {
       return csound->InitError(csound, Str("illegal delay time"));
     }
     if ((auxp = (MYFLT*)p->auxch.auxp) == NULL ||       /* new space if reqd */
@@ -322,15 +360,25 @@ int tapset(CSOUND *csound, DELTAP *p)
 int delay(CSOUND *csound, DELAY *p)
 {
     MYFLT       *ar, *asig, *curp, *endp;
-    int n, nsmps = csound->ksmps;
-
+    uint32_t offset = 0;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (UNLIKELY(p->auxch.auxp==NULL)) goto err1;  /* RWD fix */
     ar = p->ar;
+    if (csound->oparms->sampleAccurate) {
+      uint32_t early  = p->h.insdshead->ksmps_no_end;
+      offset = p->h.insdshead->ksmps_offset;
+
+      if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+      if (UNLIKELY(early)) {
+        nsmps -= early;
+        memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+      }
+    }
     asig = p->asig;
     curp = p->curp;
     endp = (MYFLT *) p->auxch.endp;
-    for (n=0; n<nsmps; n++) {
+    for (n=offset; n<nsmps; n++) {
       MYFLT in = asig[n];       /* Allow overwriting form */
       ar[n] = *curp;
       *curp = in;
@@ -341,39 +389,51 @@ int delay(CSOUND *csound, DELAY *p)
 
     return OK;
  err1:
-      return csound->PerfError(csound, Str("delay: not initialised"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("delay: not initialised"));
 }
 
 int delayr(CSOUND *csound, DELAYR *p)
 {
     MYFLT       *ar, *curp, *endp;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (UNLIKELY(p->auxch.auxp==NULL)) goto err1; /* RWD fix */
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     curp = p->curp;
     endp = (MYFLT *) p->auxch.endp;
-    for (n=0; n<nsmps; n++) {
+    for (n=offset; n<nsmps; n++) {
       ar[n] = *curp++;
       if (UNLIKELY(curp >= endp))
         curp = (MYFLT *) p->auxch.auxp;
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("delayr: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("delayr: not initialised"));
 }
 
 int delayw(CSOUND *csound, DELAYW *p)
 {
     DELAYR      *q = p->delayr;
     MYFLT       *asig, *curp, *endp;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (UNLIKELY(q->auxch.auxp==NULL)) goto err1; /* RWD fix */
     asig = p->asig;
     curp = q->curp;
     endp = (MYFLT *) q->auxch.endp;
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(early)) nsmps -= early;
+    for (n=offset; n<nsmps; n++) {
       *curp = asig[n];
       if (UNLIKELY(++curp >= endp))
         curp = (MYFLT *) q->auxch.auxp;
@@ -381,41 +441,57 @@ int delayw(CSOUND *csound, DELAYW *p)
     q->curp = curp;                                     /* now sav new curp */
     return OK;
  err1:
-    return csound->PerfError(csound, Str("delayw: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("delayw: not initialised"));
 }
 
 int deltap(CSOUND *csound, DELTAP *p)
 {
     DELAYR      *q = p->delayr;
     MYFLT       *ar, *tap, *endp;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (UNLIKELY(q->auxch.auxp==NULL)) goto err1; /* RWD fix */
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     tap = q->curp - (int32) (FL(0.5) + *p->xdlt * csound->esr);
     while (tap < (MYFLT *) q->auxch.auxp)
       tap += q->npts;
     endp = (MYFLT *) q->auxch.endp;
-    for (n=0; n<nsmps; n++) {
+    for (n=offset; n<nsmps; n++) {
       if (UNLIKELY(tap >= endp))
         tap -= q->npts;
       ar[n] = *tap++;
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("deltap: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("deltap: not initialised"));
 }
 
 int deltapi(CSOUND *csound, DELTAP *p)
 {
     DELAYR      *q = p->delayr;
     MYFLT       *ar, *tap, *prv, *begp, *endp;
-    int         n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32       idelsmps;
     MYFLT       delsmps, delfrac;
 
     if (UNLIKELY(q->auxch.auxp==NULL)) goto err1;
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     begp = (MYFLT *) q->auxch.auxp;
     endp = (MYFLT *) q->auxch.endp;
     if (!p->XINCODE) {
@@ -424,7 +500,7 @@ int deltapi(CSOUND *csound, DELTAP *p)
       delfrac = delsmps - idelsmps;
       tap = q->curp - idelsmps;
       while (tap < begp) tap += q->npts;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         if (UNLIKELY(tap >= endp))
           tap -= q->npts;
         if (UNLIKELY((prv = tap - 1) < begp))
@@ -435,7 +511,7 @@ int deltapi(CSOUND *csound, DELTAP *p)
     }
     else {
       MYFLT *timp = p->xdlt, *curq = q->curp;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         delsmps = timp[n] * csound->esr;
         idelsmps = (int32)delsmps;
         delfrac = delsmps - idelsmps;
@@ -450,7 +526,8 @@ int deltapi(CSOUND *csound, DELTAP *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("deltapi: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("deltapi: not initialised"));
 }
 
 /* ***** From Hans Mikelson ************* */
@@ -459,12 +536,19 @@ int deltapn(CSOUND *csound, DELTAP *p)
 {
     DELAYR *q = p->delayr;
     MYFLT  *ar, *tap, *begp, *endp;
-    int    n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32  idelsmps;
     MYFLT  delsmps;
 
     if (UNLIKELY(q->auxch.auxp==NULL)) goto err1;
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     begp = (MYFLT *) q->auxch.auxp;
     endp = (MYFLT *) q->auxch.endp;
     if (!p->XINCODE) {
@@ -472,7 +556,7 @@ int deltapn(CSOUND *csound, DELTAP *p)
       idelsmps = (int32)delsmps;
       tap = q->curp - idelsmps;
       while (tap < begp) tap += q->npts;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         if (UNLIKELY(tap >= endp ))
           tap -= q->npts;
         if (UNLIKELY(tap < begp))
@@ -483,7 +567,7 @@ int deltapn(CSOUND *csound, DELTAP *p)
     }
     else {
       MYFLT *timp = p->xdlt, *curq = q->curp;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         delsmps = timp[n];
         idelsmps = (int32)delsmps;
         if (UNLIKELY((tap = curq++ - idelsmps) < begp))
@@ -495,7 +579,8 @@ int deltapn(CSOUND *csound, DELTAP *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("deltapn: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("deltapn: not initialised"));
 }
 
 /* **** JPff **** */
@@ -503,12 +588,19 @@ int deltap3(CSOUND *csound, DELTAP *p)
 {
     DELAYR      *q = p->delayr;
     MYFLT       *ar, *tap, *prv, *begp, *endp;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32       idelsmps;
     MYFLT       delsmps, delfrac;
 
     if (UNLIKELY(q->auxch.auxp==NULL)) goto err1;
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     begp = (MYFLT *) q->auxch.auxp;
     endp = (MYFLT *) q->auxch.endp;
     if (!p->XINCODE) {
@@ -517,7 +609,7 @@ int deltap3(CSOUND *csound, DELTAP *p)
       delfrac = delsmps - idelsmps;
       tap = q->curp - idelsmps;
       while (tap < begp) tap += q->npts;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         MYFLT ym1, y0, y1, y2;
         if (UNLIKELY(tap >= endp))
           tap -= q->npts;
@@ -544,7 +636,7 @@ int deltap3(CSOUND *csound, DELTAP *p)
     }
     else {
       MYFLT *timp = p->xdlt, *curq = q->curp;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         MYFLT ym1, y0, y1, y2;
         delsmps = *timp++ * csound->esr;
         idelsmps = (int32)delsmps;
@@ -571,7 +663,8 @@ int deltap3(CSOUND *csound, DELTAP *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("deltap3: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("deltap3: not initialised"));
 }
 
 
@@ -596,11 +689,18 @@ int deltapx(CSOUND *csound, DELTAPX *p)                 /* deltapx opcode */
 {
     DELAYR  *q = p->delayr;
     MYFLT   *out1, *del, *buf1, *bufp, *bufend;
-    int     nn = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32   indx, maxd, xpos;
 
     if (UNLIKELY(q->auxch.auxp == NULL)) goto err1; /* RWD fix */
     out1 = p->ar; del = p->adlt;
+    if (UNLIKELY(offset)) memset(out1, '0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out1[nsmps], '\0', early*sizeof(MYFLT));
+    }
     buf1 = (MYFLT *) q->auxch.auxp;
     indx = (int32) (q->curp - buf1);
     maxd = q->npts; bufend = buf1 + maxd;
@@ -611,7 +711,7 @@ int deltapx(CSOUND *csound, DELTAPX *p)                 /* deltapx opcode */
       i2 = (p->wsize >> 1);
       /* wsize = 4: d2x = 1 - 1/3, wsize = 64: d2x = 1 - 1/36 */
       d2x = p->d2x;
-      do {
+      for (n=offset; n<nsmps; n++) {
         /* x1: fractional part of delay time */
         /* x2: sine of x1 (for interpolation) */
         /* xpos: integer part of delay time (buffer position to read from) */
@@ -636,19 +736,19 @@ int deltapx(CSOUND *csound, DELTAPX *p)                 /* deltapx opcode */
             if (UNLIKELY(++bufp >= bufend)) bufp = buf1;
             n1 -= w * w * (double)*bufp / d; d++;
           } while (--i);
-          *out1 = (MYFLT)(n1 * sin(PI * x1) / PI);
+          out1[n] = (MYFLT)(n1 * sin(PI * x1) / PI);
         }
         else {                                          /* integer sample */
           xpos = (int32) ((double)xpos + x1 + 0.5);     /* position */
           if (xpos >= maxd) xpos -= maxd;
-          *out1 = buf1[xpos];
+          out1[n] = buf1[xpos];
         }
-        out1++; indx++;
-      } while (--nn);
+        indx++;
+      }
     }
     else {                          /* window size = 4, cubic interpolation */
       double  x, am1, a0, a1, a2;
-      do {
+      for (n=offset; n<nsmps; n++) {
         am1 = (double)indx - (double)*(del++) * (double)csound->esr;
         while (am1 < 0.0) am1 += (double)maxd;
         xpos = (int32) am1; am1 -= (double)xpos;
@@ -665,23 +765,27 @@ int deltapx(CSOUND *csound, DELTAPX *p)                 /* deltapx opcode */
         x += a1 * (double)*bufp;   if (UNLIKELY(++bufp >= bufend)) bufp = buf1;
         x += a2 * (double)*bufp;
 
-        indx++; *out1++ = (MYFLT)x;
-      } while (--nn);
+        indx++; out1[n] = (MYFLT)x;
+      }
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("deltap: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("deltap: not initialised"));
 }
 
 int deltapxw(CSOUND *csound, DELTAPX *p)                /* deltapxw opcode */
 {
     DELAYR  *q = p->delayr;
     MYFLT   *in1, *del, *buf1, *bufp, *bufend;
-    int     nn = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32   indx, maxd, xpos;
 
     if (UNLIKELY(q->auxch.auxp == NULL)) goto err1; /* RWD fix */
     in1 = p->ar; del = p->adlt;
+    if (UNLIKELY(early)) nsmps -= early;
     buf1 = (MYFLT *) q->auxch.auxp;
     indx = (int32) (q->curp - buf1);
     maxd = q->npts; bufend = buf1 + maxd;
@@ -692,7 +796,7 @@ int deltapxw(CSOUND *csound, DELTAPX *p)                /* deltapxw opcode */
       i2 = (p->wsize >> 1);
       /* wsize = 4: d2x = 1 - 1/3, wsize = 64: d2x = 1 - 1/36 */
       d2x = p->d2x;
-      do {
+      for (n=offset; n<nsmps; n++) {
         /* x1: fractional part of delay time */
         /* x2: sine of x1 (for interpolation) */
         /* xpos: integer part of delay time (buffer position to read from) */
@@ -721,14 +825,14 @@ int deltapxw(CSOUND *csound, DELTAPX *p)                /* deltapxw opcode */
         else {                                          /* integer sample */
           xpos = (int32) ((double)xpos + x1 + 0.5);     /* position */
           if (UNLIKELY(xpos >= maxd)) xpos -= maxd;
-          buf1[xpos] += *in1;
+          buf1[xpos] += in1[n];
         }
-        in1++; indx++;
-      } while (--nn);
+        indx++;
+      }
     }
     else {                          /* window size = 4, cubic interpolation */
       double  x, am1, a0, a1, a2;
-      do {
+      for (n=offset; n<nsmps; n++) {
         am1 = (double)indx - (double)*(del++) * (double)csound->esr;
         while (am1 < 0.0) am1 += (double)maxd;
         xpos = (int32) am1; am1 -= (double)xpos;
@@ -738,7 +842,7 @@ int deltapxw(CSOUND *csound, DELTAPX *p)                /* deltapxw opcode */
         am1 = 0.5 * (a0 - am1) - a2;                            /* sample -1 */
         a0  = 3.0 * a2 - a0; a0++;                              /* sample 0  */
 
-        x = (double)*(in1++);
+        x = (double)in1[n];
         bufp = (xpos ? (buf1 + (xpos - 1L)) : (bufend - 1));
         while (bufp >= bufend) bufp -= maxd;
         *bufp += (MYFLT)(am1 * x); if (UNLIKELY(++bufp >= bufend)) bufp = buf1;
@@ -747,11 +851,12 @@ int deltapxw(CSOUND *csound, DELTAPX *p)                /* deltapxw opcode */
         *bufp += (MYFLT)(a2 * x);
 
         indx++;
-      } while (--nn);
+      }
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("deltap: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("deltap: not initialised"));
 }
 
 int del1set(CSOUND *csound, DELAY1 *p)
@@ -764,13 +869,20 @@ int del1set(CSOUND *csound, DELAY1 *p)
 int delay1(CSOUND *csound, DELAY1 *p)
 {
     MYFLT       *ar, *asig;
-    int         nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t nsmps = CS_KSMPS;
 
     ar = p->ar;
     /* asig = p->asig - 1; */
     asig = p->asig;
-    ar[0] = p->sav1;
-    memmove(&ar[1], asig, sizeof(MYFLT)*(nsmps-1));
+    ar[offset] = p->sav1;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    memmove(&ar[offset+1], &asig[offset], sizeof(MYFLT)*(nsmps-1-offset));
     p->sav1 = asig[nsmps-1];
     return OK;
 }
@@ -788,7 +900,7 @@ int cmbset(CSOUND *csound, COMB *p)
       return csound->InitError(csound, Str("illegal loop time"));
     }
     nbytes = lpsiz * sizeof(MYFLT);
-    if (p->auxch.auxp == NULL || nbytes != p->auxch.size) {
+    if (p->auxch.auxp == NULL || (uint32_t)nbytes != p->auxch.size) {
       csound->AuxAlloc(csound, (int32)nbytes, &p->auxch);
       p->pntr = (MYFLT *) p->auxch.auxp;
       p->prvt = FL(0.0);
@@ -806,7 +918,9 @@ int cmbset(CSOUND *csound, COMB *p)
 
 int comb(CSOUND *csound, COMB *p)
 {
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       *ar, *asig, *xp, *endp;
     MYFLT       coef = p->coef;
 
@@ -828,10 +942,14 @@ int comb(CSOUND *csound, COMB *p)
     xp = p->pntr;
     endp = (MYFLT *) p->auxch.endp;
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     asig = p->asig;
-    MYFLT out;
-    for (n=0; n<nsmps; n++) {
-      out = *xp;
+    for (n=offset; n<nsmps; n++) {
+      MYFLT out = *xp;
       *xp *= coef;
       *xp += asig[n];
       ar[n] = out;
@@ -841,12 +959,54 @@ int comb(CSOUND *csound, COMB *p)
     p->pntr = xp;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("comb: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("comb: not initialised"));
+}
+
+int invcomb(CSOUND *csound, COMB *p)
+{
+    int n, nsmps = csound->ksmps;
+    MYFLT       *ar, *asig, *xp, *endp;
+    MYFLT       coef = p->coef;
+
+    if (UNLIKELY(p->auxch.auxp==NULL)) goto err1; /* RWD fix */
+    if (p->prvt != *p->krvt) {
+      p->prvt = *p->krvt;
+      /*
+       * The argument to exp() in the following is sometimes a small
+       * enough negative number to result in a denormal (or worse)
+       * on Alpha. So if the result would be less than 1.0e-16, we
+       * just say it is zero and do not call exp().  heh 981101
+       */
+      double exp_arg = (double)(log001 * *p->ilpt / p->prvt);
+      if (UNLIKELY(exp_arg < -36.8413615))    /* ln(1.0e-16) */
+        coef = p->coef = FL(0.0);
+      else
+        coef = p->coef = (MYFLT)exp(exp_arg);
+    }
+    xp = p->pntr;
+    endp = (MYFLT *) p->auxch.endp;
+    ar = p->ar;
+    asig = p->asig;
+    MYFLT out;
+    for (n=0; n<nsmps; n++) {
+      out = *xp;
+      ar[n] = (*xp = asig[n])-coef*out;
+      if (UNLIKELY(++xp >= endp))
+        xp = (MYFLT *) p->auxch.auxp;
+    }
+    p->pntr = xp;
+    return OK;
+ err1:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("combinv: not initialised"));
 }
 
 int alpass(CSOUND *csound, COMB *p)
 {
-    int n, nsmps = csound->ksmps;
+    uint32_t    offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t    n, nsmps = CS_KSMPS;
     MYFLT       *ar, *asig, *xp, *endp;
     MYFLT       y, z;
     MYFLT       coef = p->coef;
@@ -859,8 +1019,13 @@ int alpass(CSOUND *csound, COMB *p)
     xp = p->pntr;
     endp = (MYFLT *) p->auxch.endp;
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     asig = p->asig;
-    for (n=0; n<nsmps; n++) {
+    for (n=offset; n<nsmps; n++) {
       y = *xp;
       *xp++ = z = coef * y + asig[n];
       ar[n] = y - coef * z;
@@ -870,7 +1035,8 @@ int alpass(CSOUND *csound, COMB *p)
     p->pntr = xp;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("alpass: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("alpass: not initialised"));
 }
 
 static const MYFLT revlptimes[6] = {FL(0.0297), FL(0.0371), FL(0.0411),
@@ -903,7 +1069,7 @@ int rvbset(CSOUND *csound, REVERB *p)
       p->adr5 = p->p5 = p->adr4 + *sizp++;
       p->adr6 = p->p6 = p->adr5 + *sizp++;
       if (UNLIKELY(p->adr6 + *sizp != (MYFLT *) p->auxch.endp)) {
-        csound->Die(csound, Str("revlpsiz inconsistent\n"));
+        csound->InitError(csound, Str("revlpsiz inconsistent\n"));
       }
       p->prvt = FL(0.0);
     }
@@ -924,7 +1090,9 @@ int reverb(CSOUND *csound, REVERB *p)
 {
     MYFLT       *asig, *p1, *p2, *p3, *p4, *p5, *p6, *ar, *endp;
     MYFLT       c1,c2,c3,c4,c5,c6;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (UNLIKELY(p->auxch.auxp==NULL)) goto err1; /* RWD fix */
     if (p->prvt != *p->krvt) {
@@ -955,8 +1123,13 @@ int reverb(CSOUND *csound, REVERB *p)
     endp = (MYFLT *) p->auxch.endp;
 
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     asig = p->asig;
-    for (n=0; n<nsmps; n++) {
+    for (n=offset; n<nsmps; n++) {
       MYFLT     cmbsum, y1, y2, z;
       MYFLT     sig = asig[n];
       cmbsum = *p1 + *p2 + *p3 + *p4;
@@ -986,14 +1159,15 @@ int reverb(CSOUND *csound, REVERB *p)
     p->p6 = p6;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("reverb: not intialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("reverb: not intialised"));
 }
 
 int panset(CSOUND *csound, PAN *p)
 {
     FUNC  *ftp;
 
-    if (UNLIKELY((ftp = csound->FTFind(csound, p->ifn)) == NULL))
+    if (UNLIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) == NULL))
       return NOTOK;
     p->ftp = ftp;
     p->xmul = (*p->imode == FL(0.0) ? FL(1.0) : (MYFLT)ftp->flen);
@@ -1006,7 +1180,9 @@ int pan(CSOUND *csound, PAN *p)
 {
     MYFLT   flend2, xndx_f, yndx_f, xt, yt, ch1, ch2, ch3, ch4;
     int32   xndx, yndx, flen;
-    int     n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     FUNC    *ftp;
 
     ftp = p->ftp;
@@ -1033,7 +1209,21 @@ int pan(CSOUND *csound, PAN *p)
     ch2 = ftp->ftable[xndx]        * ftp->ftable[yndx];
     ch3 = ftp->ftable[flen - xndx] * ftp->ftable[flen - yndx];
     ch4 = ftp->ftable[xndx]        * ftp->ftable[flen - yndx];
-    for (n=0; n<nsmps; n++) {
+
+    if (UNLIKELY(offset)) {
+      memset(p->r1, '\0', offset*sizeof(MYFLT));
+      memset(p->r2, '\0', offset*sizeof(MYFLT));
+      memset(p->r3, '\0', offset*sizeof(MYFLT));
+      memset(p->r4, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->r1[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&p->r2[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&p->r3[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&p->r4[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       MYFLT sig = p->asig[n];
       p->r1[n] = sig * ch1;
       p->r2[n] = sig * ch2;
@@ -1043,6 +1233,7 @@ int pan(CSOUND *csound, PAN *p)
 
     return OK;
  err1:
-    return csound->PerfError(csound, Str("pan: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pan: not initialised"));
 }
 
