@@ -12,20 +12,23 @@
   GNU Lesser General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with Csound; if OPCODE_IOBUFSnot, write to the Free Software
+  License along with Csound; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA
 */
 #include <OpcodeBase.hpp>
+#include <algorithm>
 #include <cmath>
 #include <cstdarg>
 #include <cstdio>
 #include <cstring>
 #include <map>
-#include <omp.h>
+#include <pthread.h>
 #include <string>
 #include <vector>
 
+using namespace std;
+
 extern "C"
 {
 #include <lua.h>
@@ -59,15 +62,36 @@ struct keys_t
     int noteoff_key;
 };
 
+static pthread_mutex_t lc_getrefkey = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t lc_manageLuaState = PTHREAD_MUTEX_INITIALIZER;
+
+struct CriticalSection
+{
+    CriticalSection(pthread_mutex_t &mutex_) : mutex(mutex_), status(-1)
+    {
+        status = pthread_mutex_lock(&mutex);
+    }
+    ~CriticalSection()
+    {
+        if (status >= 0) {
+            pthread_mutex_unlock(&mutex);
+        }
+    }
+    pthread_mutex_t &mutex;
+    int status;
+};
+
 /**
  * Thread-safe storage for Lua references to opcode subroutines.
  */
-keys_t &manageLuaReferenceKeys(const lua_State *L, const std::string &opcode, char operation = 'O')
+keys_t &manageLuaReferenceKeys(const lua_State *L,
+                               const std::string &opcode, char operation)
 {
-    static std::map<const lua_State *, std::map<std::string, keys_t> > luaReferenceKeys;
+    static std::map<const lua_State *,
+                    std::map<std::string, keys_t> > luaReferenceKeys;
     keys_t *keys = 0;
-    #pragma omp critical(lc_getrefkey)
     {
+        CriticalSection criticalSection(lc_getrefkey);
         switch(operation)
         {
         case 'O':
@@ -86,46 +110,64 @@ keys_t &manageLuaReferenceKeys(const lua_State *L, const std::string &opcode, ch
     return *keys;
 }
 
+
+struct LuaStateForThread
+{
+    pthread_t thread;
+    lua_State *L;
+};
+
+bool operator == (const LuaStateForThread& a, const LuaStateForThread &b)
+{
+    if (pthread_equal(a.thread, b.thread)) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
 /**
  * Thread-safe storage for Lua states (virtual machines). There is one Lua state
  * per thread, rather than per instance of Csound, in case one instance of Csound
  * is running multiple threads with multiple instances of a Lua opcode.
  */
-lua_State *manageLuaState(char operation = 'O')
+lua_State *manageLuaState(char operation)
 {
-    static std::map<int, lua_State *> luaStatesForThreads;
+    static std::vector<LuaStateForThread> luaStatesForThreads;
+    CriticalSection criticalSection(lc_manageLuaState);
+    LuaStateForThread luaStateForThread;
+    luaStateForThread.thread = pthread_self();
+    std::vector<LuaStateForThread>::iterator it =
+      std::find(luaStatesForThreads.begin(),
+                luaStatesForThreads.end(),
+                luaStateForThread);
     lua_State *L = 0;
-    #pragma omp critical(lc_manageLuaState)
+    switch(operation)
     {
-      int threadId = omp_get_thread_num();
-        switch(operation)
+    case 'O':
+    {
+        if (it == luaStatesForThreads.end())
         {
-        case 'O':
+            luaStateForThread.L = lua_open();
+            luaL_openlibs(luaStateForThread.L);
+            luaStatesForThreads.push_back(luaStateForThread);
+            L = luaStateForThread.L;
+        }
+        else
         {
-            if (luaStatesForThreads.find(threadId) == luaStatesForThreads.end())
-            {
-                L = lua_open();
-                luaL_openlibs(L);
-                luaStatesForThreads[threadId] = L;
-            }
-            else
-            {
-                L = luaStatesForThreads[threadId];
-            }
+            L = it->L;
         }
-        break;
-        case 'C':
-        {
-            L = luaStatesForThreads[threadId];
-            if (L)
-            {
-                manageLuaReferenceKeys(L, "", 'C');
-            }
-            luaStatesForThreads.erase(threadId);
-        }
-        break;
+    }
+    break;
+    case 'C':
+    {
+        if (it != luaStatesForThreads.end()) {
+            manageLuaReferenceKeys(it->L, "", 'C');
+            luaStatesForThreads.erase(it);
         }
     }
+    break;
+    }
     return L;
 }
 
@@ -145,16 +187,12 @@ public:
     int init(CSOUND *csound)
     {
         int result = OK;
-        lua_State *L = manageLuaState();
+        lua_State *L = manageLuaState('O');
         /* Ensure that Csound is available in the global environment. */
         lua_pushlightuserdata(L, csound);
         lua_setfield(L, LUA_GLOBALSINDEX, "csound");
-        const char *luacode =  csound->strarg2name(csound,
-                               (char *) 0,
-                               luacode_,
-                               (char *)"default",
-                               (int) csound->GetInputArgSMask(this));
-        //log(csound, "Executing Lua code:\n%s\n", luacode);
+        const char *luacode = ((STRINGDAT *)luacode_)->data;
+        log(csound, "Executing (L: 0x%p) Lua code:\n%s\n", L, luacode);
         result = luaL_dostring(L, luacode);
         if (result == 0)
         {
@@ -179,22 +217,27 @@ public:
  *                         output and/or input arguments may be passed.
  *                         All arguments must be passed on the right-hand
  *                         side and outputs are returned in the argument.
- *                         Requires opname_init and opname_kontrol to be defined in Lua.
+ *                         Requires opname_init and opname_kontrol to be
+ *                         defined in Lua.
  *
  * lua_iaopcall Sname, ... Calls a Lua opcode at i-rate and a-rate. Any number of
  *                         output and/or input arguments may be passed.
  *                         All arguments must be passed on the right-hand
  *                         side and outputs are returned in the argument.
- *                         Requires opname_init and opname_audio to be defined in Lua.
+ *                         Requires opname_init and opname_audio to be defined
+ *                         in Lua.
  *
  * Opcode that actually implements arbitrary Lua opcodes
  * by calling from Csound into Lua functions.
  *
  * Lua functions access elements of arguments as follows
- * (pointers to both scalars and arrays are dereferenced by the array access operator):
- * ffi.cdef(' struct arguments_t { double *a_out, double *i_in, double *i_txt, double *f_sig };');
+ * (pointers to both scalars and arrays are dereferenced by the array
+ * access operator):
+ * ffi.cdef(' struct arguments_t { double *a_out, double *i_in,
+ *                                 double *i_txt, double *f_sig };');
  * local arguments = ffi.cast("struct arguments_t *", carguments_lightuserdata)
- * for i = 0, ksmps -1 do begin carguments.a_out[i] = carguments.i_in[0] * 3 end end
+ * for i = 0, ksmps -1 do begin carguments.a_out[i] = carguments.i_in[0] * 3
+ * end end
  */
 class cslua_opcall: public OpcodeBase<cslua_opcall>
 {
@@ -213,7 +256,7 @@ public:
       * additional data, e.g. for opcode state, after the opcode output and
       * input arguments, as limited by the available space.
       */
-    MYFLT *arguments[0x3000];
+    MYFLT *arguments[1000];
     const char *opcodename;
 public:
     /**
@@ -225,20 +268,17 @@ public:
     int init(CSOUND *csound)
     {
         int result = OK;
-        opcodename = csound->strarg2name(csound,
-                                         (char *) 0,
-                                         opcodename_,
-                                         (char *)"default",
-                                         (int) csound->GetInputArgSMask(this));
-        lua_State *L = manageLuaState();
-        keys_t &keys = manageLuaReferenceKeys(L, opcodename);
+        opcodename = ((STRINGDAT *)opcodename_)->data;
+        lua_State *L = manageLuaState('O');
+        keys_t &keys = manageLuaReferenceKeys(L, opcodename, 'O');
         lua_rawgeti(L, LUA_REGISTRYINDEX, keys.init_key);
         lua_pushlightuserdata(L, csound);
         lua_pushlightuserdata(L, this);
         lua_pushlightuserdata(L, &arguments);
         if (lua_pcall(L, 3, 1, 0) != 0)
         {
-            log(csound, "Lua error in \"%s_init\": %s.\n", opcodename, lua_tostring(L, -1));
+            log(csound, "Lua error in \"%s_init\": %s.\n",
+                opcodename, lua_tostring(L, -1));
         }
         result = lua_tonumber(L, -1);
         lua_pop(L, 1);
@@ -248,15 +288,16 @@ public:
     int kontrol(CSOUND *csound)
     {
         int result = OK;
-        lua_State *L = manageLuaState();
-        keys_t &keys = manageLuaReferenceKeys(L, opcodename);
+        lua_State *L = manageLuaState('O');
+        keys_t &keys = manageLuaReferenceKeys(L, opcodename, 'O');
         lua_rawgeti(L, LUA_REGISTRYINDEX, keys.kontrol_key);
         lua_pushlightuserdata(L, csound);
         lua_pushlightuserdata(L, this);
         lua_pushlightuserdata(L, &arguments);
         if (lua_pcall(L, 3, 1, 0) != 0)
         {
-            log(csound, "Lua error in \"%s_kontrol\": %s.\n", opcodename, lua_tostring(L, -1));
+            log(csound, "Lua error in \"%s_kontrol\": %s.\n",
+                opcodename, lua_tostring(L, -1));
         }
         result = lua_tonumber(L, -1);
         lua_pop(L, 1);
@@ -266,15 +307,16 @@ public:
     int audio(CSOUND *csound)
     {
         int result = OK;
-        lua_State *L = manageLuaState();
-        keys_t &keys = manageLuaReferenceKeys(L, opcodename);
+        lua_State *L = manageLuaState('O');
+        keys_t &keys = manageLuaReferenceKeys(L, opcodename, 'O');
         lua_rawgeti(L, LUA_REGISTRYINDEX, keys.audio_key);
         lua_pushlightuserdata(L, csound);
         lua_pushlightuserdata(L, this);
         lua_pushlightuserdata(L, arguments);
         if (lua_pcall(L, 3, 1, 0) != 0)
         {
-            log(csound, "Lua error in \"%s_audio\": %s.\n", opcodename, lua_tostring(L, -1));
+            log(csound, "Lua error in \"%s_audio\": %s.\n",
+                opcodename, lua_tostring(L, -1));
         }
         result = lua_tonumber(L, -1);
         lua_pop(L, 1);
@@ -289,17 +331,21 @@ public:
  *                             side and outputs are returned in the argument.
  *                             Requires opname_init to be defined in Lua.
  *
- * lua_ikopcall_off Sname, ... Calls a Lua opcode at i-rate and k-rate. Any number of
- *                             output and/or input arguments may be passed.
+ * lua_ikopcall_off Sname, ... Calls a Lua opcode at i-rate and k-rate. Any
+ *                             number of output and/or input arguments may be
+ *                             passed.
  *                             All arguments must be passed on the right-hand
  *                             side and outputs are returned in the argument.
- *                             Requires opname_init and opname_kontrol to be defined in Lua.
+ *                             Requires opname_init and opname_kontrol to be
+ *                             defined in Lua.
  *
- * lua_iaopcall_off Sname, ... Calls a Lua opcode at i-rate and a-rate. Any number of
- *                             output and/or input arguments may be passed.
+ * lua_iaopcall_off Sname, ... Calls a Lua opcode at i-rate and a-rate. Any
+ *                             number of output and/or input arguments may be
+ *                             passed.
  *                             All arguments must be passed on the right-hand
  *                             side and outputs are returned in the argument.
- *                             Requires opname_init and opname_audio to be defined in Lua.
+ *                             Requires opname_init and opname_audio to be
+ *                             defined in Lua.
  *
  * Opcode that actually implements arbitrary Lua opcodes
  * by calling from Csound into Lua functions; this variant
@@ -309,10 +355,13 @@ public:
  * instrument with a reverb tail, and so on.
  *
  * Lua functions access elements of arguments as follows
- * (pointers to both scalars and arrays are dereferenced by the array access operator):
- * ffi.cdef(' struct arguments_t { double *a_out, double *i_in, double *i_txt, double *f_sig };');
+ * (pointers to both scalars and arrays are dereferenced by the array
+ *  access operator):
+ * ffi.cdef(' struct arguments_t { double *a_out, double *i_in,
+ *                                 double *i_txt, double *f_sig };');
  * local arguments = ffi.cast("struct arguments_t *", carguments_lightuserdata)
- * for i = 0, ksmps -1 do begin carguments.a_out[i] = carguments.i_in[0] * 3 end end
+ * for i = 0, ksmps -1 do begin carguments.a_out[i] = carguments.i_in[0] * 3
+ * end end
  */
 class cslua_opcall_off: public OpcodeNoteoffBase<cslua_opcall_off>
 {
@@ -331,7 +380,7 @@ public:
       * additional data, e.g. for opcode state, after the opcode output and
       * input arguments, as limited by the available space.
       */
-    MYFLT *arguments[0x3000];
+    MYFLT *arguments[1000];
     const char *opcodename;
 public:
     /**
@@ -343,20 +392,17 @@ public:
     int init(CSOUND *csound)
     {
         int result = OK;
-        opcodename = csound->strarg2name(csound,
-                                         (char *) 0,
-                                         opcodename_,
-                                         (char *)"default",
-                                         (int) csound->GetInputArgSMask(this));
-        lua_State *L = manageLuaState();
-        keys_t &keys = manageLuaReferenceKeys(L, opcodename);
+        opcodename = ((STRINGDAT *)opcodename_)->data;
+        lua_State *L = manageLuaState('O');
+        keys_t &keys = manageLuaReferenceKeys(L, opcodename, 'O');
         lua_rawgeti(L, LUA_REGISTRYINDEX, keys.init_key);
         lua_pushlightuserdata(L, csound);
         lua_pushlightuserdata(L, this);
         lua_pushlightuserdata(L, &arguments);
         if (lua_pcall(L, 3, 1, 0) != 0)
         {
-            log(csound, "Lua error in \"%s_init\": %s.\n", opcodename, lua_tostring(L, -1));
+            log(csound, "Lua error in \"%s_init\": %s.\n",
+                opcodename, lua_tostring(L, -1));
         }
         result = lua_tonumber(L, -1);
         lua_pop(L, 1);
@@ -366,15 +412,16 @@ public:
     int kontrol(CSOUND *csound)
     {
         int result = OK;
-        lua_State *L = manageLuaState();
-        keys_t &keys = manageLuaReferenceKeys(L, opcodename);
+        lua_State *L = manageLuaState('O');
+        keys_t &keys = manageLuaReferenceKeys(L, opcodename, 'O');
         lua_rawgeti(L, LUA_REGISTRYINDEX, keys.kontrol_key);
         lua_pushlightuserdata(L, csound);
         lua_pushlightuserdata(L, this);
         lua_pushlightuserdata(L, &arguments);
         if (lua_pcall(L, 3, 1, 0) != 0)
         {
-            log(csound, "Lua error in \"%s_kontrol\": %s.\n", opcodename, lua_tostring(L, -1));
+            log(csound, "Lua error in \"%s_kontrol\": %s.\n",
+                opcodename, lua_tostring(L, -1));
         }
         result = lua_tonumber(L, -1);
         lua_pop(L, 1);
@@ -384,15 +431,16 @@ public:
     int audio(CSOUND *csound)
     {
         int result = OK;
-        lua_State *L = manageLuaState();
-        keys_t &keys = manageLuaReferenceKeys(L, opcodename);
+        lua_State *L = manageLuaState('O');
+        keys_t &keys = manageLuaReferenceKeys(L, opcodename, 'O');
         lua_rawgeti(L, LUA_REGISTRYINDEX, keys.audio_key);
         lua_pushlightuserdata(L, csound);
         lua_pushlightuserdata(L, this);
         lua_pushlightuserdata(L, arguments);
         if (lua_pcall(L, 3, 1, 0) != 0)
         {
-            log(csound, "Lua error in \"%s_audio\": %s.\n", opcodename, lua_tostring(L, -1));
+            log(csound, "Lua error in \"%s_audio\": %s.\n",
+                opcodename, lua_tostring(L, -1));
         }
         result = lua_tonumber(L, -1);
         lua_pop(L, 1);
@@ -402,19 +450,21 @@ public:
     int noteoff(CSOUND *csound)
     {
         int result = OK;
-        lua_State *L = manageLuaState();
-        keys_t &keys = manageLuaReferenceKeys(L, opcodename);
+        lua_State *L = manageLuaState('O');
+        keys_t &keys = manageLuaReferenceKeys(L, opcodename, 'O');
         lua_rawgeti(L, LUA_REGISTRYINDEX, keys.noteoff_key);
         lua_pushlightuserdata(L, csound);
         lua_pushlightuserdata(L, this);
         lua_pushlightuserdata(L, arguments);
         if (lua_pcall(L, 3, 1, 0) != 0)
         {
-            log(csound, "Lua error in \"%s_noteoff\": %s.\n", opcodename, lua_tostring(L, -1));
+            log(csound, "Lua error in \"%s_noteoff\": %s.\n",
+                opcodename, lua_tostring(L, -1));
         }
         else
         {
-            log(csound, "Lua called \"%s_noteoff\": %s.\n", opcodename, lua_tostring(L, -1));
+            log(csound, "Lua called \"%s_noteoff\": %s.\n",
+                opcodename, lua_tostring(L, -1));
         }
         result = lua_tonumber(L, -1);
         lua_pop(L, 1);
@@ -467,26 +517,22 @@ public:
     int init(CSOUND *csound)
     {
         int result = OK;
-        lua_State *L = manageLuaState();
-        const char *opcodename =  csound->strarg2name(csound,
-                                  (char *) 0,
-                                  opcodename_,
-                                  (char *)"default",
-                                  (int) csound->GetInputArgSMask(this));
-        const char *luacode =  csound->strarg2name(csound,
-                               (char *) 0,
-                               luacode_,
-                               (char *)"default",
-                               (int) csound->GetInputArgSMask(this));
+        lua_State *L = manageLuaState('O');
+        /* Ensure that Csound is available in the global environment. */
+        lua_pushlightuserdata(L, csound);
+        lua_setfield(L, LUA_GLOBALSINDEX, "csound");
+        const char *opcodename = ((STRINGDAT *)opcodename_)->data;
+        const char *luacode = ((STRINGDAT *)luacode_)->data;
         //log(csound, "Executing Lua code:\n%s\n", luacode);
         result = luaL_dostring(L, luacode);
         if (result == 0)
         {
-            keys_t &keys = manageLuaReferenceKeys(L, opcodename);
+            keys_t &keys = manageLuaReferenceKeys(L, opcodename, 'O');
             log(csound, "Opcode: %s\n", opcodename);
             log(csound, "Result: %d\n", result);
             char init_function[0x100];
-            std::snprintf(init_function, 0x100, "%s_init", opcodename); //h.optext->t.opcod);
+            snprintf(init_function, 0x100,
+                          "%s_init", opcodename); //h.optext->t.opcod);
             lua_getglobal(L, init_function);
             if (!lua_isnil(L, 1))
             {
@@ -494,7 +540,8 @@ public:
                 lua_pop(L, 1);
             }
             char kontrol_function[0x100];
-            std::snprintf(kontrol_function, 0x100, "%s_kontrol", opcodename); //h.optext->t.opcod);
+            snprintf(kontrol_function, 0x100,
+                          "%s_kontrol", opcodename); //h.optext->t.opcod);
             lua_getglobal(L, kontrol_function);
             if (!lua_isnil(L, 1))
             {
@@ -502,7 +549,8 @@ public:
                 lua_pop(L, 1);
             }
             char audio_function[0x100];
-            std::snprintf(audio_function, 0x100, "%s_audio", opcodename); //h.optext->t.opcod);
+            snprintf(audio_function, 0x100,
+                          "%s_audio", opcodename); //h.optext->t.opcod);
             lua_getglobal(L, audio_function);
             if (!lua_isnil(L, 1))
             {
@@ -510,7 +558,8 @@ public:
                 lua_pop(L, 1);
             }
             char noteoff_function[0x100];
-            std::snprintf(noteoff_function, 0x100, "%s_noteoff", opcodename); //h.optext->t.opcod);
+            snprintf(noteoff_function, 0x100,
+                          "%s_noteoff", opcodename); //h.optext->t.opcod);
             lua_getglobal(L, noteoff_function);
             if (!lua_isnil(L, 1))
             {
@@ -539,6 +588,7 @@ extern "C"
         {
             (char*)"lua_exec",
             sizeof(cslua_exec),
+            0,
             1,
             (char*)"",
             (char*)"TNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN"
@@ -552,6 +602,7 @@ extern "C"
         {
             (char*)"lua_iopcall",
             sizeof(cslua_opcall),
+            0,
             1,
             (char*)"",
             (char*)"TNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN"
@@ -565,6 +616,7 @@ extern "C"
         {
             (char*)"lua_ikopcall",
             sizeof(cslua_opcall),
+            0,
             3,
             (char*)"",
             (char*)"TNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN"
@@ -578,6 +630,7 @@ extern "C"
         {
             (char*)"lua_iaopcall",
             sizeof(cslua_opcall),
+            0,
             5,
             (char*)"",
             (char*)"TNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN"
@@ -591,6 +644,7 @@ extern "C"
         {
             (char*)"lua_iopcall_off",
             sizeof(cslua_opcall_off),
+            0,
             1,
             (char*)"",
             (char*)"TNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN"
@@ -604,6 +658,7 @@ extern "C"
         {
             (char*)"lua_ikopcall_off",
             sizeof(cslua_opcall_off),
+            0,
             3,
             (char*)"",
             (char*)"TNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN"
@@ -617,6 +672,7 @@ extern "C"
         {
             (char*)"lua_iaopcall_off",
             sizeof(cslua_opcall_off),
+            0,
             5,
             (char*)"",
             (char*)"TNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN"
@@ -630,6 +686,7 @@ extern "C"
         {
             (char*)"lua_opdef",
             sizeof(cslua_opdef),
+            0,
             1,
             // No outputs.
             (char*)"",
@@ -649,6 +706,7 @@ extern "C"
             0,
             0,
             0,
+            0,
         }
     };
 
@@ -665,6 +723,7 @@ extern "C"
             status |= csound->AppendOpcode(csound,
                                            oentry->opname,
                                            oentry->dsblksiz,
+                                           oentry->flags,
                                            oentry->thread,
                                            oentry->outypes,
                                            oentry->intypes,
