@@ -30,7 +30,7 @@
 /* Nested allpass and Lorenz added                             */
 /* October 1998 by Hans Mikelson                               */
 /***************************************************************/
-#include "csdl.h"
+//#include "csdl.h"
 #include <math.h>
 #include "biquad.h"
 
@@ -51,12 +51,20 @@ static int biquadset(CSOUND *csound, BIQUAD *p)
 
 static int biquad(CSOUND *csound, BIQUAD *p)
 {
-    int   n = 0, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     double xn, yn;
     double xnm1 = p->xnm1, xnm2 = p->xnm2, ynm1 = p->ynm1, ynm2 = p->ynm2;
     double a0 = 1.0 / *p->a0, a1 = a0 * *p->a1, a2 = a0 * *p->a2;
     double b0 = a0 * *p->b0, b1 = a0 * *p->b1, b2 = a0 * *p->b2;
-    for (n=0; n<nsmps; n++) {
+
+    if (UNLIKELY(offset)) memset(p->out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       xn = (double)p->in[n];
       yn = b0*xn + b1*xnm1 + b2*xnm2 - a1*ynm1 - a2*ynm2;
       xnm2 = xnm1;
@@ -73,7 +81,9 @@ static int biquad(CSOUND *csound, BIQUAD *p)
 
 static int biquada(CSOUND *csound, BIQUAD *p)
 {
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out, *in;
     double xn, yn;
     MYFLT *a0 = p->a0, *a1 = p->a1, *a2 = p->a2;
@@ -81,7 +91,12 @@ static int biquada(CSOUND *csound, BIQUAD *p)
     double xnm1 = p->xnm1, xnm2 = p->xnm2, ynm1 = p->ynm1, ynm2 = p->ynm2;
     in   = p->in;
     out  = p->out;
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       xn = (double)in[n];
       yn = ( (double)b0[n] * xn + (double)b1[n] * xnm1 + (double)b2[n] * xnm2 -
              a1[n] * ynm1 - (double)a2[n] * ynm2)/ (double)a0[n];
@@ -116,7 +131,9 @@ static int moogvcfset(CSOUND *csound, MOOGVCF *p)
 
 static int moogvcf(CSOUND *csound, MOOGVCF *p)
 {
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out, *in;
     double xn;
     MYFLT *fcoptr, *resptr;
@@ -143,7 +160,12 @@ static int moogvcf(CSOUND *csound, MOOGVCF *p)
       scale = exp((1.0-pp1d2)*1.386249);      /* Scaling factor     */
       k     = res*scale;
     }
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       /* Handle a-rate modulation of fco & res. */
       if (p->fcocod) {
         fco = (double)fcoptr[n];
@@ -205,7 +227,9 @@ static int rezzyset(CSOUND *csound, REZZY *p)
 
 static int rezzy(CSOUND *csound, REZZY *p)
 {
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out, *fcoptr, *rezptr, *in;
     double fco, rez, xn, yn;
     double fqcadj, a=0.0, /* Initialisations fake */
@@ -220,11 +244,17 @@ static int rezzy(CSOUND *csound, REZZY *p)
     rez    = (double)*rezptr;
 
     /* Freq. is adjusted based on sample rate */
-    fqcadj = 0.149659863*(double)csound->esr;
+    fqcadj = 0.149659863*(double)CS_ESR;
     /* Try to keep the resonance under control     */
     if (rez < 1.0) rez = 1.0;
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (*p->mode == FL(0.0)) {    /* Low Pass */
-      if (UNLIKELY((p->rezcod==0) && (p->fcocod==0))) {/* Only need to calculate once */
+      if (UNLIKELY((p->rezcod==0) && (p->fcocod==0))) {
+        /* Only need to calculate once */
         double c = fqcadj/fco;    /* Filter constant c=1/Fco * adjustment */
         double rez2 = rez/(1.0 + exp(fco/11000.0));
         double b;
@@ -233,7 +263,7 @@ static int rezzy(CSOUND *csound, REZZY *p)
         b    = 1.0 + a + csq;     /* Normalization constant */
         invb = 1.0/b;
       }
-      for (n=0; n<nsmps; n++) { /* do ksmp times   */
+      for (n=offset; n<nsmps; n++) { /* do ksmp times   */
         /* Handle a-rate modulation of fco and rez */
         if (p->fcocod) {
           fco = (double)fcoptr[n];
@@ -252,7 +282,8 @@ static int rezzy(CSOUND *csound, REZZY *p)
         }
         xn = (double)in[n];             /* Get the next sample */
         /* Mikelson Biquad Filter Guts*/
-        yn = (1.0/sqrt(1.0+rez)*xn - (-a-2.0*csq)*ynm1 - csq*ynm2)*invb;
+        //yn = (1.0/sqrt(1.0+rez)*xn - (-a-2.0*csq)*ynm1 - csq*ynm2)*invb;
+        yn = (1.0/sqrt(1.0+rez)*xn - csq*((-a-2.0)*ynm1 + ynm2))*invb;
 
         xnm2 = xnm1; /* Update Xn-2 */
         xnm1 = xn;   /* Update Xn-1 */
@@ -264,7 +295,8 @@ static int rezzy(CSOUND *csound, REZZY *p)
     }
     else { /* High Pass Rezzy */
       double c=0.0, rez2=0.0;
-      if (UNLIKELY(p->fcocod==0 && p->rezcod==0)) {/* Only need to calculate once */
+      if (UNLIKELY(p->fcocod==0 && p->rezcod==0)) {
+        /* Only need to calculate once */
         double b;
         c = fqcadj/fco;    /* Filter constant c=1/Fco * adjustment */
         rez2 = rez/(1.0 + sqrt(sqrt(1.0/c)));
@@ -273,7 +305,7 @@ static int rezzy(CSOUND *csound, REZZY *p)
         b    = (c/rez2 + csq);
         invb = 1.0/b;
       }
-      for (n=0; n<nsmps; n++) { /* do ksmp times   */
+      for (n=offset; n<nsmps; n++) { /* do ksmp times   */
         /* Handle a-rate modulation of fco and rez */
         if (p->fcocod) {
           fco = (double)fcoptr[n];
@@ -314,7 +346,9 @@ static int rezzy(CSOUND *csound, REZZY *p)
 
 static int distort(CSOUND *csound, DISTORT *p)
 {
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out, *in;
     MYFLT pregain = *p->pregain, postgain  = *p->postgain;
     MYFLT shape1 = *p->shape1, shape2 = *p->shape2;
@@ -342,7 +376,12 @@ static int distort(CSOUND *csound, DISTORT *p)
     shape1 += pregain;
     shape2 -= pregain;
     postgain *= FL(0.5);
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       sig    = in[n];
       /* Generate tanh distortion and output the result */
       out[n] =                          /* IV - Dec 28 2002: optimised */
@@ -362,11 +401,11 @@ static int distort(CSOUND *csound, DISTORT *p)
 static int vcoset(CSOUND *csound, VCO *p)
 {
     /* Number of bytes in the delay */
-    uint32 ndel = (uint32)(*p->maxd * csound->esr);
+    uint32 ndel = (uint32)(*p->maxd * CS_ESR);
     FUNC  *ftp;    /* Pointer to a sine function */
-    MYFLT ndsave;
+    //MYFLT ndsave;
 
-    ndsave = (MYFLT) ndel;
+    //ndsave = (MYFLT) ndel;
     if (UNLIKELY((ftp = csound->FTFind(csound, p->sine)) == NULL))
       return NOTOK;
 
@@ -390,8 +429,8 @@ static int vcoset(CSOUND *csound, VCO *p)
 
     if (UNLIKELY(ndel == 0)) ndel = 1;    /* fix due to Troxler */
     if (p->aux.auxp == NULL ||
-        (int)(ndel*sizeof(MYFLT)) > p->aux.size)  /* allocate space for delay
-                                                     buffer */
+        (unsigned int)(ndel*sizeof(MYFLT)) > p->aux.size)
+      /* allocate space for delay buffer */
       csound->AuxAlloc(csound, ndel * sizeof(MYFLT), &p->aux);
     else if (*p->iskip==FL(0.0)) {
       memset(p->aux.auxp, 0, ndel*sizeof(MYFLT));
@@ -415,10 +454,13 @@ static int vco(CSOUND *csound, VCO *p)
     FUNC  *ftp;
     MYFLT *ar, *ampp, *cpsp, *ftbl;
     int32  phs, inc, lobits, dwnphs, tnp1, lenmask, maxd, indx;
-    MYFLT leaky, rtfqc, amp, fqc;
+    MYFLT leaky, /*rtfqc,*/ amp, fqc;
     MYFLT sicvt2, over2n, scal, num, denom, pulse = FL(0.0), saw = FL(0.0);
     MYFLT sqr = FL(0.0), tri = FL(0.0);
-    int   n, nsmps = csound->ksmps, knh;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int   knh;
 
     /* VDelay Inserted here */
     MYFLT *buf = (MYFLT *)p->aux.auxp;
@@ -430,7 +472,7 @@ static int vco(CSOUND *csound, VCO *p)
 
     ftp = p->ftp;
     if (UNLIKELY(buf==NULL || ftp==NULL)) goto err1;            /* RWD fix */
-    maxd = (uint32) (*p->maxd * csound->esr);
+    maxd = (uint32) (*p->maxd * CS_ESR);
     if (UNLIKELY(maxd == 0)) maxd = 1;    /* Degenerate case */
     indx = p->left;
     /* End of VDelay insert */
@@ -442,8 +484,8 @@ static int vco(CSOUND *csound, VCO *p)
     ampp = p->xamp;
     cpsp = p->xcps;
     fqc = *cpsp;
-    rtfqc = SQRT(fqc);
-    knh = (int)(csound->esr*p->nyq/fqc);
+    //rtfqc = SQRT(fqc);
+    knh = (int)(CS_ESR*p->nyq/fqc);
     if (UNLIKELY((n = (int)knh) <= 0)) {
       csound->Warning(csound, "knh=%x nyq=%f fqc=%f\n"
                       "vco knh (%d) <= 0; taken as 1\n", knh, p->nyq, fqc, n);
@@ -457,13 +499,18 @@ static int vco(CSOUND *csound, VCO *p)
     inc = (int32)(fqc * sicvt2);
     ar = p->ar;
     phs = p->lphs;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
 
 /*-----------------------------------------------------*/
 /* PWM Wave                                            */
 /*-----------------------------------------------------*/
     if (wave==2) {
       MYFLT pw = *p->pw;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         dwnphs = phs >> lobits;
         denom = *(ftbl + dwnphs);
         if (denom > FL(0.00001) || denom < -FL(0.00001)) {
@@ -484,7 +531,7 @@ static int vco(CSOUND *csound, VCO *p)
 
         /* VDelay inserted here */
         buf[indx] = pulse;
-        fv1 = (MYFLT) indx - csound->esr * pw / fqc;
+        fv1 = (MYFLT) indx - CS_ESR * pw / fqc;
 
         v1 = (int32) fv1;
         if (fv1 < FL(0.0)) v1--;
@@ -512,7 +559,7 @@ static int vco(CSOUND *csound, VCO *p)
     /*-----------------------------------------------------*/
     else if (wave==3) {
       MYFLT pw = *p->pw;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         dwnphs = phs >> lobits;
         denom = *(ftbl + dwnphs);
         if (denom > FL(0.0002) || denom < -FL(0.0002)) {
@@ -534,7 +581,7 @@ static int vco(CSOUND *csound, VCO *p)
 
         /* VDelay inserted here */
         buf[indx] = pulse;
-        fv1 = (MYFLT) indx - csound->esr * pw / fqc;
+        fv1 = (MYFLT) indx - CS_ESR * pw / fqc;
 
         v1 = (int32) fv1;
         if (fv1 < FL(0.0)) v1--;
@@ -548,7 +595,7 @@ static int vco(CSOUND *csound, VCO *p)
         v2 = (v1 < (maxd - 1L) ? v1 + 1L : 0L);
         out1 = buf[v1] + fv1 * (buf[v2] - buf[v1]);
 
-        if (UNLIKELY(++indx == maxd)) indx = 0;             /* Advance current pointer */
+        if (UNLIKELY(++indx == maxd)) indx = 0;  /* Advance current pointer */
         /* End of VDelay */
 
         /* Integrate twice and ouput */
@@ -557,14 +604,14 @@ static int vco(CSOUND *csound, VCO *p)
         p->ynm1 = sqr;
         p->ynm2 = tri;
         ar[n] =  tri * amp * fqc
-                 / (csound->esr * FL(0.42) * (FL(0.05) + pw - pw * pw));
+                 / (CS_ESR * FL(0.42) * (FL(0.05) + pw - pw * pw));
       }
     }
     /*-----------------------------------------------------*/
     /* Sawtooth Wave                                       */
     /*-----------------------------------------------------*/
     else {
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         dwnphs = phs >> lobits;
         denom = *(ftbl + dwnphs);
         if (denom > FL(0.0002) || denom < -FL(0.0002)) {
@@ -596,7 +643,7 @@ static int vco(CSOUND *csound, VCO *p)
     p->lphs = phs;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("vco: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead, Str("vco: not initialised"));
 }
 
 /***************************************************************************/
@@ -623,7 +670,9 @@ static int planet(CSOUND *csound, PLANET *p)
     MYFLT *outx, *outy, *outz;
     MYFLT   sqradius1, sqradius2, radius1, radius2, fric;
     MYFLT xxpyy, dz1, dz2, mass1, mass2, msqror1, msqror2;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     fric = p->friction;
 
@@ -637,7 +686,19 @@ static int planet(CSOUND *csound, PLANET *p)
     mass1 = *p->mass1;
     mass2 = *p->mass2;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) {
+      memset(outx, '\0', offset*sizeof(MYFLT));
+      memset(outy, '\0', offset*sizeof(MYFLT));
+      memset(outz, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&outx[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&outy[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&outz[nsmps], '\0', early*sizeof(MYFLT));
+    }
+
+    for (n=offset; n<nsmps; n++) {
       xxpyy = p->x * p->x + p->y * p->y;
       dz1 = p->s1z - p->z;
 
@@ -698,7 +759,9 @@ static int pareqset(CSOUND *csound, PAREQ *p)
 static int pareq(CSOUND *csound, PAREQ *p)
 {
     MYFLT xn, yn;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (*p->fc != p->prv_fc || *p->v != p->prv_v || *p->q != p->prv_q) {
       double omega = (double)(csound->tpidsr * *p->fc), k, kk, vkk, vk, vkdq, a0;
@@ -749,12 +812,16 @@ static int pareq(CSOUND *csound, PAREQ *p)
       a0 = 1.0 / a0;
       p->a1 *= a0; p->a2 *= a0; p->b0 *= a0; p->b1 *= a0; p->b2 *= a0;
     }
-
+    if (UNLIKELY(offset)) memset(p->out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->out[nsmps], '\0', early*sizeof(MYFLT));
+    }
     {
       double a1 = p->a1, a2 = p->a2;
       double b0 = p->b0, b1 = p->b1, b2 = p->b2;
       double xnm1 = p->xnm1, xnm2 = p->xnm2, ynm1 = p->ynm1, ynm2 = p->ynm2;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         xn = (double)p->in[n];
         yn = b0 * xn + b1 * xnm1 + b2 * xnm2 - a1 * ynm1 - a2 * ynm2;
         xnm2 = xnm1;
@@ -781,12 +848,13 @@ static int nestedapset(CSOUND *csound, NESTEDAP *p)
     if (*p->istor && p->auxch.auxp != NULL)
       return OK;
 
-    npts2 = (int32)(*p->del2 * csound->esr);
-    npts3 = (int32)(*p->del3 * csound->esr);
-    npts1 = (int32)(*p->del1 * csound->esr) - npts2 -npts3;
+    npts2 = (int32)(*p->del2 * CS_ESR);
+    npts3 = (int32)(*p->del3 * CS_ESR);
+    npts1 = (int32)(*p->del1 * CS_ESR) - npts2 -npts3;
 
-    if (UNLIKELY(((int32)(*p->del1 * csound->esr)) <=
-                 ((int32)(*p->del2 * csound->esr) + (int32)(*p->del3 * csound->esr)))) {
+    if (UNLIKELY(((int32)(*p->del1 * CS_ESR)) <=
+                 ((int32)(*p->del2 * CS_ESR) +
+                  (int32)(*p->del3 * CS_ESR)))) {
       return csound->InitError(csound, Str("illegal delay time"));
     }
     npts = npts1 + npts2 + npts3;
@@ -843,13 +911,20 @@ static int nestedap(CSOUND *csound, NESTEDAP *p)
     MYFLT   *beg1p, *beg2p, *beg3p, *end1p, *end2p, *end3p;
     MYFLT   *del1p, *del2p, *del3p;
     MYFLT   in1, g1, g2, g3;
-    int     n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (UNLIKELY(p->auxch.auxp==NULL)) goto err1; /* RWD fix */
 
     outp = p->out;
     inp  = p->in;
 
+    if (UNLIKELY(offset)) memset(outp, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&outp[nsmps], '\0', early*sizeof(MYFLT));
+    }
     /* Ordinary All-Pass Filter */
     if (*p->mode == FL(1.0)) {
 
@@ -858,7 +933,7 @@ static int nestedap(CSOUND *csound, NESTEDAP *p)
       beg1p = p->beg1p;
       g1    = *p->gain1;
 
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         in1 = inp[n];
         p->out1 = *del1p - g1*in1;
 
@@ -885,7 +960,7 @@ static int nestedap(CSOUND *csound, NESTEDAP *p)
       beg2p = p->beg2p;
       g2    = *p->gain2;
 
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         in1 = inp[n];
 
         p->out2 = *del2p  - g2 * *del1p;
@@ -926,7 +1001,7 @@ static int nestedap(CSOUND *csound, NESTEDAP *p)
       beg3p = p->beg3p;
       g3    = *p->gain3;
 
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         in1 = inp[n];
 
         p->out2 = *del2p  - g2 * *del1p;
@@ -957,7 +1032,8 @@ static int nestedap(CSOUND *csound, NESTEDAP *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("delay: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("delay: not initialised"));
 }
 
 /***************************************************************************/
@@ -979,7 +1055,10 @@ static int lorenz(CSOUND *csound, LORENZ *p)
 {
     MYFLT   *outx, *outy, *outz;
     MYFLT   x, y, z, xx, yy, s, r, b, hstep;
-    int32    n, nsmps = csound->ksmps, skip;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int32    skip;
 
     outx  = p->outx;
     outy  = p->outy;
@@ -994,7 +1073,19 @@ static int lorenz(CSOUND *csound, LORENZ *p)
     y     = p->valy;
     z     = p->valz;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) {
+      memset(outx, '\0', offset*sizeof(MYFLT));
+      memset(outy, '\0', offset*sizeof(MYFLT));
+      memset(outz, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&outx[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&outy[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&outz[nsmps], '\0', early*sizeof(MYFLT));
+    }
+
+    for (n=offset; n<nsmps; n++) {
       do {
         xx   =      x+hstep*s*(y-x);
         yy   =      y+hstep*(-x*z+r*x-y);
@@ -1036,7 +1127,9 @@ static int tbvcfset(CSOUND *csound, TBVCF *p)
 
 static int tbvcf(CSOUND *csound, TBVCF *p)
 {
-    int32 n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out, *in;
     double x;
     MYFLT *fcoptr, *resptr, *distptr, *asymptr;
@@ -1069,7 +1162,12 @@ static int tbvcf(CSOUND *csound, TBVCF *p)
       q    = q1*fco1*fco1*0.0005;
       fc   = fco1*(double)csound->onedsr*(44100.0/8.0);
     }
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       /* Handle a-rate modulation of fco & res. */
       if (p->fcocod) {
         fco = (double)fcoptr[n];
@@ -1110,7 +1208,9 @@ static int bqrezset(CSOUND *csound, REZZY *p)
 
 static int bqrez(CSOUND *csound, REZZY *p)
 {
-    int32 n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out, *fcoptr, *rezptr, *in;
     double fco, rez, xn, yn;
     double sin2 = 0.0, cos2 = 0.0, beta=0.0, alpha, gamma=0.0, mu, sigma, chi;
@@ -1133,6 +1233,12 @@ static int bqrez(CSOUND *csound, REZZY *p)
       gamma = (beta + 1.0) * cos2;
     }
 
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+
     if (mode < 3) {
       if (mode == 0) {    /* Low Pass */
         chi   = -1.0;
@@ -1151,7 +1257,7 @@ static int bqrez(CSOUND *csound, REZZY *p)
       }
       alpha = (beta + 1.0 + chi*gamma) * 0.5;
 
-      for (n=0; n<nsmps; n++) {                        /* do ksmp times   */
+      for (n=offset; n<nsmps; n++) {                        /* do ksmp times   */
         /* Handle a-rate modulation of fco and rez */
         if (p->fcocod) {
           fco = (double)fcoptr[n];
@@ -1180,7 +1286,7 @@ static int bqrez(CSOUND *csound, REZZY *p)
     }
     else if (mode == 3) {   /* Band Stop */
       alpha  = (beta + 1.0) * 0.5;
-      for (n=0; n<nsmps; n++) {                            /* do ksmp times   */
+      for (n=offset; n<nsmps; n++) {                       /* do ksmp times   */
         /* Handle a-rate modulation of fco and rez */
         if (p->fcocod) {
           fco = (double)fcoptr[n];
@@ -1208,7 +1314,7 @@ static int bqrez(CSOUND *csound, REZZY *p)
       }
     }
     else if (mode == 4) {   /* All Pass */
-      for (n=0; n<nsmps; n++) {                        /* do ksmp times   */
+      for (n=offset; n<nsmps; n++) {                        /* do ksmp times   */
         /* Handle a-rate modulation of fco and rez */
         if (p->fcocod) {
           fco = (double)fcoptr[n];
@@ -1256,32 +1362,41 @@ static int bqrez(CSOUND *csound, REZZY *p)
 
 static int mode(CSOUND *csound, MODE *p)
 {
-    int   n = 0, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
-    double kfreq = *p->kfreq*2*PI;
-    double kalpha = (csound->esr/kfreq);
+    double kfreq  = *p->kfreq*TWOPI;
+    double kalpha = (CS_ESR/kfreq);
     double kbeta  = kalpha*kalpha;
+    double d      = 0.5*kalpha;
 
-    double a0 = 1/ (kbeta+kalpha/(2* *p->kq));
-    double a1 = a0 * (1-2*kbeta);
-    double a2 = a0 * (kbeta-kalpha/(2* *p->kq));
+    double a0     = 1.0/ (kbeta+d/(*p->kq));
+    double a1     = a0 * (1.0-2.0*kbeta);
+    double a2     = a0 * (kbeta-d/(*p->kq));
 
     double xn, yn;
+    double xnm1 = p->xnm1, ynm1 = p->ynm1, ynm2 = p->ynm2;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(p->aout, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->aout[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       xn = (double)p->ain[n];
 
-      yn = a0*p->xnm1 - a1*p->ynm1 - a2*p->ynm2;
+      yn = a0*xnm1 - a1*ynm1 - a2*ynm2;
 
-      p->xnm1 = xn;
-      p->ynm2 = p->ynm1;
-      p->ynm1 = yn;
+      xnm1 = xn;
+      ynm2 = ynm1;
+      ynm1 = yn;
 
-      yn = yn*csound->esr/(2*kfreq);
+      yn = yn*d;
 
       p->aout[n] = (MYFLT)yn;
     }
-
+    p->xnm1 = xnm1;  p->ynm1 = ynm1;  p->ynm2 = ynm2;
     return OK;
 }
 
@@ -1290,21 +1405,28 @@ static int mode(CSOUND *csound, MODE *p)
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "biquad", S(BIQUAD),   5, "a", "akkkkkko",(SUBR)biquadset, NULL, (SUBR)biquad },
-{ "biquada", S(BIQUAD),  5, "a", "aaaaaaao",(SUBR)biquadset, NULL,(SUBR)biquada },
-{ "moogvcf", S(MOOGVCF), 5, "a", "axxpo", (SUBR)moogvcfset, NULL, (SUBR)moogvcf },
-{ "moogvcf2", S(MOOGVCF),5, "a", "axxoo", (SUBR)moogvcfset, NULL, (SUBR)moogvcf },
-{ "rezzy", S(REZZY),     5, "a", "axxoo", (SUBR)rezzyset, NULL, (SUBR)rezzy },
-{ "bqrez", S(REZZY),     5, "a", "axxoo", (SUBR)bqrezset, NULL, (SUBR)bqrez },
-{ "distort1", S(DISTORT),TR|4, "a", "akkkko",  NULL,   NULL,   (SUBR)distort   },
-{ "vco", S(VCO),      TR|5, "a", "xxiVppovoo",(SUBR)vcoset, NULL, (SUBR)vco },
-{ "tbvcf", S(TBVCF),     5, "a", "axxkkp",  (SUBR)tbvcfset, NULL, (SUBR)tbvcf   },
-{ "planet", S(PLANET),5,"aaa","kkkiiiiiiioo", (SUBR)planetset, NULL, (SUBR)planet},
-{ "pareq", S(PAREQ),     5, "a", "akkkoo",(SUBR)pareqset, NULL, (SUBR)pareq },
-{ "nestedap", S(NESTEDAP),5,"a", "aiiiiooooo",
+  { "biquad", S(BIQUAD),   0, 5, "a", "akkkkkko",
+                                 (SUBR)biquadset, NULL, (SUBR)biquad },
+{ "biquada", S(BIQUAD),  0, 5, "a", "aaaaaaao",
+                                 (SUBR)biquadset, NULL,(SUBR)biquada },
+{ "moogvcf", S(MOOGVCF), 0, 5, "a", "axxpo",
+                               (SUBR)moogvcfset, NULL, (SUBR)moogvcf },
+{ "moogvcf2", S(MOOGVCF),0, 5, "a", "axxoo",
+                               (SUBR)moogvcfset, NULL, (SUBR)moogvcf },
+{ "rezzy", S(REZZY),     0, 5, "a", "axxoo", (SUBR)rezzyset, NULL, (SUBR)rezzy },
+{ "bqrez", S(REZZY),     0, 5, "a", "axxoo", (SUBR)bqrezset, NULL, (SUBR)bqrez },
+{ "distort1", S(DISTORT),TR, 4, "a", "akkkko",  NULL,   NULL,   (SUBR)distort   },
+{ "vco", S(VCO),      TR, 5, "a", "xxiVppovoo",(SUBR)vcoset, NULL, (SUBR)vco },
+{ "tbvcf", S(TBVCF),     0, 5, "a", "axxkkp",
+                                 (SUBR)tbvcfset, NULL, (SUBR)tbvcf   },
+{ "planet", S(PLANET),0, 5,"aaa","kkkiiiiiiioo",
+                                  (SUBR)planetset, NULL, (SUBR)planet},
+{ "pareq", S(PAREQ),     0, 5, "a", "akkkoo",(SUBR)pareqset, NULL, (SUBR)pareq },
+{ "nestedap", S(NESTEDAP),0, 5,"a", "aiiiiooooo",
                                      (SUBR)nestedapset, NULL, (SUBR)nestedap},
-{ "lorenz", S(LORENZ), 5, "aaa", "kkkkiiiio", (SUBR)lorenzset, NULL, (SUBR)lorenz},
-{ "mode",  S(MODE),   5,      "a", "akko", (SUBR)modeset, NULL, (SUBR)mode   }
+{ "lorenz", S(LORENZ),0,  5, "aaa", "kkkkiiiio",
+                                  (SUBR)lorenzset, NULL, (SUBR)lorenz},
+{ "mode",  S(MODE),   0, 5,      "a", "akko", (SUBR)modeset, NULL, (SUBR)mode   }
 };
 
 int biquad_init_(CSOUND *csound)
@@ -1312,4 +1434,3 @@ int biquad_init_(CSOUND *csound)
     return csound->AppendOpcodes(csound, &(localops[0]),
                                  (int) (sizeof(localops) / sizeof(OENTRY)));
 }
-
