@@ -24,7 +24,7 @@
     02111-1307 USA
 */
 
-#include "csdl.h"
+#include "stdopcod.h"
 #include <ctype.h>
 #include <stdarg.h>
 #include "soundio.h"
@@ -84,7 +84,7 @@ static int register_ftable_delete(CSOUND *csound, void *p, int tableNum)
 
 /* set up and call any GEN routine */
 
-static int ftgen(CSOUND *csound, FTGEN *p)
+static int ftgen_(CSOUND *csound, FTGEN *p, int istring1, int istring2)
 {
     MYFLT   *fp;
     FUNC    *ftp;
@@ -92,7 +92,7 @@ static int ftgen(CSOUND *csound, FTGEN *p)
     int     n;
 
     *p->ifno = FL(0.0);
-    ftevt = (EVTBLK*) malloc(sizeof(EVTBLK)); /* Can use malloc direct as local */
+    ftevt =(EVTBLK*) malloc(sizeof(EVTBLK)); /* Can use malloc direct as local */
     ftevt->opcod = 'f';
     ftevt->strarg = NULL;
     fp = &ftevt->p[0];
@@ -101,25 +101,26 @@ static int ftgen(CSOUND *csound, FTGEN *p)
     fp[2] = ftevt->p2orig = FL(0.0);                    /* force time 0 */
     fp[3] = ftevt->p3orig = *p->p3;
     fp[4] = *p->p4;
-    
-    if ((n = csound->GetInputArgSMask(p)))
-      if (n&0x8) {              /* Named gen */
-        NAMEDGEN *n = (NAMEDGEN*) csound->GetNamedGens(csound);
-        while (n) {
-          if (strcmp(n->name, (char *)p->p4) == 0) {    /* Look up by name */
-            break;
+
+
+      if (istring1) {              /* Named gen */
+        NAMEDGEN *named = (NAMEDGEN*) csound->GetNamedGens(csound);
+        while (named) {
+          if (strcmp(named->name, ((STRINGDAT *) p->p4)->data) == 0) {
+            /* Look up by name */
+           break;
           }
-          n = n->next;                            /*  and round again         */
+          named = named->next;                            /*  and round again   */
         }
-        if (UNLIKELY(n == NULL)) {
+        if (UNLIKELY(named == NULL)) {
           return csound->InitError(csound,
                                    Str("Named gen \"%s\" not defined"),
                                    (char *)p->p4);
         }
-        else fp[4] = n->genum;
-        fp[5] = *p->p5;
+        else fp[4] = named->genum;
       }
-      else {  /* string argument: */
+
+      if(istring2) {  /* string argument: */
         n = (int) fp[4];
         fp[5] = SSTRCOD;
         if (n < 0)
@@ -129,10 +130,10 @@ static int ftgen(CSOUND *csound, FTGEN *p)
         case 23:
         case 28:
         case 43:
-          ftevt->strarg = (char*) p->p5;
+          ftevt->strarg = ((STRINGDAT *) p->p5)->data;
           break;
         default:
-          csound->Free(csound, ftevt);
+          free(ftevt);
           return csound->InitError(csound, Str("ftgen string arg not allowed"));
         }
       }
@@ -158,6 +159,22 @@ static int ftgen(CSOUND *csound, FTGEN *p)
     return OK;
 }
 
+static int ftgen(CSOUND *csound, FTGEN *p) {
+  return ftgen_(csound,p,0,0);
+}
+
+static int ftgen_S(CSOUND *csound, FTGEN *p) {
+  return ftgen_(csound,p,1,0);
+}
+
+static int ftgen_iS(CSOUND *csound, FTGEN *p) {
+  return ftgen_(csound,p,0,1);
+}
+
+static int ftgen_SS(CSOUND *csound, FTGEN *p) {
+  return ftgen_(csound,p,1,1);
+}
+
 static int ftgentmp(CSOUND *csound, FTGEN *p)
 {
     int   p1, fno;
@@ -171,6 +188,19 @@ static int ftgentmp(CSOUND *csound, FTGEN *p)
     return register_ftable_delete(csound, p, fno);
 }
 
+/* static int ftgentmp_S(CSOUND *csound, FTGEN *p) */
+/* { */
+/*     int   p1, fno; */
+
+/*     if (UNLIKELY(ftgen_(csound, p,0,1) != OK)) */
+/*       return NOTOK; */
+/*     p1 = (int) MYFLT2LRND(*p->p1); */
+/*     if (p1) */
+/*       return OK; */
+/*     fno = (int) MYFLT2LRND(*p->ifno); */
+/*     return register_ftable_delete(csound, p, fno); */
+/* } */
+
 static int ftfree(CSOUND *csound, FTFREE *p)
 {
     int fno = (int) MYFLT2LRND(*p->iftno);
@@ -185,32 +215,45 @@ static int ftfree(CSOUND *csound, FTFREE *p)
     return register_ftable_delete(csound, p, fno);
 }
 
-static int ftload(CSOUND *csound, FTLOAD *p)
+static int myInitError(CSOUND *csound, INSDS *p, const char *str, ...)
+{
+    return csound->InitError(csound, str);
+}
+
+static int ftload_(CSOUND *csound, FTLOAD *p, int istring)
 {
     MYFLT **argp = p->argums;
     FUNC  *ftp;
     char  filename[MAXNAME];
     int   nargs = csound->GetInputArgCnt(p) - 2;
     FILE  *file = NULL;
-    int   (*err_func)(CSOUND *, const char *, ...);
+    int   (*err_func)(CSOUND *, INSDS *, const char *, ...);
     FUNC  *(*ft_func)(CSOUND *, MYFLT *);
     void  *fd;
 
-    if (strcmp(csound->GetOpcodeName(p), "ftload") != 0) {
+    if (strncmp(csound->GetOpcodeName(p), "ftload", 6) != 0) {
       nargs--;
       ft_func = csound->FTFindP;
       err_func = csound->PerfError;
     }
     else {
       ft_func = csound->FTnp2Find;
-      err_func = csound->InitError;
+      err_func = myInitError;
     }
 
     if (UNLIKELY(nargs <= 0))
       goto err2;
 
-    csound->strarg2name(csound, filename, p->ifilno, "ftsave.",
-                                (int) csound->GetInputArgSMask(p));
+    if(!istring) {
+      if(ISSTRCOD(*p->ifilno))
+         csound->strarg2name(csound, filename, p->ifilno, "ftsave.",
+                               0);
+         else strncpy(filename, get_arg_string(csound,*p->ifilno), MAXNAME-1);
+
+    } else {
+      strncpy(filename, ((STRINGDAT *)p->ifilno)->data, MAXNAME-1);
+    }
+
     if (*p->iflag <= FL(0.0)) {
       fd = csound->FileOpen2(csound, &file, CSFILE_STD, filename, "rb",
                                "", CSFTYPE_FTABLES_BINARY, 0);
@@ -219,7 +262,7 @@ static int ftload(CSOUND *csound, FTLOAD *p)
         FUNC  header;
         int   fno = (int) MYFLT2LRND(**argp);
         MYFLT fno_f = (MYFLT) fno;
-        int   n;
+        size_t   n;
 
         memset(&header, 0, sizeof(FUNC));
         /* ***** Need to do byte order here ***** */
@@ -246,7 +289,7 @@ static int ftload(CSOUND *csound, FTLOAD *p)
         char  s[64], *s1;
         int   fno = (int) MYFLT2LRND(**argp);
         MYFLT fno_f = (MYFLT) fno;
-        int32  j;
+        uint32_t  j;
         char *endptr;
 
         memset(&header, 0, sizeof(FUNC));
@@ -306,22 +349,22 @@ static int ftload(CSOUND *csound, FTLOAD *p)
         header.fno = strtol(s1, &endptr, 10);
         if (UNLIKELY(endptr==NULL)) goto err4;
         if (UNLIKELY(NULL==fgets(s, 64, file))) goto err4;  s1 = strchr(s, ' ')+1;
-        header.gen01args.gen01 = (MYFLT)strtod(s1, &endptr);
+        header.gen01args.gen01 = (MYFLT)cs_strtod(s1, &endptr);
         if (UNLIKELY(endptr==NULL)) goto err4;
         if (UNLIKELY(NULL==fgets(s, 64, file))) goto err4;  s1 = strchr(s, ' ')+1;
-        header.gen01args.ifilno = (MYFLT)strtod(s1, &endptr);
+        header.gen01args.ifilno = (MYFLT)cs_strtod(s1, &endptr);
         if (UNLIKELY(endptr==NULL)) goto err4;
         if (UNLIKELY(NULL==fgets(s, 64, file))) goto err4;  s1 = strchr(s, ' ')+1;
-        header.gen01args.iskptim = (MYFLT)strtod(s1, &endptr);
+        header.gen01args.iskptim = (MYFLT)cs_strtod(s1, &endptr);
         if (UNLIKELY(endptr==NULL)) goto err4;
         if (UNLIKELY(NULL==fgets(s, 64, file))) goto err4;  s1 = strchr(s, ' ')+1;
-        header.gen01args.iformat = (MYFLT)strtod(s1, &endptr);
+        header.gen01args.iformat = (MYFLT)cs_strtod(s1, &endptr);
         if (UNLIKELY(endptr==NULL)) goto err4;
         if (UNLIKELY(NULL==fgets(s, 64, file))) goto err4;  s1 = strchr(s, ' ')+1;
-        header.gen01args.channel = (MYFLT)strtod(s1, &endptr);
+        header.gen01args.channel = (MYFLT)cs_strtod(s1, &endptr);
         if (UNLIKELY(endptr==NULL)) goto err4;
         if (UNLIKELY(NULL==fgets(s, 64, file))) goto err4;  s1 = strchr(s, ' ')+1;
-        header.gen01args.sample_rate = (MYFLT)strtod(s1, &endptr);
+        header.gen01args.sample_rate = (MYFLT)cs_strtod(s1, &endptr);
         if (UNLIKELY(endptr==NULL)) goto err4;
         if (UNLIKELY(NULL==fgets(s, 64, file))) goto err4;  s1 = strchr(s, ' ')+1;
         /* WARNING! skips header.gen01args.strarg from saving/loading
@@ -334,7 +377,7 @@ static int ftload(CSOUND *csound, FTLOAD *p)
         memset(&(ftp->ftable[0]), 0, sizeof(MYFLT) * (ftp->flen + 1));
         for (j = 0; j <= ftp->flen; j++) {
           if (UNLIKELY(NULL==fgets(s, 64, file))) goto err4;
-          ftp->ftable[j] = (MYFLT) strtod(s, &endptr);
+          ftp->ftable[j] = (MYFLT) cs_strtod(s, &endptr);
           if (UNLIKELY(endptr==NULL)) goto err4;
         }
         if (UNLIKELY(NULL==fgets(s, 64, file))) goto err4;
@@ -345,48 +388,75 @@ static int ftload(CSOUND *csound, FTLOAD *p)
     return OK;
  err:
     csound->FileClose(csound, fd);
-    return err_func(csound, Str("ftload: error allocating ftable"));
+    return err_func(csound, p->h.insdshead,
+                    Str("ftload: error allocating ftable"));
  err2:
-    return err_func(csound, Str("ftload: no table numbers"));
+    return err_func(csound, p->h.insdshead, Str("ftload: no table numbers"));
  err3:
-    return err_func(csound, Str("ftload: unable to open file"));
+    return err_func(csound, p->h.insdshead, Str("ftload: unable to open file"));
  err4:
     csound->FileClose(csound, fd);
-    return err_func(csound, Str("ftload: incorrect file"));
+    return err_func(csound, p->h.insdshead, Str("ftload: incorrect file"));
 }
 
+static int ftload(CSOUND *csound, FTLOAD *p)
+{
+  return ftload_(csound, p, 0);
+}
+
+static int ftload_S(CSOUND *csound, FTLOAD *p)
+{
+  return ftload_(csound, p, 1);
+}
+
+
 static int ftload_k(CSOUND *csound, FTLOAD_K *p)
 {
     if (*p->ktrig != FL(0.0))
-      return ftload(csound, &(p->p));
+      return ftload_(csound, &(p->p),0);
     return OK;
 }
 
-static int ftsave(CSOUND *csound, FTLOAD *p)
+static int ftload_kS(CSOUND *csound, FTLOAD_K *p)
+{
+    if (*p->ktrig != FL(0.0))
+      return ftload_(csound, &(p->p), 1);
+    return OK;
+}
+
+static int ftsave_(CSOUND *csound, FTLOAD *p, int istring)
 {
     MYFLT **argp = p->argums;
     char  filename[MAXNAME];
     int   nargs = csound->GetInputArgCnt(p) - 2;
     FILE  *file = NULL;
-    int   (*err_func)(CSOUND *, const char *, ...);
+    int   (*err_func)(CSOUND *, INSDS *, const char *, ...);
     FUNC  *(*ft_func)(CSOUND *, MYFLT *);
     void  *fd;
 
-    if (strcmp(csound->GetOpcodeName(p), "ftsave") != 0) {
+    if (strncmp(csound->GetOpcodeName(p), "ftsave", 6) != 0) {
       nargs--;
       ft_func = csound->FTFindP;
       err_func = csound->PerfError;
     }
     else {
       ft_func = csound->FTnp2Find;
-      err_func = csound->InitError;
+      err_func = myInitError;
     }
 
     if (UNLIKELY(nargs <= 0))
       goto err2;
 
-    csound->strarg2name(csound, filename, p->ifilno, "ftsave.",
-                                (int) csound->GetInputArgSMask(p));
+    if(!istring) {
+      if(ISSTRCOD(*p->ifilno))
+         csound->strarg2name(csound, filename, p->ifilno, "ftsave.",
+                               0);
+         else strncpy(filename, get_arg_string(csound,*p->ifilno), MAXNAME-1);
+
+    } else {
+      strncpy(filename, ((STRINGDAT *)p->ifilno)->data, MAXNAME-1);
+    }
+
     if (*p->iflag <= FL(0.0)) {
       fd = csound->FileOpen2(csound, &file, CSFILE_STD, filename, "wb",
                                "", CSFTYPE_FTABLES_BINARY, 0);
@@ -465,16 +535,26 @@ static int ftsave(CSOUND *csound, FTLOAD *p)
     return OK;
  err:
     csound->FileClose(csound, fd);
-    return err_func(csound, Str("ftsave: Bad table number. Saving is possible "
-                                "only for existing tables."));
+    return err_func(csound, p->h.insdshead,
+                    Str("ftsave: Bad table number. Saving is possible "
+                        "only for existing tables."));
  err2:
-    return err_func(csound, Str("ftsave: no table numbers"));
+    return err_func(csound, p->h.insdshead, Str("ftsave: no table numbers"));
  err3:
-    return err_func(csound, Str("ftsave: unable to open file"));
+    return err_func(csound, p->h.insdshead, Str("ftsave: unable to open file"));
  err4:
-    return err_func(csound, Str("ftsave: failed to write file"));
+    return err_func(csound, p->h.insdshead, Str("ftsave: failed to write file"));
 }
 
+static int ftsave(CSOUND *csound, FTLOAD *p){
+  return ftsave_(csound,p,0);
+}
+
+static int ftsave_S(CSOUND *csound, FTLOAD *p){
+  return ftsave_(csound,p,1);
+}
+
+
 static int ftsave_k_set(CSOUND *csound, FTLOAD_K *p)
 {
     memcpy(&(p->p.h), &(p->h), sizeof(OPDS));
@@ -488,22 +568,39 @@ static int ftsave_k_set(CSOUND *csound, FTLOAD_K *p)
 static int ftsave_k(CSOUND *csound, FTLOAD_K *p)
 {
     if (*p->ktrig != FL(0.0))
-      return ftsave(csound, &(p->p));
+      return ftsave_(csound, &(p->p), 0);
+    return OK;
+}
+
+static int ftsave_kS(CSOUND *csound, FTLOAD_K *p)
+{
+    if (*p->ktrig != FL(0.0))
+      return ftsave_(csound, &(p->p), 1);
     return OK;
 }
 
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-  { "ftgen",    S(FTGEN),     TW|1,  "i",  "iiiTTm", (SUBR) ftgen, NULL, NULL    },
-  { "ftgentmp", S(FTGEN),     TW|1,  "i",  "iiiiTm", (SUBR) ftgentmp, NULL, NULL },
-  { "ftfree",   S(FTFREE),    TW|1,  "",   "ii",     (SUBR) ftfree, NULL, NULL   },
-  { "ftsave",   S(FTLOAD),    TR|1,  "",   "Tim",    (SUBR) ftsave, NULL, NULL   },
-  { "ftload",   S(FTLOAD),    TR|1,  "",   "Tim",    (SUBR) ftload, NULL, NULL   },
-  { "ftsavek",  S(FTLOAD_K),  TW|3,  "",   "Tkim",   (SUBR) ftsave_k_set,
+  { "ftgen",    S(FTGEN),     TW, 1,  "i",  "iiiiim", (SUBR) ftgen, NULL, NULL    },
+  { "ftgen.S",    S(FTGEN),   TW, 1,  "i",  "iiiSim", (SUBR) ftgen_S, NULL, NULL  },
+  { "ftgen.iS",    S(FTGEN),  TW, 1,  "i",  "iiiiSm", (SUBR) ftgen_iS, NULL, NULL },
+  { "ftgen.SS",    S(FTGEN),  TW, 1,  "i",  "iiiSSm", (SUBR) ftgen_SS, NULL, NULL },
+  { "ftgentmp", S(FTGEN),     TW, 1,  "i",  "iiiiim", (SUBR) ftgentmp, NULL, NULL },
+  { "ftgentmp.S", S(FTGEN),   TW, 1,  "i",  "iiiiSm", (SUBR) ftgentmp, NULL, NULL },
+  { "ftfree",   S(FTFREE),    TW, 1,  "",   "ii",     (SUBR) ftfree, NULL, NULL   },
+  { "ftsave",   S(FTLOAD),    TR, 1,  "",   "iim",    (SUBR) ftsave, NULL, NULL   },
+  { "ftsave.S",   S(FTLOAD),  TR, 1,  "",   "Sim",    (SUBR) ftsave_S, NULL, NULL },
+  { "ftload",   S(FTLOAD),    TR, 1,  "",   "iim",    (SUBR) ftload, NULL, NULL   },
+    { "ftload.S",  S(FTLOAD), TR, 1,  "",   "Sim",    (SUBR) ftload_S, NULL, NULL },
+  { "ftsavek",  S(FTLOAD_K),  TW, 3,  "",   "ikim",   (SUBR) ftsave_k_set,
                                                   (SUBR) ftsave_k, NULL       },
-  { "ftloadk",  S(FTLOAD_K),  TW|3,  "",   "Tkim",   (SUBR) ftsave_k_set,
-                                                  (SUBR) ftload_k, NULL       }
+  { "ftsavek.S",  S(FTLOAD_K),  TW, 3,  "",   "Skim",   (SUBR) ftsave_k_set,
+                                                  (SUBR) ftsave_kS, NULL       },
+  { "ftloadk",  S(FTLOAD_K),  TW, 3,  "",   "ikim",   (SUBR) ftsave_k_set,
+    (SUBR) ftload_k, NULL       },
+ { "ftloadk.S",  S(FTLOAD_K),  TW, 3,  "",   "Skim",   (SUBR) ftsave_k_set,
+                                                  (SUBR) ftload_kS, NULL       }
 };
 
 int ftgen_init_(CSOUND *csound)
@@ -511,4 +608,3 @@ int ftgen_init_(CSOUND *csound)
     return csound->AppendOpcodes(csound, &(localops[0]),
                                  (int) (sizeof(localops) / sizeof(OENTRY)));
 }
-
