@@ -1,3 +1,4 @@
+
 /*  Copyright (C) 2002-2004 Gabriel Maldonado */
 
 /*  The gab library is free software; you can redistribute it */
@@ -27,7 +28,7 @@
 /*Check csystem, exitnow */
 /*Check other opcodes commented out in Oentry */
 
-#include "csdl.h"
+#include "stdopcod.h"
 #include "gab.h"
 #include <math.h>
 #include "interlocks.h"
@@ -40,7 +41,7 @@ static int krsnsetx(CSOUND *csound, KRESONX *p)
     if ((p->loop = MYFLT2LRND(*p->ord)) < 1)
       p->loop = 4; /*default value*/
     if (!*p->istor && (p->aux.auxp == NULL ||
-                       (int)(p->loop*2*sizeof(MYFLT)) > p->aux.size))
+                       (unsigned int)(p->loop*2*sizeof(MYFLT)) > p->aux.size))
       csound->AuxAlloc(csound, (long)(p->loop*2*sizeof(MYFLT)), &p->aux);
     p->yt1 = (MYFLT*)p->aux.auxp; p->yt2 = (MYFLT*)p->aux.auxp + p->loop;
     if (scale && scale != 1 && scale != 2) {
@@ -66,12 +67,12 @@ static int kresonx(CSOUND *csound, KRESONX *p) /* Gabriel Maldonado, modified */
 
     if (*p->kcf != p->prvcf) {
       p->prvcf = *p->kcf;
-      p->cosf = COS(*p->kcf * csound->tpidsr * csound->ksmps);
+      p->cosf = COS(*p->kcf * csound->tpidsr * CS_KSMPS);
       flag = 1;
     }
     if (*p->kbw != p->prvbw) {
       p->prvbw = *p->kbw;
-      p->c3 = EXP(*p->kbw * csound->mtpdsr * csound->ksmps);
+      p->c3 = EXP(*p->kbw * csound->mtpdsr * CS_KSMPS);
       flag = 1;
     }
     if (flag) {
@@ -122,25 +123,29 @@ static int fastab_set(CSOUND *csound, FASTAB *p)
 
 static int fastabw(CSOUND *csound, FASTAB *p)
 {
-    int nsmps = csound->ksmps, n;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *tab = p->table;
     MYFLT *rslt = p->rslt, *ndx = p->xndx;
+
+    if (UNLIKELY(early)) nsmps -= early;
     if (p->xmode) {
       MYFLT xbmul = p->xbmul;   /* load once */
-      for (n=0; n<nsmps; n++)  { /* for loops compile better */
+      for (n=offset; n<nsmps; n++)  { /* for loops compile better */
         int i = (int)(ndx[n]*xbmul);
         if (UNLIKELY(i > p->tablen || i<0)) {
           csound->Message(csound, "ndx: %f \n", ndx[n]);
-          return csound->PerfError(csound, Str("tabw off end"));
+          return csound->PerfError(csound, p->h.insdshead, Str("tabw off end"));
         }
         tab[i] = rslt[n];
       }
     }
     else {
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         int i = ndx[n];
         if (UNLIKELY(i > p->tablen || i<0)) {
-          return csound->PerfError(csound, Str("tabw off end"));
+          return csound->PerfError(csound, p->h.insdshead, Str("tabw off end"));
         }
         tab[i] = rslt[n];
       }
@@ -156,7 +161,7 @@ static int fastabk(CSOUND *csound, FASTAB *p)
     else
       i = (int) *p->xndx;
     if (UNLIKELY(i > p->tablen || i<0)) {
-      return csound->PerfError(csound, Str("tab off end"));
+      return csound->PerfError(csound, p->h.insdshead, Str("tab off end"));
     }
     *p->rslt =  p->table[i];
     return OK;
@@ -170,7 +175,7 @@ static int fastabkw(CSOUND *csound, FASTAB *p)
     else
       i = (int) *p->xndx;
     if (UNLIKELY(i > p->tablen || i<0)) {
-      return csound->PerfError(csound, Str("tabw off end"));
+      return csound->PerfError(csound, p->h.insdshead, Str("tabw off end"));
     }
     p->table[i] = *p->rslt;
     return OK;
@@ -179,17 +184,19 @@ static int fastabkw(CSOUND *csound, FASTAB *p)
 static int fastabi(CSOUND *csound, FASTAB *p)
 {
     FUNC *ftp;
-    int i;
+    int32 i;
 
     if (UNLIKELY((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL)) {
       return csound->InitError(csound, Str("tab_i: incorrect table number"));
     }
     if (*p->ixmode)
-      i = (int) (*p->xndx * ftp->flen);
+      i = (int32) (*p->xndx * ftp->flen);
     else
-      i = (int) *p->xndx;
-    if (UNLIKELY(i >= ftp->flen || i<0)) {
-      return csound->PerfError(csound, Str("tab_i off end: table number: %d\n"), (int) *p->xfn);
+      i = (int32) *p->xndx;
+    if (UNLIKELY(i >= (int32)ftp->flen || i<0)) {
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("tab_i off end: table number: %d\n"),
+                               (int) *p->xfn);
     }
     *p->rslt =  ftp->ftable[i];
     return OK;
@@ -198,17 +205,17 @@ static int fastabi(CSOUND *csound, FASTAB *p)
 static int fastabiw(CSOUND *csound, FASTAB *p)
 {
     FUNC *ftp;
-    int i;
+    int32 i;
     /*ftp = csound->FTFind(p->xfn); */
     if ((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL) {
       return csound->InitError(csound, Str("tabw_i: incorrect table number"));
     }
     if (*p->ixmode)
-      i = (int) (*p->xndx * ftp->flen);
+      i = (int32) (*p->xndx * ftp->flen);
     else
-      i = (int) *p->xndx;
-    if (UNLIKELY(i >= ftp->flen || i<0)) {
-        return csound->PerfError(csound, Str("tabw_i off end"));
+      i = (int32) *p->xndx;
+    if (UNLIKELY(i >= (int32)ftp->flen || i<0)) {
+        return csound->PerfError(csound, p->h.insdshead, Str("tabw_i off end"));
     }
     ftp->ftable[i] = *p->rslt;
     return OK;
@@ -216,24 +223,31 @@ static int fastabiw(CSOUND *csound, FASTAB *p)
 
 static int fastab(CSOUND *csound, FASTAB *p)
 {
-    int   i = 0, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     MYFLT *tab = p->table;
     MYFLT *rslt = p->rslt, *ndx = p->xndx;
+    if (UNLIKELY(offset)) memset(rslt, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rslt[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->xmode) {
       MYFLT xbmul = p->xbmul;
-      for (i=0; i<nsmps; i++) {
+      for (i=offset; i<nsmps; i++) {
         int n = (int) (ndx[i] * xbmul);
         if (UNLIKELY(n > p->tablen || n<0)) {
-          return csound->PerfError(csound, Str("tab off end"));
+          return csound->PerfError(csound, p->h.insdshead, Str("tab off end"));
         }
         rslt[i] = tab[n];
       }
     }
     else {
-      for (i=0; i<nsmps; i++) {
+      for (i=offset; i<nsmps; i++) {
         int n = (int) ndx[i];
         if (UNLIKELY(n > p->tablen || n<0)) {
-          return csound->PerfError(csound, Str("tab off end"));
+          return csound->PerfError(csound, p->h.insdshead, Str("tab off end"));
         }
         rslt[i] = tab[n];
       }
@@ -245,7 +259,7 @@ static CS_NOINLINE int tab_init(CSOUND *csound, TB_INIT *p, int ndx)
 {
     MYFLT             *ft;
     STDOPCOD_GLOBALS  *pp;
-    if (UNLIKELY(csound->GetTable(csound, &ft, MYFLT2LRND(*p->ifn)) < 0))
+    if (UNLIKELY(csoundGetTable(csound, &ft, MYFLT2LRND(*p->ifn)) < 0))
       return csound->InitError(csound, Str("tab_init: incorrect table number"));
     pp = (STDOPCOD_GLOBALS*) csound->stdOp_Env;
     pp->tb_ptrs[ndx] = ft;
@@ -346,7 +360,9 @@ static int nlalp_set(CSOUND *csound, NLALP *p)
 
 static int nlalp(CSOUND *csound, NLALP *p)
 {
-    int nsmps = csound->ksmps, n;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *rp;
     MYFLT *ip;
     double m0;
@@ -362,11 +378,16 @@ static int nlalp(CSOUND *csound, NLALP *p)
     knfact = (double)*p->knfact;
     tm0 = p->m0;
     tm1 = p->m1;
-    if (knfact == 0.0) { /* linear case */
+    if (UNLIKELY(offset)) memset(rp, '\0', offset*sizeof(MYFLT));
+     if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rp[nsmps], '\0', early*sizeof(MYFLT));
+    }
+   if (knfact == 0.0) { /* linear case */
       if (klfact == 0.0) { /* degenerated linear case */
         m0 = (double)ip[0] - tm1;
-        rp[0] = (MYFLT)(tm0);
-        for (n=1; n<nsmps; n++) {
+        rp[offset] = (MYFLT)(tm0);
+        for (n=offset+1; n<nsmps; n++) {
           rp[n] = (MYFLT)(m0);
           m0 = (double)ip[n];
         }
@@ -374,7 +395,7 @@ static int nlalp(CSOUND *csound, NLALP *p)
         tm0 = m0;
       }
       else { /* normal linear case */
-        for (n=0; n<nsmps; n++) {
+        for (n=offset; n<nsmps; n++) {
           m0 = (double)ip[n] - tm1;
           m1 = m0 * klfact;
           rp[n] = (MYFLT)(tm0 + m1);
@@ -384,7 +405,7 @@ static int nlalp(CSOUND *csound, NLALP *p)
       }
     } else { /* non-linear case */
       if (klfact == 0.0) { /* simplified non-linear case */
-        for (n=0; n<nsmps; n++) {
+        for (n=offset; n<nsmps; n++) {
           m0 = (double)ip[n] - tm1;
           m1 = fabs(m0) * knfact;
           rp[n] = (MYFLT)(tm0 + m1);
@@ -392,7 +413,7 @@ static int nlalp(CSOUND *csound, NLALP *p)
           tm0 = m0;
         }
       } else { /* normal non-linear case */
-        for (n=0; n<nsmps; n++) {
+        for (n=offset; n<nsmps; n++) {
           m0 = (double)ip[n] - tm1;
           m1 = m0 * klfact + fabs(m0) * knfact;
           rp[n] = (MYFLT)(tm0 + m1);
@@ -411,13 +432,13 @@ static int nlalp(CSOUND *csound, NLALP *p)
 static int adsynt2_set(CSOUND *csound,ADSYNT2 *p)
 {
     FUNC    *ftp;
-    int     count;
+    unsigned int     count;
     int32   *lphs;
     MYFLT   iphs = *p->iphs;
 
     p->inerr = 0;
 
-    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL)) {
+    if (LIKELY((ftp = csound->FTFind(csound, p->ifn)) != NULL)) {
       p->ftp = ftp;
     }
     else {
@@ -425,7 +446,7 @@ static int adsynt2_set(CSOUND *csound,ADSYNT2 *p)
       return csound->InitError(csound, Str("adsynt2: wavetable not found!"));
     }
 
-    count = (int)*p->icnt;
+    count = (unsigned int)*p->icnt;
     if (UNLIKELY(count < 1)) count = 1;
     p->count = count;
 
@@ -458,12 +479,12 @@ static int adsynt2_set(CSOUND *csound,ADSYNT2 *p)
     }
 
     if (p->lphs.auxp==NULL ||
-        p->lphs.size < (int32)(sizeof(int32)*count))
+        p->lphs.size < sizeof(int32)*count)
       csound->AuxAlloc(csound, sizeof(int32)*count, &p->lphs);
     lphs = (int32*)p->lphs.auxp;
 
     if (iphs > 1) {
-      int c;
+      unsigned int c;
       for (c=0; c<count; c++) {
         lphs[c] = ((int32)
                    ((MYFLT) ((double) (csound->Rand31(&(csound->randSeed1)) - 1)
@@ -471,13 +492,13 @@ static int adsynt2_set(CSOUND *csound,ADSYNT2 *p)
       }
     }
     else if (iphs >= 0) {
-      int c;
+      unsigned int c;
       for (c=0; c<count; c++) {
         lphs[c] = ((int32)(iphs * FMAXLEN)) & PHMASK;
       }
     }
     if (p->pamp.auxp==NULL ||
-        p->pamp.size < (int32)(sizeof(MYFLT)*p->count))
+        p->pamp.size < (uint32_t)(sizeof(MYFLT)*p->count))
       csound->AuxAlloc(csound, sizeof(MYFLT)*p->count, &p->pamp);
     else                        /* AuxAlloc clear anyway */
       memset(p->pamp.auxp, 0, sizeof(MYFLT)*p->count);
@@ -495,7 +516,10 @@ static int adsynt2(CSOUND *csound,ADSYNT2 *p)
     MYFLT   amp0, amp, cps0, cps, ampIncr, amp2;
     int32   phs, inc, lobits;
     int32   *lphs;
-    int     c, n, nsmps= csound->ksmps, count;
+    int     c, count;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (UNLIKELY(p->inerr)) {
       return csound->InitError(csound, Str("adsynt2: not initialised"));
@@ -515,7 +539,11 @@ static int adsynt2(CSOUND *csound,ADSYNT2 *p)
     count = p->count;
 
     ar = p->sr;
-    memset(ar, 0, nsmps*sizeof(MYFLT));
+    if (UNLIKELY(offset)) memset(ar, 0, nsmps*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
 
     for (c=0; c<count; c++) {
       amp2 = prevAmp[c];
@@ -523,8 +551,8 @@ static int adsynt2(CSOUND *csound,ADSYNT2 *p)
       cps = freqtbl[c] * cps0;
       inc = (int32) (cps * csound->sicvt);
       phs = lphs[c];
-      ampIncr = (amp - amp2) * csound->onedksmps;
-      for (n=0; n<nsmps; n++) {
+      ampIncr = (amp - amp2) * CS_ONEDKSMPS;
+      for (n=offset; n<nsmps; n++) {
         ar[n] += *(ftbl + (phs >> lobits)) * amp2;
         phs += inc;
         phs &= PHMASK;
@@ -543,27 +571,8 @@ static int exitnow(CSOUND *csound, EXITNOW *p)
     return OK;  /* compiler only */
 }
 
-/*-----zak opcodes */
-/* int zread(CSOUND *csound,ZKR *p) */
-/* { */
-/*      *p->rslt = zkstart[(long) *p->ndx]; */
-/*      return OK; */
-/* } */
-
-/*void a_k_set(INDIFF *p)
-{
-        p->prev = FL(0.0);
-}*/
-
 static int tabrec_set(CSOUND *csound,TABREC *p)
 {
-    /*FUNC *ftp; */
-    /*if ((ftp = csound->FTFind(p->ifn)) == NULL) { */
-    /*  csound->InitError(csound, Str("tabrec: incorrect table number")); */
-    /*  return; */
-    /*} */
-    /*p->table = ftp->ftable; */
-    /*p->tablen = ftp->flen; */
     p->recording = 0;
     p->currtic = 0;
     p->ndx = 0;
@@ -576,8 +585,9 @@ static int tabrec_k(CSOUND *csound,TABREC *p)
     if (*p->ktrig_start) {
       if (*p->kfn != p->old_fn) {
         int flen;
-        if ((flen = csound->GetTable(csound, &(p->table), (int) *p->kfn)) < 0)
-          return csound->PerfError(csound, Str("Invalid ftable no. %f"), *p->kfn);
+        if ((flen = csoundGetTable(csound, &(p->table), (int) *p->kfn)) < 0)
+          return csound->PerfError(csound, p->h.insdshead,
+                                   Str("Invalid ftable no. %f"), *p->kfn);
         p->tablen = (long) flen;
         *(p->table++) = *p->numtics;
         p->old_fn = *p->kfn;
@@ -630,8 +640,9 @@ static int tabplay_k(CSOUND *csound,TABPLAY *p)
     if (*p->ktrig) {
       if (*p->kfn != p->old_fn) {
         int flen;
-        if ((flen = csound->GetTable(csound, &(p->table), (int) *p->kfn)) < 0)
-          return csound->PerfError(csound, Str("Invalid ftable no. %f"), *p->kfn);
+        if ((flen = csoundGetTable(csound, &(p->table), (int) *p->kfn)) < 0)
+          return csound->PerfError(csound, p->h.insdshead,
+                                   Str("Invalid ftable no. %f"), *p->kfn);
         p->tablen = (long) flen;
         p->currtic = 0;
         p->ndx = 0;
@@ -707,25 +718,29 @@ static int partial_maximum_set(CSOUND *csound,P_MAXIMUM *p)
 
 static int partial_maximum(CSOUND *csound,P_MAXIMUM *p)
 {
-    int n, nsmps = csound->ksmps, flag = (int) *p->imaxflag;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int flag = (int) *p->imaxflag;
     MYFLT *a = p->asig;
     MYFLT max = p->max;
+    if (UNLIKELY(early)) nsmps -= early;
     switch(flag) {
     case 1: /* absolute maximum */
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         MYFLT temp;
         if ((temp = FABS(a[n])) > max) max = temp;
       }
       if (max > p->max) p->max = max;
       break;
     case 2: /* actual maximum */
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         if (a[n] > max) max = a[n];
       }
       if (max > p->max) p->max = max;
       break;
     case 3: /* actual minimum */
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         if (a[n] < max) max = a[n];
       }
       if (max < p->max) p->max = max;
@@ -733,14 +748,15 @@ static int partial_maximum(CSOUND *csound,P_MAXIMUM *p)
     case 4: { /* average */
         MYFLT temp = FL(0.0);
         p->counter += nsmps;
-        for (n=0; n<nsmps; n++) {
+        for (n=offset; n<nsmps; n++) {
           temp += a[n];
         }
         p->max += temp;
       }
       break;
     default:
-      return csound->PerfError(csound, Str("max_k: invalid imaxflag value"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("max_k: invalid imaxflag value"));
     }
     if (*p->ktrig) {
       if (flag == 4) {
@@ -793,86 +809,88 @@ static int mandel(CSOUND *csound,MANDEL *p)
 #define S(x)    sizeof(x)
 
 OENTRY gab_localops[] = {
-  {"resonxk", S(KRESONX),    3,   "k",    "kkkooo",
+  {"resonxk", S(KRESONX),    0, 3,   "k",    "kkkooo",
                             (SUBR) krsnsetx, (SUBR) kresonx, NULL },
-  { "tab_i",S(FASTAB),       TR|1,   "i",    "iio", (SUBR) fastabi, NULL, NULL },
-  { "tab",S(FASTAB),         TR|7,   "s",    "xio",
-                            (SUBR) fastab_set, (SUBR)fastabk, (SUBR) fastab },
-  { "tabw_i",S(FASTAB),      TW|1,   "",    "iiio", (SUBR) fastabiw, NULL, NULL },
-  { "tabw",S(FASTAB),        TW|7,   "",    "xxio",
+  { "tab_i",S(FASTAB),       TR, 1,   "i",    "iio", (SUBR) fastabi, NULL, NULL },
+  { "tab",S(FASTAB),         TR, 5,   "a",    "xio",
+                            (SUBR) fastab_set, (SUBR)NULL, (SUBR) fastab },
+  { "tab.k",S(FASTAB),         TR, 3,   "k",    "kio",
+                            (SUBR) fastab_set, (SUBR)fastabk, NULL },
+  { "tabw_i",S(FASTAB),      TW, 1,   "",    "iiio", (SUBR) fastabiw, NULL, NULL },
+  { "tabw",S(FASTAB),        TW, 7,   "",    "xxio",
                             (SUBR)fastab_set, (SUBR)fastabkw, (SUBR)fastabw },
-  { "tb0_init", S(TB_INIT),  TR|1,   "",      "i",    (SUBR)tab0_init},
-  { "tb1_init", S(TB_INIT),  TR|1,   "",      "i",    (SUBR)tab1_init},
-  { "tb2_init", S(TB_INIT),  TR|1,   "",      "i",    (SUBR)tab2_init},
-  { "tb3_init", S(TB_INIT),  TR|1,   "",      "i",    (SUBR)tab3_init},
-  { "tb4_init", S(TB_INIT),  TR|1,   "",      "i",    (SUBR)tab4_init},
-  { "tb5_init", S(TB_INIT),  TR|1,   "",      "i",    (SUBR)tab5_init},
-  { "tb6_init", S(TB_INIT),  TR|1,   "",      "i",    (SUBR)tab6_init},
-  { "tb7_init", S(TB_INIT),  TR|1,   "",      "i",    (SUBR)tab7_init},
-  { "tb8_init", S(TB_INIT),  TR|1,   "",      "i",    (SUBR)tab8_init},
-  { "tb9_init", S(TB_INIT),  TR|1,   "",      "i",    (SUBR)tab9_init},
-  { "tb10_init", S(TB_INIT), TR|1,   "",      "i",    (SUBR)tab10_init},
-  { "tb11_init", S(TB_INIT), TR|1,   "",      "i",    (SUBR)tab11_init},
-  { "tb12_init", S(TB_INIT), TR|1,   "",      "i",    (SUBR)tab12_init},
-  { "tb13_init", S(TB_INIT), TR|1,   "",      "i",    (SUBR)tab13_init},
-  { "tb14_init", S(TB_INIT), TR|1,   "",      "i",    (SUBR)tab14_init},
-  { "tb15_init", S(TB_INIT), TR|1,   "",      "i",    (SUBR)tab15_init},
-  { "tb0.k",  S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab0_k_tmp,  NULL  },
-  { "tb1.k",  S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab1_k_tmp,  NULL  },
-  { "tb2.k",  S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab2_k_tmp,  NULL  },
-  { "tb3.k",  S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab3_k_tmp,  NULL  },
-  { "tb4.k",  S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab4_k_tmp,  NULL  },
-  { "tb5.k",  S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab5_k_tmp,  NULL  },
-  { "tb6.k",  S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab6_k_tmp,  NULL  },
-  { "tb7.k",  S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab7_k_tmp,  NULL  },
-  { "tb8.k",  S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab8_k_tmp,  NULL  },
-  { "tb9.k",  S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab9_k_tmp,  NULL  },
-  { "tb10.k", S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab10_k_tmp, NULL  },
-  { "tb11.k", S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab11_k_tmp, NULL  },
-  { "tb12.k", S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab12_k_tmp, NULL  },
-  { "tb13.k", S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab13_k_tmp, NULL  },
-  { "tb14.k", S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab14_k_tmp, NULL  },
-  { "tb15.k", S(FASTB), 2,  "k",    "k",    NULL, (SUBR) tab15_k_tmp, NULL  },
+  { "tb0_init", S(TB_INIT),  TR, 1,   "",      "i",    (SUBR)tab0_init},
+  { "tb1_init", S(TB_INIT),  TR, 1,   "",      "i",    (SUBR)tab1_init},
+  { "tb2_init", S(TB_INIT),  TR, 1,   "",      "i",    (SUBR)tab2_init},
+  { "tb3_init", S(TB_INIT),  TR, 1,   "",      "i",    (SUBR)tab3_init},
+  { "tb4_init", S(TB_INIT),  TR, 1,   "",      "i",    (SUBR)tab4_init},
+  { "tb5_init", S(TB_INIT),  TR, 1,   "",      "i",    (SUBR)tab5_init},
+  { "tb6_init", S(TB_INIT),  TR, 1,   "",      "i",    (SUBR)tab6_init},
+  { "tb7_init", S(TB_INIT),  TR, 1,   "",      "i",    (SUBR)tab7_init},
+  { "tb8_init", S(TB_INIT),  TR, 1,   "",      "i",    (SUBR)tab8_init},
+  { "tb9_init", S(TB_INIT),  TR, 1,   "",      "i",    (SUBR)tab9_init},
+  { "tb10_init", S(TB_INIT), TR, 1,   "",      "i",    (SUBR)tab10_init},
+  { "tb11_init", S(TB_INIT), TR, 1,   "",      "i",    (SUBR)tab11_init},
+  { "tb12_init", S(TB_INIT), TR, 1,   "",      "i",    (SUBR)tab12_init},
+  { "tb13_init", S(TB_INIT), TR, 1,   "",      "i",    (SUBR)tab13_init},
+  { "tb14_init", S(TB_INIT), TR, 1,   "",      "i",    (SUBR)tab14_init},
+  { "tb15_init", S(TB_INIT), TR, 1,   "",      "i",    (SUBR)tab15_init},
+  { "tb0.k",  S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab0_k_tmp,  NULL  },
+  { "tb1.k",  S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab1_k_tmp,  NULL  },
+  { "tb2.k",  S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab2_k_tmp,  NULL  },
+  { "tb3.k",  S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab3_k_tmp,  NULL  },
+  { "tb4.k",  S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab4_k_tmp,  NULL  },
+  { "tb5.k",  S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab5_k_tmp,  NULL  },
+  { "tb6.k",  S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab6_k_tmp,  NULL  },
+  { "tb7.k",  S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab7_k_tmp,  NULL  },
+  { "tb8.k",  S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab8_k_tmp,  NULL  },
+  { "tb9.k",  S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab9_k_tmp,  NULL  },
+  { "tb10.k", S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab10_k_tmp, NULL  },
+  { "tb11.k", S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab11_k_tmp, NULL  },
+  { "tb12.k", S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab12_k_tmp, NULL  },
+  { "tb13.k", S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab13_k_tmp, NULL  },
+  { "tb14.k", S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab14_k_tmp, NULL  },
+  { "tb15.k", S(FASTB), 0, 2,  "k",    "k",    NULL, (SUBR) tab15_k_tmp, NULL  },
   /* tbx_t (t-rate version removed here) */
-  { "tb0.i",      S(FASTB), 1,    "i",     "i",    (SUBR) tab0_i_tmp    },
-  { "tb1.i",      S(FASTB), 1,    "i",     "i",    (SUBR) tab1_i_tmp    },
-  { "tb2.i",      S(FASTB), 1,    "i",     "i",    (SUBR) tab2_i_tmp    },
-  { "tb3.i",      S(FASTB), 1,    "i",     "i",    (SUBR) tab3_i_tmp    },
-  { "tb4.i",      S(FASTB), 1,    "i",     "i",    (SUBR) tab4_i_tmp    },
-  { "tb5.i",      S(FASTB), 1,    "i",     "i",    (SUBR) tab5_i_tmp    },
-  { "tb6.i",      S(FASTB), 1,    "i",     "i",    (SUBR) tab6_i_tmp    },
-  { "tb7.i",      S(FASTB), 1,    "i",     "i",    (SUBR) tab7_i_tmp    },
-  { "tb8.i",      S(FASTB), 1,    "i",     "i",    (SUBR) tab8_i_tmp    },
-  { "tb9.i",      S(FASTB), 1,    "i",     "i",    (SUBR) tab9_i_tmp    },
-  { "tb10.i",     S(FASTB), 1,    "i",     "i",    (SUBR) tab10_i_tmp   },
-  { "tb11.i",     S(FASTB), 1,    "i",     "i",    (SUBR) tab11_i_tmp   },
-  { "tb12.i",     S(FASTB), 1,    "i",     "i",    (SUBR) tab12_i_tmp   },
-  { "tb13.i",     S(FASTB), 1,    "i",     "i",    (SUBR) tab13_i_tmp   },
-  { "tb14.i",     S(FASTB), 1,    "i",     "i",    (SUBR) tab14_i_tmp   },
-  { "tb15.i",     S(FASTB), 1,    "i",     "i",    (SUBR) tab15_i_tmp   },
-  { "nlalp",      S(NLALP), 5,    "a",     "akkoo",
+  { "tb0.i",      S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab0_i_tmp    },
+  { "tb1.i",      S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab1_i_tmp    },
+  { "tb2.i",      S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab2_i_tmp    },
+  { "tb3.i",      S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab3_i_tmp    },
+  { "tb4.i",      S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab4_i_tmp    },
+  { "tb5.i",      S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab5_i_tmp    },
+  { "tb6.i",      S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab6_i_tmp    },
+  { "tb7.i",      S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab7_i_tmp    },
+  { "tb8.i",      S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab8_i_tmp    },
+  { "tb9.i",      S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab9_i_tmp    },
+  { "tb10.i",     S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab10_i_tmp   },
+  { "tb11.i",     S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab11_i_tmp   },
+  { "tb12.i",     S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab12_i_tmp   },
+  { "tb13.i",     S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab13_i_tmp   },
+  { "tb14.i",     S(FASTB), 0, 1,    "i",     "i",    (SUBR) tab14_i_tmp   },
+  { "tb15.i",     S(FASTB),0,  1,    "i",     "i",    (SUBR) tab15_i_tmp   },
+  { "nlalp",      S(NLALP), 0, 5,    "a",     "akkoo",
                             (SUBR) nlalp_set, NULL, (SUBR) nlalp   },
-  { "adsynt2",S(ADSYNT2),TR|5,    "a",     "kkiiiio",
+  { "adsynt2",S(ADSYNT2),TR, 5,    "a",     "kkiiiio",
                             (SUBR) adsynt2_set, NULL, (SUBR)adsynt2 },
-  { "exitnow",S(EXITNOW),   1,    "",  "", (SUBR) exitnow, NULL, NULL },
-/* { "zr_i",  S(ZKR),     1,  "i",  "i",  (SUBR)zread, NULL, NULL}, */
-/* { "zr_k",  S(ZKR),     2,  "k",  "k",  NULL, (SUBR)zread, NULL}, */
-/* { "zr_a",  S(ZAR),     5,  "a",  "a",  (SUBR)zaset, NULL, (SUBR)zar}, */
-/* { "k_i",   S(ASSIGN),  1,  "k",  "i",  (SUBR)assign}, */
-/* { "k_t",   S(ASSIGN),  2,  "k",  "t",  NULL, (SUBR)assign}, */
-/* { "a_k",   S(INDIFF),  5,  "a",  "k",  (SUBR)a_k_set,NULL, (SUBR)interp }, */
-  { "tabrec",   S(TABREC),  3,     "",      "kkkkz",
+  { "exitnow",S(EXITNOW),   0, 1,    "",  "", (SUBR) exitnow, NULL, NULL },
+/* { "zr_i",  S(ZKR),     0, 1,  "i",  "i",  (SUBR)zread, NULL, NULL}, */
+/* { "zr_k",  S(ZKR),     0, 2,  "k",  "k",  NULL, (SUBR)zread, NULL}, */
+/* { "zr_a",  S(ZAR),     0, 5,  "a",  "a",  (SUBR)zaset, NULL, (SUBR)zar}, */
+/* { "k_i",   S(ASSIGN),  0, 1,  "k",  "i",  (SUBR)assign}, */
+/* { "k_t",   S(ASSIGN),  0, 2,  "k",  "t",  NULL, (SUBR)assign}, */
+/* { "a_k",   S(INDIFF),  0, 5,  "a",  "k",  (SUBR)a_k_set,NULL, (SUBR)interp }, */
+  { "tabrec",   S(TABREC),  0, 3,     "",      "kkkkz",
                             (SUBR) tabrec_set, (SUBR) tabrec_k, NULL },
-  { "tabplay",  S(TABPLAY), TR|3,     "",      "kkkz",
+  { "tabplay",  S(TABPLAY), TR, 3,     "",      "kkkz",
                             (SUBR) tabplay_set, (SUBR) tabplay_k, NULL },
-  { "changed", S(ISCHANGED), 3,     "k",     "z",
+  { "changed", S(ISCHANGED), 0, 3,     "k",     "z",
                             (SUBR) isChanged_set, (SUBR)isChanged, NULL },
-  /*{ "ftlen_k",S(EVAL),    2,      "k",    "k", NULL,      (SUBR)ftlen   }, */
-  { "max_k",  S(P_MAXIMUM), 5,      "k",    "aki",
+  /*{ "ftlen_k",S(EVAL),    0, 2,      "k",    "k", NULL,      (SUBR)ftlen   }, */
+  { "max_k",  S(P_MAXIMUM), 0, 5,      "k",    "aki",
             (SUBR) partial_maximum_set, (SUBR) NULL, (SUBR) partial_maximum },
-/*{ "maxk",   S(P_MAXIMUM), 5,      "k",    "aki", */
+/*{ "maxk",   S(P_MAXIMUM), 0, 5,      "k",    "aki", */
 /*        (SUBR) partial_maximum_set, (SUBR) NULL, (SUBR) partial_maximum }, */
-  { "mandel",S(MANDEL),     3,      "kk",    "kkkk",
+  { "mandel",S(MANDEL),     0, 3,      "kk",    "kkkk",
                             (SUBR) mandel_set, (SUBR) mandel, NULL }
 };
 
