@@ -22,7 +22,7 @@
     02111-1307 USA
 */
 
-#include "csdl.h"
+#include "stdopcod.h"
 
 #include "newfils.h"
 #include <math.h>
@@ -35,6 +35,8 @@ static int moogladder_init(CSOUND *csound,moogladder *p)
         p->delay[i] = 0.0;
       for (i = 0; i < 3; i++)
         p->tanhstg[i] = 0.0;
+      p->oldfreq = FL(0.0);
+      p->oldres = -FL(1.0);     /* ensure calculation on first cycle */
     }
     return OK;
 }
@@ -49,40 +51,67 @@ static int moogladder_process(CSOUND *csound,moogladder *p)
     double  *delay = p->delay;
     double  *tanhstg = p->tanhstg;
     double  stg[4], input;
-    double  f, fc, fc2, fc3, fcr, acr, tune;
-    double  thermal = (1.0 / 40000.0);  /* transistor thermal voltage  */
-    int     j, k, i;
+    double  acr, tune;
+#define THERMAL (0.000025) /* (1.0 / 40000.0) transistor thermal voltage  */
+    int     j, k;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
 
     if (res < 0) res = 0;
 
-    /* sr is half the actual filter sampling rate  */
-    fc =  (double)(freq/csound->esr);
-    f  =  fc/2;
-    fc2 = fc*fc;
-    fc3 = fc2*fc;
-    /* frequency & amplitude correction  */
-    fcr = 1.8730*fc3 + 0.4955*fc2 - 0.6490*fc + 0.9988;
-    acr = -3.9364*fc2 + 1.8409*fc + 0.9968;
-    tune = (1.0 - exp(-(TWOPI*f*fcr))) / thermal;   /* filter tuning  */
+    if (p->oldfreq != freq || p->oldres != res) {
+      double  f, fc, fc2, fc3, fcr;
+      p->oldfreq = freq;
+      /* sr is half the actual filter sampling rate  */
+      fc =  (double)(freq/CS_ESR);
+      f  =  0.5*fc;
+      fc2 = fc*fc;
+      fc3 = fc2*fc;
+      /* frequency & amplitude correction  */
+      fcr = 1.8730*fc3 + 0.4955*fc2 - 0.6490*fc + 0.9988;
+      acr = -3.9364*fc2 + 1.8409*fc + 0.9968;
+      tune = (1.0 - exp(-(TWOPI*f*fcr))) / THERMAL;   /* filter tuning  */
+      p->oldres = res;
+      p->oldacr = acr;
+      p->oldtune = tune;
+    }
+    else {
+      res = p->oldres;
+      acr = p->oldacr;
+      tune = p->oldtune;
+    }
     res4 = 4.0*(double)res*acr;
 
-    for (i = 0; i < csound->ksmps; i++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i = offset; i < nsmps; i++) {
       /* oversampling  */
       for (j = 0; j < 2; j++) {
         /* filter stages  */
-        for (k = 0; k < 4; k++) {
-          if (k) {
-            input = stg[k-1];
-            stg[k] = delay[k]
-                     + tune*((tanhstg[k-1] = tanh(input*thermal))
-                             - (k != 3 ? tanhstg[k] : tanh(delay[k]*thermal)));
-          }
-          else {
-            input = in[i] - res4 /*4.0*res*acr*/ *delay[5];
-            stg[k] = delay[k] + tune*(tanh(input*thermal) - tanhstg[k]);
-          }
+        input = in[i] - res4 /*4.0*res*acr*/ *delay[5];
+        delay[0] = stg[0] = delay[0] + tune*(tanh(input*THERMAL) - tanhstg[0]);
+#if 0
+        input = stg[0];
+        stg[1] = delay[1] + tune*((tanhstg[0] = tanh(input*THERMAL)) - tanhstg[1]);
+        input = delay[1] = stg[1];
+        stg[2] = delay[2] + tune*((tanhstg[1] = tanh(input*THERMAL)) - tanhstg[2]);
+        input = delay[2] = stg[2];
+        stg[3] = delay[3] + tune*((tanhstg[2] =
+                                   tanh(input*THERMAL)) - tanh(delay[3]*THERMAL));
+        delay[3] = stg[3];
+#else
+        for (k = 1; k < 4; k++) {
+          input = stg[k-1];
+          stg[k] = delay[k]
+            + tune*((tanhstg[k-1] = tanh(input*THERMAL))
+                    - (k != 3 ? tanhstg[k] : tanh(delay[k]*THERMAL)));
           delay[k] = stg[k];
         }
+#endif
         /* 1/2-sample delay for phase compensation  */
         delay[5] = (stg[3] + delay[4])*0.5;
         delay[4] = stg[3];
@@ -96,6 +125,8 @@ static int statevar_init(CSOUND *csound,statevar *p)
 {
     if (*p->istor==FL(0.0)) {
       p->bpd = p->lpd = p->lp = 0.0;
+      p->oldfreq = FL(0.0);
+      p->oldres = FL(0.0);
     }
     if (*p->osamp<=FL(0.0)) p->ostimes = 3;
     else p->ostimes = (int) *p->osamp;
@@ -115,17 +146,43 @@ static int statevar_process(CSOUND *csound,statevar *p)
     double  bpd = p->bpd;
     double  lp  = p->lp, hp = 0.0, bp = 0.0, br = 0.0;
     double  f,q,lim;
-    int ostimes = p->ostimes,i,j;
+    int ostimes = p->ostimes,j;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
 
-    f = 2.0*sin(freq*(double)csound->pidsr/ostimes);
-    q = 1.0/res;
-    lim = 0.1*((2.0 - f) / 2.)*(1./ostimes);
-    /* csound->Message(csound, "lim: %f, q: %f \n", lim, q); */
+    if (p->oldfreq != freq || p->oldres != res) {
+      f = 2.0*sin(freq*(double)csound->pidsr/ostimes);
+      q = 1.0/res;
+      lim = ((2.0 - f) *0.05)/ostimes;
+      /* csound->Message(csound, "lim: %f, q: %f \n", lim, q); */
 
-    if (q < lim) q = lim;
+      if (q < lim) q = lim;
+      p->oldq = q;
+      p->oldf = f;
+      p->oldfreq = freq;
+      p->oldres = res;
+    }
+    else {
+      q = p->oldq;
+      f = p->oldf;
+    }
 
-    for (i=0; i<csound->ksmps;i++) {
-      for (j=0;j<ostimes;j++) {
+    if (UNLIKELY(offset)) {
+      memset(outhp, '\0', offset*sizeof(MYFLT));
+      memset(outlp, '\0', offset*sizeof(MYFLT));
+      memset(outbp, '\0', offset*sizeof(MYFLT));
+      memset(outbr, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&outhp[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&outlp[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&outbp[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&outbr[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset; i<nsmps; i++) {
+      for (j=0; j<ostimes; j++) {
 
         hp = in[i] - q*bpd - lp;
         bp = hp*f + bpd;
@@ -165,23 +222,30 @@ static int fofilter_process(CSOUND *csound,fofilter *p)
     MYFLT  freq = *p->freq;
     MYFLT  ris = *p->ris;
     MYFLT  dec = *p->dec;
-    double  *delay = p->delay,ang,fsc,rad1,rad2;
+    double  *delay = p->delay,ang,fsc,rrad1,rrad2;
     double  w1,y1,w2,y2;
-    int i;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
 
     ang = (double)csound->tpidsr*freq;         /* pole angle */
     fsc = sin(ang) - 3.0;                      /* freq scl   */
-    rad1 =  pow(10.0, fsc/(dec*csound->esr));  /* filter radii */
-    rad2 =  pow(10.0, fsc/(ris*csound->esr));
+    rrad1 =  pow(10.0, fsc/(dec*CS_ESR));  /* filter radii */
+    rrad2 =  pow(10.0, fsc/(ris*CS_ESR));
 
-    for (i=0;i<csound->ksmps;i++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset;i<nsmps;i++) {
 
-      w1  = in[i] + 2.0*rad1*cos(ang)*delay[0] - rad1*rad1*delay[1];
+      w1  = in[i] + 2.0*rrad1*cos(ang)*delay[0] - rrad1*rrad1*delay[1];
       y1 =  w1 - delay[1];
       delay[1] = delay[0];
       delay[0] = w1;
 
-      w2  = in[i] + 2.0*rad2*cos(ang)*delay[2] - rad2*rad2*delay[3];
+      w2  = in[i] + 2.0*rrad2*cos(ang)*delay[2] - rrad2*rrad2*delay[3];
       y2 =  w2 - delay[3];
       delay[3] = delay[2];
       delay[2] = w2;
@@ -193,11 +257,11 @@ static int fofilter_process(CSOUND *csound,fofilter *p)
 }
 
 static OENTRY localops[] = {
-  {"moogladder", sizeof(moogladder), 5, "a", "akkp",
+  {"moogladder", sizeof(moogladder), 0, 5, "a", "akkp",
                     (SUBR) moogladder_init, NULL, (SUBR) moogladder_process },
-  {"statevar", sizeof(statevar), 5, "aaaa", "akkop",
+  {"statevar", sizeof(statevar), 0, 5, "aaaa", "akkop",
                     (SUBR) statevar_init, NULL, (SUBR) statevar_process     },
-  {"fofilter", sizeof(fofilter), 5, "a", "akkkp",
+  {"fofilter", sizeof(fofilter), 0, 5, "a", "akkkp",
                     (SUBR) fofilter_init, NULL, (SUBR) fofilter_process     }
 };
 
@@ -206,4 +270,3 @@ int newfils_init_(CSOUND *csound)
     return csound->AppendOpcodes(csound, &(localops[0]),
                                  (int) (sizeof(localops) / sizeof(OENTRY)));
 }
-
