@@ -51,54 +51,70 @@ static int pan2run(CSOUND *csound, PAN2 *p)
     int type = p->type;
     MYFLT *ain = p->asig;
     MYFLT *al = p->aleft, *ar = p->aright;
-    int n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+
+    if (UNLIKELY(offset)) {
+      memset(ar, '\0', offset*sizeof(MYFLT));
+      memset(al, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&al[nsmps], '\0', early*sizeof(MYFLT));
+    }
     switch (type) {
-    case 0: {
-      MYFLT kangl = PI_F*FL(0.5) * *p->pan;
-      for (n=0; n<nsmps; n++) {
-        if (XINARG2) kangl = PI_F*FL(0.5) * p->pan[n];
-        ar[n] = ain[n] * SIN(kangl);
+    case 0:
+      {
+        MYFLT kangl = PI_F*FL(0.5) * *p->pan;
+        for (n=offset; n<nsmps; n++) {
+          if (XINARG2) kangl = PI_F*FL(0.5) * p->pan[n];
+          ar[n] = ain[n] * SIN(kangl);
         al[n] = ain[n] * COS(kangl);
+        }
+        break;
       }
-      break;
-    }
-    case 1: {
-      MYFLT kangl = *p->pan;
-      for (n=0; n<nsmps; n++) {
-        if (XINARG2) kangl = p->pan[n];
-        ar[n] = ain[n] * SQRT(kangl);
-        al[n] = ain[n] * SQRT(FL(1.0)-kangl);
+    case 1:
+      {
+        MYFLT kangl = *p->pan;
+        for (n=offset; n<nsmps; n++) {
+          if (XINARG2) kangl = p->pan[n];
+          ar[n] = ain[n] * SQRT(kangl);
+          al[n] = ain[n] * SQRT(FL(1.0)-kangl);
+        }
+        break;
       }
-      break;
-    }
-    case 2: {
-      MYFLT kangl = *p->pan;
-      for (n=0; n<nsmps; n++) {
-        if (XINARG2) kangl = p->pan[n];
-        ar[n] = ain[n] * kangl;
-        al[n] = ain[n] * (FL(1.0)-kangl);
+    case 2:
+      {
+        MYFLT kangl = *p->pan;
+        for (n=offset; n<nsmps; n++) {
+          if (XINARG2) kangl = p->pan[n];
+          ar[n] = ain[n] * kangl;
+          al[n] = ain[n] * (FL(1.0)-kangl);
+        }
+        break;
       }
-      break;
-    }
-    case 3: {
-      MYFLT kangl = *p->pan, cc, ss, l, r;
-      for (n=0; n<nsmps; n++) {
-        if (XINARG2) kangl = p->pan[n];
-        cc = COS(PI*kangl*FL(0.5));
-        ss = SIN(PI*kangl*FL(0.5));
-        l = SQRT2*(cc+ss)*0.5;
-        r = SQRT2*(cc-ss)*0.5;
-        al[n] = ain[n] * l;
-        ar[n] = ain[n] * r;
+    case 3:
+      {
+        MYFLT kangl = *p->pan, cc, ss, l, r;
+        for (n=offset; n<nsmps; n++) {
+          if (XINARG2) kangl = p->pan[n];
+          cc = COS(PI*kangl*FL(0.5));
+          ss = SIN(PI*kangl*FL(0.5));
+          l = SQRT2*(cc+ss)*0.5;
+          r = SQRT2*(cc-ss)*0.5;
+          al[n] = ain[n] * l;
+          ar[n] = ain[n] * r;
+        }
       }
     }
-    }
     return OK;
 }
 
 static OENTRY pan2_localops[] =
 {
-  { "pan2", sizeof(PAN2), 5, "aa", "axo", (SUBR) pan2set, 0, (SUBR) pan2run }
+  { "pan2", sizeof(PAN2), 0, 5, "aa", "axo", (SUBR) pan2set, 0, (SUBR) pan2run }
 };
 
-LINKAGE1(pan2_localops)
+LINKAGE_BUILTIN(pan2_localops)
