@@ -1,6 +1,6 @@
 /*
 Partikkel - a granular synthesis module for Csound 5
-Copyright (C) 2006-2009 yvind Brandtsegg, Torgeir Strand Henriksen,
+Copyright (C) 2006-2009 Oeyvind Brandtsegg, Torgeir Strand Henriksen,
 Thom Johansen
 
 This library is free software; you can redistribute it and/or
@@ -23,14 +23,14 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include <math.h>
 
 #define INITERROR(x) csound->InitError(csound, Str("partikkel: " x))
-#define PERFERROR(x) csound->PerfError(csound, Str("partikkel: " x))
+#define PERFERROR(x) csound->PerfError(csound, p->h.insdshead,Str("partikkel: " x))
 #define WARNING(x) csound->Warning(csound, Str("partikkel: " x))
 
 /* Assume csound and p pointers are always available */
 #define frand() (csound->RandMT(&p->randstate)/(double)(0xffffffff))
 
 /* linear interpolation between x and y by z
- * NOTE: arguments evaluated more than once, don't pass anything with side
+ * NOTE: arguments evaluated more than once, do not pass anything with side
  * effects
  */
 #define lrp(x, y, z) ((x) + ((y) - (x))*(z))
@@ -43,7 +43,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 /* here follows routines for maintaining a linked list of grains */
 
 /* initialises a linked list of NODEs */
-static int init_pool(GRAINPOOL *s, unsigned max_grains)
+static void init_pool(GRAINPOOL *s, unsigned max_grains)
 {
     unsigned i;
     NODE **p = &s->grainlist;
@@ -58,7 +58,6 @@ static int init_pool(GRAINPOOL *s, unsigned max_grains)
         node->next = NULL;
         p = &(node->next);
     }
-    return 1;
 }
 
 /* returns pointer to new node */
@@ -103,7 +102,7 @@ static int setup_globals(CSOUND *csound, PARTIKKEL *p)
         int i;
 
         if (UNLIKELY(csound->CreateGlobalVariable(csound, "partikkel",
-                                                  sizeof(PARTIKKEL_GLOBALS)) != 0))
+                                               sizeof(PARTIKKEL_GLOBALS)) != 0))
             return INITERROR("could not allocate globals");
         pg = csound->QueryGlobalVariable(csound, "partikkel");
         pg->rootentry = NULL;
@@ -138,7 +137,7 @@ static int setup_globals(CSOUND *csound, PARTIKKEL *p)
     }
     p->globals = pg;
     if ((int)*p->opcodeid == 0) {
-        /* opcodeid 0 means we don't bother with the sync opcode */
+        /* opcodeid 0 means we do not bother with the sync opcode */
         p->globals_entry = NULL;
         return OK;
     }
@@ -151,8 +150,9 @@ static int setup_globals(CSOUND *csound, PARTIKKEL *p)
     if (*pe == NULL) {
         *pe = csound->Malloc(csound, sizeof(PARTIKKEL_GLOBALS_ENTRY));
         (*pe)->id = *p->opcodeid;
+        (*pe)->partikkel = p;
         /* allocate table for sync data */
-        (*pe)->synctab = csound->Calloc(csound, 2*csound->ksmps*sizeof(MYFLT));
+        (*pe)->synctab = csound->Calloc(csound, 2*CS_KSMPS*sizeof(MYFLT));
         (*pe)->next = NULL;
     }
     p->globals_entry = *pe;
@@ -178,7 +178,7 @@ static inline MYFLT lrplookup(FUNC *tab, unsigned phase, MYFLT zscale,
 }
 
 /* Why not use csound->intpow ? */
-static inline double intpow(MYFLT x, unsigned n)
+static inline double intpow_(MYFLT x, unsigned n)
 {
     double ans = 1.0;
 
@@ -220,7 +220,7 @@ static inline MYFLT dsf(FUNC *tab, GRAIN *grain, double beta, MYFLT zscale,
 
 static int partikkel_init(CSOUND *csound, PARTIKKEL *p)
 {
-    int32 size;
+    uint32_t size;
     int ret;
 
     if ((ret = setup_globals(csound, p)) != OK)
@@ -264,6 +264,13 @@ static int partikkel_init(CSOUND *csound, PARTIKKEL *p)
     p->wavgaintab = *p->waveamps >= FL(0.0)
                     ? csound->FTFind(csound, p->waveamps)
                     : p->globals->zzhhhhz_tab;
+    if (*p->pantable >= FL(0.0)) {
+        p->pantab = csound->FTFind(csound, p->pantable);
+        if (!p->pantab)
+            return INITERROR("unable to load panning function table");
+    } else {
+        p->pantab = NULL; /* use default linear panning function */
+    }
 
     if (UNLIKELY(!p->disttab))
         return INITERROR("unable to load distribution table");
@@ -295,11 +302,14 @@ static int partikkel_init(CSOUND *csound, PARTIKKEL *p)
     p->fmampindex = 0;
     p->distindex = 0;
     p->synced = 0;
+    p->graininc = 0.0;
+
     /* allocate memory for the grain mix buffer */
-    size = csound->ksmps*sizeof(MYFLT);
+    size = CS_KSMPS*sizeof(MYFLT);
     if (p->aux.auxp == NULL || p->aux.size < size)
         csound->AuxAlloc(csound, size, &p->aux);
-    memset(p->aux.auxp, 0, sizeof(MYFLT)*csound->ksmps);
+    else
+      memset(p->aux.auxp, 0, size);
 
     /* allocate memory for the grain pool and initialize it*/
     if (UNLIKELY(*p->max_grains < FL(1.0)))
@@ -393,20 +403,31 @@ static int schedule_grain(CSOUND *csound, PARTIKKEL *p, NODE *node, int32 n,
         return_grain(&p->gpool, node);
         return PERFERROR("channel mask specifies non-existing output channel");
     }
-    grain->gain1 = FL(1.0) - (maskchannel - chan);
-    grain->gain2 = maskchannel - chan;
+    /* use panning law table if specified */
+    if (p->pantab != NULL) {
+        unsigned tabsize = p->pantab->flen/8;
+        unsigned i1 = (unsigned)((FL(1.0) - maskchannel + 2*chan)*tabsize);
+        unsigned i2 = (unsigned)(maskchannel*tabsize);
+
+        grain->gain1 = p->pantab->ftable[i1];
+        grain->gain2 = p->pantab->ftable[i2];
+    } else {
+        grain->gain1 = FL(1.0) - (maskchannel - chan);
+        grain->gain2 = maskchannel - chan;
+    }
+
     grain->chan1 = chan;
     grain->chan2 = p->num_outputs > chan + 1 ? chan + 1 : 0;
 
     /* duration in samples */
-    samples = (int)((csound->esr*(*p->duration)/1000.0) + 0.5);
+    samples = (int)((CS_ESR*(*p->duration)/1000.0) + 0.5);
     /* if grainlength is below one sample, we'll just cancel it */
     if (samples <= 0) {
         return_grain(&p->gpool, node);
         return OK;
     }
     rcp_samples = 1.0/(double)samples;
-    grain->start = n + offset*csound->esr;
+    grain->start = n + offset*CS_ESR;
     grain->stop = grain->start + samples;
     /* implement sub-sample grain placement for synchronous grains */
     if (offset == 0.0 && p->graininc > 1e-6)
@@ -428,8 +449,7 @@ static int schedule_grain(CSOUND *csound, PARTIKKEL *p, NODE *node, int32 n,
         curwav->table = i != WAV_TRAINLET ? p->wavetabs[i] : p->costab;
         curwav->gain = wavgains[wavgainsindex + i + 2]*graingain;
 
-        /* check if waveform gets zero volume. if so, let's mark it as unused
-         * and move on to the next one */
+        /* drop wavetables with close to zero gain */
         if (fabs(curwav->gain) < FL(1e-8)) {
             curwav->table = NULL;
             continue;
@@ -443,14 +463,14 @@ static int schedule_grain(CSOUND *csound, PARTIKKEL *p, NODE *node, int32 n,
             /* limit dsf harmonics to nyquist to avoid aliasing.
              * minumum number of harmonics is 2, since 1 would yield just dc,
              * which we remove anyway */
-            nh = 0.5*csound->esr/fabs(maxfreq);
+            nh = 0.5*CS_ESR/fabs(maxfreq);
             if (nh > fabs(*p->harmonics))
                 nh = fabs(*p->harmonics);
             grain->harmonics = (unsigned)nh + 1;
             if (grain->harmonics < 2)
                 grain->harmonics = 2;
             grain->falloff = *p->falloff;
-            grain->falloff_pow_N = intpow(grain->falloff, grain->harmonics);
+            grain->falloff_pow_N = intpow_(grain->falloff, grain->harmonics);
             /* normalize trainlets to uniform peak, using geometric sum */
             if (FABS(grain->falloff) > FL(0.9999) &&
                 FABS(grain->falloff) < FL(1.0001))
@@ -526,10 +546,12 @@ static int schedule_grain(CSOUND *csound, PARTIKKEL *p, NODE *node, int32 n,
 /* this function schedules the grains that are bound to happen this k-period */
 static int schedule_grains(CSOUND *csound, PARTIKKEL *p)
 {
-    int32 n;
+    uint32_t koffset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     NODE *node;
     MYFLT **waveformparams = &p->waveform1;
-    MYFLT grainfreq = *p->grainfreq;
+    MYFLT grainfreq = fabs(*p->grainfreq);
 
     /* krate table lookup, first look up waveform ftables */
     for (n = 0; n < 4; ++n) {
@@ -546,12 +568,9 @@ static int schedule_grains(CSOUND *csound, PARTIKKEL *p)
     if (UNLIKELY(!p->fmenvtab))
         return PERFERROR("unable to load FM envelope table");
 
+    if (UNLIKELY(early)) nsmps -= early;
     /* start grain scheduling */
-    for (n = 0; n < csound->ksmps; ++n) {
-        if (p->grainfreq_arate)
-            grainfreq = p->grainfreq[n];
-        p->graininc = fabs(grainfreq*csound->onedsr);
-
+    for (n = koffset; n < nsmps; ++n) {
         if (p->sync[n] >= FL(1.0)) {
             /* we got a full sync pulse, hardsync grain clock if needed */
             if (!p->synced) {
@@ -576,7 +595,7 @@ static int schedule_grains(CSOUND *csound, PARTIKKEL *p)
         if (p->grainphase >= 1.0) {
             double offset;
 
-            do 
+            do
                 p->grainphase -= 1.0;
             while (p->grainphase >= 1.0);
             /* schedule new synchronous or synced grain */
@@ -590,19 +609,19 @@ static int schedule_grains(CSOUND *csound, PARTIKKEL *p)
                 /* negative distrib, choose sequential point in table */
                 offset = p->disttab->ftable[p->distindex++];
                 offset *= -*p->distribution;
-                if (p->distindex >= p->disttab->flen)
+                if ((uint32_t)p->distindex >= p->disttab->flen)
                     p->distindex = 0;
             }
             /* convert offset to seconds, also limiting it to 10 seconds to
              * avoid accidentally filling grain pool with grains which will
              * spawn in half a day */
             if (grainfreq < FL(0.001))
-                offset = 0; 
+                offset = 0;
             else if ((offset - p->grainphase)/grainfreq > 10.)
-                offset = 10.;
+                offset = 10.0;
             else
-                offset = (offset - p->grainphase)/grainfreq; 
-            /* prepare actual grain creation */
+                offset = (offset - p->grainphase)/grainfreq;
+
             /* check if there are any grains left in the pool */
             if (!p->gpool.free_nodes) {
                 if (!p->out_of_voices_warning) {
@@ -628,12 +647,79 @@ static int schedule_grains(CSOUND *csound, PARTIKKEL *p)
 
         /* store away the scheduler phase for use in partikkelsync */
         if (p->globals_entry)
-            p->globals_entry->synctab[csound->ksmps + n] = p->grainphase;
+            p->globals_entry->synctab[CS_KSMPS + n] = p->grainphase;
+
+        if (p->grainfreq_arate)
+            grainfreq = fabs(p->grainfreq[n]);
+        p->graininc = grainfreq*csound->onedsr;
         p->grainphase += p->graininc;
     }
     return OK;
 }
 
+/* Main synthesis loops */
+/* NOTE: the main synthesis loop is duplicated for both wavetable and
+ * trainlet synthesis for speed */
+static inline void render_wave(PARTIKKEL *p, GRAIN *grain, WAVEDATA *wav,
+                               MYFLT *buf, unsigned stop)
+{
+    unsigned n;
+    double fmenvphase = grain->envphase;
+
+    /* wavetable synthesis */
+    for (n = grain->start; n < stop; ++n) {
+        double tablen = (double)wav->table->flen;
+        unsigned x0;
+        MYFLT frac, fmenv;
+
+        /* make sure phase accumulator stays within bounds */
+        while (UNLIKELY(wav->phase >= tablen))
+            wav->phase -= tablen;
+        while (UNLIKELY(wav->phase < 0.0))
+            wav->phase += tablen;
+
+        /* sample table lookup with linear interpolation */
+        x0 = (unsigned)wav->phase;
+        frac = (MYFLT)(wav->phase - x0);
+        buf[n] += lrp(wav->table->ftable[x0], wav->table->ftable[x0 + 1],
+                      frac)*wav->gain;
+
+        fmenv = grain->fmenvtab->ftable[(size_t)(fmenvphase*FMAXLEN)
+                                        >> grain->fmenvtab->lobits];
+        fmenvphase += grain->envinc;
+        wav->phase += wav->delta + wav->delta*p->fm[n]*grain->fmamp*fmenv;
+        /* apply sweep */
+        wav->delta = wav->delta*wav->sweepdecay + wav->sweepoffset;
+     }
+}
+
+static inline void render_trainlet(PARTIKKEL *p, GRAIN *grain, WAVEDATA *wav,
+                                   MYFLT *buf, unsigned stop)
+{
+    unsigned n;
+    double fmenvphase = grain->envphase;
+
+    /* trainlet synthesis */
+    for (n = grain->start; n < stop; ++n) {
+        MYFLT fmenv;
+
+        while (UNLIKELY(wav->phase >= 1.0))
+            wav->phase -= 1.0;
+        while (UNLIKELY(wav->phase < 0.0))
+            wav->phase += 1.0;
+
+        /* dsf/trainlet synthesis */
+        buf[n] += wav->gain*dsf(p->costab, grain, wav->phase, p->zscale,
+                                p->cosineshift);
+
+        fmenv = grain->fmenvtab->ftable[(size_t)(fmenvphase*FMAXLEN)
+                                        >> grain->fmenvtab->lobits];
+        fmenvphase += grain->envinc;
+        wav->phase += wav->delta + wav->delta*p->fm[n]*grain->fmamp*fmenv;
+        wav->delta = wav->delta*wav->sweepdecay + wav->sweepoffset;
+    }
+}
+
 /* do the actual waveform synthesis */
 static inline void render_grain(CSOUND *csound, PARTIKKEL *p, GRAIN *grain)
 {
@@ -641,73 +727,23 @@ static inline void render_grain(CSOUND *csound, PARTIKKEL *p, GRAIN *grain)
     unsigned n;
     MYFLT *out1 = *(&(p->output1) + grain->chan1);
     MYFLT *out2 = *(&(p->output1) + grain->chan2);
-    unsigned stop = grain->stop > csound->ksmps
-                    ? csound->ksmps : grain->stop;
+    unsigned stop = grain->stop > CS_KSMPS
+                    ? CS_KSMPS : grain->stop;
     MYFLT *buf = (MYFLT *)p->aux.auxp;
 
-    if (grain->start >= csound->ksmps)
+    if (grain->start >= CS_KSMPS)
         return; /* grain starts at a later kperiod */
     for (i = 0; i < 5; ++i) {
         WAVEDATA *curwav = &grain->wav[i];
-        double fmenvphase = grain->envphase;
-        MYFLT fmenv;
 
         /* check if ftable is to be rendered */
         if (curwav->table == NULL)
             continue;
 
-        /* NOTE: the main synthesis loop is duplicated for both wavetable and
-         * trainlet synthesis for speed */
-        if (i != WAV_TRAINLET) {
-            /* wavetable synthesis */
-            for (n = grain->start; n < stop; ++n) {
-                double tablen = (double)curwav->table->flen;
-                unsigned x0;
-                MYFLT frac;
-
-                /* make sure phase accumulator stays within bounds */
-                while (UNLIKELY(curwav->phase >= tablen))
-                    curwav->phase -= tablen;
-                while (UNLIKELY(curwav->phase < 0.0))
-                    curwav->phase += tablen;
-
-                /* sample table lookup with linear interpolation */
-                x0 = (unsigned)curwav->phase;
-                frac = (MYFLT)(curwav->phase - x0);
-                buf[n] += lrp(curwav->table->ftable[x0],
-                              curwav->table->ftable[x0 + 1],
-                              frac)*curwav->gain;
-
-                fmenv = grain->fmenvtab->ftable[(size_t)(fmenvphase*FMAXLEN)
-                                                >> grain->fmenvtab->lobits];
-                fmenvphase += grain->envinc;
-                curwav->phase += curwav->delta
-                                 + curwav->delta*p->fm[n]*grain->fmamp*fmenv;
-                /* apply sweep */
-                curwav->delta = curwav->delta*curwav->sweepdecay
-                                + curwav->sweepoffset;
-            }
-        } else {
-            /* trainlet synthesis */
-            for (n = grain->start; n < stop; ++n) {
-                while (UNLIKELY(curwav->phase >= 1.0))
-                    curwav->phase -= 1.0;
-                while (UNLIKELY(curwav->phase < 0.0))
-                    curwav->phase += 1.0;
-
-                /* dsf/trainlet synthesis */
-                buf[n] += curwav->gain*dsf(p->costab, grain, curwav->phase,
-                                           p->zscale, p->cosineshift);
-
-                fmenv = grain->fmenvtab->ftable[(size_t)(fmenvphase*FMAXLEN)
-                                                >> grain->fmenvtab->lobits];
-                fmenvphase += grain->envinc;
-                curwav->phase += curwav->delta
-                                 + curwav->delta*p->fm[n]*grain->fmamp*fmenv;
-                curwav->delta = curwav->delta*curwav->sweepdecay
-                                + curwav->sweepoffset;
-            }
-        }
+        if (i != WAV_TRAINLET)
+            render_wave(p, grain, curwav, buf, stop);
+        else
+            render_trainlet(p, grain, curwav, buf, stop);
     }
 
     /* apply envelopes */
@@ -755,7 +791,8 @@ static inline void render_grain(CSOUND *csound, PARTIKKEL *p, GRAIN *grain)
 
 static int partikkel(CSOUND *csound, PARTIKKEL *p)
 {
-    int ret, n;
+    int ret;
+    unsigned int n;
     NODE **nodeptr;
     MYFLT **outputs = &p->output1;
 
@@ -767,7 +804,7 @@ static int partikkel(CSOUND *csound, PARTIKKEL *p)
 
     /* clear output buffers, we'll be accumulating our outputs */
     for (n = 0; n < p->num_outputs; ++n)
-        memset(outputs[n], 0, sizeof(MYFLT)*csound->ksmps);
+        memset(outputs[n], 0, sizeof(MYFLT)*CS_KSMPS);
 
     /* prepare to traverse grain list */
     nodeptr = &p->grainroot;
@@ -777,16 +814,16 @@ static int partikkel(CSOUND *csound, PARTIKKEL *p)
         /* render current grain to outputs */
         render_grain(csound, p, grain);
         /* check if grain is finished */
-        if (grain->stop <= csound->ksmps) {
+        if (grain->stop <= CS_KSMPS) {
             /* grain is finished, deactivate it */
             *nodeptr = return_grain(&p->gpool, *nodeptr);
         } else {
             /* extend grain lifetime with one k-period and find next grain */
-            if (csound->ksmps > grain->start)
+            if (CS_KSMPS > grain->start)
                 grain->start = 0; /* grain is active */
             else
-                grain->start -= csound->ksmps; /* grain is not yet active */
-            grain->stop -= csound->ksmps;
+                grain->start -= CS_KSMPS; /* grain is not yet active */
+            grain->stop -= CS_KSMPS;
             nodeptr = &((*nodeptr)->next);
         }
     }
@@ -794,7 +831,7 @@ static int partikkel(CSOUND *csound, PARTIKKEL *p)
 }
 
 /* partikkelsync stuff */
-static int partikkelsync_init(CSOUND *csound, PARTIKKELSYNC *p)
+static int partikkelsync_init(CSOUND *csound, PARTIKKEL_SYNC *p)
 {
     PARTIKKEL_GLOBALS *pg;
     PARTIKKEL_GLOBALS_ENTRY *pe;
@@ -818,36 +855,137 @@ static int partikkelsync_init(CSOUND *csound, PARTIKKELSYNC *p)
     return OK;
 }
 
-static int partikkelsync(CSOUND *csound, PARTIKKELSYNC *p)
+static int partikkelsync(CSOUND *csound, PARTIKKEL_SYNC *p)
 {
     /* write sync pulse data */
-    memcpy(p->syncout, p->ge->synctab, csound->ksmps*sizeof(MYFLT));
+    memcpy(p->syncout, p->ge->synctab, CS_KSMPS*sizeof(MYFLT));
     /* write scheduler phase data, if user wanted it */
     if (p->output_schedphase) {
-        memcpy(p->schedphaseout, p->ge->synctab + csound->ksmps,
-               csound->ksmps*sizeof(MYFLT));
+        memcpy(p->schedphaseout, p->ge->synctab + CS_KSMPS,
+               CS_KSMPS*sizeof(MYFLT));
     }
     /* clear first half of sync table to get rid of old sync pulses */
-    memset(p->ge->synctab, 0, csound->ksmps*sizeof(MYFLT));
+    memset(p->ge->synctab, 0, CS_KSMPS*sizeof(MYFLT));
     return OK;
 }
 
-static OENTRY localops[] = {
+static int get_global_entry(CSOUND *csound, PARTIKKEL_GLOBALS_ENTRY **entry,
+                            MYFLT opcodeid, const char *prefix)
+{
+    PARTIKKEL_GLOBALS *pg;
+    PARTIKKEL_GLOBALS_ENTRY *pe;
+
+    pg = csound->QueryGlobalVariable(csound, "partikkel");
+    if (UNLIKELY(pg == NULL))
+        return csound->InitError(csound,
+                                 Str("%s: partikkel not initialized"), prefix);
+    /* try to find entry corresponding to our opcodeid */
+    pe = pg->rootentry;
+    while (pe != NULL && pe->id != opcodeid)
+        pe = pe->next;
+
+    if (UNLIKELY(pe == NULL))
+        return csound->InitError(csound,
+                                 Str("%s: could not find opcode id"), prefix);
+    *entry = pe;
+    return OK;
+}
+
+static int partikkelget_init(CSOUND *csound, PARTIKKEL_GET *p)
+{
+    return get_global_entry(csound, &p->ge, *p->opcodeid, "partikkelget");
+}
+
+static int partikkelget(CSOUND *csound, PARTIKKEL_GET *p)
+{
+    PARTIKKEL *partikkel = p->ge->partikkel;
+
+    switch ((int)*p->index) {
+    case 0:
+        *p->valout = (MYFLT)partikkel->gainmaskindex;
+        break;
+    case 1:
+        *p->valout = (MYFLT)partikkel->wavfreqstartindex;
+        break;
+    case 2:
+        *p->valout = (MYFLT)partikkel->wavfreqendindex;
+        break;
+    case 3:
+        *p->valout = (MYFLT)partikkel->fmampindex;
+        break;
+    case 4:
+        *p->valout = (MYFLT)partikkel->channelmaskindex;
+        break;
+    case 5:
+        *p->valout = (MYFLT)partikkel->wavgainindex;
+        break;
+    }
+    return OK;
+}
+
+static int partikkelset_init(CSOUND *csound, PARTIKKEL_SET *p)
+{
+    return get_global_entry(csound, &p->ge, *p->opcodeid, "partikkelset");
+}
+
+static int partikkelset(CSOUND *csound, PARTIKKEL_SET *p)
+{
+    PARTIKKEL *partikkel = p->ge->partikkel;
+
+    switch ((int)*p->index) {
+    case 0:
+        partikkel->gainmaskindex = (unsigned)*p->value;
+        break;
+    case 1:
+        partikkel->wavfreqstartindex = (unsigned)*p->value;
+        break;
+    case 2:
+        partikkel->wavfreqendindex = (unsigned)*p->value;
+        break;
+    case 3:
+        partikkel->fmampindex = (unsigned)*p->value;
+        break;
+    case 4:
+        partikkel->channelmaskindex = (unsigned)*p->value;
+        break;
+    case 5:
+        partikkel->wavgainindex = (unsigned)*p->value;
+        break;
+    }
+    return OK;
+}
+
+static OENTRY partikkel_localops[] = {
     {
-        "partikkel", sizeof(PARTIKKEL), 5,
+        "partikkel", sizeof(PARTIKKEL), TR, 5,
         "ammmmmmm",
-        "xkiakiiikkkkikkiiaikikkkikkkkkiaaaakkkkio",
+        "xkiakiiikkkkikkiiaikikkkikkkkkiaaaakkkkioj",
         (SUBR)partikkel_init,
         (SUBR)NULL,
         (SUBR)partikkel
     },
     {
-        "partikkelsync", sizeof(PARTIKKELSYNC), 5,
+        "partikkelsync", sizeof(PARTIKKEL_SYNC), TR, 5,
         "am", "i",
         (SUBR)partikkelsync_init,
         (SUBR)NULL,
         (SUBR)partikkelsync
+    },
+    {
+        "partikkelget", sizeof(PARTIKKEL_GET), TR, 3,
+        "k", "ki",
+        (SUBR)partikkelget_init,
+        (SUBR)partikkelget,
+        (SUBR)NULL
+    },
+    {
+        "partikkelset", sizeof(PARTIKKEL_SET), TR, 3,
+        "", "kki",
+        (SUBR)partikkelset_init,
+        (SUBR)partikkelset,
+        (SUBR)NULL
     }
 };
 
-LINKAGE
+LINKAGE_BUILTIN(partikkel_localops)
+
