@@ -2,7 +2,7 @@
 /*
     pitch.c:
 
-    Copyright (C) 1999 John ffitch, Istvan Varga, Peter Neubcker,
+    Copyright (C) 1999 John ffitch, Istvan Varga, Peter Neubäcker,
                        rasmus ekman, Phil Burk
 
     This file is part of Csound.
@@ -73,7 +73,7 @@ int pitchset(CSOUND *csound, PITCH *p)  /* pitch - uses spectra technology */
                                 /* End of rms */
                                 /* Initialise spectrum */
     /* for mac roundoff */
-    p->timcount = (int)(csound->ekr * *p->iprd + FL(0.001));
+    p->timcount = (int)(CS_EKR * *p->iprd + FL(0.001));
     nocts = (int)*p->iocts; if (UNLIKELY(nocts<=0)) nocts = 6;
     nfreqs = (int)*p->ifrqs; if (UNLIKELY(nfreqs<=0)) nfreqs = 12;
     ncoefs = nocts * nfreqs;
@@ -100,7 +100,7 @@ int pitchset(CSOUND *csound, PITCH *p)  /* pitch - uses spectra technology */
       p->nfreqs = nfreqs;
       p->curq = Q;
       p->ncoefs = ncoefs;
-      dwnp->srate = csound->esr;
+      dwnp->srate = CS_ESR;
       hicps = dwnp->srate * 0.375;            /* top freq is 3/4 pi/2 ...   */
       oct = log(hicps / ONEPT) / LOGTWO;      /* octcps()  (see aops.c)     */
       dwnp->looct = (MYFLT)(oct - nocts);     /* true oct val of lowest frq */
@@ -249,7 +249,10 @@ int pitch(CSOUND *csound, PITCH *p)
     double      c1 = p->c1, c2 = p->c2;
 
     MYFLT   a, b, *dftp, SIG, yt1, yt2;
-    int     nocts, n, nsmps = csound->ksmps, winlen;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int     nocts, winlen;
     DOWNDAT *downp = &p->downsig;
     OCTDAT  *octp;
     SPECDAT *specp;
@@ -258,7 +261,12 @@ int pitch(CSOUND *csound, PITCH *p)
                                 /* RMS */
     q = p->prvq;
     asig = p->asig;
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(asig, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&asig[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       MYFLT as = asig[n]*DFLT_DBFS/csound->e0dbfs; /* Normalise.... */
       q = c1 * as * as + c2 * q;
       SIG = as;                              /* for each source sample: */
@@ -273,7 +281,7 @@ int pitch(CSOUND *csound, PITCH *p)
         if (UNLIKELY(curp >= octp->endp))
           curp = octp->begp;                /*    & modulo the pointer */
         octp->curp = curp;
-        if (UNLIKELY(!(--nocts)))  break;             /*  if lastoct, break      */
+        if (UNLIKELY(!(--nocts)))  break;   /*  if lastoct, break      */
         coefp = bicoefs;  ytp = octp->feedback;
         for (nfilt = 3; nfilt--; ) {        /*  apply triple biquad:   */
           yt2    = *ytp++; yt1 = *ytp--;          /* get prev feedback */
@@ -333,7 +341,7 @@ int pitch(CSOUND *csound, PITCH *p)
         *dftp++ = c;                          /* store in out spectrum   */
       }
     }
-    specp->ktimstamp = csound->kcounter;      /* time-stamp the output   */
+    specp->ktimstamp = CS_KCNT;      /* time-stamp the output   */
 
  nxt:
                                 /* specptrk */
@@ -420,8 +428,7 @@ int pitch(CSOUND *csound, PITCH *p)
       kval = realbin / specp->nfreqs;           /*     & cvt to true decoct */
 
       if (p->playing == STARTING) {             /* STARTING mode:           */
-        if ((absdiff = kval - p->kvalsav) < FL(0.0))
-          absdiff = -absdiff;
+        absdiff = FABS(kval - p->kvalsav);// < FL(0.0)) absdiff = -absdiff;
         confirms = (int)(absdiff * p->confact); /* get interval dependency  */
         if (UNLIKELY(p->jmpcount < confirms)) {
           p->jmpcount += 1;               /* if not enough confirms,  */
@@ -433,8 +440,7 @@ int pitch(CSOUND *csound, PITCH *p)
           p->kinc = FL(0.0);
         }
       } else {                                  /* PLAYING mode:            */
-        if ((absdiff = kval - p->kval) < FL(0.0))
-          absdiff = -absdiff;
+        absdiff = FABS(kval - p->kval);
         confirms = (int)(absdiff * p->confact); /* get interval dependency  */
         if (p->jmpcount < confirms) {
           p->jmpcount += 1;               /* if not enough confirms,  */
@@ -452,7 +458,8 @@ int pitch(CSOUND *csound, PITCH *p)
     *p->kamp = p->kavl * FL(4.0);
     return OK;
  err1:
-    return csound->PerfError(csound, Str("pitch: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pitch: not initialised"));
 }
 
 /* Multiply and accumulate opcodes */
@@ -460,7 +467,7 @@ int pitch(CSOUND *csound, PITCH *p)
 int macset(CSOUND *csound, SUM *p)
 {
     if (UNLIKELY((((int)p->INOCOUNT)&1)==1)) {
-      return csound->PerfError(csound,
+      return csound->PerfError(csound, p->h.insdshead,
                                Str("Must have even number of arguments in mac\n"));
     }
     return OK;
@@ -468,9 +475,18 @@ int macset(CSOUND *csound, SUM *p)
 
 int maca(CSOUND *csound, SUM *p)
 {
-    int nsmps=csound->ksmps, count=(int) p->INOCOUNT, j, k;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t k, nsmps = CS_KSMPS;
+    int count=(int) p->INOCOUNT, j;
     MYFLT *ar = p->ar, **args = p->argums;
-    for (k=0; k<nsmps; k++) {
+
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (k=offset; k<nsmps; k++) {
       MYFLT ans = FL(0.0);
       for (j=0; j<count; j +=2)
         ans += args[j][k] * args[j+1][k];
@@ -481,9 +497,17 @@ int maca(CSOUND *csound, SUM *p)
 
 int mac(CSOUND *csound, SUM *p)
 {
-    int nsmps=csound->ksmps, count=(int) p->INOCOUNT, j, k;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t k, nsmps = CS_KSMPS;
+    int count=(int) p->INOCOUNT, j;
     MYFLT *ar = p->ar, **args = p->argums;
-    for (k=0; k<nsmps; k++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (k=offset; k<nsmps; k++) {
       MYFLT ans = FL(0.0);
       for (j=0; j<count; j +=2)
         ans += *args[j] * args[j+1][k];
@@ -559,14 +583,35 @@ int clockread(CSOUND *csound, CLKRD *p)
     return OK;
 }
 
+int scratchread(CSOUND *csound, SCRATCHPAD *p)
+{
+    int index = MYFLT2LRND(*p->index);
+    if (index<0 || index>3)
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("scratchpad index out of range"));
+    *p->val = p->h.insdshead->scratchpad[index];
+    return OK;
+}
+
+int scratchwrite(CSOUND *csound, SCRATCHPAD *p)
+{
+    int index = MYFLT2LRND(*p->index);
+    if (index<0 || index>3)
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("scratchpad index out of range"));
+    p->h.insdshead->scratchpad[index] = *p->val;
+    return OK;
+}
+
+
 /* ************************************************************ */
-/* Opcodes from Peter Neubcker                                 */
+/* Opcodes from Peter Neubäcker                                 */
 /* ************************************************************ */
 
 int adsyntset(CSOUND *csound, ADSYNT *p)
 {
     FUNC    *ftp;
-    int     count;
+    unsigned int     count;
     int32   *lphs;
 
     p->inerr = 0;
@@ -579,12 +624,12 @@ int adsyntset(CSOUND *csound, ADSYNT *p)
       return csound->InitError(csound, Str("adsynt: wavetable not found!"));
     }
 
-    count = (int)*p->icnt;
+    count = (unsigned int)*p->icnt;
     if (UNLIKELY(count < 1))
       count = 1;
     p->count = count;
 
-    if (LIKELY((ftp = csound->FTFind(csound, p->ifreqtbl)) != NULL)) {
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifreqtbl)) != NULL)) {
       p->freqtp = ftp;
     }
     else {
@@ -597,7 +642,7 @@ int adsyntset(CSOUND *csound, ADSYNT *p)
                     "adsynt: partial count is greater than freqtable size!"));
     }
 
-    if (LIKELY((ftp = csound->FTFind(csound, p->iamptbl)) != NULL)) {
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->iamptbl)) != NULL)) {
       p->amptp = ftp;
     }
     else {
@@ -632,13 +677,17 @@ int adsynt(CSOUND *csound, ADSYNT *p)
 {
     FUNC    *ftp, *freqtp, *amptp;
     MYFLT   *ar, *ftbl, *freqtbl, *amptbl;
-    MYFLT   amp0, amp, cps0, cps;
+    MYFLT    amp0, amp, cps0, cps;
     int32    phs, inc, lobits;
-    int32    *lphs;
-    int     n, nsmps = csound->ksmps, c, count;
+    int32   *lphs;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int      c, count;
 
     if (UNLIKELY(p->inerr)) {
-      return csound->PerfError(csound, Str("adsynt: not initialised"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("adsynt: not initialised"));
     }
     ftp = p->ftp;
     ftbl = ftp->ftable;
@@ -655,13 +704,14 @@ int adsynt(CSOUND *csound, ADSYNT *p)
 
     ar = p->sr;
     memset(ar, 0, nsmps*sizeof(MYFLT));
+    if (UNLIKELY(early)) nsmps -= early;
 
     for (c=0; c<count; c++) {
       amp = amptbl[c] * amp0;
       cps = freqtbl[c] * cps0;
       inc = (int32) (cps * csound->sicvt);
       phs = lphs[c];
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         ar[n] += *(ftbl + (phs >> lobits)) * amp;
         phs += inc;
         phs &= PHMASK;
@@ -676,7 +726,7 @@ int hsboscset(CSOUND *csound, HSBOSC *p)
     FUNC        *ftp;
     int         octcnt, i;
 
-    if (LIKELY((ftp = csound->FTFind(csound, p->ifn)) != NULL)) {
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL)) {
       p->ftp = ftp;
       if (UNLIKELY(*p->ioctcnt < 2))
         octcnt = 3;
@@ -691,7 +741,7 @@ int hsboscset(CSOUND *csound, HSBOSC *p)
       }
     }
     else p->ftp = NULL;
-    if (LIKELY((ftp = csound->FTFind(csound, p->imixtbl)) != NULL)) {
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->imixtbl)) != NULL)) {
       p->mixtp = ftp;
     }
     else p->mixtp = NULL;
@@ -704,17 +754,20 @@ int hsboscil(CSOUND *csound, HSBOSC   *p)
     MYFLT       fract, v1, amp0, amp, *ar, *ftab, *mtab;
     int32       phs, inc, lobits;
     int32       phases[10];
-    int         n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       tonal, bright, freq, ampscl;
     int         octcnt = p->octcnt;
     MYFLT       octstart, octoffs, octbase;
     int         octshift, i, mtablen;
-    MYFLT       hesr = csound->esr / FL(2.0);
+    MYFLT       hesr = CS_ESR / FL(2.0);
 
     ftp = p->ftp;
     mixtp = p->mixtp;
     if (UNLIKELY(ftp==NULL || mixtp==NULL)) {
-      return csound->PerfError(csound, Str("hsboscil: not initialised"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("hsboscil: not initialised"));
     }
 
     tonal = *p->ktona;
@@ -748,6 +801,7 @@ int hsboscil(CSOUND *csound, HSBOSC   *p)
     lobits = ftp->lobits;
     ar = p->sr;
     memset(ar, 0, nsmps*sizeof(MYFLT));
+    if (UNLIKELY(early)) nsmps -= early;
 
     for (i=0; i<octcnt; i++) {
       phs = phases[i];
@@ -755,7 +809,7 @@ int hsboscil(CSOUND *csound, HSBOSC   *p)
       if (UNLIKELY(freq > hesr))
         amp = FL(0.0);
       inc = (int32)(freq * csound->sicvt);
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         fract = PFRAC(phs);
         ftab = ftp->ftable + (phs >> lobits);
         v1 = *ftab++;
@@ -774,7 +828,9 @@ int hsboscil(CSOUND *csound, HSBOSC   *p)
 int pitchamdfset(CSOUND *csound, PITCHAMDF *p)
 {
     MYFLT srate, downs;
-    int32  size, minperi, maxperi, downsamp, upsamp, msize, bufsize, interval;
+    int32  size, minperi, maxperi, downsamp, upsamp, msize, bufsize;
+    uint32_t interval;
+    uint32_t nsmps = CS_KSMPS;
 
     p->inerr = 0;
 
@@ -782,13 +838,13 @@ int pitchamdfset(CSOUND *csound, PITCHAMDF *p)
     if (downs < (-FL(1.9))) {
       upsamp = (int)MYFLT2LONG((-downs));
       downsamp = 0;
-      srate = csound->esr * (MYFLT)upsamp;
+      srate = CS_ESR * (MYFLT)upsamp;
     }
     else {
       downsamp = (int)MYFLT2LONG(downs);
       if (UNLIKELY(downsamp < 1))
         downsamp = 1;
-      srate = csound->esr / (MYFLT)downsamp;
+      srate = CS_ESR / (MYFLT)downsamp;
       upsamp = 0;
     }
 
@@ -803,12 +859,12 @@ int pitchamdfset(CSOUND *csound, PITCHAMDF *p)
     if (*p->iexcps < 1)
         interval = maxperi;
     else
-        interval = (int32)(srate / *p->iexcps);
-    if (interval < csound->ksmps) {
+        interval = (uint32_t)(srate / *p->iexcps);
+    if (interval < nsmps) {
       if (downsamp)
-        interval = csound->ksmps / downsamp;
+        interval = nsmps / downsamp;
       else
-        interval = csound->ksmps * upsamp;
+        interval = nsmps * upsamp;
     }
 
     size = maxperi + interval;
@@ -935,7 +991,7 @@ int pitchamdf(CSOUND *csound, PITCHAMDF *p)
     MYFLT  newval, delta;
     int32  readp = p->readp;
     int32  interval = size - maxperi;
-    int    nsmps = csound->ksmps;
+    int    nsmps = CS_KSMPS;
     int    i;
     int32  i1, i2;
     MYFLT  val, rms;
@@ -943,7 +999,8 @@ int pitchamdf(CSOUND *csound, PITCHAMDF *p)
     MYFLT  acc, accmin, diff;
 
     if (UNLIKELY(p->inerr)) {
-      return csound->PerfError(csound, Str("pitchamdf: not initialised"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("pitchamdf: not initialised"));
     }
 
     if (upsamp) {
@@ -1123,7 +1180,8 @@ int kphsorbnk(CSOUND *csound, PHSORBNK *p)
     int     index = (int)(*p->kindx);
 
     if (UNLIKELY(curphs == NULL)) {
-      return csound->PerfError(csound, Str("phasorbnk: not initialised"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("phasorbnk: not initialised"));
     }
 
     if (UNLIKELY(index<0 || index>=size)) {
@@ -1142,7 +1200,9 @@ int kphsorbnk(CSOUND *csound, PHSORBNK *p)
 
 int phsorbnk(CSOUND *csound, PHSORBNK *p)
 {
-    int     n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   *rs;
     double  phase, incr;
     double  *curphs = (double*)p->curphs.auxp;
@@ -1150,7 +1210,8 @@ int phsorbnk(CSOUND *csound, PHSORBNK *p)
     int     index = (int)(*p->kindx);
 
     if (UNLIKELY(curphs == NULL)) {
-      return csound->PerfError(csound, Str("phasorbnk: not initialised"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("phasorbnk: not initialised"));
     }
 
     if (UNLIKELY(index<0 || index>=size)) {
@@ -1160,9 +1221,14 @@ int phsorbnk(CSOUND *csound, PHSORBNK *p)
 
     rs = p->sr;
     phase = curphs[index];
+    if (UNLIKELY(offset)) memset(rs, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rs[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->XINCODE) {
       MYFLT *cps = p->xcps;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         incr = (double)(cps[n] * csound->onedsr);
         rs[n] = (MYFLT)phase;
         phase += incr;
@@ -1174,7 +1240,7 @@ int phsorbnk(CSOUND *csound, PHSORBNK *p)
     }
     else {
       incr = (double)(*p->xcps * csound->onedsr);
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         rs[n] = (MYFLT)phase;
         phase += incr;
         if (UNLIKELY(phase >= 1.0))
@@ -1241,10 +1307,16 @@ int pinkish(CSOUND *csound, PINKISH *p)
 {
     MYFLT       *aout, *ain;
     double      c0, c1, c2, c3, c4, c5, c6, nxtin, nxtout;
-    int    n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     aout = p->aout;
     ain = p->xin;
-
+    if (UNLIKELY(offset)) memset(aout, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&aout[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (*p->imethod == GARDNER_PINK) {  /* Gardner method (default) */
       GardnerPink_perf(csound,p);
     }
@@ -1253,7 +1325,7 @@ int pinkish(CSOUND *csound, PINKISH *p)
       /* Get filter states */
       c0 = p->b0; c1 = p->b1; c2 = p->b2;
       c3 = p->b3; c4 = p->b4; c5 = p->b5; c6 = p->b6;
-      for (n=0;n<nsmps;n++) {
+      for (n=offset;n<nsmps;n++) {
         nxtin = (double)ain[n];
         c0 = c0 * 0.99886 + nxtin * 0.0555179;
         c1 = c1 * 0.99332 + nxtin * 0.0750759;
@@ -1273,7 +1345,7 @@ int pinkish(CSOUND *csound, PINKISH *p)
       /* Get filter states */
       c0 = p->b0; c1 = p->b1; c2 = p->b2;
 
-      for (n=0;n<nsmps;n++) {      /* Paul Kellet's "economy" pink filter */
+      for (n=offset;n<nsmps;n++) {      /* Paul Kellet's "economy" pink filter */
         nxtin = (double)ain[n];
         c0 = c0 * 0.99765 + nxtin * 0.0990460;
         c1 = c1 * 0.96300 + nxtin * 0.2965164;
@@ -1392,7 +1464,8 @@ int GardnerPink_perf(CSOUND *csound, PINKISH *p)
     MYFLT *aout, *amp, scalar;
     int32 *rows, rowIndex, indexMask, randSeed, newRandom;
     int32 runningSum, sum, ampinc;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t n, nsmps = CS_KSMPS - p->h.insdshead->ksmps_no_end;
 
     aout        = p->aout;
     amp         = p->xin;
@@ -1404,11 +1477,11 @@ int GardnerPink_perf(CSOUND *csound, PINKISH *p)
     rows        = &(p->grd_Rows[0]);
     randSeed    = p->randSeed;
 
-    for (n=0; n<nsmps;n++) {
+    for (n=offset; n<nsmps;n++) {
       /* Increment and mask index. */
       rowIndex = (rowIndex + 1) & indexMask;
 
-      /* If index is zero, don't update any random values. */
+      /* If index is zero, do not update any random values. */
       if ( rowIndex != 0 ) {
         /* Determine how many trailing zeros in PinkIndex. */
         /* This algorithm will hang if n==0 so test first. */
@@ -1467,9 +1540,8 @@ int clip_set(CSOUND *csound, CLIP *p)
 {
     int meth = (int)MYFLT2LONG(*p->imethod);
     p->meth = meth;
-    p->arg = *p->iarg;
+    p->arg = FABS(*p->iarg);
     p->lim = *p->limit;
-    if (p->arg < FL(0.0)) p->arg = - p->arg;
     switch (meth) {
     case 0:                     /* Bram de Jong method */
       if (p->arg > FL(1.0) || p->arg < FL(0.0)) p->arg = FL(0.999);
@@ -1493,14 +1565,21 @@ int clip_set(CSOUND *csound, CLIP *p)
 int clip(CSOUND *csound, CLIP *p)
 {
     MYFLT *aout = p->aout, *ain = p->ain;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT a = p->arg, k1 = p->k1, k2 = p->k2;
     MYFLT limit = p->lim;
     MYFLT rlim = FL(1.0)/limit;
 
+    if (UNLIKELY(offset)) memset(aout, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&aout[nsmps], '\0', early*sizeof(MYFLT));
+    }
     switch (p->meth) {
     case 0:                     /* Soft clip with division */
-      for (n=0;n<nsmps;n++) {
+      for (n=offset;n<nsmps;n++) {
         MYFLT x = ain[n];
         if (x>=FL(0.0)) {
           if (UNLIKELY(x>limit)) x = k2;
@@ -1517,7 +1596,7 @@ int clip(CSOUND *csound, CLIP *p)
       }
       return OK;
     case 1:
-      for (n=0;n<nsmps;n++) {
+      for (n=offset;n<nsmps;n++) {
         MYFLT x = ain[n];
         if (UNLIKELY(x>=limit))
             x = limit;
@@ -1529,7 +1608,7 @@ int clip(CSOUND *csound, CLIP *p)
       }
       return OK;
     case 2:
-      for (n=0;n<nsmps;n++) {
+      for (n=offset;n<nsmps;n++) {
         MYFLT x = ain[n];
         if (UNLIKELY(x>=limit))
           x = limit;
@@ -1550,23 +1629,30 @@ int clip(CSOUND *csound, CLIP *p)
 
 int impulse_set(CSOUND *csound, IMPULSE *p)
 {
-    p->next = (int)MYFLT2LONG(*p->offset * csound->esr);
+    p->next = (unsigned int)MYFLT2LONG(*p->offset * CS_ESR);
     return OK;
 }
 
 int impulse(CSOUND *csound, IMPULSE *p)
 {
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int next = p->next;
     MYFLT *ar = p->ar;
     if (next<0) next = -next;
-    if (UNLIKELY(next < csound->ksmps)) {          /* Impulse in this frame */
+    if (UNLIKELY(next < (int32)nsmps)) { /* Impulse in this frame */
       MYFLT frq = *p->freq;     /* Freq at k-rate */
       int sfreq;                /* Converted to samples */
       if (frq == FL(0.0)) sfreq = INT_MAX; /* Zero means infinite */
       else if (frq < FL(0.0)) sfreq = -(int)frq; /* Negative cnts in sample */
-      else sfreq = (int)(frq*csound->esr); /* Normal case */
-      for (n=0;n<nsmps;n++) {
+      else sfreq = (int)(frq*CS_ESR); /* Normal case */
+      if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+      if (UNLIKELY(early)) {
+        nsmps -= early;
+        memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+      }
+      for (n=offset;n<nsmps;n++) {
         if (UNLIKELY(next-- == 0)) {
           ar[n] = *p->amp;
           next = sfreq - 1;     /* Note can be less than k-rate */
@@ -1615,11 +1701,11 @@ int trnset(CSOUND *csound, TRANSEG *p)
       MYFLT dur = **argp++;
       MYFLT alpha = **argp++;
       MYFLT nxtval = **argp++;
-      MYFLT d = dur * csound->esr;
+      MYFLT d = dur * CS_ESR;
       if ((segp->cnt = (int32)MYFLT2LONG(d)) < 0)
         segp->cnt = 0;
       else
-        segp->cnt = (int32)(dur * csound->ekr);
+        segp->cnt = (int32)(dur * CS_EKR);
         segp->nxtpt = nxtval;
       segp->val = val;
       if (alpha == FL(0.0)) {
@@ -1669,11 +1755,11 @@ int trnset_bkpt(CSOUND *csound, TRANSEG *p)
       MYFLT d;
       dur -= totdur;
       totdur += dur;
-      d = dur * csound->esr;
+      d = dur * CS_ESR;
       if ((segp->cnt = (int32)MYFLT2LONG(d)) < 0)
         segp->cnt = 0;
       else
-        segp->cnt = (int32)(dur * csound->ekr);
+        segp->cnt = (int32)(dur * CS_EKR);
         segp->nxtpt = nxtval;
       segp->val = val;
       if (alpha == FL(0.0)) {
@@ -1696,7 +1782,8 @@ int ktrnseg(CSOUND *csound, TRANSEG *p)
 {
     *p->rslt = p->curval;               /* put the cur value    */
     if (UNLIKELY(p->auxch.auxp==NULL)) { /* RWD fix */
-      csound->Die(csound, Str("Error: transeg not initialised (krate)\n"));
+      csound->PerfError(csound,p->h.insdshead,
+                        Str("Error: transeg not initialised (krate)\n"));
     }
     if (p->segsrem) {                   /* done if no more segs */
       if (--p->curcnt <= 0) {           /* if done cur segment  */
@@ -1716,11 +1803,11 @@ int ktrnseg(CSOUND *csound, TRANSEG *p)
         p->curx = FL(0.0);
       }
       if (p->alpha == FL(0.0))
-        p->curval += p->curinc*csound->ksmps;   /* advance the cur val  */
+        p->curval += p->curinc*CS_KSMPS;   /* advance the cur val  */
       else
         p->curval = p->cursegp->val + p->curinc *
           (FL(1.0) - EXP(p->curx));
-      p->curx += (MYFLT)csound->ksmps*p->alpha;
+      p->curx += (MYFLT)CS_KSMPS*p->alpha;
     }
     return OK;
 }
@@ -1728,10 +1815,18 @@ int ktrnseg(CSOUND *csound, TRANSEG *p)
 int trnseg(CSOUND *csound, TRANSEG *p)
 {
     MYFLT  val, *rs = p->rslt;
-    int         n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     NSEG        *segp = p->cursegp;
     if (UNLIKELY(p->auxch.auxp==NULL)) {
-      return csound->PerfError(csound, Str("transeg: not initialised (arate)\n"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("transeg: not initialised (arate)\n"));
+    }
+    if (UNLIKELY(offset)) memset(rs, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rs[nsmps], '\0', early*sizeof(MYFLT));
     }
     val = p->curval;                      /* sav the cur value    */
     if (p->segsrem) {                     /* if no more segs putk */
@@ -1753,13 +1848,13 @@ int trnseg(CSOUND *csound, TRANSEG *p)
         p->curval = val;
       }
       if (p->alpha == FL(0.0)) {
-        for (n=0; n<nsmps; n++) {
+        for (n=offset; n<nsmps; n++) {
           rs[n] = val;
           val += p->curinc;
         }
       }
       else {
-        for (n=0; n<nsmps; n++) {
+        for (n=offset; n<nsmps; n++) {
           rs[n] = val;
           p->curx += p->alpha;
           val = segp->val + p->curinc *
@@ -1769,7 +1864,7 @@ int trnseg(CSOUND *csound, TRANSEG *p)
       p->curval = val;
       return OK;
 putk:
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         rs[n] = val;
       }
     }
@@ -1806,15 +1901,16 @@ int trnsetr(CSOUND *csound, TRANSEG *p)
       double dur = (double)**argp++;
       MYFLT alpha = **argp++;
       MYFLT nxtval = **argp++;
-      MYFLT d = dur * csound->esr;
+      MYFLT d = dur * CS_ESR;
       if ((segp->cnt = (int32)(d + FL(0.5))) < 0)
         segp->cnt = 0;
       else
-        segp->cnt = (int32)(dur * csound->ekr);
+        segp->cnt = (int32)(dur * CS_EKR);
         segp->nxtpt = nxtval;
       segp->val = val;
       if (alpha == FL(0.0)) {
         segp->c1 = (nxtval-val)/d;
+        //printf("alpha zero val=%f c1=%f\n", segp->val, segp->c1);
       }
       else {
         p->lastalpha = alpha;
@@ -1832,6 +1928,14 @@ int trnsetr(CSOUND *csound, TRANSEG *p)
     p->xtra = relestim;
     if (relestim > p->h.insdshead->xtratim)
       p->h.insdshead->xtratim = (int)relestim;
+    /* {  */
+    /*   int i; */
+    /*   int nseg = p->INOCOUNT / 3; */
+    /*   NSEG *segp = p->cursegp; */
+    /*   for (i=0; i<nseg; i++) */
+    /*     printf("cnt=%d alpha=%f val=%f nxtpt=%f c1=%f\n", */
+    /*      segp[i].cnt, segp[i].alpha, segp[i].val, segp[i].nxtpt, segp[i].c1); */
+    /* } */
     return OK;
 }
 
@@ -1839,11 +1943,13 @@ int ktrnsegr(CSOUND *csound, TRANSEG *p)
 {
     *p->rslt = p->curval;               /* put the cur value    */
     if (UNLIKELY(p->auxch.auxp==NULL)) { /* RWD fix */
-      csound->Die(csound, Str("Error: transeg not initialised (krate)\n"));
+      csound->PerfError(csound,p->h.insdshead,
+                        Str("Error: transeg not initialised (krate)\n"));
     }
     if (p->segsrem) {                   /* done if no more segs */
-        NSEG        *segp;
+      NSEG        *segp;
       if (p->h.insdshead->relesing && p->segsrem > 1) {
+        //printf("releasing\n");
         while (p->segsrem > 1) {        /* reles flag new:      */
           segp = ++p->cursegp;          /*   go to last segment */
           p->segsrem--;
@@ -1851,6 +1957,8 @@ int ktrnsegr(CSOUND *csound, TRANSEG *p)
         segp->cnt = p->xtra>=0 ? p->xtra : p->h.insdshead->xtratim;
         if (segp->alpha == FL(0.0)) {
           segp->c1 = (p->finalval-p->curval)/segp->cnt;
+          //printf("finalval = %f curval = %f, cnt = %d c1 = %f\n",
+          //       p->finalval, p->curval, segp->cnt, segp->c1);
         }
         else {
           /* this is very wrong */
@@ -1866,20 +1974,24 @@ int ktrnsegr(CSOUND *csound, TRANSEG *p)
           if (!(--p->segsrem)) return OK;    /*   seg Z now done all */
         segp = ++p->cursegp;            /*   find the next      */
       newm:
+        //printf("curcnt = %d seg/cnt = %d\n", p->curcnt, segp->cnt);
         if (!(p->curcnt = segp->cnt)) { /*   nonlen = discontin */
           p->curval = segp->nxtpt;      /*   poslen = new slope */
+          //printf("curval = %f\n", p->curval);
           goto chk1;
         }
         p->curinc = segp->c1;
         p->alpha = segp->alpha;
         p->curx = FL(0.0);
       }
-      if (p->alpha == FL(0.0))
-        p->curval += p->curinc*csound->ksmps;   /* advance the cur val  */
+      if (p->alpha == FL(0.0)) {
+        p->curval += p->curinc/**CS_KSMPS*/;   /* advance the cur val  */
+        //printf("curval = %f\n", p->curval);
+      }
       else
         p->curval = p->cursegp->val + p->curinc *
           (FL(1.0) - EXP(p->curx));
-      p->curx += (MYFLT)csound->ksmps*p->alpha;
+      p->curx += /* (MYFLT)CS_KSMPS* */ p->alpha;
     }
     return OK;
 }
@@ -1887,9 +1999,17 @@ int ktrnsegr(CSOUND *csound, TRANSEG *p)
 int trnsegr(CSOUND *csound, TRANSEG *p)
 {
     MYFLT  val, *rs = p->rslt;
-    int         n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     if (UNLIKELY(p->auxch.auxp==NULL)) {
-      return csound->PerfError(csound, Str("transeg: not initialised (arate)\n"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("transeg: not initialised (arate)\n"));
+    }
+    if (UNLIKELY(offset)) memset(rs, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rs[nsmps], '\0', early*sizeof(MYFLT));
     }
     val = p->curval;                      /* sav the cur value    */
     if (LIKELY(p->segsrem)) {             /* if no more segs putk */
@@ -1931,14 +2051,14 @@ int trnsegr(CSOUND *csound, TRANSEG *p)
         p->curval = val;
       }
       if (p->alpha == FL(0.0)) {
-        for (n=0; n<nsmps; n++) {
+        for (n=offset; n<nsmps; n++) {
           rs[n] = val;
           val += p->curinc;
         }
       }
       else {
         segp = p->cursegp;
-        for (n=0; n<nsmps; n++) {
+        for (n=offset; n<nsmps; n++) {
           rs[n] = val;
           p->curx += p->alpha;
           val = segp->val + p->curinc * (FL(1.0) - EXP(p->curx));
@@ -1947,7 +2067,7 @@ int trnsegr(CSOUND *csound, TRANSEG *p)
       p->curval = val;
       return OK;
     putk:
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         rs[n] = val;
       }
     }
@@ -1968,7 +2088,9 @@ int varicolset(CSOUND *csound, VARI *p)
 
 int varicol(CSOUND *csound, VARI *p)
 {
-    int         n, nsmps = csound->ksmps;
+    uint32_t    offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t    n, nsmps = CS_KSMPS;
     MYFLT       beta = *p->beta;
     MYFLT       sq1mb2 = p->sq1mb2;
     MYFLT       lastx = p->last;
@@ -1983,7 +2105,12 @@ int varicol(CSOUND *csound, VARI *p)
        ampmod = p->ampmod = FL(0.785)/(FL(1.0)+p->lastbeta);
     }
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(rslt, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rslt[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       MYFLT rnd = FL(2.0) * (MYFLT) rand_31(csound) / FL(2147483645) - FL(1.0);
       lastx = lastx * beta + sq1mb2 * rnd;
       rslt[n] = lastx * *kamp * ampmod;
@@ -2015,7 +2142,9 @@ int lpf18set(CSOUND *csound, LPF18 *p)
 
 int lpf18db(CSOUND *csound, LPF18 *p)
 {
-    int         n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT kfcn = FL(2.0) * *p->fco * csound->onedsr;
     MYFLT kp   = ((-FL(2.7528)*kfcn + FL(3.0429))*kfcn +
                   FL(1.718))*kfcn - FL(0.9984);
@@ -2034,7 +2163,12 @@ int lpf18db(CSOUND *csound, LPF18 *p)
     MYFLT lastin = p->lastin;
     double value = 1.0+(dist*(1.5+2.0*(double)kres*(1.0-(double)kfcn)));
 
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       MYFLT ax1   = lastin;
       MYFLT ay11  = ay1;
       MYFLT ay31  = ay2;
@@ -2061,10 +2195,10 @@ int lpf18db(CSOUND *csound, LPF18 *p)
 int wavesetset(CSOUND *csound, BARRI *p)
 {
     if (*p->len == FL(0.0))
-      p->length = 1 + (int)(p->h.insdshead->p3 * csound->esr * FL(0.5));
+      p->length = 1 + (int)(p->h.insdshead->p3 * CS_ESR * FL(0.5));
     else
       p->length = 1 + (int)*p->len;
-    if (UNLIKELY(p->length <= 1)) p->length = (int)csound->esr;
+    if (UNLIKELY(p->length <= 1)) p->length = (int)CS_ESR;
     csound->AuxAlloc(csound, (int32)p->length*sizeof(MYFLT), &p->auxch);
     p->cnt = 1;
     p->start = 0;
@@ -2082,9 +2216,16 @@ int waveset(CSOUND *csound, BARRI *p)
     MYFLT *out = p->ar;
     int   index = p->end;
     MYFLT *insert = (MYFLT*)(p->auxch.auxp) + index;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->noinsert) goto output;
-    for (n=0;n<nsmps;n++) {                        /* Deal with inputs */
+    for (n=offset;n<nsmps;n++) {                        /* Deal with inputs */
       *insert++ = in[n];
       if (++index ==  p->start) {
         p->noinsert = 1;
@@ -2099,7 +2240,7 @@ int waveset(CSOUND *csound, BARRI *p)
     p->end = index;
     index = p->current;
     insert = (MYFLT*)(p->auxch.auxp) + index;
-    for (n=0;n<nsmps;n++) {
+    for (n=offset;n<nsmps;n++) {
       MYFLT samp = *insert++;
       index ++;
       if (index==p->length) {
@@ -2154,9 +2295,12 @@ int medfilt(CSOUND *csound, MEDFILT *p)
     int maxwind = p->maxwind;
     int kwind = MYFLT2LONG(*p->kwind);
     int index = p->ind;
-    int n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     if (UNLIKELY(p->b.auxp==NULL)) {
-      return csound->PerfError(csound, Str("median: not initialised (arate)\n"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("median: not initialised (arate)\n"));
     }
     if (UNLIKELY(kwind > maxwind)) {
       csound->Warning(csound,
@@ -2164,7 +2308,12 @@ int medfilt(CSOUND *csound, MEDFILT *p)
                       kwind, maxwind);
       kwind = maxwind;
     }
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(aout, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&aout[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       MYFLT x = asig[n];
       buffer[index++] = x;
 
@@ -2203,7 +2352,8 @@ int kmedfilt(CSOUND *csound, MEDFILT *p)
     int kwind = MYFLT2LONG(*p->kwind);
     int index = p->ind;
     if (UNLIKELY(p->b.auxp==NULL)) {
-      return csound->PerfError(csound, Str("median: not initialised (krate)\n"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("median: not initialised (krate)\n"));
     }
     if (UNLIKELY(kwind > maxwind)) {
       csound->Warning(csound,
