@@ -1,5 +1,5 @@
 /*
-    crossfm.c:
+    pvlock.c:
 
     Copyright (C) 2009 V Lazzarini
 
@@ -28,25 +28,32 @@
 
 typedef struct dats{
   OPDS h;
-  MYFLT *out[MAXOUTS], *time, *kamp, *kpitch, *knum, *klock, *iN, *idecim, *konset, *offset, *dbthresh;
-  int cnt, hsize, curframe, N, decim,tscale,nchans;
+  MYFLT *out[MAXOUTS], *time, *kamp, *kpitch, *knum, *klock, *iN,
+        *idecim, *konset, *offset, *dbthresh;
+  int cnt, hsize, curframe, N, decim,tscale;
+  unsigned int nchans;
   double pos;
   MYFLT accum;
-  AUXCH outframe[MAXOUTS], win, bwin[MAXOUTS], fwin[MAXOUTS], nwin[MAXOUTS], prev[MAXOUTS], framecount[MAXOUTS];
+  AUXCH outframe[MAXOUTS], win, bwin[MAXOUTS], fwin[MAXOUTS],
+        nwin[MAXOUTS], prev[MAXOUTS], framecount[MAXOUTS];
 } DATASPACE;
 
 
 static int sinit(CSOUND *csound, DATASPACE *p)
 {
 
-  int N =  *p->iN, i,size, nchans;
+    int N =  *p->iN, ui;
+    unsigned int nchans, i;
+    unsigned int size;
     int decim = *p->idecim;
 
+
     if (N) {
+
       for (i=0; N; i++){
         N >>= 1;
       }
-      N = (int) pow(2., i-1.);
+      N = (int) pow(2.0, i-1);
     } else N = 2048;
     if (decim == 0) decim = 4;
 
@@ -55,11 +62,13 @@ static int sinit(CSOUND *csound, DATASPACE *p)
     p->curframe = 0;
 
     nchans = csound->GetOutputArgCnt(p);
+
   if (UNLIKELY(nchans < 1 || nchans > MAXOUTS))
-      csound->Die(csound, Str("invalid number of output arguments"));
+      csound->InitError(csound, Str("invalid number of output arguments"));
     p->nchans = nchans;
 
     for (i=0; i < nchans; i++){
+
     size = (N+2)*sizeof(MYFLT);
     if (p->fwin[i].auxp == NULL || p->fwin[i].size < size)
       csound->AuxAlloc(csound, size, &p->fwin[i]);
@@ -82,24 +91,31 @@ static int sinit(CSOUND *csound, DATASPACE *p)
     if (p->win.auxp == NULL || p->win.size < size)
       csound->AuxAlloc(csound, size, &p->win);
 
-    for (i=0; i < N; i++)
-      ((MYFLT *)p->win.auxp)[i] = 0.5 - 0.5*cos(i*2*PI/N);
+    {
+      MYFLT x = FL(2.0)*PI_F/N;
+      for (ui=0; ui < N; ui++)
+        ((MYFLT *)p->win.auxp)[ui] = FL(0.5) - FL(0.5)*COS((MYFLT)ui*x);
+    }
 
     p->N = N;
     p->decim = decim;
+
     return OK;
 }
 
 
 static int sprocess(CSOUND *csound, DATASPACE *p)
 {
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
     MYFLT pitch = *p->kpitch, *time = p->time, lock = *p->klock,
       *out, amp =*p->kamp;
     MYFLT *tab, frac;
     FUNC *ft;
     int N = p->N, hsize = p->hsize, cnt = p->cnt, nchans = p->nchans;
-    int ksmps = csound->GetKsmps(csound), n;
-    int sizefrs, size, post, i, j, spos = p->pos;
+    int nsmps = CS_KSMPS, n;
+    int sizefrs, size, post, i, j;
+    long spos = p->pos;
     double pos;
     MYFLT *fwin, *bwin, in,
       *prev, *win = (MYFLT *) p->win.auxp;
@@ -108,7 +124,22 @@ static int sprocess(CSOUND *csound, DATASPACE *p)
     int *framecnt;
     int curframe = p->curframe, decim = p->decim;
 
-    for (n=0; n < ksmps; n++) {
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      for (j=0; j < nchans; j++) {
+      out = p->out[j];
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+      }
+    }
+    if (UNLIKELY(offset)) {
+     for (j=0; j < nchans; j++) {
+        out = p->out[j];
+        memset(out, '\0', offset*sizeof(MYFLT));
+     }
+    }
+
+
+    for (n=offset; n < nsmps; n++) {
 
       if (cnt == hsize) {
         /* audio samples are stored in a function table */
@@ -117,9 +148,10 @@ static int sprocess(CSOUND *csound, DATASPACE *p)
         size = ft->flen;
 
         if (UNLIKELY((int) ft->nchanls != nchans))
-          return csound->PerfError(csound, Str("number of output arguments "
-                                               "inconsistent with number of "
-                                               "sound file channels"));
+          return csound->PerfError(csound, p->h.insdshead,
+                                   Str("number of output arguments "
+                                       "inconsistent with number of "
+                                       "sound file channels"));
 
 
 
@@ -127,10 +159,10 @@ static int sprocess(CSOUND *csound, DATASPACE *p)
            time[n] is current read position in secs
            esr is sampling rate
         */
-        spos  = hsize*(int)((time[n])*csound->esr/hsize);
+        spos  = hsize*(long)((time[n])*CS_ESR/hsize);
         sizefrs = size/nchans;
-        while(spos > sizefrs - N) spos -= sizefrs;
-        while(spos <= hsize)  spos += sizefrs;
+        while(spos > sizefrs) spos -= sizefrs;
+        while(spos <= 0)  spos += sizefrs;
         pos = spos;
 
         for (j = 0; j < nchans; j++) {
@@ -150,17 +182,21 @@ static int sprocess(CSOUND *csound, DATASPACE *p)
             frac = pos  - post;
             post *= nchans;
             post += j;
-            if (post >= 0 && post < size)
-              in = tab[post] + frac*(tab[post+nchans] - tab[post]);
-            else in =  (MYFLT) 0;
+            while (post < 0) post += size;
+            while (post >= size) post -= size;
+
+            in = tab[post] + frac*(tab[post+nchans] - tab[post]);
+
             fwin[i] = in * win[i]; /* window it */
             /* back windo, bwin */
             post = (int) (pos - hsize*pitch);
             post *= nchans;
             post += j;
-            if (post >= 0 && post < size)
-              in =  tab[post] + frac*(tab[post+nchans] - tab[post]);
-            else in =  (MYFLT) 0;
+            while(post < 0) post += size;
+            while(post >= size) post -= size;
+
+            in = tab[post] + frac*(tab[post+nchans] - tab[post]);
+
             bwin[i] = in * win[i];  /* window it */
             /* increment read pos according to pitch transposition */
             pos += pitch;
@@ -269,13 +305,13 @@ static int sprocess(CSOUND *csound, DATASPACE *p)
 
 static int sinit2(CSOUND *csound, DATASPACE *p)
 {
-    int size,i;
+    unsigned int size,i;
     sinit(csound, p);
     size = p->N*sizeof(MYFLT);
     for (i=0; i < p->nchans; i++)
     if (p->nwin[i].auxp == NULL || p->nwin[i].size < size)
       csound->AuxAlloc(csound, size, &p->nwin[i]);
-    p->pos = *p->offset*csound->esr + p->hsize;
+    p->pos = *p->offset*CS_ESR + p->hsize;
     p->tscale  = 0;
     p->accum = 0;
     return OK;
@@ -283,12 +319,14 @@ static int sinit2(CSOUND *csound, DATASPACE *p)
 
 static int sprocess2(CSOUND *csound, DATASPACE *p)
 {
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
     MYFLT pitch = *p->kpitch, time = *p->time, lock = *p->klock;
     MYFLT *out, amp =*p->kamp;
     MYFLT *tab,frac,  dbtresh = *p->dbthresh;
     FUNC *ft;
     int N = p->N, hsize = p->hsize, cnt = p->cnt, sizefrs, nchans = p->nchans;
-    int  ksmps = csound->GetKsmps(csound), n;
+    int  nsmps = CS_KSMPS, n;
     int size, post, i, j;
     double pos, spos = p->pos;
     MYFLT *fwin, *bwin;
@@ -299,7 +337,21 @@ static int sprocess2(CSOUND *csound, DATASPACE *p)
     int *framecnt, curframe = p->curframe;
     int decim = p->decim;
 
-    for (n=0; n < ksmps; n++) {
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      for (j=0; j < nchans; j++) {
+      out = p->out[j];
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+      }
+    }
+    if (UNLIKELY(offset)) {
+     for (j=0; j < nchans; j++) {
+        out = p->out[j];
+        memset(out, '\0', offset*sizeof(MYFLT));
+     }
+    }
+
+    for (n=offset; n < nsmps; n++) {
 
       if (cnt == hsize){
         ft = csound->FTnp2Find(csound,p->knum);
@@ -319,13 +371,14 @@ static int sprocess2(CSOUND *csound, DATASPACE *p)
           p->tscale = 1;
         }
         if (UNLIKELY((int) ft->nchanls != nchans))
-          return csound->PerfError(csound, Str("number of output arguments "
-                                               "inconsistent with number of "
-                                               "sound file channels"));
+          return csound->PerfError(csound, p->h.insdshead,
+                                   Str("number of output arguments "
+                                       "inconsistent with number of "
+                                       "sound file channels"));
 
         sizefrs = size/nchans;
-        while(spos > sizefrs - N - hsize) spos -= sizefrs;
-        while(spos <= hsize)  spos += sizefrs;
+        while(spos > sizefrs) spos -= sizefrs;
+        while(spos <= 0)  spos += sizefrs;
         pos = spos;
 
         for (j = 0; j < nchans; j++) {
@@ -342,9 +395,11 @@ static int sprocess2(CSOUND *csound, DATASPACE *p)
             frac = pos  - post;
             post *= nchans;
             post += j;
-            if (post >= 0 && post < size)
-              in = tab[post] + frac*(tab[post+nchans] - tab[post]);
-            else in =  (MYFLT) 0;
+
+            while(post < 0) post += size;
+            while(post >= size) post -= size;
+            in = tab[post] + frac*(tab[post+nchans] - tab[post]);
+
             fwin[i] = in * win[i];
 
             post = (int) (pos - hsize*pitch);
@@ -357,8 +412,9 @@ static int sprocess2(CSOUND *csound, DATASPACE *p)
             post = (int) pos + hsize;
             post *= nchans;
             post += j;
-            if (post >= 0 && post < size) in =  tab[post];
-            else in =  (MYFLT) 0;
+            while(post < 0) post += size;
+            while(post >= size) post -= size;
+            in = tab[post] + frac*(tab[post+nchans] - tab[post]);
             nwin[i] = in * win[i];
             pos += pitch;
           }
@@ -530,12 +586,12 @@ static int pvslockproc(CSOUND *csound, PVSLOCK *p)
 }
 
 static OENTRY pvlock_localops[] = {
-  {"mincer", sizeof(DATASPACE), 5, "mm", "akkkkoo",
+  {"mincer", sizeof(DATASPACE), 0, 5, "mm", "akkkkoo",
                                                (SUBR)sinit, NULL,(SUBR)sprocess },
-  {"temposcal", sizeof(DATASPACE), 5, "mm", "kkkkkooPOP",
+  {"temposcal", sizeof(DATASPACE), 0, 5, "mm", "kkkkkooPOP",
                                                (SUBR)sinit2, NULL,(SUBR)sprocess2 },
-  {"pvslock", sizeof(PVSLOCK), 3, "f", "fk", (SUBR) pvslockset,
+  {"pvslock", sizeof(PVSLOCK), 0, 3, "f", "fk", (SUBR) pvslockset,
          (SUBR) pvslockproc},
 };
 
-LINKAGE1(pvlock_localops)
+LINKAGE_BUILTIN(pvlock_localops)
