@@ -28,7 +28,7 @@
 *   3 March 1996 John ffitch                                   *
 \***************************************************************/
 
-#include "csdl.h"
+#include "stdopcod.h"
 #include "repluck.h"
 
 static int wgpsetin(CSOUND *, WGPLUCK2 *);
@@ -54,9 +54,9 @@ static int wgpsetin(CSOUND *csound, WGPLUCK2 *p)
     DelayLine   *lower_rail;
     MYFLT   plk = *p->plk;
                                 /* Initialize variables....*/
-    npts = (int)(csound->esr / *p->icps);/* Length of full delay */
+    npts = (int)(CS_ESR / *p->icps);/* Length of full delay */
     while (npts < 512) {        /* Minimum rail length is 256 */
-      npts += (int)(csound->esr / *p->icps);
+      npts += (int)(CS_ESR / *p->icps);
       scale++;
     }
     rail_len = npts/2/* + 1*/;      /* but only need half length */
@@ -150,7 +150,9 @@ static MYFLT getvalue(DelayLine *dl, int position)
 static int wgpluck(CSOUND *csound, WGPLUCK2 *p)
 {
     MYFLT   *ar, *ain;
-    int     n,nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   yp0,ym0,ypM,ymM;
     DelayLine   *upper_rail;
     DelayLine   *lower_rail;
@@ -170,7 +172,8 @@ static int wgpluck(CSOUND *csound, WGPLUCK2 *p)
     reflect    = FL(1.0) - (FL(1.0) - reflect)/(MYFLT)scale; /* For over sapling */
     upper_rail = (DelayLine*)p->upper.auxp;
     lower_rail = (DelayLine*)p->lower.auxp;
-    pickup     = (int)((MYFLT)OVERCNT * *(p->pickup) * p->rail_len); /* fract delays */
+    /* fractional delays */
+    pickup     = (int)((MYFLT)OVERCNT * *(p->pickup) * p->rail_len);
     pickfrac   = pickup & OVERMSK;
     pickup     = pickup>>OVERSHT;
     if (UNLIKELY(pickup<0 || pickup > p->rail_len)) {
@@ -180,7 +183,12 @@ static int wgpluck(CSOUND *csound, WGPLUCK2 *p)
       pickup   = pickup>>OVERSHT;
     }
 
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       MYFLT s, s1;
       s = getvalue(upper_rail, pickup) + getvalue(lower_rail, pickup);
       s1 = getvalue(upper_rail, pickup+1) + getvalue(lower_rail, pickup+1);
@@ -237,7 +245,7 @@ static int wgpluck(CSOUND *csound, WGPLUCK2 *p)
 static int stresonset(CSOUND *csound, STRES *p)
 {
     int n;
-    p->size = (int) (csound->esr/20);   /* size of delay line */
+    p->size = (int) (CS_ESR/20);   /* size of delay line */
     csound->AuxAlloc(csound, p->size*sizeof(MYFLT), &p->aux);
     p->Cdelay = (MYFLT*) p->aux.auxp; /* delay line */
     p->LPdelay = p->APdelay = FL(0.0); /* reset the All-pass and Low-pass delays */
@@ -254,7 +262,10 @@ static int streson(CSOUND *csound, STRES *p)
     MYFLT g = *p->ifdbgain;
     MYFLT freq;
     double a, s, w, sample, tdelay, fracdelay;
-    int delay, n, nsmps = csound->ksmps;
+    int delay;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int rp = p->rpointer, wp = p->wpointer;
     int size = p->size;
     MYFLT       APdelay = p->APdelay;
@@ -263,12 +274,17 @@ static int streson(CSOUND *csound, STRES *p)
 
     freq = *p->afr;
     if (freq < FL(20.0)) freq = FL(20.0);   /* lowest freq is 20 Hz */
-    tdelay = csound->esr/freq;
+    tdelay = CS_ESR/freq;
     delay = (int) (tdelay - 0.5); /* comb delay */
     fracdelay = tdelay - (delay + 0.5); /* fractional delay */
     vdt = size - delay;       /* set the var delay */
     a = (1.0-fracdelay)/(1.0+fracdelay);   /* set the all-pass gain */
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       /* GetSample(p); */
       MYFLT tmpo;
       rp = (vdt + wp);
@@ -291,9 +307,9 @@ static int streson(CSOUND *csound, STRES *p)
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "repluck", S(WGPLUCK2), 5, "a",  "ikikka",(SUBR)wgpsetin, NULL, (SUBR)wgpluck},
-{ "wgpluck2",S(WGPLUCK2), 5, "a",  "ikikk", (SUBR)wgpset,   NULL, (SUBR)wgpluck},
-{ "streson", S(STRES),    5, "a",  "akk",  (SUBR)stresonset, NULL, (SUBR)streson}
+{ "repluck", S(WGPLUCK2), 0, 5, "a",  "ikikka",(SUBR)wgpsetin, NULL, (SUBR)wgpluck},
+{ "wgpluck2",S(WGPLUCK2), 0, 5, "a",  "ikikk", (SUBR)wgpset,   NULL, (SUBR)wgpluck},
+{ "streson", S(STRES),    0, 5, "a",  "akk",  (SUBR)stresonset, NULL, (SUBR)streson}
 };
 
 int repluck_init_(CSOUND *csound)
