@@ -37,7 +37,7 @@ void DOWNset(CSOUND *p, DOWNDAT *downdp, int32 npts)
 {
     int32 nbytes = npts * sizeof(MYFLT);
 
-    if (downdp->auxch.auxp == NULL || downdp->auxch.size != nbytes)
+    if (downdp->auxch.auxp == NULL || downdp->auxch.size != (uint32_t)nbytes)
       p->AuxAlloc(p, nbytes, &downdp->auxch);
     downdp->npts = npts;
 }
@@ -46,7 +46,7 @@ void SPECset(CSOUND *p, SPECDAT *specdp, int32 npts)
 {
     int32 nbytes = npts * sizeof(MYFLT);
 
-    if (specdp->auxch.auxp == NULL || nbytes != specdp->auxch.size)
+    if (specdp->auxch.auxp == NULL || (uint32_t)nbytes != specdp->auxch.size)
       p->AuxAlloc(p, nbytes, &specdp->auxch);
     specdp->npts = npts;
 }
@@ -65,14 +65,15 @@ int spectset(CSOUND *csound, SPECTRUM *p)
     SPECDAT *specp = p->wsig;
 
     /* for mac roundoff */
-    p->timcount = (int)(csound->ekr * *p->iprd + FL(0.001));
+    p->timcount = (int)(CS_EKR * *p->iprd + FL(0.001));
     nocts = (int)*p->iocts;
     nfreqs = (int)*p->ifrqs;
     ncoefs = nocts * nfreqs;
     Q = *p->iq;
     hanning = (*p->ihann) ? 1 : 0;
     p->dbout = (int)*p->idbout;
-    if ((p->disprd = (int)(csound->ekr * *p->idisprd)) < 0)  p->disprd = 0;
+    if ((p->disprd = (int)(CS_EKR * *p->idisprd)) < 0)
+      p->disprd = 0;
 
     if (UNLIKELY(p->timcount <= 0))
       return csound->InitError(csound, Str("illegal iprd"));
@@ -107,12 +108,12 @@ int spectset(CSOUND *csound, SPECTRUM *p)
                       (hanning) ? "hanning":"hamming", outstring[p->dbout]);
 
       if (p->h.optext->t.intype == 'k') {
-        dwnp->srate = csound->ekr;            /* define the srate           */
+        dwnp->srate = CS_EKR;            /* define the srate */
         p->nsmps = 1;
       }
       else {
-        dwnp->srate = csound->esr;
-        p->nsmps = csound->ksmps;
+        dwnp->srate = CS_ESR;
+        p->nsmps = CS_KSMPS;
       }
       hicps = dwnp->srate * 0.375;            /* top freq is 3/4 pi/2 ...   */
       oct = log(hicps / ONEPT) / LOGTWO;      /* octcps()  (see aops.c)     */
@@ -248,13 +249,17 @@ int spectrum(CSOUND *csound, SPECTRUM *p)
 {
     MYFLT   a, b, *dftp, *sigp = p->signal, SIG, yt1, yt2;
     int     nocts, nsmps = p->nsmps, winlen;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
     DOWNDAT *downp = &p->downsig;
     OCTDAT  *octp;
     SPECDAT *specp;
     double  c;
 
+    if (UNLIKELY(early)) nsmps -= early;
     do {
       SIG = *sigp++;                        /* for each source sample:     */
+      if (offset--) SIG = FL(0.0);          /* for sample accuracy         */
       octp = downp->octdata;                /*   align onto top octave     */
       nocts = downp->nocts;
       do {                                  /*   then for each oct:        */
@@ -280,10 +285,10 @@ int spectrum(CSOUND *csound, SPECTRUM *p)
       } while (!(++octp->scount & 01) && octp++); /* send alt samps to nxtoct */
     } while (--nsmps);
 
-    if (p->disprd)                         /* if displays requested,   */
-      if (!(--p->dcountdown)) {            /*   on countdown           */
+    if (p->disprd)                               /* if displays requested,   */
+      if (!(--p->dcountdown)) {                  /*   on countdown           */
         linocts(downp, (MYFLT *)p->auxch2.auxp); /*   linearize the oct bufs */
-        csound->display(csound, &p->octwindow);    /*      & display           */
+        csound->display(csound, &p->octwindow);  /*      & display           */
         p->dcountdown = p->disprd;
       }
 
@@ -300,7 +305,7 @@ int spectrum(CSOUND *csound, SPECTRUM *p)
       int    len, *lenp, *offp, nfreqs;
       MYFLT    *begp, *curp, *endp, *linbufp;
       int      len2;
-      octp--;                                /* for each oct (low to high)   */
+      octp--;                                /* for each oct (low to high) */
       begp = octp->begp;
       curp = octp->curp;
       endp = octp->endp;
@@ -343,7 +348,7 @@ int spectrum(CSOUND *csound, SPECTRUM *p)
         *dftp++ = (MYFLT)c;                  /* store in out spectrum   */
       }
     }
-    specp->ktimstamp = csound->kcounter;     /* time-stamp the output   */
+    specp->ktimstamp = CS_KCNT;     /* time-stamp the output   */
     return OK;
 }
 
@@ -356,20 +361,21 @@ int spectrum(CSOUND *csound, SPECTRUM *p)
 /*     DOWNDAT *downp = p->dsig; */
 /*     SPECDAT *specp = p->wsig; */
 
-/*     p->timcount = csound->ekr * *p->iprd; */
+/*     p->timcount = CS_EKR * *p->iprd; */
 /*     nfreqs = *p->ifrqs; */
 /*     Q = *p->iq; */
 /*     hanning = (*p->ihann) ? 1 : 0; */
 /*     if ((p->dbout = *p->idbout) && p->dbout != 1 && p->dbout != 2) { */
-/*       return csound->InitError(csound, Str("noctdft: unknown dbout code of %d"), */
+/*       return csound->InitError(csound,
+                                  Str("noctdft: unknown dbout code of %d"), */
 /*                                        p->dbout); */
 /*     } */
 /*     nocts = downp->nocts; */
 /*     ncoefs = nocts * nfreqs; */
-/*     if (nfreqs != p->nfreqs || Q != p->curq         /\* if anything changed *\/ */
+/*     if (nfreqs != p->nfreqs || Q != p->curq  /\* if anything changed *\/ */
 /*         || p->timcount <= 0 || Q <= 0. */
 /*         || hanning != p->hanning */
-/*         || ncoefs != p->ncoefs) {                   /\*     make new tables *\/ */
+/*         || ncoefs != p->ncoefs) {       /\*     make new tables *\/ */
 /*       double      basfrq, curfrq, frqmlt, Qfactor; */
 /*       double      theta, a, windamp, onedws, pidws; */
 /*       MYFLT       *sinp, *cosp; */
@@ -390,16 +396,17 @@ int spectrum(CSOUND *csound, SPECTRUM *p)
 /*       p->curq = Q; */
 /*       p->hanning = hanning; */
 /*       p->ncoefs = ncoefs; */
-/*       basfrq = downp->hifrq/2.0 * TWOPI/downp->srate; /\* oct below retuned top *\/ */
+/*       basfrq = downp->hifrq/2.0 * TWOPI/downp->srate;
+                                                /\* oct below retuned top *\/ */
 /*       frqmlt = pow(2.0,1.0/(double)nfreqs);    /\* nfreq interval mult *\/ */
 /*       Qfactor = TWOPI * Q;      /\* Was incorrect value for 2pi?? *\/ */
 /*       curfrq = basfrq; */
 /*       for (sumk=0,wsizp=p->winlen,n=nfreqs; n--; ) { */
-/*         *wsizp++ = k = Qfactor/curfrq + 0.5;         /\* calc window sizes  *\/ */
-/*         sumk += k;                                   /\*   and find total   *\/ */
+/*         *wsizp++ = k = Qfactor/curfrq + 0.5; /\* calc window sizes  *\/ */
+/*         sumk += k;                           /\*   and find total   *\/ */
 /*         curfrq *= frqmlt; */
 /*       } */
-/*       if ((windsiz = *(p->winlen)) > nsamps) {        /\* chk longest windsiz *\/ */
+/*       if ((windsiz = *(p->winlen)) > nsamps) {/\* chk longest windsiz *\/ */
 /*         return csound->InitError(csound, Str("Q %4.1f needs %d samples, " */
 /*                                              "octdown has just %d"), */
 /*                                          Q, windsiz, nsamps); */
@@ -407,14 +414,16 @@ int spectrum(CSOUND *csound, SPECTRUM *p)
 /*       else csound->Message(csound, Str("noctdft: Q %4.1f uses %d of " */
 /*                                        "%d samps per octdown\n"), */
 /*                                    Q, windsiz, nsamps); */
-/*       auxsiz = (nsamps + 2*sumk) * sizeof(MYFLT);    /\* calc local space reqd *\/ */
-/*       csound->AuxAlloc(csound, (size_t)auxsiz, &p->auxch); /\* & alloc auxspace  *\/ */
+/*       auxsiz = (nsamps + 2*sumk) * sizeof(MYFLT);/\* calc local space reqd *\/ */
+/*       csound->AuxAlloc(csound, (size_t)auxsiz, &p->auxch);
+         /\* & alloc auxspace  *\/ */
 /*       fltp = (MYFLT *) p->auxch.auxp; */
-/*       p->linbufp = fltp;          fltp += nsamps; /\* linbuf must handle nsamps *\/ */
+/*       p->linbufp = fltp;          fltp += nsamps;
+         /\* linbuf must handle nsamps *\/ */
 /*       p->sinp = sinp = fltp;      fltp += sumk; */
-/*       p->cosp = cosp = fltp;                         /\* cos gets rem sumk  *\/ */
+/*       p->cosp = cosp = fltp;                 /\* cos gets rem sumk  *\/ */
 /*       wsizp = p->winlen; */
-/*       for (curfrq=basfrq,n=nfreqs; n--; ) {           /\* now fill tables *\/ */
+/*       for (curfrq=basfrq,n=nfreqs; n--; ) {     /\* now fill tables *\/ */
 /*         windsiz = *wsizp++; */
 /*         onedws = 1.0 / windsiz; */
 /*         pidws = PI / windsiz; */
@@ -428,7 +437,7 @@ int spectrum(CSOUND *csound, SPECTRUM *p)
 /*           *sinp++ = windamp * sin(theta); */
 /*           *cosp++ = windamp * cos(theta); */
 /*         } */
-/*         curfrq *= frqmlt;                           /\*   step by log freq  *\/ */
+/*         curfrq *= frqmlt;                     /\*   step by log freq  *\/ */
 /*       } */
 /*       if (*p->idsines != FL(0.0)) { */
 /*         /\* if reqd, display windowed sines immediately *\/ */
@@ -437,14 +446,14 @@ int spectrum(CSOUND *csound, SPECTRUM *p)
 /*         csound->display(csound, &p->dwindow); */
 /*       } */
 /*       SPECset(csound, */
-/*               specp, (long)ncoefs);                /\* prep the spec dspace *\/ */
-/*       specp->downsrcp = downp;                     /\*  & record its source *\/ */
+/*               specp, (long)ncoefs);          /\* prep the spec dspace *\/ */
+/*       specp->downsrcp = downp;               /\*  & record its source *\/ */
 /*     } */
-/*     specp->nfreqs = p->nfreqs;                 /\* save the spec descriptors *\/ */
+/*     specp->nfreqs = p->nfreqs;            \* save the spec descriptors *\/ */
 /*     specp->dbout = p->dbout; */
-/*     specp->ktimstamp = 0;                      /\* init specdata to not new  *\/ */
+/*     specp->ktimstamp = 0;                 \* init specdata to not new  *\/ */
 /*     specp->ktimprd = p->timcount; */
-/*     p->countdown = p->timcount;                /\*     & prime the countdown *\/ */
+/*     p->countdown = p->timcount;           \*     & prime the countdown *\/ */
 /*     return OK; */
 /* } */
 
@@ -458,11 +467,13 @@ int spectrum(CSOUND *csound, SPECTRUM *p)
 /*     MYFLT   a, b; */
 /*     double  c; */
 
-/*     if ((--p->countdown))  return;    /\* if not yet time for new spec, return *\/ */
+/*     if ((--p->countdown))  return;
+       /\* if not yet time for new spec, return *\/ */
 /*     if (p->auxch.auxp==NULL) { /\* RWD fix *\/ */
-/*       return csound->PerfError(csound, Str("noctdft: not initialised")); */
+/*       return csound->PerfError(csound, p->h.insdshead, */
+/*                                Str("noctdft: not initialised")); */
 /*     } */
-/*     p->countdown = p->timcount;            /\* else reset counter & proceed:   *\/ */
+/*     p->countdown = p->timcount;      /\* else reset counter & proceed:   *\/ */
 /*     downp = p->dsig; */
 /*     specp = p->wsig; */
 /*     nocts = downp->nocts; */
@@ -472,38 +483,38 @@ int spectrum(CSOUND *csound, SPECTRUM *p)
 /*       MYFLT  *bufp, *sinp, *cosp; */
 /*       int    len, *lenp, nfreqs; */
 /*       MYFLT   *begp, *curp, *endp; */
-/*       octp--;                              /\* for each octave (low to high)   *\/ */
+/*       octp--;                        /\* for each octave (low to high)   *\/ */
 /*       begp = octp->begp; */
 /*       curp = octp->curp; */
 /*       endp = octp->endp; */
 /*       wrap = curp - begp; */
 /*       bufp = p->linbufp; */
-/*       while (curp < endp)                    /\*   copy circbuf to linbuf   *\/ */
+/*       while (curp < endp)              /\*   copy circbuf to linbuf   *\/ */
 /*         *bufp++ = *curp++; */
 /*       for (curp=begp,len=wrap; len--; ) */
 /*         *bufp++ = *curp++; */
-/*       cosp = p->cosp;                        /\*   get start windowed sines *\/ */
+/*       cosp = p->cosp;                  /\*   get start windowed sines *\/ */
 /*       sinp = p->sinp; */
 /*       lenp = p->winlen; */
-/*       for (nfreqs=p->nfreqs; nfreqs--; ) {   /\*   now for each freq this oct: *\/ */
+/*       for (nfreqs=p->nfreqs; nfreqs--; ) { /\* now for each freq this oct: *\/ */
 /*         a = 0.0; */
 /*         b = 0.0; */
 /*         bufp = p->linbufp; */
-/*         for (len = *lenp++; len--; bufp++) {    /\*  apply windowed sine seg *\/ */
+/*         for (len = *lenp++; len--; bufp++) {/\*  apply windowed sine seg *\/ */
 /*           a += *bufp * *cosp++; */
 /*           b += *bufp * *sinp++; */
 /*         } */
-/*         c = a*a + b*b;                          /\*  get magnitude squared   *\/ */
-/*         if (!(p->dbout))                        /\*    & optionally convert  *\/ */
-/*           c = sqrt(c);                          /\*    to  mag or db         *\/ */
+/*         c = a*a + b*b;                    /\*  get magnitude squared   *\/ */
+/*         if (!(p->dbout))                  /\*    & optionally convert  *\/ */
+/*           c = sqrt(c);                    /\*    to  mag or db         *\/ */
 /*         else if (p->dbout == 1) { */
 /*           if (c < .001) c = .001; */
 /*           c = 10. * log10(c); */
 /*         } */
-/*         *dftp++ = c;                            /\* store in out spectrum   *\/ */
+/*         *dftp++ = c;                       /\* store in out spectrum   *\/ */
 /*       } */
 /*     } */
-/*     specp->ktimstamp = csound->kcounter;        /\* time-stamp the output   *\/ */
+/*     specp->ktimstamp = CS_KCNT;   /\* time-stamp the output   *\/ */
 /*     return OK; */
 /* } */
 
@@ -514,7 +525,7 @@ int spdspset(CSOUND *csound, SPECDISP *p)
     if (UNLIKELY(p->wsig->auxch.auxp==NULL)) {
       return csound->InitError(csound, Str("specdisp: not initialised"));
     }
-    if (UNLIKELY((p->timcount = (int)(csound->ekr * *p->iprd)) <= 0)) {
+    if (UNLIKELY((p->timcount = (int)(CS_EKR * *p->iprd)) <= 0)) {
       return csound->InitError(csound, Str("illegal iperiod"));
     }
     if (!(p->dwindow.windid)) {
@@ -552,7 +563,8 @@ int specdisp(CSOUND *csound, SPECDISP *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("specdisp: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("specdisp: not initialised"));
 }
 
 int sptrkset(CSOUND *csound, SPECPTRK *p)
@@ -570,7 +582,8 @@ int sptrkset(CSOUND *csound, SPECPTRK *p)
       p->fundp = (MYFLT *) p->wfund.auxch.auxp;
       p->winpts = npts;
         }
-    if ((p->ftimcnt = (int)(csound->ekr**p->ifprd)) > 0) {/* if displaying wfund */
+    if ((p->ftimcnt = (int)(CS_EKR**p->ifprd)) > 0) {
+      /* if displaying wfund */
       SPECDISP *fdp = &p->fdisplay;
       fdp->h = p->h;
       fdp->wsig = &p->wfund;                    /*  pass the param pntrs */
@@ -597,7 +610,7 @@ int sptrkset(CSOUND *csound, SPECPTRK *p)
     nfreqs = (MYFLT)inspecp->nfreqs;
     for (nn = 1; nn <= ptlmax; nn += inc)
       *dstp++ = (int) ((log((double) nn) / LOGTWO) * nfreqs + 0.5);
-    if ((rolloff = *p->irolloff) == 0. || rolloff == 1. || nptls == 1) {
+    if ((rolloff = *p->irolloff) == 0.0 || rolloff == 1.0 || nptls == 1) {
       p->rolloff = 0;
       weightsum = (MYFLT)nptls;
     } else {
@@ -623,7 +636,7 @@ int sptrkset(CSOUND *csound, SPECPTRK *p)
     fendp = fundp + inspecp->npts;
     if (flop < fundp) flop = fundp;
     if (fhip > fendp) fhip = fendp;
-    if (flop >= fhip) {         /* chk hi-lo range valid */
+    if (UNLIKELY(flop >= fhip)) {         /* chk hi-lo range valid */
       return csound->InitError(csound, Str("illegal lo-hi values"));
     }
     for (fp = fundp; fp < flop; )
@@ -683,7 +696,7 @@ int specptrk(CSOUND *csound, SPECPTRK *p)
 {
     SPECDAT *inspecp = p->wsig;
 
-    if (inspecp->ktimstamp == csound->kcounter) {   /* if inspectrum is new: */
+    if (inspecp->ktimstamp == CS_KCNT) {   /* if inspectrum is new: */
       MYFLT *inp = (MYFLT *) inspecp->auxch.auxp;
       MYFLT *endp = inp + inspecp->npts;
       MYFLT *inp2, sum, *fp;
@@ -693,8 +706,7 @@ int specptrk(CSOUND *csound, SPECPTRK *p)
       int32 lobin, hibin;
 
       if (UNLIKELY(inp==NULL)) goto err1;             /* RWD fix */
-      if ((kvar = *p->kvar) < FL(0.0))
-        kvar = -kvar;
+      kvar = FABS(*p->kvar);
       kval = p->playing == PLAYING ? p->kval : p->kvalsav;
       lobin = (int32)((kval-kvar) * inspecp->nfreqs); /* set lims of frq interest */
       hibin = (int32)((kval+kvar) * inspecp->nfreqs);
@@ -774,8 +786,7 @@ int specptrk(CSOUND *csound, SPECPTRK *p)
       kval = realbin / inspecp->nfreqs;        /*     & cvt to true decoct */
 
       if (p->playing == STARTING) {            /* STARTING mode:           */
-        if ((absdiff = kval - p->kvalsav) < FL(0.0))
-          absdiff = -absdiff;
+        absdiff = FABS(kval - p->kvalsav);
         confirms = (int)(absdiff * p->confact); /* get interval dependency  */
         if (p->jmpcount < confirms) {
           p->jmpcount += 1;                /* if not enough confirms,  */
@@ -787,8 +798,7 @@ int specptrk(CSOUND *csound, SPECPTRK *p)
           p->kinc = FL(0.0);
         }
       } else {                                 /* PLAYING mode:            */
-        if ((absdiff = kval - p->kval) < FL(0.0))
-          absdiff = -absdiff;
+        absdiff = FABS(kval - p->kval);
         confirms = (int)(absdiff * p->confact); /* get interval dependency  */
         if (p->jmpcount < confirms) {
           p->jmpcount += 1;                /* if not enough confirms,  */
@@ -816,7 +826,8 @@ int specptrk(CSOUND *csound, SPECPTRK *p)
       specdisp(csound,&p->fdisplay);
     return OK;
  err1:
-    return csound->PerfError(csound, Str("specptrk: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("specptrk: not initialised"));
 }
 
 int spsumset(CSOUND *csound, SPECSUM *p)
@@ -832,7 +843,7 @@ int specsum(CSOUND *csound, SPECSUM *p)
 {
     SPECDAT *specp = p->wsig;
     if (UNLIKELY(specp->auxch.auxp==NULL)) goto err1; /* RWD fix */
-    if (specp->ktimstamp == csound->kcounter) { /* if spectrum is new   */
+    if (specp->ktimstamp == CS_KCNT) { /* if spectrum is new   */
       MYFLT *valp = (MYFLT *) specp->auxch.auxp;
       MYFLT sum = FL(0.0);
       int32 n,npts = specp->npts;                /*   sum all the values */
@@ -848,7 +859,8 @@ int specsum(CSOUND *csound, SPECSUM *p)
       p->kval += p->kinc;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("specsum: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("specsum: not initialised"));
 }
 
 int spadmset(CSOUND *csound, SPECADDM *p)
@@ -887,7 +899,7 @@ int specaddm(CSOUND *csound, SPECADDM *p)
     if (UNLIKELY((p->wsig1->auxch.auxp==NULL) || /* RWD fix */
                  (p->wsig2->auxch.auxp==NULL) ||
                  (p->waddm->auxch.auxp==NULL))) goto err1;
-    if (p->wsig1->ktimstamp == csound->kcounter) {  /* if inspec1 is new:     */
+    if (p->wsig1->ktimstamp == CS_KCNT) {  /* if inspec1 is new:     */
       MYFLT *in1p = (MYFLT *) p->wsig1->auxch.auxp;
       MYFLT *in2p = (MYFLT *) p->wsig2->auxch.auxp;
       MYFLT *outp = (MYFLT *) p->waddm->auxch.auxp;
@@ -897,11 +909,12 @@ int specaddm(CSOUND *csound, SPECADDM *p)
       for (n=0;n<npts;n++) {
         outp[n] = in1p[n] + in2p[n] * mul2;         /* out = in1 + in2 * mul2 */
       }
-      p->waddm->ktimstamp = csound->kcounter;  /* mark the output spec as new */
+      p->waddm->ktimstamp = CS_KCNT;  /* mark the output spec as new */
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("specaddm: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("specaddm: not initialised"));
 }
 
 int spdifset(CSOUND *csound, SPECDIFF *p)
@@ -913,9 +926,9 @@ int spdifset(CSOUND *csound, SPECDIFF *p)
 
     if ((npts = inspecp->npts) != p->specsave.npts) { /* if inspec not matched  */
       SPECset(csound,
-              &p->specsave, (int32)npts);              /*   reinit the save spec */
+              &p->specsave, (int32)npts);             /*   reinit the save spec */
       SPECset(csound,
-              p->wdiff, (int32)npts);                  /*   & the out diff spec  */
+              p->wdiff, (int32)npts);                 /*   & the out diff spec  */
       p->wdiff->downsrcp = inspecp->downsrcp;
     }
     p->wdiff->ktimprd = inspecp->ktimprd;            /* pass the other specinfo */
@@ -942,7 +955,7 @@ int specdiff(CSOUND *csound, SPECDIFF *p)
         (p->specsave.auxch.auxp==NULL)
         ||
                  (p->wdiff->auxch.auxp==NULL))) goto err1;
-    if (inspecp->ktimstamp == csound->kcounter) {   /* if inspectrum is new: */
+    if (inspecp->ktimstamp == CS_KCNT) {   /* if inspectrum is new: */
       MYFLT *newp = (MYFLT *) inspecp->auxch.auxp;
       MYFLT *prvp = (MYFLT *) p->specsave.auxch.auxp;
       MYFLT *difp = (MYFLT *) p->wdiff->auxch.auxp;
@@ -959,11 +972,12 @@ int specdiff(CSOUND *csound, SPECDIFF *p)
         else difp[n] = FL(0.0);               /* else enter zero         */
         prvp[n] = newval;                     /* sav newval for nxt time */
       }
-      p->wdiff->ktimstamp = csound->kcounter; /* mark the output spec as new */
+      p->wdiff->ktimstamp = CS_KCNT; /* mark the output spec as new */
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("specdiff: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("specdiff: not initialised"));
 }
 
 int spsclset(CSOUND *csound, SPECSCAL *p)
@@ -1031,7 +1045,7 @@ int specscal(CSOUND *csound, SPECSCAL *p)
         (p->wscaled->auxch.auxp==NULL)
         ||
         (p->fscale==NULL)) goto err1;
-    if (inspecp->ktimstamp == csound->kcounter) {   /* if inspectrum is new: */
+    if (inspecp->ktimstamp == CS_KCNT) {   /* if inspectrum is new: */
       SPECDAT *outspecp = p->wscaled;
       MYFLT *inp = (MYFLT *) inspecp->auxch.auxp;
       MYFLT *outp = (MYFLT *) outspecp->auxch.auxp;
@@ -1052,11 +1066,12 @@ int specscal(CSOUND *csound, SPECSCAL *p)
           outp[n] = inp[n] * sclp[n];             /* no thresh: rescale only */
         }
       }
-      outspecp->ktimstamp = csound->kcounter;     /* mark the outspec as new */
+      outspecp->ktimstamp = CS_KCNT;     /* mark the outspec as new */
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("specscal: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("specscal: not initialised"));
 }
 
 int sphstset(CSOUND *csound, SPECHIST *p)
@@ -1068,9 +1083,9 @@ int sphstset(CSOUND *csound, SPECHIST *p)
 
     if ((npts = inspecp->npts) != p->accumer.npts) { /* if inspec not matched   */
       SPECset(csound,
-              &p->accumer, (int32)npts);              /*   reinit the accum spec */
+              &p->accumer, (int32)npts);             /*   reinit the accum spec */
       SPECset(csound,
-              p->wacout, (int32)npts);                /*    & the output spec    */
+              p->wacout, (int32)npts);               /*    & the output spec    */
       p->wacout->downsrcp = inspecp->downsrcp;
     }
     p->wacout->ktimprd = inspecp->ktimprd;           /* pass the other specinfo */
@@ -1080,7 +1095,7 @@ int sphstset(CSOUND *csound, SPECHIST *p)
     outp = (MYFLT *) p->wacout->auxch.auxp;
     if (UNLIKELY(lclp==NULL || outp==NULL)) { /* RWD fix */
       return csound->InitError(csound,
-                               Str("spechist: local buffers not intiialised"));
+                               Str("spechist: local buffers not initialised"));
     }
     memset(lclp,0,npts*sizeof(MYFLT));      /* clr local & out spec bufs */
     memset(outp,0,npts*sizeof(MYFLT));
@@ -1096,7 +1111,7 @@ int spechist(CSOUND *csound, SPECHIST *p)
         (p->accumer.auxch.auxp==NULL)
         ||
                  (p->wacout->auxch.auxp==NULL))) goto err1;
-    if (inspecp->ktimstamp == csound->kcounter) {   /* if inspectrum is new: */
+    if (inspecp->ktimstamp == CS_KCNT) {   /* if inspectrum is new: */
       MYFLT *newp = (MYFLT *) inspecp->auxch.auxp;
       MYFLT *acup = (MYFLT *) p->accumer.auxch.auxp;
       MYFLT *outp = (MYFLT *) p->wacout->auxch.auxp;
@@ -1108,11 +1123,12 @@ int spechist(CSOUND *csound, SPECHIST *p)
         acup[n] = newval;                   /* sav in accumulator   */
         outp[n] = newval;                   /* & copy to output     */
       }
-      p->wacout->ktimstamp = csound->kcounter; /* mark the output spec as new */
+      p->wacout->ktimstamp = CS_KCNT; /* mark the output spec as new */
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("spechist: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("spechist: not initialised"));
 }
 
 int spfilset(CSOUND *csound, SPECFILT *p)
@@ -1124,7 +1140,7 @@ int spfilset(CSOUND *csound, SPECFILT *p)
 
     if ((npts = inspecp->npts) != outspecp->npts) {  /* if inspec not matched */
       SPECset(csound,
-              outspecp, (int32)npts);                 /*   reinit the out spec */
+              outspecp, (int32)npts);                /*   reinit the out spec */
       csound->AuxAlloc(csound,
                        (size_t)npts*2* sizeof(MYFLT),
                        &p->auxch);                   /*   & local auxspace  */
@@ -1158,7 +1174,7 @@ int spfilset(CSOUND *csound, SPECFILT *p)
     {
       int32 nn;
       MYFLT *flp = p->coefs;
-      double halftim, reittim = inspecp->ktimprd * csound->onedkr;
+      double halftim, reittim = inspecp->ktimprd * CS_ONEDKR;
       for (nn=0;nn<npts;nn++) {
         if ((halftim = flp[nn]) > 0.)
           flp[nn] = (MYFLT)pow(0.5, reittim/halftim);
@@ -1180,7 +1196,7 @@ int spfilset(CSOUND *csound, SPECFILT *p)
 
 int specfilt(CSOUND *csound, SPECFILT *p)
 {
-    if (p->wsig->ktimstamp == csound->kcounter) {   /* if input spec is new,  */
+    if (p->wsig->ktimstamp == CS_KCNT) {   /* if input spec is new,  */
       SPECDAT *inspecp = p->wsig;
       SPECDAT *outspecp = p->wfil;
       MYFLT *newp = (MYFLT *) inspecp->auxch.auxp;
@@ -1196,61 +1212,83 @@ int specfilt(CSOUND *csound, SPECFILT *p)
         outp[n] = curval = persp[n];               /*   output current point  */
         persp[n] = coefp[n] * curval + newp[n];    /*   decay & addin newval  */
       }
-      outspecp->ktimstamp = csound->kcounter;      /* mark output spec as new */
+      outspecp->ktimstamp = CS_KCNT;      /* mark output spec as new */
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("specfilt: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("specfilt: not initialised"));
 }
 
 #define S       sizeof
 
 static OENTRY spectra_localops[] = {
-{ "spectrum", S(SPECTRUM),7, "w", "siiiqoooo",
+{ "spectrum", S(SPECTRUM),_QQ, 7, "w", "siiiqoooo",
                                    (SUBR)spectset,(SUBR)spectrum,(SUBR)spectrum},
-{ "specaddm", S(SPECADDM),5, "w",  "wwp",  (SUBR)spadmset,NULL,  (SUBR)specaddm},
-{ "specdiff", S(SPECDIFF),5, "w",  "w",    (SUBR)spdifset,NULL,  (SUBR)specdiff},
-{ "specscal", S(SPECSCAL),5, "w",  "wii",  (SUBR)spsclset,NULL,  (SUBR)specscal},
-{ "spechist", S(SPECHIST),5, "w",  "w",    (SUBR)sphstset,NULL,  (SUBR)spechist},
-{ "specfilt", S(SPECFILT),5, "w",  "wi",   (SUBR)spfilset,NULL,  (SUBR)specfilt},
-{ "specptrk", S(SPECPTRK),5, "kk", "wkiiiiiioqooo",
-                                            (SUBR)sptrkset,NULL,(SUBR)specptrk},
-{ "specsum",  S(SPECSUM), 5, "k",  "wo",   (SUBR)spsumset,NULL,  (SUBR)specsum },
-{ "specdisp", S(SPECDISP),5, "",   "wio",  (SUBR)spdspset,NULL,  (SUBR)specdisp},
-{ "pitch", S(PITCH),     5,    "kk", "aiiiiqooooojo",
+{ "specaddm", S(SPECADDM),_QQ, 5, "w", "wwp", (SUBR)spadmset,NULL,  (SUBR)specaddm},
+{ "specdiff", S(SPECDIFF),_QQ, 5, "w", "w",   (SUBR)spdifset,NULL,  (SUBR)specdiff},
+{ "specscal", S(SPECSCAL),_QQ, 5, "w", "wii", (SUBR)spsclset,NULL,  (SUBR)specscal},
+{ "spechist", S(SPECHIST),_QQ, 5, "w", "w",   (SUBR)sphstset,NULL,  (SUBR)spechist},
+{ "specfilt", S(SPECFILT),_QQ, 5, "w", "wi",  (SUBR)spfilset,NULL,  (SUBR)specfilt},
+{ "specptrk", S(SPECPTRK),_QQ, 5, "kk", "wkiiiiiioqooo",
+                                             (SUBR)sptrkset,NULL,(SUBR)specptrk},
+{ "specsum",  S(SPECSUM), _QQ, 5, "k", "wo",  (SUBR)spsumset,NULL,  (SUBR)specsum },
+{ "specdisp", S(SPECDISP),_QQ, 5, "",  "wio", (SUBR)spdspset,NULL,  (SUBR)specdisp},
+{ "pitch", S(PITCH),   0,  5,    "kk", "aiiiiqooooojo",
                                              (SUBR)pitchset, NULL, (SUBR)pitch },
-{ "maca", S(SUM),        5,  "a", "y",    (SUBR)macset,      NULL, (SUBR)maca  },
-{ "mac", S(SUM),         5,  "a", "Z",    (SUBR)macset,      NULL, (SUBR)mac   },
-{ "clockon", S(CLOCK),   3,  "",  "i",    (SUBR)clockset, (SUBR)clockon, NULL  },
-{ "clockoff", S(CLOCK),  3,  "",  "i",    (SUBR)clockset, (SUBR)clockoff, NULL },
-{ "readclock", S(CLKRD), 1,  "i", "i",    (SUBR)clockread, NULL, NULL          },
-{ "pitchamdf",S(PITCHAMDF),5,"kk","aiioppoo",
+{ "maca", S(SUM),      0,  5,  "a", "y",    (SUBR)macset,      NULL, (SUBR)maca  },
+{ "mac", S(SUM),       0,  5,  "a", "Z",    (SUBR)macset,      NULL, (SUBR)mac   },
+{ "clockon", S(CLOCK), 0,  3,  "",  "i",    (SUBR)clockset, (SUBR)clockon, NULL  },
+{ "clockoff", S(CLOCK),0,  3,  "",  "i",    (SUBR)clockset, (SUBR)clockoff, NULL },
+{ "readclock", S(CLKRD),0, 1,  "i", "i",    (SUBR)clockread, NULL, NULL          },
+{ "readscratch", S(SCRATCHPAD),0, 1, "i", "o", (SUBR)scratchread, NULL, NULL     },
+{ "writescratch", S(SCRATCHPAD),0, 1, "", "io", (SUBR)scratchwrite, NULL, NULL   },
+{ "pitchamdf",S(PITCHAMDF),0,5,"kk","aiioppoo",
                                      (SUBR)pitchamdfset, NULL, (SUBR)pitchamdf },
-{ "hsboscil",S(HSBOSC), TR|5, "a", "kkkiiioo",(SUBR)hsboscset,NULL,(SUBR)hsboscil },
-{ "phasorbnk", S(PHSORBNK),7,"s", "xkio",
-                                (SUBR)phsbnkset, (SUBR)kphsorbnk, (SUBR)phsorbnk },
-{ "adsynt",S(HSBOSC), TR|5,  "a", "kkiiiio", (SUBR)adsyntset, NULL, (SUBR)adsynt },
-{ "mpulse", S(IMPULSE),  5,  "a", "kko",  (SUBR)impulse_set, NULL, (SUBR)impulse },
-{ "lpf18", S(LPF18),     5,  "a", "akkko",  (SUBR)lpf18set, NULL, (SUBR)lpf18db   },
-{ "waveset", S(BARRI),   5,  "a", "ako",   (SUBR)wavesetset,  NULL, (SUBR)waveset},
-{ "pinkish", S(PINKISH),  5, "a", "xoooo", (SUBR)pinkset, NULL, (SUBR)pinkish },
-{ "noise",  S(VARI),   5,    "a", "xk",   (SUBR)varicolset, NULL, (SUBR)varicol },
-{ "transeg", S(TRANSEG), 7,  "s", "iiim", (SUBR)trnset,(SUBR)ktrnseg,(SUBR)trnseg},
-{ "transegb", S(TRANSEG), 7,  "s", "iiim",
-                              (SUBR)trnset_bkpt,(SUBR)ktrnseg,(SUBR)trnseg    },
-{ "transegr", S(TRANSEG), 7, "s", "iiim",
-                              (SUBR)trnsetr,(SUBR)ktrnsegr,(SUBR)trnsegr      },
-{ "clip", S(CLIP),       5,  "a", "aiiv", (SUBR)clip_set, NULL, (SUBR)clip  },
-{ "cpuprc", S(CPU_PERC), 1,     "",     "Ti",   (SUBR)cpuperc, NULL, NULL   },
-{ "maxalloc", S(CPU_PERC), 1,   "",     "Ti",   (SUBR)maxalloc, NULL, NULL  },
+{ "hsboscil",S(HSBOSC), TR, 5, "a", "kkkiiioo",
+
+                                    (SUBR)hsboscset,NULL,(SUBR)hsboscil },
+{ "phasorbnk", S(PHSORBNK),0,5,"a", "xkio",
+                                (SUBR)phsbnkset, (SUBR)NULL, (SUBR)phsorbnk },
+{ "phasorbnk.k", S(PHSORBNK),0,3,"k", "xkio",
+                                (SUBR)phsbnkset, (SUBR)kphsorbnk, NULL},
+{ "adsynt",S(HSBOSC), TR, 5,  "a", "kkiiiio", (SUBR)adsyntset, NULL, (SUBR)adsynt },
+{ "mpulse", S(IMPULSE), 0, 5,  "a", "kko",
+                                    (SUBR)impulse_set, NULL, (SUBR)impulse },
+{ "lpf18", S(LPF18),    0, 5,  "a", "akkko",  (SUBR)lpf18set, NULL, (SUBR)lpf18db },
+{ "waveset", S(BARRI),  0, 5,  "a", "ako",   (SUBR)wavesetset, NULL, (SUBR)waveset},
+{ "pinkish", S(PINKISH), 0, 5, "a", "xoooo", (SUBR)pinkset, NULL, (SUBR)pinkish },
+{ "noise",  S(VARI),  0, 5,    "a", "xk",   (SUBR)varicolset, NULL, (SUBR)varicol },
+{ "transeg", S(TRANSEG),0, 3,  "k", "iiim",
+                                           (SUBR)trnset,(SUBR)ktrnseg, NULL},
+{ "transeg.a", S(TRANSEG),0, 5,  "a", "iiim",
+                                           (SUBR)trnset,NULL,(SUBR)trnseg},
+{ "transegb", S(TRANSEG),0, 3,  "k", "iiim",
+                              (SUBR)trnset_bkpt,(SUBR)ktrnseg,(SUBR)NULL},
+{ "transegb.a", S(TRANSEG),0, 5,  "a", "iiim",
+                              (SUBR)trnset_bkpt,NULL,(SUBR)trnseg    },
+{ "transegr", S(TRANSEG),0, 3, "k", "iiim",
+                              (SUBR)trnsetr,(SUBR)ktrnsegr,(SUBR)NULL },
+{ "transegr.a", S(TRANSEG),0, 5, "a", "iiim",
+                              (SUBR)trnsetr,NULL,(SUBR)trnsegr      },
+{ "clip", S(CLIP),      0, 5,  "a", "aiiv", (SUBR)clip_set, NULL, (SUBR)clip  },
+{ "cpuprc", S(CPU_PERC),0, 1,     "",     "Si",   (SUBR)cpuperc_S, NULL, NULL   },
+{ "maxalloc", S(CPU_PERC),0, 1,   "",     "Si",   (SUBR)maxalloc_S, NULL, NULL  },
+{ "cpuprc", S(CPU_PERC),0, 1,     "",     "ii",   (SUBR)cpuperc, NULL, NULL   },
+{ "maxalloc", S(CPU_PERC),0, 1,   "",     "ii",   (SUBR)maxalloc, NULL, NULL  },
 { "active", 0xffff                                                          },
-{ "active.i", S(INSTCNT),1,     "i",    "To",    (SUBR)instcount, NULL, NULL },
-{ "active.k", S(INSTCNT),2,     "k",    "Uo",    NULL, (SUBR)instcount, NULL },
-{ "p.i", S(PFUN),        1,     "i",    "i",     (SUBR)pfun, NULL, NULL     },
-{ "p.k", S(PFUNK),       3,     "k",    "k", (SUBR)pfunk_init, (SUBR)pfunk, NULL },
-{ "mute", S(MUTE), 1,          "",      "To",   (SUBR)mute_inst             },
-{ "median", S(MEDFILT),  5,     "a", "akio", (SUBR)medfiltset, NULL, (SUBR)medfilt},
-{ "mediank", S(MEDFILT), 5,     "k", "kkio", (SUBR)medfiltset, (SUBR)kmedfilt},
+{ "active.iS", S(INSTCNT),0,1,     "i",    "Soo",   (SUBR)instcount_S, NULL, NULL },
+{ "active.kS", S(INSTCNT),0,2,     "k",    "Soo",   NULL, (SUBR)instcount_S, NULL },
+{ "active.i", S(INSTCNT),0,1,     "i",    "ioo",   (SUBR)instcount, NULL, NULL },
+{ "active.k", S(INSTCNT),0,2,     "k",    "koo",   NULL, (SUBR)instcount, NULL },
+{ "p.i", S(PFUN),        0,1,     "i",    "i",     (SUBR)pfun, NULL, NULL     },
+{ "p.k", S(PFUNK),       0,3,     "k",    "k",
+                                          (SUBR)pfunk_init, (SUBR)pfunk, NULL },
+{ "mute", S(MUTE), 0,1,          "",      "So",   (SUBR)mute_inst_S             },
+{ "mute.i", S(MUTE), 0,1,          "",      "io",   (SUBR)mute_inst             },
+{ "median", S(MEDFILT), 0, 5,     "a", "akio",
+                                        (SUBR)medfiltset, NULL, (SUBR)medfilt },
+{ "mediank", S(MEDFILT), 0,5,     "k", "kkio", (SUBR)medfiltset, (SUBR)kmedfilt},
 };
 
-LINKAGE1(spectra_localops)
+LINKAGE_BUILTIN(spectra_localops)
