@@ -23,7 +23,7 @@
 
 /* ugsc.c -- Opcodes from Sean Costello <costello@seanet.com> */
 
-#include "csdl.h"
+#include "stdopcod.h"
 #include "ugsc.h"
 
 /* svfilter.c
@@ -51,7 +51,9 @@ static int svf(CSOUND *csound, SVF *p)
     MYFLT *low, *high, *band, *in, ynm1, ynm2;
     MYFLT low2, high2, band2;
     MYFLT kfco = *p->kfco, kq = *p->kq;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     /* calculate frequency and Q coefficients */
     f1 = FL(2.0) * (MYFLT)sin((double)(kfco * csound->pidsr));
@@ -77,7 +79,18 @@ static int svf(CSOUND *csound, SVF *p)
     /* equations derived from Hal Chamberlin, "Musical Applications
      * of Microprocessors.
      */
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) {
+      memset(low,  '\0', offset*sizeof(MYFLT));
+      memset(high, '\0', offset*sizeof(MYFLT));
+      memset(band, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&low[nsmps],  '\0', early*sizeof(MYFLT));
+      memset(&high[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&band[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       low[n]  = low2 = ynm2 + f1 * ynm1;
       high[n] = high2 = scale * in[n] - low2 - q1 * ynm1;
       band[n] = band2 = f1 * high2 + ynm1;
@@ -107,39 +120,47 @@ static int hilbertset(CSOUND *csound, HILBERT *p)
     /* pole values taken from Bernie Hutchins, "Musical Engineer's Handbook" */
     double poles[12] = {0.3609, 2.7412, 11.1573, 44.7581, 179.6242, 798.4578,
                         1.2524, 5.5671, 22.3423, 89.6271, 364.7914, 2770.1114};
-    double polefreq[12], rc[12], alpha[12], beta[12];
+    double polefreq, rc, alpha, beta;
     /* calculate coefficients for allpass filters, based on sampling rate */
     for (j=0; j<12; j++) {
-      /*      p->coef[j] = (1 - (15 * PI * pole[j]) / csound->esr) /
-              (1 + (15 * PI * pole[j]) / csound->esr); */
-      polefreq[j] = poles[j] * 15.0;
-      rc[j] = 1.0 / (2.0 * PI * polefreq[j]);
-      alpha[j] = 1.0 / rc[j];
-      beta[j] = (1.0 - (alpha[j] * 0.5 * (double)csound->onedsr)) /
-                (1.0 + (alpha[j] * 0.5 * (double)csound->onedsr));
+      /*      p->coef[j] = (1 - (15 * PI * pole[j]) / CS_ESR) /
+              (1 + (15 * PI * pole[j]) / CS_ESR); */
+      polefreq = poles[j] * 15.0;
+      rc = 1.0 / (2.0 * PI * polefreq);
+      alpha = 1.0 / rc;
+      alpha = alpha * 0.5 * (double)csound->onedsr;
+      beta = (1.0 - alpha) / (1.0 + alpha);
       p->xnm1[j] = p->ynm1[j] = FL(0.0);
-      p->coef[j] = -(MYFLT)beta[j];
+      p->coef[j] = -(MYFLT)beta;
     }
     return OK;
 }
 
 static int hilbert(CSOUND *csound, HILBERT *p)
 {
-    MYFLT xn1 = FL(0.0), yn1 = FL(0.0), xn2 = FL(0.0), yn2 = FL(0.0);
+    MYFLT xn1, yn1, xn2, yn2;
     MYFLT *out1, *out2, *in;
-    MYFLT *xnm1, *ynm1, *coef;
-    int n, nsmps = csound->ksmps;
+    MYFLT *coef;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int j;
 
-    xnm1 = p->xnm1;
-    ynm1 = p->ynm1;
     coef = p->coef;
     out1 = p->out1;
     out2 = p->out2;
     in = p->in;
 
-    /* These two loops could be jammed */
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) {
+      memset(out1, '\0', offset*sizeof(MYFLT));
+      memset(out2, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out1[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&out2[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       xn1 = in[n];
       /* 6th order allpass filter for sine output. Structure is
        * 6 first-order allpass sections in series. Coefficients
@@ -215,7 +236,9 @@ static int resonr(CSOUND *csound, RESONZ *p)
     MYFLT *out, *in;
     double xn, yn, xnm1, xnm2, ynm1, ynm2;
     MYFLT kcf = *p->kcf, kbw = *p->kbw;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     r = exp((double)(kbw * csound->mpidsr));
     c1 = 2.0 * r * cos((double)(kcf * csound->tpidsr));
@@ -235,7 +258,12 @@ static int resonr(CSOUND *csound, RESONZ *p)
     ynm1 = p->ynm1;
     ynm2 = p->ynm2;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       xn = (double)in[n];
       out[n] = (MYFLT)(yn = scale * (xn - r * xnm2) + c1 * ynm1 - c2 * ynm2);
       xnm2 = xnm1;
@@ -268,7 +296,9 @@ static int resonz(CSOUND *csound, RESONZ *p)
     MYFLT *out, *in;
     double xn, yn, xnm1, xnm2, ynm1, ynm2;
     MYFLT kcf = *p->kcf, kbw = *p->kbw;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     r = exp(-(double)(kbw * csound->pidsr));
     c1 = 2.0 * r * cos((double)(csound->tpidsr*kcf));
@@ -291,7 +321,12 @@ static int resonz(CSOUND *csound, RESONZ *p)
     ynm1 = p->ynm1;
     ynm2 = p->ynm2;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       xn = (double)in[n];
       out[n] = (MYFLT)(yn = scale * (xn - xnm2) + c1 * ynm1 - c2 * ynm2);
       xnm2 = xnm1;
@@ -312,8 +347,9 @@ static int phaser1set(CSOUND *csound, PHASER1 *p)
     int   loop = (int) MYFLT2LONG(*p->iorder);
     int32  nBytes = (int32) loop * (int32) sizeof(MYFLT);
 
-    if (*p->istor == FL(0.0) || p->auxx.auxp == NULL || p->auxx.size<nBytes ||
-                                p->auxy.auxp == NULL || p->auxy.size<nBytes) {
+    if (*p->istor == FL(0.0) || p->auxx.auxp == NULL ||
+        (int)p->auxx.size<nBytes || p->auxy.auxp == NULL ||
+        (int)p->auxy.size<nBytes) {
       csound->AuxAlloc(csound, nBytes, &p->auxx);
       csound->AuxAlloc(csound, nBytes, &p->auxy);
       p->xnm1 = (MYFLT *) p->auxx.auxp;
@@ -324,7 +360,7 @@ static int phaser1set(CSOUND *csound, PHASER1 *p)
       void    *tmp1, *tmp2;
       size_t  oldSize1 = (size_t) p->auxx.size;
       size_t  oldSize2 = (size_t) p->auxy.size;
-      tmp1 = csound->Malloc(csound, oldSize1 + oldSize2);
+      tmp1 = malloc(oldSize1 + oldSize2);
       tmp2 = (char*) tmp1 + (int) oldSize1;
       memcpy(tmp1, p->auxx.auxp, oldSize1);
       memcpy(tmp2, p->auxy.auxp, oldSize2);
@@ -332,7 +368,7 @@ static int phaser1set(CSOUND *csound, PHASER1 *p)
       csound->AuxAlloc(csound, nBytes, &p->auxy);
       memcpy(p->auxx.auxp, tmp1, oldSize1);
       memcpy(p->auxy.auxp, tmp2, oldSize2);
-      csound->Free(csound, tmp1);
+      free(tmp1);
       p->xnm1 = (MYFLT *) p->auxx.auxp;
       p->ynm1 = (MYFLT *) p->auxy.auxp;
     }
@@ -344,20 +380,19 @@ static int phaser1(CSOUND *csound, PHASER1 *p)
 {
     MYFLT xn = FL(0.0), yn = FL(0.0);
     MYFLT *out, *in;
-    MYFLT *xnm1, *ynm1, feedback;
-    MYFLT coef = *p->kcoef, fbgain = *p->fbgain;
+    MYFLT feedback;
+    MYFLT coef = FABS(*p->kcoef), fbgain = *p->fbgain;
     MYFLT beta, wp;
-    int nsmps = csound->ksmps;
-    int i, j;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
+    int j;
 
-    xnm1 = p->xnm1;
-    ynm1 = p->ynm1;
     feedback = p->feedback;
     out = p->out;
     in = p->in;
 
-    if (coef <= FL(0.0))
-      coef = -coef;     /* frequency will "fold over" if <= 0 Hz */
+    //if (coef<=FL(0.0)) coef = -coef; /* frequency will "fold over" if <= 0 Hz */
     /* next two lines implement bilinear z-transform, to convert
      * frequency value into a useable coefficient for the
      * allpass filters.
@@ -365,7 +400,12 @@ static int phaser1(CSOUND *csound, PHASER1 *p)
     wp = csound->pidsr * coef;
     beta = (FL(1.0) - wp)/(FL(1.0) + wp);
 
-    for (i=0; i<nsmps; i++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset; i<nsmps; i++) {
       xn = in[i] + feedback * fbgain;
       for (j=0; j < p->loop; j++) {
         /* Difference equation for 1st order
@@ -385,7 +425,7 @@ static int phaser1(CSOUND *csound, PHASER1 *p)
 
 static int phaser2set(CSOUND *csound, PHASER2 *p)
 {
-    int modetype, j;
+    int modetype;
     int loop;
 
     p->modetype = modetype = (int)*p->mode;
@@ -399,8 +439,9 @@ static int phaser2set(CSOUND *csound, PHASER2 *p)
     csound->AuxAlloc(csound, (size_t)loop*sizeof(MYFLT), &p->aux2);
     p->nm1 = (MYFLT *) p->aux1.auxp;
     p->nm2 = (MYFLT *) p->aux2.auxp;
-    for (j=0; j< loop; j++)
-      p->nm1[j] = p->nm2[j] = FL(0.0);
+    /* *** This is unnecessary as AuxAlloc zeros *** */
+    /* for (j=0; j< loop; j++) */
+    /*   p->nm1[j] = p->nm2[j] = FL(0.0); */
     return OK;
 }
 
@@ -413,7 +454,9 @@ static int phaser2(CSOUND *csound, PHASER2 *p)
     MYFLT b, a, r, freq;
     MYFLT temp;
     MYFLT *nm1, *nm2, feedback;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int j;
 
     nm1 = p->nm1;
@@ -432,7 +475,13 @@ static int phaser2(CSOUND *csound, PHASER2 *p)
     if (ksep <= FL(0.0))
       ksep = -ksep;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      MYFLT kk = FL(1.0);
       xn = in[n] + feedback * fbgain;
       /* The following code is used to determine
        * how the frequencies of the notches are calculated.
@@ -443,8 +492,11 @@ static int phaser2(CSOUND *csound, PHASER2 *p)
       for (j=0; j < p->loop; j++) {
         if (p->modetype == 1)
           freq = kbf + (kbf * ksep * j);
-        else
-          freq = kbf * csound->intpow(ksep,(int32)j);
+        else {
+          freq = kbf * kk;
+          kk *= ksep;
+          //freq = kbf * csound->intpow(ksep,(int32)j);
+        }
         /* Note similarities of following equations to
          * equations in resonr/resonz. The 2nd-order
          * allpass filter used here is similar to the
@@ -460,10 +512,10 @@ static int phaser2(CSOUND *csound, PHASER2 *p)
         /* Difference equations for implementing canonical
          * 2nd order section. (Direct Form II)
          */
-        temp = xn - b * p->nm1[j] - a * p->nm2[j];
-        yn = a * temp + b * p->nm1[j] + nm2[j];
-        p->nm2[j] = p->nm1[j];
-        p->nm1[j] = temp;
+        temp = xn - b * nm1[j] - a * nm2[j];
+        yn = a * temp + b * nm1[j] + nm2[j];
+        nm2[j] = nm1[j];
+        nm1[j] = temp;
         xn = yn;
       }
       out[n] = yn;
@@ -489,11 +541,12 @@ static int lp2(CSOUND *csound, LP2 *p)
     MYFLT *out, *in;
     double yn, ynm1, ynm2;
     MYFLT kfco = *p->kfco, kres = *p->kres;
-    int nsmps = csound->ksmps;
-    int n;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     temp = (double)(csound->mpidsr * kfco / kres);
-      /* (-PI_F * kfco / (kres * csound->esr)); */
+      /* (-PI_F * kfco / (kres * CS_ESR)); */
     a = 2.0 * cos((double) (kfco * csound->tpidsr)) * exp(temp);
     b = exp(temp+temp);
     c = 1.0 - a + b;
@@ -503,7 +556,12 @@ static int lp2(CSOUND *csound, LP2 *p)
     ynm1 = p->ynm1;
     ynm2 = p->ynm2;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       out[n] = (MYFLT)(yn = a * ynm1 - b * ynm2 + c * (double)in[n]);
       ynm2 = ynm1;
       ynm1 = yn;
@@ -516,13 +574,13 @@ static int lp2(CSOUND *csound, LP2 *p)
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "svfilter", S(SVF),    5, "aaa", "akko", (SUBR)svfset, NULL, (SUBR)svf     },
-{ "hilbert", S(HILBERT), 5, "aa",  "a",    (SUBR)hilbertset, NULL, (SUBR)hilbert },
-{ "resonr", S(RESONZ),   5, "a",   "akkoo", (SUBR)resonzset, NULL, (SUBR)resonr  },
-{ "resonz", S(RESONZ),   5, "a",   "akkoo", (SUBR)resonzset, NULL, (SUBR)resonz  },
-{ "lowpass2", S(LP2),    5, "a",   "akko",  (SUBR)lp2_set, NULL, (SUBR)lp2       },
-{ "phaser2", S(PHASER2), 5, "a", "akkkkkk",(SUBR)phaser2set, NULL, (SUBR)phaser2 },
-{ "phaser1", S(PHASER1), 5, "a",   "akkko", (SUBR)phaser1set, NULL, (SUBR)phaser1 }
+{ "svfilter", S(SVF),    0, 5, "aaa", "akko", (SUBR)svfset, NULL, (SUBR)svf     },
+{ "hilbert", S(HILBERT), 0, 5, "aa", "a", (SUBR)hilbertset, NULL, (SUBR)hilbert },
+{ "resonr", S(RESONZ),   0, 5, "a",  "akkoo", (SUBR)resonzset, NULL, (SUBR)resonr},
+{ "resonz", S(RESONZ),   0, 5, "a",  "akkoo", (SUBR)resonzset, NULL, (SUBR)resonz},
+{ "lowpass2", S(LP2),    0, 5, "a",  "akko",  (SUBR)lp2_set, NULL, (SUBR)lp2     },
+{ "phaser2", S(PHASER2), 0, 5, "a", "akkkkkk",(SUBR)phaser2set,NULL,(SUBR)phaser2},
+{ "phaser1", S(PHASER1), 0, 5, "a", "akkko", (SUBR)phaser1set, NULL,(SUBR)phaser1}
 };
 
 int ugsc_init_(CSOUND *csound)
