@@ -1,7 +1,7 @@
 /*
     argdecode.c:
 
-    Copyright (C) 1998 John ffitch
+    Copyright (C) 1998-2013 John ffitch, Victor Lazzarini
 
     This file is part of Csound.
 
@@ -42,7 +42,7 @@ extern void strset_option(CSOUND *csound, char *s);     /* from str_ops.c */
 
 /* IV - Feb 19 2005 */
 
-static void set_stdin_assign(CSOUND *csound, int type, int state)
+static inline void set_stdin_assign(CSOUND *csound, int type, int state)
 {
     if (state)
       csound->stdin_assign_flg |= type;
@@ -50,7 +50,7 @@ static void set_stdin_assign(CSOUND *csound, int type, int state)
       csound->stdin_assign_flg &= (~type);
 }
 
-static void set_stdout_assign(CSOUND *csound, int type, int state)
+static inline void set_stdout_assign(CSOUND *csound, int type, int state)
 {
     if (state)
       csound->stdout_assign_flg |= type;
@@ -63,9 +63,7 @@ static const char *shortUsageList[] = {
   Str_noop("--help\tprint long usage options"),
   Str_noop("-U unam\trun utility program unam"),
   Str_noop("-C\tuse Cscore processing of scorefile"),
-#ifdef PARCS
-  Str_noop("-j N\tuse N processes"),
-#endif
+  Str_noop("-j N\tuse N threads in performance"),
   Str_noop("-I\tI-time only orch run"),
   Str_noop("-n\tno sound onto disk"),
   Str_noop("-i fnam\tsound input filename"),
@@ -123,13 +121,14 @@ static const char *shortUsageList[] = {
 
 static const char *longUsageList[] = {
   "--format={wav,aiff,au,raw,paf,svx,nist,voc,ircam,w64,mat4,mat5",
-  "          pvf,xi,htk,sds,avr,wavex,sd2,flac,caf,WVE,ogg,MPC,W64}",
-  "--format={alaw,ulaw,schar,uchar,float,short,long,24bit}",
+  "          pvf,xi,htk,sds,avr,wavex,sd2,flac,caf,wve,ogg,mpc2k,rf64}",
+  "--format={alaw,ulaw,schar,uchar,float,double,short,long,24bit,vorbis}",
   Str_noop("\t\t\tSet output file format"),
   Str_noop("--aiff\t\t\tSet AIFF format"),
   Str_noop("--au\t\t\tSet AU format"),
   Str_noop("--wave\t\t\tSet WAV format"),
   Str_noop("--ircam\t\t\tSet IRCAM format"),
+  Str_noop("--ogg\t\t\tSet OGG/VORBIS format"),
   Str_noop("--noheader\t\tRaw format"),
   Str_noop("--nopeaks\t\tDo not write peak information"),
   " ",
@@ -138,30 +137,36 @@ static const char *longUsageList[] = {
   Str_noop("--postscriptdisplay\tSuppress graphics, use Postscript displays"),
   " ",
   Str_noop("--defer-gen1\t\tDefer GEN01 soundfile loads until performance time"),
-  Str_noop("--iobufsamps=N\t\tSample frames (or -kprds) per software sound I/O buffer"),
+  Str_noop("--iobufsamps=N\t\tSample frames (or -kprds) per software "
+           "sound I/O buffer"),
   Str_noop("--hardwarebufsamps=N\tSamples per hardware sound I/O buffer"),
   Str_noop("--cscore\t\tUse Cscore processing of scorefile"),
+  Str_noop("--orc\t\t\tUse orchfile without scorefile"),
   " ",
   Str_noop("--midifile=FNAME\tRead MIDIfile event stream from file"),
   Str_noop("--midioutfile=FNAME\tWrite MIDI output to file FNAME"),
   Str_noop("--midi-device=FNAME\tRead MIDI realtime events from device"),
   Str_noop("--terminate-on-midi\tTerminate the performance when miditrack is done"),
   " ",
-  Str_noop("--heartbeat=N\t\tPrint a heartbeat style 1, 2 or 3 at each soundfile write"),
+  Str_noop("--heartbeat=N\t\tPrint a heartbeat style 1, 2 or 3 at "
+           "each soundfile write"),
   Str_noop("--notify\t\tNotify (ring the bell) when score or miditrack is done"),
-  Str_noop("--rewrite\t\tContinually rewrite header while writing soundfile (WAV/AIFF)"),
+  Str_noop("--rewrite\t\tContinually rewrite header while writing "
+           "soundfile (WAV/AIFF)"),
   " ",
   Str_noop("--input=FNAME\t\tSound input filename"),
   Str_noop("--output=FNAME\t\tSound output filename"),
   Str_noop("--logfile=FNAME\t\tLog output to file"),
   " ",
   Str_noop("--nosound\t\tNo sound onto disk or device"),
-  Str_noop("--tempo=N\t\tUse uninterpreted beats of the score, initially at tempo N"),
+  Str_noop("--tempo=N\t\tUse uninterpreted beats of the score, initially"
+           " at tempo N"),
   Str_noop("--i-only\t\tI-time only orch run"),
   Str_noop("--syntax-check-only\tStop after checking orchestra and score syntax"),
   Str_noop("--control-rate=N\tOrchestra krate override"),
   Str_noop("--sample-rate=N\t\tOrchestra srate override"),
-  Str_noop("--score-in=FNAME\tRead Line-oriented realtime score events from device"),
+  Str_noop("--score-in=FNAME\tRead Line-oriented realtime score events"
+           " from device"),
   Str_noop("--messagelevel=N\ttty message level, sum of:"),
   Str_noop("--messageolevel=O\ttty message level in octal, of:"),
   Str_noop("\t\t\t\t1=note amps, 2=out-of-range msg, 4=warnings,"),
@@ -179,7 +184,6 @@ static const char *longUsageList[] = {
   Str_noop("\t\t\t1=use CSD line #s (default), 0=use ORC/SCO-relative line #s"),
   Str_noop("--extract-score=FNAME\tExtract from score.srt using extract file"),
   Str_noop("--keep-sorted-score"),
-  Str_noop("--expression-opt\tOptimise use of temporary variables in expressions"),
   Str_noop("--env:NAME=VALUE\tSet environment variable NAME to VALUE"),
   Str_noop("--env:NAME+=VALUE\tAppend VALUE to environment variable NAME"),
   Str_noop("--strsetN=VALUE\t\tSet strset table at index N to VALUE"),
@@ -209,12 +213,15 @@ static const char *longUsageList[] = {
   Str_noop("--midi-velocity-amp=N\tRoute MIDI note on message"),
   Str_noop("\t\t\tvelocity number to pfield N as amplitude"),
   Str_noop("--no-default-paths\tTurn off relative paths from CSD/ORC/SCO"),
-#if ENABLE_NEW_PARSER
-  Str_noop("--new-parser\t\tUse new Bison-based parser"),
-  Str_noop("--old-parser\t\tUse old parser"),
-#endif
+  Str_noop("--sample-accurate\t\tUse sample-accurate timing of score events"),
+  Str_noop("--realtime\t\trealtime priority mode"),
+  Str_noop("--nchnls=N\t\t override number of audio channels"),
+  Str_noop("--nchnls_i=N\t\t override number of input audio channels"),
+  Str_noop("--0dbfs=N\t\t override 0dbfs (max positive signal amplitude)"),
+  Str_noop("--sinesize\t\tlength of internal sine table"),
   " ",
   Str_noop("--help\t\t\tLong help"),
+
   NULL
 };
 
@@ -245,7 +252,7 @@ static void longusage(CSOUND *p)
     dump_cfg_variables(p);
     p->Message(p, Str("\nShort form:\n"));
     print_short_usage(p);
-    p->LongJmp(p, 0);
+    // p->LongJmp(p, 0);
 }
 
 void dieu(CSOUND *csound, char *s, ...)
@@ -258,8 +265,10 @@ void dieu(CSOUND *csound, char *s, ...)
     va_start(args, s);
     csound->ErrMsgV(csound, Str("Csound Command ERROR:\t"), s, args);
     va_end(args);
-
+    if(csound->info_message_request == 0){
+      csound->info_message_request = 0;
     csound->LongJmp(csound, 1);
+    }
 }
 
 void set_output_format(OPARMS *O, char c)
@@ -281,6 +290,10 @@ void set_output_format(OPARMS *O, char c)
       O->outformat = AE_FLOAT;   /* float soundfile */
       break;
 
+    case 'd':
+      O->outformat = AE_DOUBLE;  /* double soundfile */
+      break;
+
     case 's':
       O->outformat = AE_SHORT;   /* short_int soundfile*/
       break;
@@ -301,6 +314,10 @@ void set_output_format(OPARMS *O, char c)
       O->outformat = AE_FLOAT;   /* float soundfile (for rescaling) */
       break;
 
+    case 'v':
+      O->outformat = AE_VORBIS;  /* Xiph Vorbis encoding */
+      break;
+
     default:
       return; /* do nothing */
     };
@@ -313,9 +330,9 @@ typedef struct  {
 
 static const SAMPLE_FORMAT_ENTRY sample_format_map[] = {
   { "alaw",   'a' },  { "schar",  'c' },  { "uchar",  '8' },
-  { "float",  'f' },  { "long",   'l' },
+  { "float",  'f' },  { "double", 'd' },  { "long",   'l' },
   { "short",  's' },  { "ulaw",   'u' },  { "24bit",  '3' },
-  { NULL, '\0' }
+  { "vorbis", 'v' },  { NULL, '\0' }
 };
 
 typedef struct {
@@ -333,18 +350,10 @@ static const SOUNDFILE_TYPE_ENTRY file_type_map[] = {
     { "pvf",    TYP_PVF   },  { "xi",     TYP_XI    },
     { "htk",    TYP_HTK   },  { "sds",    TYP_SDS   },
     { "avr",    TYP_AVR   },  { "wavex",  TYP_WAVEX },
-#if defined(HAVE_LIBSNDFILE) && HAVE_LIBSNDFILE >= 1011
     { "sd2",    TYP_SD2   },
-#  if HAVE_LIBSNDFILE >= 1013
     { "flac",   TYP_FLAC  },  { "caf",    TYP_CAF   },
-#  endif
-#  if HAVE_LIBSNDFILE >= 1018
-    { "WVE",    TYP_WVE   },  { "ogg",    TYP_OGG   },
-#  endif
-#  if HAVE_LIBSNDFILE >= 1019
-    { "MPC",    TYPE_MPC2K }, { "W64",    TYP_RF64  },
-#  endif
-#endif
+    { "wve",    TYP_WVE   },  { "ogg",    TYP_OGG   },
+    { "mpc2k",  TYP_MPC2K },  { "rf64",   TYP_RF64  },
     { NULL , -1 }
 };
 
@@ -418,6 +427,10 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       O->inbufsamps = O->outbufsamps = atoi(s);
       return 1;
     }
+    else if (!(strcmp (s, "orc"))) {
+      csound->use_only_orchfile = 1;    /* orchfile without scorefile */
+      return 1;
+    }
     else if (!(strcmp (s, "cscore"))) {
       O->usingcscore = 1;               /* use cscore processing  */
       return 1;
@@ -440,7 +453,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       O->FMidiname = s;                 /* Midifile name */
       if (!strcmp(O->FMidiname, "stdin")) {
         set_stdin_assign(csound, STDINASSIGN_MIDIFILE, 1);
-#if defined(WIN32) || defined(mac_classic)
+#if defined(WIN32)
         csoundDie(csound, Str("-F: stdin not supported on this platform"));
 #endif
       }
@@ -492,7 +505,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
         csoundDie(csound, Str("input cannot be stdout"));
       if (strcmp(O->infilename, "stdin") == 0) {
         set_stdin_assign(csound, STDINASSIGN_SNDFILE, 1);
-#if defined(WIN32) || defined(mac_classic)
+#if defined(WIN32)
         csoundDie(csound, Str("stdin audio not supported"));
 #endif
       }
@@ -518,6 +531,11 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       O->filetyp = TYP_IRCAM;           /* IRCAM output request */
       return 1;
     }
+    else if (!(strcmp (s, "ogg"))) {
+      O->filetyp = TYP_OGG;             /* OGG output request   */
+      O->outformat = AE_VORBIS;         /* Xiph Vorbis encoding */
+      return 1;
+    }
     /*
       -k N orchestra krate override
      */
@@ -541,9 +559,6 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       O->Linename = s;
       if (!strcmp(O->Linename, "stdin")) {
         set_stdin_assign(csound, STDINASSIGN_LINEIN, 1);
-#if defined(mac_classic)
-        csoundDie(csound, Str("-L: stdin not supported on this platform"));
-#endif
       }
       else
         set_stdin_assign(csound, STDINASSIGN_LINEIN, 0);
@@ -644,7 +659,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       O->Midiname = s;
       if (!strcmp(O->Midiname, "stdin")) {
         set_stdin_assign(csound, STDINASSIGN_MIDIDEV, 1);
-#if defined(WIN32) || defined(mac_classic)
+#if defined(WIN32)
         csoundDie(csound, Str("-M: stdin not supported on this platform"));
 #endif
       }
@@ -671,7 +686,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
         dieu(csound, Str("-o cannot be stdin"));
       if (strcmp(O->outfilename, "stdout") == 0) {
         set_stdout_assign(csound, STDOUTASSIGN_SNDFILE, 1);
-#if defined(WIN32) || defined(mac_classic)
+#if defined(WIN32)
         csoundDie(csound, Str("stdout audio not supported"));
 #endif
       }
@@ -738,8 +753,15 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       int retval;
       s += 8;
       if (*s=='\0') dieu(csound, Str("no utility name"));
+
       retval = csoundRunUtility(csound, s, argc, argv);
-      csound->LongJmp(csound, retval);
+     if(retval) {
+                  csound->info_message_request = 1;
+                  csound->orchname = NULL;
+                  return 0;
+              }
+       else csound->LongJmp(csound, retval);
+     return 1;
     }
     /* -v */
     else if (!(strcmp (s, "verbose"))) {
@@ -764,11 +786,13 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       if (*s != '\0') {
         if (isdigit(*s)) full = *s++ - '0';
       }
-      csoundLoadExternals(csound);
-      if (csoundInitModules(csound) != 0)
-        csound->LongJmp(csound, 1);
+      /* VL: moved to csoundReset() in csound.c
+              csoundLoadExternals(csound);
+              if (csoundInitModules(csound) != 0)
+              csound->LongJmp(csound, 1); */
       list_opcodes(csound, full);
-      csound->LongJmp(csound, 0);
+      return 1;
+      //csound->LongJmp(csound, 0);
     }
     /* -Z */
     else if (!(strcmp (s, "dither"))) {
@@ -852,48 +876,52 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
     }
     else if (!(strcmp(s, "help"))) {
       longusage(csound);
-      csound->LongJmp(csound, 0);
+      csound->info_message_request = 1;
+      return 1;
+      //csound->LongJmp(csound, 0);
     }
-#ifdef ENABLE_NEW_PARSER
-    else if (!(strcmp(s, "new-parser"))) {
-      O->newParser = 1;             /* Use New Parser */
+    else if (!(strcmp(s, "sample-accurate"))) {
+      O->sampleAccurate = 1;
       return 1;
     }
-#ifdef PARCS
-    else if (!(strncmp(s, "weight-info=", 12))) {
-      s += 12;
-      if (*s=='\0') dieu(csound, Str("no weight-info"));
-      csound->weight_info = s;
+    else if (!(strcmp(s, "realtime"))) {
+      O->realtime = 1;
       return 1;
     }
-    else if (!(strncmp(s, "weight-dump=", 12))) {
-      s += 12;
-      if (*s=='\0') dieu(csound, Str("no weight-dump"));
-      csound->weight_dump = s;
+    else if (!(strncmp(s, "nchnls=", 7))) {
+      s += 7;
+      O->nchnls_override = atoi(s);
       return 1;
     }
-    else if (!(strncmp(s, "weights=", 8))) {
-      s += 8;
-      if (*s=='\0') dieu(csound, Str("no weights"));
-      csound->weights = s;
+    else if (!(strncmp(s, "0dbfs=", 6))) {
+      s += 6;
+      O->e0dbfs_override = atoi(s);
       return 1;
     }
-    else if (!(strcmp(s, "compute-weights"))) {
-      O->calculateWeights = 1;
+    else if (!(strncmp(s, "nchnls_i=", 9))) {
+      s += 9;
+      O->nchnls_i_override = atoi(s);
       return 1;
     }
-#endif
-    else if (!(strcmp(s, "old-parser"))) {
-      O->newParser = 0;             /* Use Old Parser */
-      return 1;
+    else if (!(strncmp(s, "sinesize=", 9))) {
+      {
+        int i = 1, n;
+        s += 9;
+        n = atoi(s);
+        while (i<=n && i< MAXLEN) i <<= 1;
+        csound->sinelength = i;
+        return 1;
+      }
+    }
+    else if (!(strcmp(s, "new-parser")) ||
+             !(strcmp(s, "old-parser"))) {
+        return 1;  /* ignore flag, this is here for backwards compatibility */
     }
-#endif
-
     csoundErrorMsg(csound, Str("unknown long option: '--%s'"), s);
     return 0;
 }
 
-int argdecode(CSOUND *csound, int argc, char **argv_)
+PUBLIC int argdecode(CSOUND *csound, int argc, char **argv_)
 {
     OPARMS  *O = csound->oparms;
     char    *s, **argv;
@@ -927,7 +955,12 @@ int argdecode(CSOUND *csound, int argc, char **argv_)
             FIND(Str("no utility name"));
             {
               int retval = csoundRunUtility(csound, s, argc, argv);
-              csound->LongJmp(csound, retval);
+              if(retval) {
+                  csound->info_message_request = 1;
+                  csound->orchname = NULL;
+                  goto end;
+              }
+              else csound->LongJmp(csound, retval);
             }
             break;
           case 'C':
@@ -947,7 +980,7 @@ int argdecode(CSOUND *csound, int argc, char **argv_)
               csoundDie(csound, Str("input cannot be stdout"));
             if (strcmp(O->infilename, "stdin") == 0) {
               set_stdin_assign(csound, STDINASSIGN_SNDFILE, 1);
-#if defined(WIN32) || defined(mac_classic)
+#if defined(WIN32)
               csoundDie(csound, Str("stdin audio not supported"));
 #endif
             }
@@ -963,7 +996,7 @@ int argdecode(CSOUND *csound, int argc, char **argv_)
               dieu(csound, Str("-o cannot be stdin"));
             if (strcmp(O->outfilename, "stdout") == 0) {
               set_stdout_assign(csound, STDOUTASSIGN_SNDFILE, 1);
-#if defined(WIN32) || defined(mac_classic)
+#if defined(WIN32)
               csoundDie(csound, Str("stdout audio not supported"));
 #endif
             }
@@ -1059,9 +1092,6 @@ int argdecode(CSOUND *csound, int argc, char **argv_)
             s += (int) strlen(s);
             if (!strcmp(O->Linename, "stdin")) {
               set_stdin_assign(csound, STDINASSIGN_LINEIN, 1);
-#if defined(mac_classic)
-              csoundDie(csound, Str("-L: stdin not supported on this platform"));
-#endif
             }
             else
               set_stdin_assign(csound, STDINASSIGN_LINEIN, 0);
@@ -1073,7 +1103,7 @@ int argdecode(CSOUND *csound, int argc, char **argv_)
             s += (int) strlen(s);
             if (!strcmp(O->Midiname, "stdin")) {
               set_stdin_assign(csound, STDINASSIGN_MIDIDEV, 1);
-#if defined(WIN32) || defined(mac_classic)
+#if defined(WIN32)
               csoundDie(csound, Str("-M: stdin not supported on this platform"));
 #endif
             }
@@ -1087,7 +1117,7 @@ int argdecode(CSOUND *csound, int argc, char **argv_)
             s += (int) strlen(s);
             if (!strcmp(O->FMidiname, "stdin")) {
               set_stdin_assign(csound, STDINASSIGN_MIDIFILE, 1);
-#if defined(WIN32) || defined(mac_classic)
+#if defined(WIN32)
               csoundDie(csound, Str("-F: stdin not supported on this platform"));
 #endif
             }
@@ -1129,12 +1159,14 @@ int argdecode(CSOUND *csound, int argc, char **argv_)
               if (*s != '\0') {
                 if (isdigit(*s)) full = *s++ - '0';
               }
+              /* VL: moved to csoundReset() in csound.c
               csoundLoadExternals(csound);
               if (csoundInitModules(csound) != 0)
-                csound->LongJmp(csound, 1);
+              csound->LongJmp(csound, 1); */
               list_opcodes(csound, full);
             }
-            csound->LongJmp(csound, 0);
+            csound->info_message_request = 1;
+            // csound->LongJmp(csound, 0);
             break;
           case 'Z':
             {
@@ -1192,6 +1224,7 @@ int argdecode(CSOUND *csound, int argc, char **argv_)
             while (*(++s));
             break;
           default:
+            if(csound->info_message_request == 0)
             dieu(csound, Str("unknown flag -%c"), c);
           }
         }
@@ -1200,7 +1233,7 @@ int argdecode(CSOUND *csound, int argc, char **argv_)
         /* 0: normal, 1: ignore, 2: fail */
         if (csound->orcname_mode == 2) {
           csound->Die(csound, Str("error: orchestra and score name not "
-                                  "allowed in .csoundrc"));
+                                  "allowed in .csound6rc"));
         }
         else if (csound->orcname_mode == 0) {
           if (csound->orchname == NULL)
@@ -1214,6 +1247,243 @@ int argdecode(CSOUND *csound, int argc, char **argv_)
         }
       }
     } while (--argc);
+ end:
     return 1;
 }
 
+PUBLIC int csoundSetOption(CSOUND *csound, char *option){
+  /* if already compiled and running, return */
+  csound->info_message_request = 1;
+  if(csound->engineStatus & CS_STATE_COMP) return 1;
+  char *args[2] = {"csound", option};
+  return (argdecode(csound, 1, args) ? 0 : 1);
+}
+
+PUBLIC void csoundSetParams(CSOUND *csound, CSOUND_PARAMS *p){
+
+  /* if already compiled and running, return */
+  if(csound->engineStatus & CS_STATE_COMP) return;
+
+  OPARMS *oparms = csound->oparms;
+  /* simple ON/OFF switches */
+  oparms->odebug = p->debug_mode;
+  oparms->displays = p->displays;
+  oparms->graphsoff = p->ascii_graphs;
+  oparms->postscript = p->postscript_graphs;
+  oparms->usingcscore = p->use_cscore;
+  oparms->ringbell = p->ring_bell;
+  oparms->gen01defer = p->defer_gen01_load;
+  oparms->termifend =  p->terminate_on_midi;
+  oparms->noDefaultPaths = p->no_default_paths;
+  oparms->syntaxCheckOnly = p->syntax_check_only;
+  oparms->sampleAccurate = p->sample_accurate;
+  oparms->realtime = p->realtime_mode;
+  oparms->useCsdLineCounts = p->csd_line_counts;
+  oparms->heartbeat = p->heartbeat;
+  oparms->ringbell = p->ring_bell;
+
+  /* message level */
+  if(p->message_level > 0)
+   oparms->msglevel = p->message_level;
+
+  /* tempo / beatmode */
+  if(p->tempo > 0) {
+    oparms->cmdTempo = p->tempo;
+    oparms->Beatmode = 1;
+  }
+  /* buffer frames */
+  if(p->buffer_frames > 0)
+    oparms->inbufsamps = oparms->outbufsamps = p->buffer_frames;
+
+  /* hardware buffer frames */
+  if(p->hardware_buffer_frames > 0)
+    oparms->oMaxLag = p->hardware_buffer_frames;
+
+  /* multicore threads */
+  if(p->number_of_threads > 1)
+    oparms->numThreads = p->number_of_threads;
+
+  /* MIDI interop */
+  if(p->midi_key > 0) oparms->midiKey = p->midi_key;
+  else if(p->midi_key_cps > 0) oparms->midiKeyCps = p->midi_key_cps;
+  else if(p->midi_key_pch > 0) oparms->midiKeyPch = p->midi_key_pch;
+  else if(p->midi_key_oct > 0) oparms->midiKeyOct = p->midi_key_oct;
+
+  if(p->midi_velocity > 0) oparms->midiVelocity = p->midi_velocity;
+  else if(p->midi_velocity_amp > 0) oparms->midiVelocityAmp = p->midi_velocity_amp;
+
+  /* CSD line counts */
+  if(p->csd_line_counts > 0) oparms->useCsdLineCounts = p->csd_line_counts;
+
+  /* kr override */
+  if(p->control_rate_override > 0) oparms->kr_override = p->control_rate_override;
+
+  /* sr override */
+  if(p->sample_rate_override > 0) oparms->sr_override = p->sample_rate_override;
+
+  oparms->nchnls_override = p->nchnls_override;
+  oparms->nchnls_i_override = p->nchnls_i_override;
+  oparms->e0dbfs_override = p->e0dbfs_override;
+}
+
+PUBLIC void csoundGetParams(CSOUND *csound, CSOUND_PARAMS *p){
+
+  OPARMS *oparms = csound->oparms;
+
+  p->debug_mode = oparms->odebug;
+  p->displays = oparms->displays;
+  p->ascii_graphs = oparms->graphsoff;
+  p->postscript_graphs = oparms->postscript;
+  p->use_cscore = oparms->usingcscore;
+  p->ring_bell = oparms->ringbell;
+  p->defer_gen01_load = oparms->gen01defer;
+  p->terminate_on_midi = oparms->termifend;
+  p->no_default_paths = oparms->noDefaultPaths;
+  p->syntax_check_only = oparms->syntaxCheckOnly;
+  p->sample_accurate = oparms->sampleAccurate;
+  p->realtime_mode = oparms->realtime;
+  p->message_level = oparms->msglevel;
+  p->tempo = oparms->cmdTempo;
+  p->buffer_frames = oparms->outbufsamps;
+  p->hardware_buffer_frames = oparms->oMaxLag;
+  p->number_of_threads = oparms->numThreads;
+  p->midi_key = oparms->midiKey;
+  p->midi_key_cps = oparms->midiKeyCps;
+  p->midi_key_pch = oparms->midiKeyPch;
+  p->midi_key_oct = oparms->midiKeyOct;
+  p->midi_velocity = oparms->midiVelocity;
+  p->midi_velocity_amp = oparms->midiVelocityAmp;
+  p->csd_line_counts = oparms->useCsdLineCounts;
+  p->control_rate_override = oparms->kr_override;
+  p->sample_rate_override = oparms->sr_override;
+  p->nchnls_override = oparms->nchnls_override;
+  p->nchnls_i_override = oparms->nchnls_i_override;
+  p->e0dbfs_override = oparms->e0dbfs_override;
+  p->heartbeat = oparms->heartbeat;
+  p->ring_bell = oparms->ringbell;
+}
+
+
+PUBLIC void csoundSetOutput(CSOUND *csound, char *name, char *type, char *format){
+
+  OPARMS *oparms = csound->oparms;
+  char *typename;
+
+  /* if already compiled and running, return */
+  if(csound->engineStatus & CS_STATE_COMP) return;
+
+  oparms->outfilename =
+    csound->Malloc(csound, strlen(name) + 1); /* will be freed by memRESET */
+  strcpy(oparms->outfilename, name);
+  if (strcmp(oparms->outfilename, "stdout") == 0) {
+        set_stdout_assign(csound, STDOUTASSIGN_SNDFILE, 1);
+#if defined(WIN32)
+        csound->Warning(csound, Str("stdout not supported on this platform"));
+#endif
+      }
+  else set_stdout_assign(csound, STDOUTASSIGN_SNDFILE, 0);
+
+  oparms->sfwrite = 1;
+  if(type != NULL){
+  int i=0;
+  while((typename = file_type_map[i].format) != NULL) {
+    if(!strcmp(type,typename)) break;
+    i++;
+    }
+  if(typename != NULL) {
+    oparms->filetyp = file_type_map[i].type;
+  }
+  }
+  if(format != NULL){
+   int i=0;
+   while((typename = sample_format_map[i].longformat) != NULL) {
+    if(!strcmp(type,typename)) break;
+    i++;
+    }
+  if(format != NULL) {
+    set_output_format(oparms, sample_format_map[i].shortformat);
+  }
+  }
+}
+
+PUBLIC void csoundSetInput(CSOUND *csound, char *name) {
+  OPARMS *oparms = csound->oparms;
+
+  /* if already compiled and running, return */
+  if(csound->engineStatus & CS_STATE_COMP) return;
+
+  oparms->infilename =
+    csound->Malloc(csound, strlen(name)); /* will be freed by memRESET */
+  strcpy(oparms->infilename, name);
+  if (strcmp(oparms->infilename, "stdin") == 0) {
+        set_stdin_assign(csound, STDINASSIGN_SNDFILE, 1);
+#if defined(WIN32)
+        csound->Warning(csound, Str("stdin not supported on this platform"));
+#endif
+      }
+      else
+        set_stdin_assign(csound, STDINASSIGN_SNDFILE, 0);
+  oparms->sfread = 1;
+}
+
+PUBLIC void csoundSetMIDIInput(CSOUND *csound, char *name) {
+   OPARMS *oparms = csound->oparms;
+
+   /* if already compiled and running, return */
+  if(csound->engineStatus & CS_STATE_COMP) return;
+
+   oparms->Midiname =
+     csound->Malloc(csound, strlen(name)); /* will be freed by memRESET */
+   strcpy(oparms->Midiname, name);
+   if (!strcmp(oparms->Midiname, "stdin")) {
+        set_stdin_assign(csound, STDINASSIGN_MIDIDEV, 1);
+#if defined(WIN32)
+        csound->Warning(csound, Str("stdin not supported on this platform"));
+#endif
+      }
+      else
+        set_stdin_assign(csound, STDINASSIGN_MIDIDEV, 0);
+      oparms->Midiin = 1;
+}
+
+PUBLIC void csoundSetMIDIFileInput(CSOUND *csound, char *name) {
+   OPARMS *oparms = csound->oparms;
+
+   /* if already compiled and running, return */
+  if(csound->engineStatus & CS_STATE_COMP) return;
+
+   oparms->FMidiname =
+     csound->Malloc(csound, strlen(name)); /* will be freed by memRESET */
+   strcpy(oparms->FMidiname, name);
+   if (!strcmp(oparms->FMidiname, "stdin")) {
+        set_stdin_assign(csound, STDINASSIGN_MIDIFILE, 1);
+#if defined(WIN32)
+        csound->Warning(csound, Str("stdin not supported on this platform"));
+#endif
+      }
+      else
+        set_stdin_assign(csound, STDINASSIGN_MIDIFILE, 0);
+      oparms->FMidiin = 1;
+}
+
+PUBLIC void csoundSetMIDIFileOutput(CSOUND *csound, char *name) {
+   OPARMS *oparms = csound->oparms;
+
+   /* if already compiled and running, return */
+  if(csound->engineStatus & CS_STATE_COMP) return;
+
+   oparms->FMidioutname =
+     csound->Malloc(csound, strlen(name)); /* will be freed by memRESET */
+   strcpy(oparms->FMidioutname, name);
+}
+
+PUBLIC void csoundSetMIDIOutput(CSOUND *csound, char *name) {
+   OPARMS *oparms = csound->oparms;
+
+    /* if already compiled and running, return */
+  if(csound->engineStatus & CS_STATE_COMP) return;
+
+   oparms->Midioutname =
+     csound->Malloc(csound, strlen(name)); /* will be freed by memRESET */
+   strcpy(oparms->Midioutname, name);
+}
