@@ -27,15 +27,11 @@
 #include "csmodule.h"
 #include "corfile.h"
 
-#ifdef ENABLE_NEW_PARSER
 #include "csound_orc.h"
-#endif
 
-#ifdef PARCS
 #include "cs_par_base.h"
 #include "cs_par_orc_semantics.h"
 #include "cs_par_dispatch.h"
-#endif
 
 #if defined(USE_OPENMP)
 #include <omp.h>
@@ -45,98 +41,79 @@
 extern  void    dieu(CSOUND *, char *, ...);
 extern  int     argdecode(CSOUND *, int, char **);
 extern  int     init_pvsys(CSOUND *);
-extern  char    *get_sconame(CSOUND *);
+//extern  char    *get_sconame(CSOUND *);
 extern  void    print_benchmark_info(CSOUND *, const char *);
 extern  void    openMIDIout(CSOUND *);
 extern  int     read_unified_file(CSOUND *, char **, char **);
-extern  OENTRY  opcodlst_1[];
 extern  uintptr_t  kperfThread(void * cs);
-#if defined(ENABLE_NEW_PARSER)
-extern void cs_init_math_constants_macros(CSOUND *csound,void *yyscanner);
-extern void cs_init_omacros(CSOUND *csound, NAMES *nn);
-#endif
+extern void cs_init_math_constants_macros(CSOUND *csound, PRE_PARM *yyscanner);
+extern void cs_init_omacros(CSOUND *csound, PRE_PARM*, NAMES *nn);
 
-static void create_opcodlst(CSOUND *csound)
-{
-    OENTRY  *saved_opcodlst = csound->opcodlst;
-    int     old_cnt = 0, err;
+void checkOptions(CSOUND *csound) {
+      const char  *csrcname;
+      const char  *home_dir;
+      FILE        *csrc = NULL;
+      void        *fd = NULL;
+      char *s;
+      /* IV - Feb 17 2005 */
+      csrcname = csoundGetEnv(csound, "CSOUND6RC");
+      if (csrcname != NULL && csrcname[0] != '\0') {
+        fd = csound->FileOpen2(csound, &csrc, CSFILE_STD, csrcname, "r", NULL,
+                               CSFTYPE_OPTIONS, 0);
+        if (fd == NULL)
+          csoundMessage(csound, Str("WARNING: cannot open csound6rc file %s\n"),
+                                csrcname);
+        else
+          csound->Message(csound, Str("Reading options from $CSOUND6RC: %s \n"),
+                           csrcname);
+      }
+      if (fd == NULL && ((home_dir = csoundGetEnv(csound, "HOME")) != NULL &&
+                         home_dir[0] != '\0')) {
+        s = csoundConcatenatePaths(csound, home_dir, ".csound6rc");
+        fd = csound->FileOpen2(csound, &csrc, CSFILE_STD, s, "r", NULL,
+                               CSFTYPE_OPTIONS, 0);
+        if (fd != NULL)
+          csound->Message(csound, Str("Reading options from $HOME/.csound6rc\n"));
+        mfree(csound, s);
+      }
+      /* read global .csound6rc file (if exists) */
+      if (fd != NULL) {
 
-    if (saved_opcodlst != NULL) {
-      csound->opcodlst = NULL;
-      if (csound->oplstend != NULL)
-        old_cnt = (int) ((OENTRY*) csound->oplstend - (OENTRY*) saved_opcodlst);
-      csound->oplstend = NULL;
-      memset(csound->opcode_list, 0, sizeof(int) * 256);
-    }
-    /* Basic Entry1 stuff */
-    err = csoundAppendOpcodes(csound, &(opcodlst_1[0]), -1);
-    /* Add opcodes registered by host application */
-    if (old_cnt)
-      err |= csoundAppendOpcodes(csound, saved_opcodlst, old_cnt);
-    if (saved_opcodlst != NULL)
-      free(saved_opcodlst);
-    if (err)
-      csoundDie(csound, Str("Error allocating opcode list"));
+        readOptions(csound, csrc, 0);
+        csound->FileClose(csound, fd);
+      }
+      /* check for .csound6rc in current directory */
+      fd = csound->FileOpen2(csound, &csrc, CSFILE_STD, ".csound6rc", "r", NULL,
+                             CSFTYPE_OPTIONS, 0);
+      if (fd != NULL) {
+        readOptions(csound, csrc, 0);
+        csound->Message(csound,
+                        Str("Reading options from local directory .csound6rc \n"));
+        csound->FileClose(csound, fd);
+      }
 }
 
-PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
+
+PUBLIC int csoundCompileArgs(CSOUND *csound, int argc, char **argv)
 {
     OPARMS  *O = csound->oparms;
     char    *s;
-    char    *sortedscore = NULL;
-    //    char    *xtractedscore = "score.xtr";
     FILE    *xfile = NULL;
     int     n;
     int     csdFound = 0;
     char    *fileDir;
 
-    /* IV - Feb 05 2005: find out if csoundPreCompile() needs to be called */
-    if (csound->engineState != CS_STATE_PRE) {
-      csound->printerrormessagesflag = (void*)1234;
-      if ((n = csoundPreCompile(csound)) != CSOUND_SUCCESS)
-        return n;
-    }
 
     if ((n = setjmp(csound->exitjmp)) != 0) {
       return ((n - CSOUND_EXITJMP_SUCCESS) | CSOUND_EXITJMP_SUCCESS);
     }
 
-    init_pvsys(csound);
-    /* utilities depend on this as well as orchs; may get changed by an orch */
-    dbfs_init(csound, DFLT_DBFS);
-    csound->csRtClock = (RTCLOCK*) csound->Calloc(csound, sizeof(RTCLOCK));
-    csoundInitTimerStruct(csound->csRtClock);
-    csound->engineState |= CS_STATE_COMP | CS_STATE_CLN;
-
-#ifndef USE_DOUBLE
-#ifdef BETA
-    csound->Message(csound, Str("Csound version %s beta (float samples) %s\n"),
-                            CS_PACKAGE_VERSION, __DATE__);
-#else
-    csound->Message(csound, Str("Csound version %s (float samples) %s\n"),
-                            CS_PACKAGE_VERSION, __DATE__);
-#endif
-#else
-#ifdef BETA
-    csound->Message(csound, Str("Csound version %s beta (double samples) %s\n"),
-                            CS_PACKAGE_VERSION, __DATE__);
-#else
-    csound->Message(csound, Str("Csound version %s (double samples) %s\n"),
-                            CS_PACKAGE_VERSION, __DATE__);
-#endif
-#endif
-    {
-      char buffer[128];
-      sf_command(NULL, SFC_GET_LIB_VERSION, buffer, 128);
-      csound->Message(csound, "%s\n", buffer);
+    if(csound->engineStatus & CS_STATE_COMP){
+      csound->Message(csound, Str("Csound is already started, call csoundReset()\n"
+                                  "before starting again \n"));
+       return CSOUND_ERROR;
     }
 
-    /* do not know file type yet */
-    O->filetyp = -1;
-    O->sfheader = 0;
-    csound->peakchunks = 1;
-    create_opcodlst(csound);
-
     if (--argc <= 0) {
       dieu(csound, Str("insufficient arguments"));
     }
@@ -144,50 +121,56 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
     csound->orcname_mode = 0;   /* 0: normal, 1: ignore, 2: fail */
     if (argdecode(csound, argc, argv) == 0)
       csound->LongJmp(csound, 1);
-    /* do not allow orc/sco/csd name in .csoundrc */
+    /* do not allow orc/sco/csd name in .csound6rc */
     csound->orcname_mode = 2;
-    {
-      const char  *csrcname;
-      const char  *home_dir;
-      FILE        *csrc = NULL;
-      void        *fd = NULL;
-      /* IV - Feb 17 2005 */
-      csrcname = csoundGetEnv(csound, "CSOUNDRC");
-      if (csrcname != NULL && csrcname[0] != '\0') {
-        fd = csound->FileOpen2(csound, &csrc, CSFILE_STD, csrcname, "r", NULL,
-                               CSFTYPE_OPTIONS, 0);
-        if (fd == NULL)
-          csoundMessage(csound, Str("WARNING: cannot open csoundrc file %s\n"),
-                                csrcname);
-        else
-          csound->Message(csound, Str("Reading options from $CSOUNDRC: %s \n"),
-                           csrcname);
-      }
-      if (fd == NULL && ((home_dir = csoundGetEnv(csound, "HOME")) != NULL &&
-                         home_dir[0] != '\0')) {
-        s = csoundConcatenatePaths(csound, home_dir, ".csoundrc");
-        fd = csound->FileOpen2(csound, &csrc, CSFILE_STD, s, "r", NULL,
-                               CSFTYPE_OPTIONS, 0);
-        if (fd != NULL)
-          csound->Message(csound, Str("Reading options from $HOME/.csoundrc\n"));
-        mfree(csound, s);
-      }
-      /* read global .csoundrc file (if exists) */
-      if (fd != NULL) {
+    checkOptions(csound);
+    /* { */
+    /*   const char  *csrcname; */
+    /*   const char  *home_dir; */
+    /*   FILE        *csrc = NULL; */
+    /*   void        *fd = NULL; */
+    /*   /\* IV - Feb 17 2005 *\/ */
+     
+    /*   csrcname = csoundGetEnv(csound, "CSOUND6RC"); */
+    /*   if (csrcname != NULL && csrcname[0] != '\0') { */
+    /*     fd = csound->FileOpen2(csound, &csrc, CSFILE_STD, csrcname, "r", NULL, */
+    /*                            CSFTYPE_OPTIONS, 0); */
+    /*     if (fd == NULL) */
+    /*       csoundMessage(csound, Str("WARNING: cannot open csound6rc file %s\n"), */
+    /*                             csrcname); */
+    /*     else */
+    /*       csound->Message(csound, Str("Reading options from $CSOUND6RC: %s \n"), */
+    /*                        csrcname); */
+    /*   } */
+    /*   if (fd == NULL && ((home_dir = csoundGetEnv(csound, "HOME")) != NULL && */
+    /*                      home_dir[0] != '\0')) { */
+    /*     csound->Message(csound, Str("Reading options from $HOME/.csound6rc\n")); */
+    /*     s = csoundConcatenatePaths(csound, home_dir, ".csound6rc"); */
+        
+    /*     fd = csound->FileOpen2(csound, &csrc, CSFILE_STD, s, "r", NULL, */
+    /*                            CSFTYPE_OPTIONS, 0); */
 
-        readOptions(csound, csrc, 0);
-        csound->FileClose(csound, fd);
-      }
-      /* check for .csoundrc in current directory */
-      fd = csound->FileOpen2(csound, &csrc, CSFILE_STD, ".csoundrc", "r", NULL,
-                             CSFTYPE_OPTIONS, 0);
-      if (fd != NULL) {
-        readOptions(csound, csrc, 0);
-        csound->Message(csound,
-                        "Reading options from local directory .csoundrc \n");
-        csound->FileClose(csound, fd);
-      }
-    }
+    /*     csound->Message(csound, Str("%s=%p\n"), s,fd); */
+    /*     if (fd != NULL) */
+    /*       csound->Message(csound, Str("Reading options from $HOME/.csound6rc\n")); */
+    /*     mfree(csound, s); */
+    /*   } */
+    /*   /\* read global .csound6rc file (if exists) *\/ */
+    /*   if (fd != NULL) { */
+
+    /*     readOptions(csound, csrc, 0); */
+    /*     csound->FileClose(csound, fd); */
+    /*   } */
+    /*   /\* check for .csound6rc in current directory *\/ */
+    /*   fd = csound->FileOpen2(csound, &csrc, CSFILE_STD, ".csound6rc", "r", NULL, */
+    /*                          CSFTYPE_OPTIONS, 0); */
+    /*   if (fd != NULL) { */
+    /*     readOptions(csound, csrc, 0); */
+    /*     csound->Message(csound, */
+    /*                     Str("Reading options from local directory .csound6rc \n")); */
+    /*     csound->FileClose(csound, fd); */
+    /*   } */
+    /* } */
     if (csound->delayederrormessages) {
       if (O->msglevel>8)
         csound->Warning(csound, csound->delayederrormessages);
@@ -195,37 +178,39 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
       csound->delayederrormessages = NULL;
     }
     /* check for CSD file */
-    if (csound->orchname == NULL)
-      dieu(csound, Str("no orchestra name"));
-    else if (csound->scorename == NULL || csound->scorename[0] == (char) 0) {
-      int   tmp = (int) strlen(csound->orchname) - 4;
-      if (tmp >= 0 && csound->orchname[tmp] == '.' &&
-          tolower(csound->orchname[tmp + 1]) == 'c' &&
-          tolower(csound->orchname[tmp + 2]) == 's' &&
-          tolower(csound->orchname[tmp + 3]) == 'd') {
-        /* FIXME: allow orc/sco/csd name in CSD file: does this work ? */
-        csound->orcname_mode = 0;
-        csound->Message(csound, "UnifiedCSD:  %s\n", csound->orchname);
-
-        /* Add directory of CSD file to search paths before orchname gets
-         * replaced with temp orch name if default paths is enabled */
-        if (!O->noDefaultPaths) {
-          fileDir = csoundGetDirectoryForPath(csound, csound->orchname);
-          csoundAppendEnv(csound, "SADIR", fileDir);
-          csoundAppendEnv(csound, "SSDIR", fileDir);
-          csoundAppendEnv(csound, "INCDIR", fileDir);
-          csoundAppendEnv(csound, "MFDIR", fileDir);
-          mfree(csound, fileDir);
-        }
-
-        csound->csdname = csound->orchname; /* save original CSD name */
-        if (!read_unified_file(csound, &(csound->orchname),
-                                       &(csound->scorename))) {
-          csound->Die(csound, Str("Reading CSD failed ... stopping"));
-        }
-
-        csdFound = 1;
+    if (csound->orchname == NULL) {
+      if(csound->info_message_request) {
+        csound->info_message_request = 0;
+        csound->LongJmp(csound, 1);
       }
+      else dieu(csound, Str("no orchestra name"));
+
+    }
+    else if (csound->use_only_orchfile == 0
+             && (csound->scorename == NULL || csound->scorename[0] == (char) 0)
+             && csound->orchname[0] != '\0') {
+      /* FIXME: allow orc/sco/csd name in CSD file: does this work ? */
+      csound->orcname_mode = 0;
+      csound->Message(csound, "UnifiedCSD:  %s\n", csound->orchname);
+
+      /* Add directory of CSD file to search paths before orchname gets
+       * replaced with temp orch name if default paths is enabled */
+      if (!O->noDefaultPaths) {
+        fileDir = csoundGetDirectoryForPath(csound, csound->orchname);
+        csoundAppendEnv(csound, "SADIR", fileDir);
+        csoundAppendEnv(csound, "SSDIR", fileDir);
+        csoundAppendEnv(csound, "INCDIR", fileDir);
+        csoundAppendEnv(csound, "MFDIR", fileDir);
+        mfree(csound, fileDir);
+      }
+
+      csound->csdname = csound->orchname; /* save original CSD name */
+      if (!read_unified_file(csound, &(csound->orchname),
+                                       &(csound->scorename))) {
+        csound->Die(csound, Str("Reading CSD failed ... stopping"));
+      }
+
+      csdFound = 1;
     }
 
     /* IV - Feb 19 2005: run a second pass of argdecode so that */
@@ -242,50 +227,11 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
         (csound->stdout_assign_flg & (csound->stdout_assign_flg - 1)) != 0) {
       csound->Die(csound, Str("error: multiple uses of stdout"));
     }
-    /* done parsing csoundrc, CSD, and command line options */
-    /* if sound file type is still not known, check SFOUTYP */
-    if (O->filetyp <= 0) {
-      const char  *envoutyp;
-      envoutyp = csoundGetEnv(csound, "SFOUTYP");
-      if (envoutyp != NULL && envoutyp[0] != '\0') {
-        if (strcmp(envoutyp, "AIFF") == 0)
-          O->filetyp = TYP_AIFF;
-        else if (strcmp(envoutyp, "WAV") == 0 || strcmp(envoutyp, "WAVE") == 0)
-          O->filetyp = TYP_WAV;
-        else if (strcmp(envoutyp, "IRCAM") == 0)
-          O->filetyp = TYP_IRCAM;
-        else if (strcmp(envoutyp, "RAW") == 0)
-          O->filetyp = TYP_RAW;
-        else {
-          dieu(csound, Str("%s not a recognised SFOUTYP env setting"),
-                       envoutyp);
-        }
-      }
-      else
-#if !defined(__MACH__) && !defined(mac_classic)
-        O->filetyp = TYP_WAV;   /* default to WAV if even SFOUTYP is unset */
-#else
-        O->filetyp = TYP_AIFF;  /* ... or AIFF on the Mac */
-#endif
-    }
-    /* everything other than a raw sound file has a header */
-    O->sfheader = (O->filetyp == TYP_RAW ? 0 : 1);
-    if (O->Linein || O->Midiin || O->FMidiin)
-      O->RTevents = 1;
-    if (!O->sfheader)
-      O->rewrt_hdr = 0;         /* cannot rewrite header of headerless file */
-    if (O->sr_override || O->kr_override) {
-      if (!O->sr_override || !O->kr_override)
-        dieu(csound, Str("srate and krate overrides must occur jointly"));
-    }
-    if (!O->outformat)                      /* if no audioformat yet  */
-      O->outformat = AE_SHORT;              /*  default to short_ints */
-    O->sfsampsize = sfsampsize(FORMAT2SF(O->outformat));
-    O->informat = O->outformat;             /* informat default */
+    /* done parsing csound6rc, CSD, and command line options */
 
     if (csound->scorename == NULL && csound->scorestr==NULL) {
       /* No scorename yet */
-      csound->scorestr = corfile_create_r("f0 42000\n");
+      csound->scorestr = corfile_create_r("f0 800000000000.0\n");
       corfile_flush(csound->scorestr);
       if (O->RTevents)
         csound->Message(csound, Str("realtime performance using dummy "
@@ -316,52 +262,38 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
       if (csound->orchstr==NULL)
         csound->Die(csound,
                     Str("Failed to open input file %s\n"), csound->orchname);
-#ifdef ENABLE_NEW_PARSER
-      if (O->newParser) corfile_puts("\n#exit\n", csound->orchstr);
-#endif
+      corfile_puts("\n#exit\n", csound->orchstr);
       corfile_putc('\0', csound->orchstr);
       corfile_putc('\0', csound->orchstr);
       //csound->orchname = NULL;
     }
     if (csound->xfilename != NULL)
       csound->Message(csound, "xfilename: %s\n", csound->xfilename);
-    /* IV - Oct 31 2002: moved orchestra compilation here, so that named */
-    /* instrument numbers are known at the score read/sort stage */
+
     csoundLoadExternals(csound);    /* load plugin opcodes */
-    /* IV - Jan 31 2005: initialise external modules */
+     /* VL: added this also to csoundReset() in csound.c   */
     if (csoundInitModules(csound) != 0)
       csound->LongJmp(csound, 1);
+     if(csoundCompileOrc(csound, NULL) != 0){
+        csoundDie(csound, Str("cannot compile orchestra \n"));
+     }
+     csound->modules_loaded = 1;
 
-#ifdef ENABLE_NEW_PARSER
-    if (O->newParser) {
-      int new_orc_parser(CSOUND *);
-      if (new_orc_parser(csound)) {
-        csoundDie(csound, Str("Stopping on parser failure\n"));
-      }
+    s = csoundQueryGlobalVariable(csound, "_RTMIDI");
+    if(csound->enableHostImplementedMIDIIO == 1) {
+    strcpy(s, "hostbased");
+    csoundSetConfigurationVariable(csound,"rtmidi", s);
     }
-    else {
-      csound->Message(csound, "********************\n");
-      csound->Message(csound, "* USING OLD PARSER *\n");
-      csound->Message(csound, "********************\n");
-      otran(csound);                  /* read orcfile, setup desblks & spaces */
-    }
-#else
-    otran(csound);                  /* read orcfile, setup desblks & spaces */
-#endif
-#if defined(USE_OPENMP)
-    if (csound->oparms->numThreads > 1) {
-      omp_set_num_threads(csound->oparms->numThreads);
-      csound->Message(csound, "OpenMP enabled: requested %d threads.\n",
-                      csound->oparms->numThreads);
-    }
-#endif
+
+
 
     /* IV - Jan 28 2005 */
     print_benchmark_info(csound, Str("end of orchestra compile"));
     if (!csoundYield(csound))
       return -1;
     /* IV - Oct 31 2002: now we can read and sort the score */
-    if ((n = strlen(csound->scorename)) > 4 &&  /* if score ?.srt or ?.xtr */
+    if (csound->scorename != NULL &&
+        (n = strlen(csound->scorename)) > 4 &&  /* if score ?.srt or ?.xtr */
         (!strcmp(csound->scorename + (n - 4), ".srt") ||
          !strcmp(csound->scorename + (n - 4), ".xtr"))) {
       csound->Message(csound, Str("using previous %s\n"), csound->scorename);
@@ -370,7 +302,7 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
       csound->scorestr = copy_to_corefile(csound, csound->scorename, NULL, 1);
     }
     else {
-      sortedscore = NULL;
+      //sortedscore = NULL;
       if (csound->scorestr==NULL) {
         csound->scorestr = copy_to_corefile(csound, csound->scorename, NULL, 1);
         if (csound->scorestr==NULL)
@@ -412,7 +344,142 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
     if (O->Midioutname != NULL || O->FMidioutname != NULL)
       openMIDIout(csound);
 
-#ifdef PARCS
+    return CSOUND_SUCCESS;
+}
+
+extern int  playopen_dummy(CSOUND *, const csRtAudioParams *parm);
+extern void rtplay_dummy(CSOUND *, const MYFLT *outBuf, int nbytes);
+extern int  recopen_dummy(CSOUND *, const csRtAudioParams *parm);
+extern int  rtrecord_dummy(CSOUND *, MYFLT *inBuf, int nbytes);
+extern void rtclose_dummy(CSOUND *);
+extern int  audio_dev_list_dummy(CSOUND *, CS_AUDIODEVICE *, int);
+extern int  midi_dev_list_dummy(CSOUND *csound, CS_MIDIDEVICE *list, int isOutput);
+extern int DummyMidiInOpen(CSOUND *csound, void **userData,
+                           const char *devName);
+extern int DummyMidiRead(CSOUND *csound, void *userData,
+                         unsigned char *buf, int nbytes);
+extern int DummyMidiOutOpen(CSOUND *csound, void **userData,
+                     const char *devName);
+extern int DummyMidiWrite(CSOUND *csound, void *userData,
+                   const unsigned char *buf, int nbytes);
+
+
+PUBLIC int csoundStart(CSOUND *csound) // DEBUG
+{
+    OPARMS  *O = csound->oparms;
+    int     n;
+
+    /* if a CSD was not used, check options */
+    if(csound->csdname == NULL)
+          checkOptions(csound);
+
+   if(csound->engineStatus & CS_STATE_COMP){
+       csound->Message(csound, "Csound is already started, call csoundReset()\n"
+                                "before starting again \n");
+       return CSOUND_ERROR;
+    }
+
+
+
+
+   { /* test for dummy module request */
+    char *s;
+     if((s = csoundQueryGlobalVariable(csound, "_RTAUDIO")) != NULL)
+       if(strcmp(s, "null") == 0 || strcmp(s, "Null") == 0 ||
+           strcmp(s, "NULL") == 0) {
+        csound->Message(csound, Str("setting dummy interface\n"));
+      csound->SetPlayopenCallback(csound, playopen_dummy);
+      csound->SetRecopenCallback(csound, recopen_dummy);
+      csound->SetRtplayCallback(csound, rtplay_dummy);
+      csound->SetRtrecordCallback(csound, rtrecord_dummy);
+      csound->SetRtcloseCallback(csound, rtclose_dummy);
+      csound->SetAudioDeviceListCallback(csound, audio_dev_list_dummy);
+        }
+
+     /* and midi */
+  if(csound->enableHostImplementedMIDIIO == 0){
+  if((s = csoundQueryGlobalVariable(csound, "_RTMIDI")) != NULL)
+    if(strcmp(s, "null") == 0 || strcmp(s, "Null") == 0 ||
+     strcmp(s, "NULL") == 0) {
+     csound->SetMIDIDeviceListCallback(csound, midi_dev_list_dummy);
+     csound->SetExternalMidiInOpenCallback(csound, DummyMidiInOpen);
+     csound->SetExternalMidiReadCallback(csound,  DummyMidiRead);
+     csound->SetExternalMidiInCloseCallback(csound, NULL);
+     csound->SetExternalMidiOutOpenCallback(csound,  DummyMidiOutOpen);
+     csound->SetExternalMidiWriteCallback(csound, DummyMidiWrite);
+     csound->SetExternalMidiOutCloseCallback(csound, NULL);
+     }
+     }
+  else {
+   s = csoundQueryGlobalVariable(csound, "_RTMIDI");
+   strcpy(s, "hostbased");
+   csoundSetConfigurationVariable(csound,"rtmidi", s);
+  }
+   }
+
+
+   /* VL 30-12-12 csoundInitModules is always called here now to enable
+       Csound to start without calling csoundCompile, but directly from
+       csoundCompileOrc() and csoundReadSco()
+    */
+   if(csound->modules_loaded == 0){
+    csoundLoadExternals(csound);    /* load plugin opcodes */
+    if (csoundInitModules(csound) != 0)
+           csound->LongJmp(csound, 1);
+    csound->modules_loaded = 1;
+   }
+    if (csound->instr0 == NULL) { /* compile empty instr 1 to allow csound to
+                                     start with no orchestra */
+        csoundCompileOrc(csound, "instr 1 \n endin \n");
+     }
+
+    if ((n = setjmp(csound->exitjmp)) != 0) {
+      return ((n - CSOUND_EXITJMP_SUCCESS) | CSOUND_EXITJMP_SUCCESS);
+    }
+
+
+
+    /* if sound file type is still not known, check SFOUTYP */
+    if (O->filetyp <= 0) {
+      const char  *envoutyp;
+      envoutyp = csoundGetEnv(csound, "SFOUTYP");
+      if (envoutyp != NULL && envoutyp[0] != '\0') {
+        if (strcmp(envoutyp, "AIFF") == 0)
+          O->filetyp = TYP_AIFF;
+        else if (strcmp(envoutyp, "WAV") == 0 || strcmp(envoutyp, "WAVE") == 0)
+          O->filetyp = TYP_WAV;
+        else if (strcmp(envoutyp, "IRCAM") == 0)
+          O->filetyp = TYP_IRCAM;
+        else if (strcmp(envoutyp, "RAW") == 0)
+          O->filetyp = TYP_RAW;
+        else {
+          dieu(csound, Str("%s not a recognised SFOUTYP env setting"),
+               envoutyp);
+        }
+      }
+      else
+#if !defined(__MACH__)
+        O->filetyp = TYP_WAV;   /* default to WAV if even SFOUTYP is unset */
+#else
+      O->filetyp = TYP_AIFF;  /* ... or AIFF on the Mac */
+#endif
+    }
+    /* everything other than a raw sound file has a header */
+    O->sfheader = (O->filetyp == TYP_RAW ? 0 : 1);
+    if (O->Linein || O->Midiin || O->FMidiin)
+      O->RTevents = 1;
+    if (!O->sfheader)
+      O->rewrt_hdr = 0;         /* cannot rewrite header of headerless file */
+    if (O->sr_override || O->kr_override) {
+      if (!O->sr_override || !O->kr_override)
+        dieu(csound, Str("srate and krate overrides must occur jointly"));
+    }
+    if (!O->outformat)                      /* if no audioformat yet  */
+      O->outformat = AE_SHORT;              /*  default to short_ints */
+    O->sfsampsize = sfsampsize(FORMAT2SF(O->outformat));
+    O->informat = O->outformat;             /* informat default */
+
+
     if (O->numThreads > 1) {
       void csp_barrier_alloc(CSOUND *, pthread_barrier_t **, int);
       int i;
@@ -442,11 +509,15 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
       }
 
       csound->WaitBarrier(csound->barrier2);
-
-      csp_parallel_compute_spec_setup(csound);
     }
-#endif
-
-
+    csound->engineStatus |= CS_STATE_COMP;
     return musmon(csound);
 }
+
+PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv){
+
+  int result = csoundCompileArgs(csound,argc,argv);
+
+  if(result == CSOUND_SUCCESS) return csoundStart(csound);
+  else return result;
+}
