@@ -56,23 +56,7 @@ const uint32_t csScoSortMask = 4;
 const uint32_t csMidiScoMask = 8;
 const uint32_t csPlayScoMask = 16;
 
-typedef struct namelst {
-  char           *name;
-  struct namelst *next;
-} NAMELST;
-
-typedef struct {
-    NAMELST *toremove;
-    char    orcname[L_tmpnam + 4];
-    char    sconame[L_tmpnam + 4];
-    char    midname[L_tmpnam + 4];
-    int     midiSet;
-    int     csdlinecount;
-    char    *orcstr;
-    char    *scostr;
-} ONE_FILE_GLOBALS;
-
-#define ST(x)   (((ONE_FILE_GLOBALS*) csound->oneFileGlobals)->x)
+#define STA(x)   (csound->onefileStatics.x)
 
 #if !defined(WIN32)
 char *mytmpnam(char *name)
@@ -115,7 +99,8 @@ CS_NOINLINE char *csoundTmpFileName(CSOUND *csound, char *buf, const char *ext)
           snprintf(buf, nBytes, "%s/csound-XXXXXX", tmpdir);
         else
           strcpy(buf, "/tmp/csound-XXXXXX");
-        umask(0077); /* ensure exclusive access on buggy implementations of mkstemp */
+        umask(0077);
+        /* ensure exclusive access on buggy implementations of mkstemp */
         if (UNLIKELY((fd = mkstemp(buf)) < 0))
           csound->Die(csound, Str(" *** cannot create temporary file"));
         close(fd);
@@ -164,20 +149,19 @@ CS_NOINLINE char *csoundTmpFileName(CSOUND *csound, char *buf, const char *ext)
     return buf;
 }
 
-static void alloc_globals(CSOUND *csound)
+static inline void alloc_globals(CSOUND *csound)
 {
-    if (UNLIKELY(csound->oneFileGlobals == NULL)) {
-      csound->oneFileGlobals = mcalloc(csound, sizeof(ONE_FILE_GLOBALS));
+    /* if (UNLIKELY(csound->oneFileGlobals == NULL)) { */
+    /*   csound->oneFileGlobals = mcalloc(csound, sizeof(ONE_FILE_GLOBALS)); */
       /* count lines from 0 so that it adds OK to orc/sco counts */
-      ST(csdlinecount) = 0;
-    }
+    STA(csdlinecount) = 0;
 }
 
-char *get_sconame(CSOUND *csound)
-{
-    alloc_globals(csound);
-    return ST(sconame);
-}
+/* char *get_sconame(CSOUND *csound) */
+/* { */
+/*     //alloc_globals(csound); */
+/*     return STA(sconame); */
+/* } */
 
 static char *my_fgets(CSOUND *csound, char *s, int n, FILE *stream)
 {
@@ -191,7 +175,7 @@ static char *my_fgets(CSOUND *csound, char *s, int n, FILE *stream)
         break; /* add NULL even if ferror(), spec says 'indeterminate' */
       }
       if (ch == '\n' || ch == '\r') {   /* end of line ? */
-        ++(ST(csdlinecount));           /* count the lines */
+        ++(STA(csdlinecount));          /* count the lines */
         *(s++) = '\n';                  /* convert */
         if (ch == '\r') {
           ch = getc(stream);
@@ -209,18 +193,18 @@ static char *my_fgets(CSOUND *csound, char *s, int n, FILE *stream)
 void remove_tmpfiles(CSOUND *csound)            /* IV - Feb 03 2005 */
 {                               /* use one fn to delete all temporary files */
     alloc_globals(csound);
-    while (ST(toremove) != NULL) {
-      NAMELST *nxt = ST(toremove)->next;
+    while (STA(toremove) != NULL) {
+      NAMELST *nxt = STA(toremove)->next;
 #ifdef BETA
       csoundMessage(csound, Str("Removing temporary file %s ...\n"),
-                            ST(toremove)->name);
+                            STA(toremove)->name);
 #endif
-      if (remove(ST(toremove)->name))
+      if (remove(STA(toremove)->name))
         csoundMessage(csound, Str("WARNING: could not remove %s\n"),
-                              ST(toremove)->name);
-      mfree(csound, ST(toremove)->name);
-      mfree(csound, ST(toremove));
-      ST(toremove) = nxt;
+                              STA(toremove)->name);
+      mfree(csound, STA(toremove)->name);
+      mfree(csound, STA(toremove));
+      STA(toremove) = nxt;
     }
 }
 
@@ -231,8 +215,8 @@ void add_tmpfile(CSOUND *csound, char *name)    /* IV - Feb 03 2005 */
     tmp = (NAMELST*) mmalloc(csound, sizeof(NAMELST));
     tmp->name = (char*) mmalloc(csound, strlen(name) + 1);
     strcpy(tmp->name, name);
-    tmp->next = ST(toremove);
-    ST(toremove) = tmp;
+    tmp->next = STA(toremove);
+    STA(toremove) = tmp;
 }
 
 /* readingCsOptions should be non-zero when readOptions() is called
@@ -244,7 +228,7 @@ int readOptions(CSOUND *csound, FILE *unf, int readingCsOptions)
     char  *argv[CSD_MAX_ARGS];
     char    buffer[CSD_MAX_LINE_LEN];
 
-    alloc_globals(csound);
+    //alloc_globals(csound);
     while (my_fgets(csound, buffer, CSD_MAX_LINE_LEN, unf) != NULL) {
       p = buffer;
       /* Remove trailing spaces; rather heavy handed */
@@ -286,9 +270,15 @@ int readOptions(CSOUND *csound, FILE *unf, int readingCsOptions)
               csoundDie(csound, Str("More than %d arguments in <CsOptions>"),
                         CSD_MAX_ARGS);
             argv[++argc] = ++p;
-            while (*p != '"' && *p != '\0') p++;
-
-            if (*p == '"') *p = '\0';
+            while (*p != '"' && *p != '\0') {
+              if (*p == '\\' && *(p+1) != '\0')
+                p++;
+              p++;
+            }
+            if (*p == '"') {
+              /* ETX char used to mark the limits of a string */
+              *p = (isspace(*(p+1)) ? '\0' : 3);
+            }
             break;
           }
 
@@ -319,6 +309,27 @@ int readOptions(CSOUND *csound, FILE *unf, int readingCsOptions)
           *p = '\0';
           break;
         }
+        else if (*p=='"') {
+          int is_escape = 0;
+          char *old = NULL;
+          *p=3; /* ETX char used to mark the limits of a string */
+          while ((*p != '"' || is_escape) && *p != '\0') {
+            if (is_escape)
+              *old = 0x18; /* CAN char used to mark a removable character */
+            is_escape = (*p == '\\' ? !is_escape : 0);
+            old = p;
+            p++;
+          }
+          if (*p == '"') {
+            if (isspace(*(p+1))) {
+              *p = '\0';
+              break;
+            }
+            else {
+              *p = 3;
+            }
+          }
+        }
         p++;
       }
 #ifdef _DEBUG
@@ -338,8 +349,8 @@ int readOptions(CSOUND *csound, FILE *unf, int readingCsOptions)
     }
     if (UNLIKELY(readingCsOptions))
       csoundErrorMsg(csound, Str("Missing end tag </CsOptions>"));
-    else
-      ST(csdlinecount) = 0;
+    /* else */
+    /*   STA(csdlinecount) = 0; */
  return FALSE;
 }
 
@@ -351,15 +362,13 @@ static int createOrchestra(CSOUND *csound, FILE *unf)
     CORFIL *incore = corfile_create_w();
     char  buffer[CSD_MAX_LINE_LEN];
 
-    csound->orcLineOffset = ST(csdlinecount)+1;
+    csound->orcLineOffset = STA(csdlinecount)+1;
     while (my_fgets(csound, buffer, CSD_MAX_LINE_LEN, unf)!= NULL) {
       p = buffer;
       while (*p == ' ' || *p == '\t') p++;
       if (strstr(p, "</CsInstruments>") == p) {
         //corfile_flush(incore);
-#ifdef ENABLE_NEW_PARSER
-        if (csound->oparms->newParser) corfile_puts("\n#exit\n", incore);
-#endif
+        corfile_puts("\n#exit\n", incore);
         corfile_putc('\0', incore);
         corfile_putc('\0', incore);
         csound->orchstr = incore;
@@ -379,7 +388,7 @@ static int createScore(CSOUND *csound, FILE *unf)
 
     if (csound->scorestr == NULL)
       csound->scorestr = corfile_create_w();
-    csound->scoLineOffset = ST(csdlinecount);
+    csound->scoLineOffset = STA(csdlinecount);
     while (my_fgets(csound, buffer, CSD_MAX_LINE_LEN, unf)!= NULL) {
       p = buffer;
       while (*p == ' ' || *p == '\t') p++;
@@ -414,7 +423,7 @@ static int createExScore(CSOUND *csound, char *p, FILE *unf)
     *q = '\0';
     strcpy(prog, p+5); /* after "<CsExScore " */
     /* Generate score name */
-    csoundTmpFileName(csound, ST(sconame), ".sco");
+    csoundTmpFileName(csound, STA(sconame), ".sco");
     csoundTmpFileName(csound, extname, ".ext");
     fd = csoundFileOpenWithType(csound, &scof, CSFILE_STD, extname, "w", NULL,
                                 CSFTYPE_SCORE, 1);
@@ -425,31 +434,32 @@ static int createExScore(CSOUND *csound, char *p, FILE *unf)
     if (UNLIKELY(fd == NULL))
       return FALSE;
 
-    csound->scoLineOffset = ST(csdlinecount);
+    csound->scoLineOffset = STA(csdlinecount);
     while (my_fgets(csound, buffer, CSD_MAX_LINE_LEN, unf)!= NULL) {
       p = buffer;
       if (strstr(p, "</CsScore>") == p) {
         char sys[1024];
         csoundFileClose(csound, fd);
-        sprintf(sys, "%s %s %s", prog, extname, ST(sconame));
+        sprintf(sys, "%s %s %s", prog, extname, STA(sconame));
         if (UNLIKELY(system(sys) != 0)) {
           csoundErrorMsg(csound, Str("External generation failed"));
           remove(extname);
-          remove(ST(sconame));
+          remove(STA(sconame));
           return FALSE;
         }
         remove(extname);
         if (csound->scorestr == NULL)
           csound->scorestr = corfile_create_w();
-        fd = csoundFileOpen(csound, &scof, CSFILE_STD, ST(sconame), "r", NULL);
+        fd = csoundFileOpenWithType(csound, &scof, CSFILE_STD, STA(sconame),
+                                    "r", NULL, CSFTYPE_SCORE, 0);
         if (UNLIKELY(fd == NULL)) {
-          remove(ST(sconame));
+          remove(STA(sconame));
           return FALSE;
         }
         while (my_fgets(csound, buffer, CSD_MAX_LINE_LEN, scof)!= NULL)
           corfile_puts(buffer, csound->scorestr);
         csoundFileClose(csound, fd);
-        remove(ST(sconame));
+        remove(STA(sconame));
         return TRUE;
       }
       else fputs(buffer, scof);
@@ -467,11 +477,11 @@ static void read_base64(CSOUND *csound, FILE *in, FILE *out)
     while ((c = getc(in)) != '=' && c != '<') {
       while (c == ' ' || c == '\t' || c == '\n' || c == '\r') {
         if (c == '\n') {               /* count lines */
-          ++(ST(csdlinecount));
+          ++(STA(csdlinecount));
           c = getc(in);
         }
         else if (c == '\r') {
-          ++(ST(csdlinecount));
+          ++(STA(csdlinecount));
           c = getc(in);
           if (c == '\n') c = getc(in); /* DOS format */
         }
@@ -523,18 +533,18 @@ static int createMIDI2(CSOUND *csound, FILE *unf)
     char  buffer[CSD_MAX_LINE_LEN];
 
     /* Generate MIDI file name */
-    csoundTmpFileName(csound, ST(midname), ".mid");
-    fd = csoundFileOpenWithType(csound, &midf, CSFILE_STD, ST(midname), "wb", NULL,
-                                CSFTYPE_STD_MIDI, 1);
+    csoundTmpFileName(csound, STA(midname), ".mid");
+    fd = csoundFileOpenWithType(csound, &midf, CSFILE_STD, STA(midname),
+                                "wb", NULL, CSFTYPE_STD_MIDI, 1);
     if (UNLIKELY(fd == NULL)) {
       csoundDie(csound, Str("Cannot open temporary file (%s) for MIDI subfile"),
-                        ST(midname));
+                        STA(midname));
     }
     csound->tempStatus |= csMidiScoMask;
     read_base64(csound, unf, midf);
     csoundFileClose(csound, fd);
-    add_tmpfile(csound, ST(midname));               /* IV - Feb 03 2005 */
-    ST(midiSet) = TRUE;
+    add_tmpfile(csound, STA(midname));               /* IV - Feb 03 2005 */
+    STA(midiSet) = TRUE;
     while (TRUE) {
       if (my_fgets(csound, buffer, CSD_MAX_LINE_LEN, unf)!= NULL) {
         p = buffer;
@@ -604,7 +614,8 @@ static int createFile(CSOUND *csound, char *buffer, FILE *unf)
     strcpy(filename, p);
 //sscanf(buffer, "<CsFileB filename=\"%s\">", filename);
 //    if (filename[0] != '\0' &&
-//       filename[strlen(filename) - 1] == '>' && filename[strlen(filename) - 2] == '"')
+//       filename[strlen(filename) - 1] == '>' &&
+//       filename[strlen(filename) - 2] == '"')
 //    filename[strlen(filename) - 2] = '\0';
     if (UNLIKELY((smpf = fopen(filename, "rb")) != NULL)) {
       fclose(smpf);
@@ -690,7 +701,7 @@ static int checkLicence(CSOUND *csound, FILE *unf)
         csound->SF_csd_licence = licence;
         return TRUE;
       }
-      csoundMessage(csound, "%s", p);
+      csoundMessage(csound, p);
       len += strlen(p);
       licence = mrealloc(csound, licence, len);
       strncat(licence, p, len);
@@ -732,8 +743,8 @@ int read_unified_file(CSOUND *csound, char **pname, char **score)
       return 0;
     }
     alloc_globals(csound);
-    ST(orcname)[0] = ST(sconame)[0] = ST(midname)[0] = '\0';
-    ST(midiSet) = FALSE;
+    STA(orcname)[0] = STA(sconame)[0] = STA(midname)[0] = '\0';
+    STA(midiSet) = FALSE;
 #ifdef _DEBUG
     csoundMessage(csound, "Calling unified file system with %s\n", name);
 #endif
@@ -749,10 +760,10 @@ int read_unified_file(CSOUND *csound, char **pname, char **score)
                strstr(p, "</CsoundSynthesiser>") == p) {
         if (csound->scorestr != NULL)
           corfile_flush(csound->scorestr);
-        *pname = ST(orcname);
-        *score = ST(sconame);
-        if (ST(midiSet)) {
-          csound->oparms->FMidiname = ST(midname);
+        *pname = STA(orcname);
+        *score = STA(sconame);
+        if (STA(midiSet)) {
+          csound->oparms->FMidiname = STA(midname);
           csound->oparms->FMidiin = 1;
         }
         csoundFileClose(csound, fd);
@@ -822,15 +833,15 @@ int read_unified_file(CSOUND *csound, char **pname, char **score)
         csoundMessage(csound, Str("unknown CSD tag: %s\n"), buffer);
       }
     }
-      if (UNLIKELY(!started)) {
+    if (UNLIKELY(!started)) {
       csoundMessage(csound,
                     Str("Could not find <CsoundSynthesizer> tag in CSD file.\n"));
       result = FALSE;
     }
-    *pname = ST(orcname);
-    *score = ST(sconame);
-    if (ST(midiSet)) {
-      csound->oparms->FMidiname = ST(midname);
+    *pname = STA(orcname);
+    *score = STA(sconame);
+    if (STA(midiSet)) {
+      csound->oparms->FMidiname = STA(midname);
       csound->oparms->FMidiin = 1;
     }
     csoundFileClose(csound, fd);
