@@ -25,7 +25,8 @@
 #endif
 
 #ifndef HAVE_GETTIMEOFDAY
-#if defined(LINUX) || defined(__unix) || defined(__unix__) || defined(__MACH__)
+#if defined(LINUX)    || defined(__unix)   || defined(__unix__) || \
+    defined(__MACH__) || defined(__HAIKU__)
 #define HAVE_GETTIMEOFDAY 1
 #endif
 #endif
@@ -39,7 +40,7 @@
 #include <process.h>
 
 #ifndef MSVC
-void __stdcall GetSystemTimeAsFileTime(FILETIME*);
+//void __stdcall GetSystemTimeAsFileTime(FILETIME*);
 #endif
 
 void gettimeofday_(struct timeval* p, void* tz /* IGNORED */)
@@ -85,24 +86,6 @@ PUBLIC void csoundSleep(size_t milliseconds)
     Sleep((DWORD) milliseconds);
 }
 
-#elif defined(mac_classic)
-
-static CS_NOINLINE void notImplementedWarning_(const char *name)
-{
-    fprintf(stderr, Str("%s() is not implemented on this platform.\n"), name);
-}
-
-PUBLIC long csoundRunCommand(const char * const *argv, int noWait)
-{
-    notImplementedWarning_("csoundRunCommand");
-    return -1L;
-}
-
-PUBLIC void csoundSleep(size_t milliseconds)
-{
-    notImplementedWarning_("csoundSleep");
-}
-
 #else
 
 #include <sys/wait.h>
@@ -168,7 +151,8 @@ PUBLIC void csoundSleep(size_t milliseconds)
 
 
 
-#if defined(LINUX) || defined(__MACH__) || defined(WIN32)
+#if defined(LINUX) || defined(__MACH__) || defined(__HAIKU__) || \
+    defined(ANDROID) || defined(WIN32)
 
 #include <errno.h>
 #include <pthread.h>
@@ -183,7 +167,7 @@ PUBLIC void csoundSleep(size_t milliseconds)
 #define BARRIER_SERIAL_THREAD (-1)
 
 #if !defined(HAVE_PTHREAD_BARRIER_INIT)
-#ifndef __MACH__
+#if !defined(__MACH__) && !defined(__HAIKU__) && !defined(ANDROID)
 
 typedef struct barrier {
     pthread_mutex_t mut;
@@ -225,7 +209,7 @@ PUBLIC uintptr_t csoundJoinThread(void *thread)
     }
 }
 
-#if !defined(ANDROID) && (defined(LINUX) || defined(WIN32))
+#if !defined(ANDROID) && (defined(LINUX) || defined(__HAIKU__) || defined(WIN32))
 
 PUBLIC void *csoundCreateThreadLock(void)
 {
@@ -382,7 +366,7 @@ PUBLIC void csoundNotifyThreadLock(void *threadLock)
 
 PUBLIC void csoundDestroyThreadLock(void *threadLock)
 {
-    CsoundThreadLock_t  *p;
+     CsoundThreadLock_t  *p;
 
     if (threadLock == NULL)
       return;
@@ -391,6 +375,7 @@ PUBLIC void csoundDestroyThreadLock(void *threadLock)
     pthread_cond_destroy(&(p->c));
     pthread_mutex_destroy(&(p->m));
     free(threadLock);
+
 }
 
 #endif  /* !LINUX */
@@ -416,7 +401,7 @@ PUBLIC void *csoundCreateBarrier(unsigned int max)
   pthread_barrier_t *barrier =
     (pthread_barrier_t *) malloc(sizeof(pthread_barrier_t));
   int status = pthread_barrier_init(barrier, 0, max-1);
-  fprintf(stderr, "Create barrier %d => %p ($d)\n", max, barrier, status);
+  fprintf(stderr, "Create barrier %d => %p (%d)\n", max, barrier, status);
   if (status) return 0;
   return barrier;
 #endif
