@@ -24,9 +24,10 @@
 */
 
 #include <stdio.h>
-#include <m_pd.h>
+#include <m_pd.h>  
+#include <pthread.h>
 #include "csound.h"
-
+     
 #define CS_MAX_CHANS 32
 #define MAXMESSTRING 16384
 
@@ -72,6 +73,7 @@ typedef struct t_csoundapi_ {
   char     *csmess;
   t_symbol *curdir;
   midi_queue *mq;
+  char *orc;
 } t_csoundapi;
 
 static int set_channel_value(t_csoundapi *x, t_symbol *channel, MYFLT value);
@@ -79,9 +81,9 @@ static MYFLT get_channel_value(t_csoundapi *x, char *channel);
 static channelname *create_channel(channelname *ch, char *channel);
 static void destroy_channels(channelname *ch);
 static void in_channel_value_callback(CSOUND *csound,
-                                      const char *name, MYFLT *val);
+                                      const char *name, void *val, const void *channelType);
 static void out_channel_value_callback(CSOUND *csound,
-                                       const char *name, MYFLT val);
+                                       const char *name, void *val, const void *channelType);
 static void csoundapi_event(t_csoundapi *x, t_symbol *s,
                             int argc, t_atom *argv);
 static void csoundapi_run(t_csoundapi *x, t_floatarg f);
@@ -95,7 +97,7 @@ static void csoundapi_dsp(t_csoundapi *x, t_signal **sp);
 static t_int *csoundapi_perform(t_int *w);
 static void csoundapi_channel(t_csoundapi *x, t_symbol *s,
                               int argc, t_atom *argv);
-static void csoundapi_control(t_csoundapi *x, t_symbol *s, float f);
+static void csoundapi_control(t_csoundapi *x, t_symbol *s,t_float f);
 static void csoundapi_set_channel(t_csoundapi *x, t_symbol *s, int argc,
                                   t_atom *argv);
 static void message_callback(CSOUND *,int attr, const char *format,va_list valist);
@@ -119,11 +121,13 @@ static int close_midi_callback(CSOUND *cs, void *userData);
 static void csoundapi_tabset(t_csoundapi *x, t_symbol *tab, t_float f);
 static void csoundapi_tabget(t_csoundapi *x, t_symbol *tab, t_float f);
 
+static void csoundapi_compile(t_csoundapi *x,  t_symbol *orc);
 
-PUBLIC void csoundapi_tilde_setup(void)
+
+PUBLIC void csound6_tilde_setup(void)
 {
     csoundapi_class =
-      class_new(gensym("csoundapi~"), (t_newmethod) csoundapi_new,
+      class_new(gensym("csound6~"), (t_newmethod) csoundapi_new,
                 (t_method) csoundapi_destroy, sizeof(t_csoundapi),
                 CLASS_DEFAULT, A_GIMME, 0);
     class_addmethod(csoundapi_class, (t_method) csoundapi_dsp, gensym("dsp"),
@@ -168,6 +172,8 @@ PUBLIC void csoundapi_tilde_setup(void)
                     gensym("touch"), A_DEFFLOAT, A_DEFFLOAT, 0);
     class_addmethod(csoundapi_class, (t_method) csoundapi_bend,
                     gensym("bend"), A_DEFFLOAT, A_DEFFLOAT, 0);
+    class_addmethod(csoundapi_class, (t_method) csoundapi_compile,
+                    gensym("compile"), A_DEFSYMBOL, 0);
 
     CLASS_MAINSIGNALIN(csoundapi_class, t_csoundapi, f);
 
@@ -177,11 +183,11 @@ PUBLIC void csoundapi_tilde_setup(void)
       v3 = v1 % 10;
       v2 = (v1 / 10) % 100;
       v1 = v1 / 1000;
-      post("\ncsoundapi~ 1.01\n"
+      post("\ncsound6~ 1.01\n"
            " A PD csound class using the Csound %d.%02d.%d API\n"
            "(c) V Lazzarini, 2005-2007\n", v1, v2, v3);
     }
-    csoundInitialize(NULL,NULL,0);
+    /* csoundInitialize(NULL,NULL,0); */
 
 }
 
@@ -199,8 +205,7 @@ int isCsoundFile(char *in)
     }
     if (*in == '.') {
       for(i=0; i<6;i++)
-        if (strcmp(in,extensions[i])==0);
-      return 1;
+        if (strcmp(in,extensions[i])==0) return 1;
     }
     return 0;
 }
@@ -214,6 +219,7 @@ static void *csoundapi_new(t_symbol *s, int argc, t_atom *argv)
 
     x->csound = (CSOUND *) csoundCreate(x);
     outlet_new(&x->x_obj, gensym("signal"));
+    x->orc = NULL;
     x->numlets = 1;
     x->result = 1;
     x->run = 1;
@@ -227,7 +233,8 @@ static void *csoundapi_new(t_symbol *s, int argc, t_atom *argv)
     if (argc == 1 && argv[0].a_type == A_FLOAT) {
       x->numlets = (t_int) atom_getfloat(&argv[0]);
       for (i = 1; i < x->numlets && i < CS_MAX_CHANS; i++) {
-        inlet_new(&x->x_obj, &x->x_obj.ob_pd, gensym("signal"), gensym("signal"));
+        inlet_new(&x->x_obj, &x->x_obj.ob_pd, gensym("signal"),
+                  gensym("signal"));
         outlet_new(&x->x_obj, gensym("signal"));
       }
     }
@@ -241,37 +248,36 @@ static void *csoundapi_new(t_symbol *s, int argc, t_atom *argv)
 #ifndef WIN32
           if(*cmdl[i] != '/')
 #else
-            if(cmdl[i][1] != ':')
+          if(cmdl[i][1] != ':')
 #endif
-              {
-                char *tmp = cmdl[i];
-                cmdl[i] = 
-                  (char *)  malloc(strlen(tmp) + strlen(x->curdir->s_name) + 2);
-                strcpy(cmdl[i], x->curdir->s_name);
-                strcat(cmdl[i],"/");
-                strcat(cmdl[i],tmp);
-                post(cmdl[i]);
-                free(tmp);
-              }
+            {
+              char *tmp = cmdl[i];
+              cmdl[i] =
+                (char *)  malloc(strlen(tmp) + strlen(x->curdir->s_name) + 2);
+              strcpy(cmdl[i], x->curdir->s_name);
+              strcat(cmdl[i],"/");
+              strcat(cmdl[i],tmp);
+              post(cmdl[i]);
+              free(tmp);
+            }
         }
         post(cmdl[i]);
       }
       cmdl[i] = "-d";
       x->argnum = argc + 2;
       x->cmdl = cmdl;
-      x->result = csoundPreCompile(x->csound);
-      if (x->result == CSOUND_SUCCESS) {
-        csoundSetHostImplementedAudioIO(x->csound, 1, 0);
-        csoundSetInputValueCallback(x->csound, in_channel_value_callback);
-        csoundSetOutputValueCallback(x->csound, out_channel_value_callback);
 
-        csoundSetExternalMidiInOpenCallback(x->csound, open_midi_callback);
-        csoundSetExternalMidiReadCallback(x->csound, read_midi_callback);
-        csoundSetExternalMidiInCloseCallback(x->csound, close_midi_callback);
+      csoundSetHostImplementedAudioIO(x->csound, 1, 0);
+      csoundSetInputChannelCallback(x->csound, in_channel_value_callback);
+      csoundSetOutputChannelCallback(x->csound, out_channel_value_callback);
+
+      csoundSetExternalMidiInOpenCallback(x->csound, open_midi_callback);
+      csoundSetExternalMidiReadCallback(x->csound, read_midi_callback);
+      csoundSetExternalMidiInCloseCallback(x->csound, close_midi_callback);
+
+      csoundSetMessageCallback(x->csound, message_callback);
+      x->result = csoundCompile(x->csound, x->argnum, cmdl);
 
-        csoundSetMessageCallback(x->csound, message_callback);
-        x->result = csoundCompile(x->csound, x->argnum, cmdl);
-      }
 
       if (!x->result) {
         x->end = 0;
@@ -287,7 +293,7 @@ static void *csoundapi_new(t_symbol *s, int argc, t_atom *argv)
         x->pos = 0;
       }
       else
-        post("csoundapi~ warning: could not compile");
+        post("csound6~ warning: could not compile");
     }
     x->ctlout = outlet_new(&x->x_obj, gensym("list"));
     x->bangout = outlet_new(&x->x_obj, gensym("bang"));
@@ -319,7 +325,7 @@ static void csoundapi_dsp(t_csoundapi *x, t_signal **sp)
       dsp_add((t_perfroutine) csoundapi_perform, 1, x);
     }
     else
-      post("csoundapi~ warning: orchestra not compiled");
+      post("csound6~ warning: orchestra not compiled");
 }
 
 static t_int *csoundapi_perform(t_int *w)
@@ -335,7 +341,7 @@ static t_int *csoundapi_perform(t_int *w)
     t_sample *out[CS_MAX_CHANS], *in[CS_MAX_CHANS];
     t_int   i, n, end = x->end, run = x->run;
     MYFLT  *csout, *csin;
-
+ 
     csout = csoundGetSpout(x->csound);
     csin = csoundGetSpin(x->csound);
 
@@ -376,8 +382,9 @@ static t_int *csoundapi_perform(t_int *w)
 static void csoundapi_event(t_csoundapi *x, t_symbol *s, int argc, t_atom *argv)
 {
     char    type[10];
-    MYFLT   pFields[64];
+    MYFLT   *pFields;
     int     num = argc - 1, i;
+    pFields  = (MYFLT *) malloc(num*sizeof(MYFLT));
 
     if (!x->result) {
       atom_string(&argv[0], type, 10);
@@ -389,10 +396,11 @@ static void csoundapi_event(t_csoundapi *x, t_symbol *s, int argc, t_atom *argv)
         x->end = 0;
       }
       else
-        post("csoundapi~ warning: invalid realtime score event");
+        post("csound6~ warning: invalid realtime score event");
     }
     else
-      post("csoundapi~ warning: not compiled");
+      post("csound6~ warning: not compiled");
+    free(pFields);
 }
 
 static void csoundapi_reset(t_csoundapi *x)
@@ -405,16 +413,11 @@ static void csoundapi_reset(t_csoundapi *x)
       }
 
       csoundReset(x->csound);
-      x->result = csoundPreCompile(x->csound);
-      if (x->result == CSOUND_SUCCESS) {
-        csoundSetHostImplementedAudioIO(x->csound, 1, 0);
-
-        csoundSetExternalMidiInOpenCallback(x->csound, open_midi_callback);
-        csoundSetExternalMidiReadCallback(x->csound, read_midi_callback);
-        csoundSetExternalMidiInCloseCallback(x->csound, close_midi_callback);
-
-        x->result = csoundCompile(x->csound, x->argnum, x->cmdl);
-      }
+      csoundSetHostImplementedAudioIO(x->csound, 1, 0);
+      csoundSetExternalMidiInOpenCallback(x->csound, open_midi_callback);
+      csoundSetExternalMidiReadCallback(x->csound, read_midi_callback);
+      csoundSetExternalMidiInCloseCallback(x->csound, close_midi_callback);
+      x->result = csoundCompile(x->csound, x->argnum, x->cmdl);
 
       if (!x->result) {
         x->end = 0;
@@ -435,7 +438,7 @@ static void csoundapi_rewind(t_csoundapi *x)
       x->cleanup = 1;
     }
     else
-      post("csoundapi~ warning: not compiled");
+      post("csound6~ warning: not compiled");
 }
 
 static void csoundapi_open(t_csoundapi *x, t_symbol *s, int argc, t_atom *argv)
@@ -461,34 +464,32 @@ static void csoundapi_open(t_csoundapi *x, t_symbol *s, int argc, t_atom *argv)
 #ifndef WIN32
         if(*cmdl[i] != '/')
 #else
-          if(cmdl[i][1] != ':')
+        if(cmdl[i][1] != ':')
 #endif
-            {
-              char *tmp = cmdl[i];
-              cmdl[i] =
-                (char *)  malloc(strlen(tmp) + strlen(x->curdir->s_name) + 2);
-              strcpy(cmdl[i], x->curdir->s_name);
-              strcat(cmdl[i],"/");
-              strcat(cmdl[i],tmp);
-              post(cmdl[i]);
-              free(tmp);
-            }
+          {
+            char *tmp = cmdl[i];
+            cmdl[i] =
+              (char *)  malloc(strlen(tmp) + strlen(x->curdir->s_name) + 2);
+            strcpy(cmdl[i], x->curdir->s_name);
+            strcat(cmdl[i],"/");
+            strcat(cmdl[i],tmp);
+            post(cmdl[i]);
+            free(tmp);
+          }
       }
       post(cmdl[i]);
     }
     cmdl[i] = "-d";
     x->argnum = argc + 2;
     x->cmdl = cmdl;
-    x->result = csoundPreCompile(x->csound);
-    if (x->result == CSOUND_SUCCESS) {
-      csoundSetHostImplementedAudioIO(x->csound, 1, 0);
+    csoundSetHostImplementedAudioIO(x->csound, 1, 0);
 
-      csoundSetExternalMidiInOpenCallback(x->csound, open_midi_callback);
-      csoundSetExternalMidiReadCallback(x->csound, read_midi_callback);
-      csoundSetExternalMidiInCloseCallback(x->csound, close_midi_callback);
+    csoundSetExternalMidiInOpenCallback(x->csound, open_midi_callback);
+    csoundSetExternalMidiReadCallback(x->csound, read_midi_callback);
+    csoundSetExternalMidiInCloseCallback(x->csound, close_midi_callback);
+    x->run = 0;
+    x->result = csoundCompile(x->csound, x->argnum, cmdl);
 
-      x->result = csoundCompile(x->csound, x->argnum, cmdl);
-    }
 
     if (!x->result) {
       x->end = 0;
@@ -498,12 +499,13 @@ static void csoundapi_open(t_csoundapi *x, t_symbol *s, int argc, t_atom *argv)
       x->pos = 0;
       csoundSetHostData(x->csound, x);
       if (x->chans != x->numlets)
-        post("csoundapi~ warning: number of orchestra channels (%d)\n"
+        post("csound6~ warning: number of orchestra channels (%d)\n"
              "does not match number of PD in/outlets (%d)\n"
              "some channels will be muted", x->chans, x->numlets);
+      x->run = 1;
     }
     else
-      post("csoundapi~ warning: could not compile");
+      post("csound6~ warning: could not compile");
 }
 
 static void csoundapi_run(t_csoundapi *x, t_floatarg f)
@@ -530,10 +532,10 @@ static void csoundapi_tabset(t_csoundapi *x, t_symbol *tab, t_float f)
         }
       }
       else {
-        post ("csoundapi~: could not find array\n");
+        post ("csound6~: could not find array\n");
         return;
       }
-    } else post("csoundapi~: csound table %d not found \n", (int) f);
+    } else post("csound6~: csound table %d not found \n", (int) f);
 }
 
 static void csoundapi_tabget(t_csoundapi *x,  t_symbol *tab, t_float f)
@@ -554,10 +556,62 @@ static void csoundapi_tabget(t_csoundapi *x,  t_symbol *tab, t_float f)
         garray_redraw(pdarray);
       }
       else {
-        post ("csoundapi~: could not find array\n");
+        post ("csound6~: could not find array\n");
         return;
       }
-    } else post("csoundapi~: csound table %d not found \n", (int) f);
+    } else post("csound6~: csound table %d not found \n", (int) f);
+}
+
+void *thread_func(void *p){
+    t_csoundapi *pp = (t_csoundapi *) p;
+    int size = 0;
+    char c;
+    char *orc, *orcfile;
+    FILE *fp;
+
+#ifndef WIN32
+    if(*(pp->orc) != '/')
+#else
+      if(*(pp->orc) != ':')
+#endif
+        {
+          orcfile =
+            (char *)  malloc(strlen(pp->orc) + strlen(pp->curdir->s_name) + 2);
+          strcpy(orcfile, pp->curdir->s_name);
+          strcat(orcfile,"/");
+          strcat(orcfile,pp->orc);
+        }
+      else orcfile = pp->orc;
+    post(orcfile);
+
+    fp = fopen(orcfile, "rb");
+    if(fp != NULL) {
+      while(!feof(fp))
+        size += fread(&c,1,1,fp);
+
+      if(size==0) {
+        fclose(fp);
+        return NULL;
+      }
+
+      orc = (char *) malloc(size+1);
+      fseek(fp, 0, SEEK_SET);
+      fread(orc,1,size,fp);
+      csoundCompileOrc(pp->csound, orc);
+      fclose(fp);
+      free(orc);
+    } else post("csound6~: could not open %s \n", pp->orc);
+
+    free(orcfile);
+    return NULL;
+}
+
+
+static void csoundapi_compile(t_csoundapi *x,  t_symbol *orc) {
+    pthread_t thread;
+    if(x->orc != NULL) free(x->orc);
+    x->orc = strdup(orc->s_name);
+    pthread_create(&thread, NULL, thread_func, x);
 }
 
 
@@ -634,26 +688,29 @@ static void csoundapi_channel(t_csoundapi *x, t_symbol *s,
     }
 }
 
-static void csoundapi_control(t_csoundapi *x, t_symbol *s, float f)
+static void csoundapi_control(t_csoundapi *x, t_symbol *s, t_float f)
 {
     if (!set_channel_value(x, s, f))
       post("channel not found");
 }
 
 static void in_channel_value_callback(CSOUND *csound,
-                                      const char *name, MYFLT *val)
+                                      const char *name, void *valp,
+                                      const void *channelType)
 {
     t_csoundapi *x = (t_csoundapi *) csoundGetHostData(csound);
+    MYFLT *vp = (MYFLT *) valp;
+    *vp = get_channel_value(x, (char *) name);
 
-    *val = get_channel_value(x, (char *) name);
 }
 
 static void out_channel_value_callback(CSOUND *csound,
-                                       const char *name, MYFLT val)
+                                       const char *name, void *valp,
+                                       const void *channelType)
 {
     t_atom  at[2];
     t_csoundapi *x = (t_csoundapi *) csoundGetHostData(csound);
-
+    MYFLT val = *((MYFLT *) valp);
     SETFLOAT(&at[1], (t_float) val);
     SETSYMBOL(&at[0], gensym((char *) name));
     outlet_list(x->ctlout, gensym("list"), 2, at);
@@ -661,24 +718,19 @@ static void out_channel_value_callback(CSOUND *csound,
 
 static void csoundapi_set_channel(t_csoundapi *x, t_symbol *s,
                                   int argc, t_atom *argv){
-    CSOUND *p = x->csound;
-    MYFLT *pval;
+    CSOUND *csound = x->csound;
     int i;
     char chn[64];
-    int len = csoundGetStrVarMaxLen(p);
 
     for(i=0; i < argc; i+=2){
       atom_string(&argv[i],chn,64);
       if(i+1 < argc){
         if(argv[i+1].a_type == A_SYMBOL) {
-          csoundGetChannelPtr(p,&pval,chn,
-                              CSOUND_STRING_CHANNEL | CSOUND_INPUT_CHANNEL);
-          atom_string(&argv[i+1],(char *)pval,len);
+          t_symbol *mess = atom_getsymbol(&argv[i+1]);
+          csoundSetStringChannel(csound, chn, mess->s_name);
         }
         else if (argv[i+1].a_type == A_FLOAT) {
-          csoundGetChannelPtr(p,&pval,chn,
-                              CSOUND_CONTROL_CHANNEL | CSOUND_INPUT_CHANNEL);
-          *pval = atom_getfloat(&argv[i+1]);
+          csoundSetControlChannel(csound, chn, atom_getfloat(&argv[i+1]));
         }
       }
     }
@@ -706,10 +758,6 @@ static void message_callback(CSOUND *csound,
 }
 
 
-
-
-
-
 static int open_midi_callback(CSOUND *cs, void **userData, const char *dev)
 {
     t_csoundapi *x = (t_csoundapi *) csoundGetHostData(cs);
@@ -762,7 +810,8 @@ static int read_midi_callback(CSOUND *cs, void *userData,
 #define MIDI_COMMON                                                     \
   midi_queue *mq = x->mq;                                               \
   if (mq == NULL) {                                                     \
-    post("WARNING: midi disabled (launch with '-+rtmidi=null -M0' to enable)"); \
+    post("WARNING: midi disabled (launch with"                          \
+         " '-+rtmidi=null -M0' to enable)");                            \
     return;                                                             \
   }                                                                     \
   if (!canvas_dspstate) {                                               \
@@ -773,12 +822,12 @@ static int read_midi_callback(CSOUND *cs, void *userData,
   unsigned char *val = mq->values;                                      \
   int mm;
 
-#define MIDI_COMMAND(N, M)                                              \
-  mm = ((int) N) - 1;                                                   \
-  if ((mm & 0x0f) != mm) {                                              \
-                          error("midi channel out of range: %d", mm + 1); \
-                          return;                                       \
-                          }                                             \
+#define MIDI_COMMAND(N, M)                              \
+  mm = ((int) N) - 1;                                   \
+  if ((mm & 0x0f) != mm) {                              \
+    error("midi channel out of range: %d", mm + 1);     \
+    return;                                             \
+  }                                                     \
   val[wp++] = (mm | M);
 
 #define MIDI_DATA(N, M, S)                      \
